var OData;
(function (OData) {
    OData.WARMUP = "$warmup";
    OData.LOGOUT = "$logout";
    OData.RESET_MODEL = "$resetModel";
    OData.REQUEST_TWO_FACTOR_TOKEN = "$requestTwoFactorToken";
    OData.VALIDATE_TWO_FACTOR_TOKEN = "$validateTwoFactorToken";
    var ODataClient = /** @class */ (function () {
        function ODataClient(baseUrl, username, password) {
            this.baseUrl = baseUrl;
            this.username = username;
            if (username) {
                this.authenticationHash = Base64.encode(username + ":" + password);
            }
        }
        ODataClient.prototype.preflight = function (callback) {
            var xmlHttp = new XMLHttpRequest();
            xmlHttp.open("GET", this.baseUrl.split("/api")[0], false);
            xmlHttp.withCredentials = false;
            xmlHttp.send(null);
            callback();
        };
        ODataClient.prototype.authenticate = function () {
            var url = Utils.CombineUrl(this.baseUrl, OData.WARMUP);
            var deferred = jQuery.Deferred(), xmlHttp = new XMLHttpRequest();
            xmlHttp.open("GET", url, true);
            // Some WS implementations (i.e. BEA WebLogic Server 10.0 JAX-WS) don't support Challenge/Response HTTP BASIC, so we send authorization headers in the first request
            if (this.authenticationHash) {
                xmlHttp.setRequestHeader("Authorization", "Basic " + this.authenticationHash);
            }
            xmlHttp.withCredentials = true;
            var appl = Model.TSFApplicationManager.Current.activeAppl;
            xmlHttp.timeout = appl != null ? appl.extendedProperties.defaultAuthenticationTimeout : AppConfig.defaultAuthenticationTimeout;
            xmlHttp.onerror = xmlHttp.ontimeout = function () {
                deferred.reject(Model.TranslKeys.noServerConnection);
            };
            xmlHttp.onload = function (e) {
                if (xmlHttp.readyState == 4) {
                    if (xmlHttp.status === 204) {
                        deferred.resolve();
                    }
                    else if (xmlHttp.status === 401 && xmlHttp.getResponseHeader("X-2FA")) {
                        deferred.reject({
                            TWOFA: xmlHttp.getResponseHeader("X-2FA"),
                            TWOFAFallbacks: xmlHttp.getResponseHeader("X-2FA-Fallback-Types")
                        }, RequestStatus.TWO_FACTOR_AUTH_REQUESTED_REASON);
                    }
                    else if (xmlHttp.status === 403 && xmlHttp.statusText === RequestStatus.EXPIRED_REASON) {
                        deferred.reject(Model.TranslKeys.loginFailPasswordExpired, RequestStatus.EXPIRED_REASON);
                    }
                    else if (xmlHttp.status >= 400 && xmlHttp.status < 600 && xmlHttp.status != 401) {
                        deferred.reject([Model.TranslKeys.serverInvalidResponse, null, [["[status_code]", xmlHttp.status + ""]]]);
                    }
                    else {
                        deferred.reject(Model.TranslKeys.loginFail);
                    }
                }
            };
            try {
                this.preflight(function () {
                    xmlHttp.send(null);
                });
            }
            catch (error) {
                deferred.reject(Model.TranslKeys.loginError);
            }
            // When the password is wrong, iOS won't throw a timeout and xmlHttp.readystate is 1
            // Just wait till after the timer should have ended and throw a password error
            // On browser which don't support timeout, show the no server connection message
            var timer = window.setTimeout(function () {
                if ('timeout' in xmlHttp) {
                    deferred.reject(Model.TranslKeys.loginFail);
                }
                else {
                    deferred.reject(Model.TranslKeys.noServerConnection);
                }
            }, xmlHttp.timeout + 1000);
            deferred.always(function () {
                clearTimeout(timer);
            });
            return deferred.promise();
        };
        ODataClient.prototype.invoke = function (relativeUri, method, parameters, callback) {
            var _this = this;
            var url = Utils.CombineUrl(this.baseUrl, relativeUri);
            var xmlHttp = new XMLHttpRequest();
            xmlHttp.open(method, url, true);
            xmlHttp.withCredentials = true;
            // Some WS implementations (i.e. BEA WebLogic Server 10.0 JAX-WS) don't support Challenge/Response HTTP BASIC, so we send authorization headers in the first request
            if (this.authenticationHash) {
                xmlHttp.setRequestHeader("Authorization", "Basic " + this.authenticationHash);
            }
            xmlHttp.setRequestHeader("Content-Type", "application/json;charset=UTF-8");
            xmlHttp.setRequestHeader("Accept", "application/json;odata.metadata=none;");
            var appl = Model.TSFApplicationManager.Current.activeAppl;
            var obj = parameters && parameters.parameters;
            var requestBody = (obj && Object.keys(obj).length > 0 ? Utils.serializeJSON(obj) : "");
            // Timeout is only available for async XMLHttpRequests.
            xmlHttp.timeout = appl != null ? appl.extendedProperties.defaultRequestTimeout : AppConfig.defaultRequestTimeout;
            xmlHttp.onreadystatechange = function () {
                if (xmlHttp.readyState == 4)
                    _this.onSendODataRequest(method, xmlHttp, requestBody, callback);
            };
            try {
                xmlHttp.send(requestBody);
            }
            catch (error) {
                callback(null, RequestStatus.createFromXMLHttpRequest(xmlHttp, null));
            }
            return xmlHttp;
        };
        ODataClient.prototype.onSendODataRequest = function (method, xmlHttp, requestBody, callback) {
            var reader = null;
            if (xmlHttp.responseText && !xmlHttp.responseXML) {
                var result = void 0;
                try {
                    result = Utils.deserializeJSON(xmlHttp.responseText);
                }
                catch (e) {
                    callback(null, RequestStatus.createFromXMLHttpRequest(xmlHttp, reader));
                    return;
                }
                // TODO: Move this to the requestbuilder.get method (and maybe check the get_gui as it is calling the invoke directly)
                if (Array.isArray(result.value)) {
                    result = result.value;
                }
                else {
                    result = [result];
                }
                reader = new IndexedDb.IndexedDbArrayReader(result);
            }
            if (callback) {
                callback(reader, RequestStatus.createFromXMLHttpRequest(xmlHttp, reader));
            }
        };
        ODataClient.prototype.createParameterCollection = function () {
            return new ODataParameterCollection();
        };
        return ODataClient;
    }());
    OData.ODataClient = ODataClient;
    var ODataParameterCollection = /** @class */ (function () {
        function ODataParameterCollection() {
            this.parameters = {};
        }
        ODataParameterCollection.prototype.add = function (name, value) {
            this.parameters[name] = value;
            return this;
        };
        ODataParameterCollection.prototype.get = function (name) {
            return this.parameters[name];
        };
        ODataParameterCollection.prototype.clone = function () {
            var ret = new ODataParameterCollection();
            ret.parameters = jQuery.extend(true, {}, this.parameters);
            return ret;
        };
        return ODataParameterCollection;
    }());
    var RequestBuilder = /** @class */ (function () {
        function RequestBuilder(application, contractId) {
            this.application = application;
            this.contractId = contractId;
            this.paramUrl = undefined;
            this.callbacks = [];
            this.request = null;
            this.serviceUrl = this.contractId;
            this.params = new ODataParameterCollection();
        }
        RequestBuilder.fromTSFQuery = function (application, query) {
            var builder = new RequestBuilder(application, query.serviceId);
            if (query.usePagination) {
                builder.addPaginationContract(query.pageIndex, query.pageSize);
            }
            builder.addSelectionContract(query.selectClause.selectParts);
            builder.addWhereClause(query.whereClause);
            builder.addSortContract(query.currentSorting);
            builder.addFileOptionsContract({
                IncludeHashCodes: true,
                IncludeFiles: false,
                IncludeThumbnails: false
            });
            return builder;
        };
        RequestBuilder.prototype.setCallbackMethod = function (callback) {
            this.callbacks.push(callback);
            return this;
        };
        RequestBuilder.prototype.invoke = function (method) {
            var deferred = jQuery.Deferred();
            var relativeUrl = this.serviceUrl;
            if (this.paramUrl)
                relativeUrl += this.paramUrl;
            this.request = this.application.oDataClient.invoke(relativeUrl, method, this.params, function (reader, requestStatus) {
                if (!requestStatus.requestSuccess && requestStatus.request.statusText == RequestStatus.TWO_FACTOR_AUTH_REQUESTED_REASON) {
                    jQuery.mobile.pageContainer.pagecontainer("change", "twoFactorValidation.html");
                }
                deferred.resolve(reader, requestStatus);
            });
            return deferred.promise();
        };
        RequestBuilder.prototype.abortRequest = function () {
            if (this.request !== null) {
                this.request.aborted = true;
                this.request.abort();
                this.emitCallbacks(null, new RequestStatus(false));
            }
        };
        RequestBuilder.prototype.emitCallbacks = function (reader, status) {
            for (var i = 0; i < this.callbacks.length; i++) {
                var callback = this.callbacks[i];
                callback(reader, status);
            }
            // Clear callback list for this call.
            this.callbacks = [];
        };
        RequestBuilder.prototype.addSelectionContract = function (columns) {
            if (columns.length > 0) {
                this.addUrlParameter("$select", columns.join(","));
            }
            return this;
        };
        RequestBuilder.prototype.addFilterContract = function (filter) {
            if (filter.subContracts.length > 0) {
                throw "invalid filter contract because it contains subContracts";
            }
            var result = "";
            var addAnd = false;
            for (var _i = 0, _a = filter.conditions; _i < _a.length; _i++) {
                var condition = _a[_i];
                if (addAnd)
                    result += " and ";
                addAnd = true;
                result += buildFilterCondition(condition);
            }
            this.addUrlParameter("$filter", result);
            return this;
        };
        RequestBuilder.prototype.addWhereClause = function (whereClause) {
            var result = "";
            if (whereClause.contextFilters.length > 0) {
                result += this.appendFilterConditions(whereClause.contextFilters, "and");
            }
            if (whereClause.combinedFilterConditions.length > 0) {
                if (result != "") {
                    result += " and ";
                }
                result += "(";
                result += this.appendFilterConditions(whereClause.combinedFilterConditions, "or");
                result += ")";
            }
            if (whereClause.userFilters.length > 0) {
                if (result != "") {
                    result += " and ";
                }
                result += this.appendFilterConditions(whereClause.userFilters, "and");
            }
            if (whereClause.orredConditions.length > 0) {
                if (result != "") {
                    result += " and ";
                }
                result += "(";
                var addOr = false;
                for (var _i = 0, _a = whereClause.orredConditions; _i < _a.length; _i++) {
                    var orredConditionList = _a[_i];
                    if (orredConditionList.length > 0) {
                        if (addOr)
                            result += " or ";
                        result += this.appendFilterConditions(orredConditionList, "and");
                        addOr = true;
                    }
                }
                result += ")";
            }
            if (result != "") {
                this.addUrlParameter("$filter", result);
            }
            if (whereClause.prefilterState && whereClause.prefilterState.length > 0) {
                var prefilterResult = whereClause.prefilterState.filter(function (s) { return s.selected; }).map(function (x) { return x.prefilter.prefilterId; }).join(",");
                if (prefilterResult != "") {
                    this.addUrlParameter("$prefilter", prefilterResult);
                }
            }
            return this;
        };
        RequestBuilder.prototype.appendFilterConditions = function (filterConditions, conditionOperator) {
            var result = "";
            var addOperator = false;
            for (var _i = 0, filterConditions_1 = filterConditions; _i < filterConditions_1.length; _i++) {
                var filter = filterConditions_1[_i];
                if (addOperator)
                    result += " " + conditionOperator + " ";
                result += buildFilterCondition(filter);
                addOperator = true;
            }
            return result;
        };
        RequestBuilder.prototype.addSortContract = function (columns) {
            if (columns !== null && columns.length > 0) {
                this.addUrlParameter("$orderby", columns
                    .sort(function (c1, c2) { return c1.sortOrder - c2.sortOrder; })
                    .map(function (c) { return c.name + " " + (c.direction == 1 ? "desc" : "asc"); })
                    .join(","));
            }
            return this;
        };
        RequestBuilder.prototype.addPaginationContract = function (page, pageSize) {
            this.addUrlParameter("$top", pageSize + "");
            this.addUrlParameter("$skip", (page * pageSize) + "");
            return this;
        };
        RequestBuilder.prototype.addFileOptionsContract = function (options) {
            this.addUrlParameter("$IncludeFiles", options.IncludeFiles + "");
            this.addUrlParameter("$IncludeHashCodes", options.IncludeHashCodes + "");
            if (options.IncludeThumbnails) {
                this.addUrlParameter("$IncludeThumbnails", options.IncludeThumbnails + "");
                this.addUrlParameter("$ThumbnailMaxHeight", options.ThumbnailMaxHeight + "");
                this.addUrlParameter("$ThumbnailMaxWidth", options.ThumbnailMaxWidth + "");
            }
            return this;
        };
        RequestBuilder.prototype.getEmptyReader = function () {
            // Cheat it with an empty array. Length has to return 0;
            return new IndexedDb.IndexedDbArrayReader([]);
        };
        RequestBuilder.prototype.localSync = function (synctoken) {
            var _this = this;
            this.serviceUrl += ".LocalSync";
            this.params.add("synctoken", synctoken || null);
            this.invoke("POST").always(function (reader, status) {
                _this.emitCallbacks(reader, status);
            });
        };
        RequestBuilder.prototype.get = function () {
            var _this = this;
            this.invoke("GET").always(function (reader, status) {
                _this.emitCallbacks(reader, status);
            });
        };
        RequestBuilder.prototype.create = function (newRow) {
            var _this = this;
            for (var _i = 0, _a = newRow.conditions; _i < _a.length; _i++) {
                var condition = _a[_i];
                this.params.add(condition.objProp.objPropId, condition.value);
            }
            this.invoke("POST").always(function (reader, status) {
                _this.emitCallbacks(reader, status);
            });
        };
        RequestBuilder.prototype.put = function (referenceContract, newRow) {
            var _this = this;
            var pkFilter = referenceContract.toPkBookmark().conditions.map(function (value) { return value.oDataUrlName + "=" + serializeValue(value.oDataUrlValue, value.objProp); });
            this.serviceUrl += "(" + pkFilter.join(",") + ")";
            for (var _i = 0, _a = newRow.conditions; _i < _a.length; _i++) {
                var condition = _a[_i];
                this.params.add(condition.objProp.objPropId, condition.value);
            }
            this.invoke("PATCH").always(function (reader, status) {
                _this.emitCallbacks(reader, status);
            });
        };
        // Delete is uppercase, the android browser (on the emulator) gives a syntax error when lowercase because delete is a keyword.
        RequestBuilder.prototype.Delete = function (row) {
            var _this = this;
            var pkFilter = row.toPkBookmark().conditions.map(function (value) { return value.oDataUrlName + "=" + serializeValue(value.oDataUrlValue, value.objProp); });
            this.serviceUrl += "(" + pkFilter.join(",") + ")";
            this.invoke("DELETE").always(function (reader, status) {
                _this.emitCallbacks(reader, status);
            });
        };
        RequestBuilder.prototype.executeLayout = function (layoutContract, layoutmode, importMode) {
            var _this = this;
            this.serviceUrl += ".ExecuteLayout";
            this.params.add("layout_mode", layoutmode);
            this.params.add("import_mode", importMode);
            for (var key in layoutContract) {
                this.params.add(key, layoutContract[key]);
            }
            this.invoke("POST").always(function (reader, status) {
                _this.emitCallbacks(reader, status);
            });
        };
        RequestBuilder.prototype.executeDefault = function (defaultContract, layoutmode, importMode, from) {
            var _this = this;
            this.serviceUrl += ".ExecuteDefault";
            this.params.add("default_mode", layoutmode);
            this.params.add("import_mode", importMode);
            if (from) {
                this.params.add("cursor_from_col_id", from.objPropId);
            }
            for (var key in defaultContract) {
                this.params.add(key, defaultContract[key]);
            }
            this.invoke("POST").always(function (reader, status) {
                _this.emitCallbacks(reader, status);
            });
        };
        RequestBuilder.prototype.executeContext = function (contextContract) {
            var _this = this;
            this.serviceUrl += ".ExecuteContext";
            for (var key in contextContract) {
                this.params.add(key, contextContract[key]);
            }
            this.params.add("active_target_ref_id", null);
            this.invoke("POST").always(function (reader, status) {
                _this.emitCallbacks(reader, status);
            });
        };
        RequestBuilder.prototype.executeBadge = function (objView, badgeContract) {
            var _this = this;
            this.serviceUrl += ".ExecuteBadge";
            if (objView.objViewId != null) {
                this.params.add("variant_id", objView.objViewId);
            }
            for (var key in badgeContract) {
                this.params.add(key, badgeContract[key]);
            }
            this.invoke("POST").always(function (reader, status) {
                _this.emitCallbacks(reader, status);
            });
        };
        RequestBuilder.prototype.executeTask = function (contract) {
            var _this = this;
            for (var key in contract) {
                this.params.add(key, contract[key]);
            }
            this.invoke("POST").always(function (reader, status) {
                _this.emitCallbacks(reader, status);
            });
        };
        RequestBuilder.prototype.addUrlParameter = function (key, value) {
            if (this.paramUrl == undefined) {
                this.paramUrl = "?";
            }
            else {
                this.paramUrl += "&";
            }
            this.paramUrl += key + "=" + value;
        };
        RequestBuilder.prototype.allowCustomIdentitiesInContracts = function () {
            return false;
        };
        return RequestBuilder;
    }());
    OData.RequestBuilder = RequestBuilder;
    function buildFilterCondition(condition) {
        var name = condition.oDataUrlName;
        var comparison = condition.comparision;
        var filterValue = condition.dbFilterValue;
        var filterUntilValue = condition.filterUntilValue;
        var prop = condition.listProp;
        switch (comparison) {
            case Enum.FilterCondition.EqualTo:
            case Enum.FilterCondition.NotEqualTo:
            case Enum.FilterCondition.GreaterThan:
            case Enum.FilterCondition.SmallerThan:
            case Enum.FilterCondition.GreaterThanOrEqualTo:
            case Enum.FilterCondition.SmallerThanOrEqualTo:
            case Enum.FilterCondition.IsNull:
            case Enum.FilterCondition.IsNotNull:
            case Enum.FilterCondition.DateEqualTo:
            case Enum.FilterCondition.DateNotEqualTo:
                return "(" + name + " " + filterConditionToOperator[comparison] + " " + serializeValue(filterValue, prop) + ")";
            case Enum.FilterCondition.Between:
                return "(" + name + " ge " + serializeValue(filterValue, prop) + " and " + name + " le " + serializeValue(filterUntilValue, prop) + ")";
            case Enum.FilterCondition.NotBetween:
                return "(" + name + " lt " + serializeValue(filterValue, prop) + " or " + name + " gt " + serializeValue(filterUntilValue, prop) + ")";
            case Enum.FilterCondition.StartsWith:
            case Enum.FilterCondition.DoesNotStartWith:
                var param = void 0;
                if (typeof filterValue === "number") {
                    param = "startswith(cast(" + name + ", 'Edm.String'), " + serializeValue(filterValue.toString(), prop) + ")";
                }
                else if (typeof filterValue === "string") {
                    param = "startswith(cast(" + name + ", 'Edm.String'), " + serializeValue(filterValue, prop) + ")";
                }
                if (comparison === Enum.FilterCondition.DoesNotStartWith) {
                    param += "eq false";
                }
                return param;
            case Enum.FilterCondition.Contains:
            case Enum.FilterCondition.DoesNotContain:
                if (typeof filterValue === "number") {
                    param = "contains(cast(" + name + ", 'Edm.String'), " + serializeValue(filterValue.toString(), prop) + ")";
                }
                else if (typeof filterValue === "string") {
                    param = "contains(cast(" + name + ", 'Edm.String'), " + serializeValue(filterValue, prop) + ")";
                }
                if (comparison === Enum.FilterCondition.DoesNotContain) {
                    param += "eq false";
                }
                return param;
            default:
                return "";
        }
    }
    var filterConditionToOperator = (function () {
        var result = {};
        result[Enum.FilterCondition.EqualTo] = "eq";
        result[Enum.FilterCondition.NotEqualTo] = "ne";
        result[Enum.FilterCondition.GreaterThan] = "gt";
        result[Enum.FilterCondition.SmallerThan] = "lt";
        result[Enum.FilterCondition.GreaterThanOrEqualTo] = "ge";
        result[Enum.FilterCondition.SmallerThanOrEqualTo] = "le";
        result[Enum.FilterCondition.IsNull] = "eq";
        result[Enum.FilterCondition.IsNotNull] = "ne";
        result[Enum.FilterCondition.DateEqualTo] = "eq";
        result[Enum.FilterCondition.DateNotEqualTo] = "ne";
        return result;
    })();
    var escapeRegex = /'/g;
    function serializeValue(value, objProp) {
        var serializedValue = "";
        if (value === null || value == undefined) {
            serializedValue = "null";
        }
        switch (typeof value) {
            case "string":
                var escapedValue = value.replace(escapeRegex, "''");
                if (objProp.dttpId === Model.DttpId.guid) {
                    serializedValue = escapedValue;
                }
                else {
                    serializedValue = "'" + escapedValue + "'";
                }
                break;
            case "number":
            case "boolean":
                serializedValue = value + "";
                break;
            case "object":
                if (value instanceof Date) {
                    // Encode Date objects as ISO 8601 strings without converting them to UTC time.
                    // Indicium will ignore the timezone, but it is necessary to provide it in order
                    // to abide by the OData v4.0 URL conventions.
                    serializedValue = "" + moment(value).format("YYYY-MM-DDTHH:mm:ss.SSSZ");
                }
                else {
                    console.log("Value that can't be serialized: ");
                    console.log(value);
                    throw "Invalid value, unable to serialize";
                }
                break;
        }
        return encodeURIComponent(serializedValue);
    }
})(OData || (OData = {}));
