jQuery(document).on('pagebeforecreate', '#maintenance', function () {
    var page = jQuery(this);
    var manager = Model.TSFApplicationManager.Current, appl = manager.activeAppl, content = page.children(".ui-content");
    if (!appl || !appl.model) {
        Logger.log("Application model is incomplete", LogType.ERROR);
        Model.TSFApplicationManager.Current.dispose(true);
        jQuery.mobile.pageContainer.pagecontainer("change", jQuery("#login"));
        return;
    }
    // Hide buttons, there aren't any tables for offline so they won't do anything.
    if (!appl.model.containsOfflineObjs) {
        content.find(".sync,.queue,.synclog").css("display", "none");
    }
    // Set black or white icon for the back button.
    var color = Utils.Color.hexToColor(jQuery("a.back-button").css("color"));
    Utils.SetIconBlackOrWhite(jQuery("a.back-button"), color);
    // Set black or white icon for all the other buttons.
    Utils.SetIconBlackOrWhite(jQuery(this).find("button"), color);
    var refreshObjectModel = content.find(".refreshObjectModel");
    refreshObjectModel.text(appl.language.transl(Model.TranslKeys.refreshObjectModel));
    refreshObjectModel.click(function (evt) {
        if (AppConfig.online) {
            manager.activeAppl.serverSync.getUnsyncedAmount().done(function (unsyncedAmount) {
                var questionTransl = appl.language.transl(Model.TranslKeys.refreshObjectModelQuestion);
                if (unsyncedAmount !== 0) {
                    questionTransl += " " + appl.language.transl(Model.TranslKeys.amountOfOfflineChangesLost);
                    questionTransl = questionTransl.replace("[unsyncedAmount]", unsyncedAmount + "");
                }
                var promise = ConfirmationPopup.CreateYesNoPopup().initialize(appl.language.transl(Model.TranslKeys.refreshObjectModel), questionTransl);
                promise.done(function (option) {
                    if (option == Enum.ConfirmationOption.Yes) {
                        refreshObjectModelInternal();
                    }
                });
            });
        }
        else {
            var errorPopup = ConfirmationPopup.CreateOkPopup();
            errorPopup.initialize(appl.language.transl(Model.TranslKeys.refreshObjectModel), appl.language.transl(Model.TranslKeys.deviceOffline));
        }
    });
    function refreshObjectModelInternal() {
        PopupManager.resetQueue();
        var refreshObjectModelFunc = function () {
            var promise = manager.refreshObjectModel();
            promise.done(function () {
                // Use the manager here, because the old appl doesn't exist anymore
                Logger.log(Model.TranslKeys.refreshObjectModel, LogType.LOG);
                if (typeof Raven !== "undefined") {
                    Raven.captureBreadcrumb({
                        message: Model.TranslKeys.refreshObjectModel,
                        category: "action"
                    });
                }
                StyleSheetGenerator.generateFullStyle(manager.activeAppl);
                manager.activeAppl.localSync.fullSynchronize(true)
                    .always(function () {
                    manager.activeAppl.localSync.startLocalSyncInterval();
                });
            });
            promise.fail(function (error) {
                var popup = ConfirmationPopup.CreateOkPopup();
                popup.initialize(AppConfig.language.transl(Model.TranslKeys.refreshObjectModel), error).always(function () {
                    // Dispose everything and logout just like when the user actually logs out.
                    // We can't go back to the menu because the model and data are removed.
                    Model.TSFApplicationManager.Current.dispose(true);
                    jQuery.mobile.pageContainer.pagecontainer("change", jQuery("#login"));
                });
            });
        };
        // Wait for localSync to complete or it will crash (at least on the shim)
        var localSyncPromise = manager.activeAppl.localSync.getSynchronisationPromise();
        if (localSyncPromise.state() === "pending") {
            Utils.showLoader(manager.activeAppl.language.transl(Model.TranslKeys.waitingForSynchronisation));
        }
        localSyncPromise.always(function () {
            refreshObjectModelFunc();
        });
        localSyncPromise.fail(function (error) {
            var popup = ConfirmationPopup.CreateOkPopup();
            var syncErrorText = AppConfig.language.transl(Model.TranslKeys.errorDuringSynchronization);
            if (error) {
                syncErrorText += "\r\n" + error;
            }
            popup.initialize(AppConfig.language.transl(Model.TranslKeys.refreshObjectModel), syncErrorText);
        });
    }
    var download = content.find(".sync");
    download.text(appl.language.transl(Model.TranslKeys.syncData));
    download.click(function (evt) {
        if (AppConfig.online) {
            manager.activeAppl.localSync.fullSynchronize(true);
        }
        else {
            var errorPopup = ConfirmationPopup.CreateOkPopup();
            errorPopup.initialize(appl.language.transl(Model.TranslKeys.syncData), appl.language.transl(Model.TranslKeys.deviceOffline));
        }
    });
    var queue = content.find(".queue");
    queue.text(appl.language.transl(Model.TranslKeys.offlineQueue));
    queue.click(function (evt) {
        jQuery.mobile.pageContainer.pagecontainer("change", "synchronize.html");
    });
    var synclog = content.find(".synclog");
    synclog.text(appl.language.transl(Model.TranslKeys.synclog));
    synclog.click(function (evt) {
        jQuery.mobile.pageContainer.pagecontainer("change", "synclog.html");
    });
    var $changePassword = content.find(".change-password");
    if (AppConfig.enableChangePassword) {
        $changePassword.show().text(appl.language.transl(Model.TranslKeys.changePassword));
        $changePassword.click(function () {
            NavigationManager.navigate("changePassword.html", {
                data: {
                    action: "changePassword"
                }
            });
        });
    }
    var header = page.children("div[data-role='header']");
    var footer = page.children("div[data-role='footer']");
    header.children('h1').text(appl.language.transl(Model.TranslKeys.maintenance));
    // Make sure that the height of the main div is set. Otherwise scrolling on iOS doesn't work.
    page.one("pageshow", function () {
        var height = window.innerHeight;
        height -= header.outerHeight() + footer.outerHeight();
        page.find(".content-panel").css('height', height + 'px');
    });
});
