;
(function ($) {
    var allowCommaInPrecision = (function () {
        var input = document.createElement("input");
        input.type = "number";
        input.value = "1,3";
        // this feature detection doesn't seem to work on iOS where it does work
        return input.value != "" || Utils.getiOSVersion() != null;
    })();
    var getInputValue = function () {
        return this.value || this.textContent;
    };
    var setInputValue = function (value) {
        if (this.value) {
            this.value = value;
        }
        else if (this.textContent) {
            this.textContent = value;
        }
    };
    $.fn.extend({
        /**
        * Forces the value of an input field to be numeric.
        *
        * @return {jQuery}
        */
        numeric: function (hasPrecision, allowNegative) {
            // On some devices it was possible to use the , which we broke with an update of our inputvalidation.js
            // we do allow it on some devices now. Note on some devices when you type a , input.value returns an empty string so we can't fix it
            // we could also automatically change the , to a . but that requires some more work
            var optPrecisionExpr = allowCommaInPrecision ? '(\\.|,)?[\\d]*$' : '\\.?[\\d]*$'; // Optional precision expression for positiveExpr and negativeExpr.
            var expressions = [
                '^([\\d]+)+' + (hasPrecision ? optPrecisionExpr : '$'),
                '^[-]+([\\d]+)*' + (hasPrecision ? optPrecisionExpr : '$'),
                '^\\.{1}[\\d]*$' // Handles edge case for positive values starting with ','.
            ];
            this.data('numValidationExpr', new RegExp(expressions[0] + (allowNegative ? '|' + expressions[1] : '') + (hasPrecision ? '|' + expressions[2] : '')));
            var tmpValue = null;
            this.bind('keydown.numeric', function (e) {
                tmpValue = getInputValue.call(this);
            });
            this.bind('keyup.numeric', function (e) {
                if (!valueIsValid.call(this, getInputValue.call(this))) {
                    setInputValue.call(this, tmpValue);
                }
            });
            var numIsValid = function (str, strict) {
                var strict = (strict !== undefined) ? strict : true;
                var sel = jQuery(this).selectionPos(), before = getInputValue.call(this).substring(0, sel.start), after = getInputValue.call(this).substring(sel.end), newText = before + str + after;
                return valueIsValid.call(this, newText);
            };
            var valueIsValid = function (val) {
                var matchExpr = jQuery(this).data('numValidationExpr');
                return matchExpr.test(val) || val == '';
            };
            this.bind('keypress.numeric', function (e) {
                if (!valueIsValid.call(this, getInputValue.call(this))) {
                    e.preventDefault();
                    return;
                }
                tmpValue = getInputValue.call(this);
                var key = String.fromCharCode(e.which);
                if (!e.ctrlKey && (e.which >= 32) && !numIsValid.apply(this, [key, false])) {
                    e.preventDefault();
                }
                if (e.keyCode == 45) {
                    if (allowNegative) {
                        if (getInputValue.call(this).charAt(0) != '-') {
                            setInputValue.call(this, '-' + getInputValue.call(this));
                        }
                        else {
                            e.preventDefault();
                        }
                    }
                    else {
                        e.preventDefault();
                    }
                }
            });
            this.bind('cut.numeric', function (e) {
                // We use a timer to check the value immediately after it was entered.
                var oldValue = getInputValue.call(this), caretPos = jQuery(this).selectionPos().start;
                var input = this;
                window.setTimeout(function () {
                    if (!numIsValid.call(input, input.value)) {
                        input.value = oldValue;
                        jQuery(input).moveCaretTo(caretPos);
                    }
                }, 0);
            });
            this.bind('paste.numeric', function (e) {
                // We use a timer to check the value immediately after it was entered.
                var oldValue = getInputValue.call(this), caretPos = jQuery(this).selectionPos().start;
                var input = this;
                window.setTimeout(function () {
                    if (!numIsValid.call(input, input.value) || input.value == "") {
                        input.value = oldValue;
                        jQuery(input).moveCaretTo(caretPos);
                    }
                }, 0);
            });
            this.bind('blur.numeric', function (e) {
                if (getInputValue.call(this)) {
                    var value = parseFloat(getInputValue.call(this).replace(',', '.'));
                    if (isNaN(value))
                        value = '';
                    setInputValue.call(this, value + '');
                }
                else {
                    setInputValue.call(this, value + '');
                }
            });
            return this;
        },
        /**
        * Forces the max length of a textarea field's value.
        *
        * @return {jQuery}
        */
        maxLength: function (numeric, len, prec) {
            var reg = (prec !== undefined) ? new RegExp('^[-+]?\\d{0,' + (len - prec) + '}(?:(?:\\.|,)\\d{0,' + prec + '})?$', 'm') : null, timer = null, tmpValue = null, 
            // Defined as var to put it in the correct context(this= the input) and it can be called directly wihout .apply(context, params)
            lenIsValid = function (str) {
                // Ignore group seperators
                var val = str || '';
                return (reg) ? reg.test(val) : (val.length <= len);
            };
            this.bind('cut.maxLength', function (e) {
                var oldValue = getInputValue.call(this);
                var that = this;
                window.setTimeout(function () {
                    if (!lenIsValid(that.value)) {
                        that.value = oldValue;
                    }
                }, 0);
            });
            this.bind('keydown.maxLength', function (e) {
                tmpValue = getInputValue.call(this);
            });
            this.bind('keypress.maxLength', function (e) {
                if (this.type != "number") {
                    handleInput.call(this, e);
                }
                else {
                    if (!lenIsValid(getInputValue.call(this))) {
                        e.preventDefault();
                        return;
                    }
                    tmpValue = getInputValue.call(this);
                }
            });
            this.bind('keyup.maxLength', function (e) {
                if (!lenIsValid.call(this, getInputValue.call(this))) {
                    setInputValue.call(this, tmpValue);
                }
            });
            this.bind('paste.maxLength', function (e) {
                // Use a timer to check the value immediately after it was entered.
                var pos = jQuery(this).selectionPos();
                var input = this, oldValue = getInputValue.call(this), before = getInputValue.call(this).substring(0, pos.start), after = getInputValue.call(this).substring(pos.end);
                if (this.type != "number" && typeof e.originalEvent !== 'undefined' && typeof e.originalEvent.clipboardData !== 'undefined') {
                    var currentLength = before.length + after.length;
                    var clipboardText = e.originalEvent.clipboardData.getData("Text");
                    var diff = len - currentLength;
                    // Calculate available length and strip the rest of the data;
                    // when pasting in between this means the pasted data is cut off - same as the Windows gui does.
                    if (clipboardText.length > diff) {
                        clipboardText = clipboardText.substring(0, diff);
                        input.value = before + clipboardText + after;
                        jQuery(this).moveCaretTo(before.length + clipboardText.length);
                        e.preventDefault();
                        return;
                    }
                    // Latest Firefox and Chrome support this because the originalEvent is a ClipboardEvent.
                    if (!lenIsValid(before + clipboardText + after)) {
                        e.preventDefault();
                    }
                }
                // Always do this, because on some Android devices the clipboardText stuff doesn't work
                window.setTimeout(function () {
                    if (!lenIsValid(input.value) || input.value == "") {
                        input.value = oldValue;
                        jQuery(input).moveCaretTo(pos.end);
                    }
                }, 0);
            });
            this.bind('blur.maxLength', function (e) {
                var input = jQuery(this);
                var value = input.val();
                if (!lenIsValid(value)) {
                    var max = (prec !== undefined) ? (len + prec + 1) : len;
                    input.val(value.substring(0, max));
                }
                if (numeric && value != '' && prec > 0) {
                    var newValue = parseFloat(value.replace(',', '.'));
                    if (isNaN(newValue)) {
                        newValue = '';
                    }
                    else {
                        newValue = newValue.toFixed(prec);
                    }
                    if (value != newValue)
                        input.val(newValue);
                }
            });
            var handleInput = function (e, keydownCode) {
                var val = (getInputValue.call(this) != null) ? getInputValue.call(this) : '', pos = jQuery(this).selectionPos(), before = val.substring(0, pos.start - (keydownCode == 8 && pos.start > 0 ? 1 : 0)), after = val.substring(pos.end + (keydownCode == 46 && val.length > pos.end ? 1 : 0)), c = e.which == undefined || keydownCode ? '' : String.fromCharCode(e.which), value = before + c + after;
                //e.which is undefined if called from a trigger (calculator)
                if (!e.ctrlKey && (e.which >= 32 || keydownCode || e.which == undefined) && !lenIsValid(value)) {
                    e.preventDefault();
                }
            };
            return this;
        },
        /**
        * Moves the cursor relative to its current location.
        *
        * @return {jQuery}
        */
        moveCaret: function (rel) {
            this.each(function () {
                var pos = jQuery(this).selectionPos();
                try {
                    // Other browsers and IE9
                    if (typeof this.selectionStart == "number" && typeof this.selectionEnd == "number") {
                        this.selectionStart = this.selectionEnd = pos.start + rel >= 0 ? pos.start + rel : 0;
                    }
                    else if (this.createTextRange) {
                        var range = this.createTextRange();
                        range.move('character', pos.start + rel >= 0 ? pos.start + rel : 0);
                        range.select();
                    }
                }
                catch (e) {
                    // doesn't work in input type=number
                }
            });
            return this;
        },
        /**
        * Moves the cursor to the specified location.
        *
        * @return {jQuery}
        */
        moveCaretTo: function (pos) {
            this.each(function () {
                // Other browsers and IE9
                try {
                    if (typeof this.selectionStart == "number" && typeof this.selectionEnd == "number") {
                        this.selectionStart = this.selectionEnd = pos;
                    }
                    else if (this.createTextRange) {
                        var range = this.createTextRange();
                        range.move('character', pos);
                        range.select();
                    }
                }
                catch (e) {
                    // doesn't work in input type=number
                }
            });
            return this;
        },
        /**
        * Returns the start and end positions of the selection in an input field.
        *
        * http://javascript.nwbox.com/cursor_position/
        *
        * @return Object containing start and end positions.
        */
        selectionPos: function () {
            var start = 0, end = 0;
            try {
                var input = this[0];
                /*if (this[0]) {
                    if (this[0].createTextRange) {
                        var r = document.selection.createRange();
                        end = r.text.length;
                        r.moveEnd('character', this[0].value.length);
                        start = (r.text == '') ? this[0].value.length : this[0].value.lastIndexOf(r.text);

                        //r.moveStart('character', -this[0].value.length);
                        end = start + end;
                    }
                    else {
                        start = this[0].selectionStart;
                        end = this[0].selectionEnd;
                    }
                }*/
                if (typeof input.selectionStart == "number" && typeof input.selectionEnd == "number") {
                    // Non-IE browsers and IE 9
                    start = input.selectionStart;
                    end = input.selectionEnd;
                }
                else if (document.selection && document.selection.createRange) {
                    // For IE up to version 8
                    var selectionRange = document.selection.createRange();
                    var textInputRange = input.createTextRange();
                    var precedingRange = input.createTextRange();
                    var bookmark = selectionRange.getBookmark();
                    textInputRange.moveToBookmark(bookmark);
                    precedingRange.setEndPoint("EndToStart", textInputRange);
                    start = precedingRange.text.length;
                    end = start + selectionRange.text.length;
                }
            }
            catch (e) {
                // invalid state, happens on electron on input type="number";
                // we can't really do the right thing here because we can't get the cursor position, but the user is probably
                // working at the end of the text
                if (e.code == 11 && input.value) {
                    start = end = input.value.length;
                }
            }
            return {
                start: start,
                end: end
            };
        }
    });
}(jQuery));
