var NavigationManager = /** @class */ (function () {
    function NavigationManager() {
        jQuery(document).ready(function () {
            jQuery.mobile.pageContainer.on("pagebeforechange", function (evt, args) {
                if (args.options.role === "popup"
                    || (args.options.fromPage && args.options.fromPage.data("url") === args.options.pageUrl)) {
                    // don't do anything when a popup is opened, or when the page transitions to the same page.
                    // eg closing the popup (abstractscreen popup for add, edit, copy)
                    return;
                }
                // toPage is a string when the page change is requested. You can cancel it then.
                // NOTE: Closing a popup raises one pagebeforechange with toPage as string.
                if (typeof args.toPage == "string") {
                    if (NavigationManager.currentNavigationAction != null ||
                        // Same fix as below for iOS 9
                        (jQuery.mobile.activePage.is("#login") && (/index.html$/i).test(args.toPage))) {
                        evt.preventDefault();
                    }
                    else {
                        Utils.showLoader();
                    }
                }
                else {
                    // special case the login pgae because on iOS 9 it navigates twice to the index.html causing the gui to hang
                    // Looks something with the change(jQuery(#login)) method (we normally provide a string)
                    // iOS 8 works though, there is something fishy going on with the `navigate/popstate` events
                    // It does work when adding changeHash: false but that breaks the back button
                    if (jQuery.mobile.activePage.is("#login") && args.toPage.is("#login")) {
                        event.preventDefault();
                        return;
                    }
                    // The page is actually loaded when toPage is an object.
                    NavigationManager.currentNavigationAction = NavigationManager.waitForNavigation().always(function () {
                        Utils.hideLoader();
                        NavigationManager.currentNavigationAction = null;
                        jQuery.mobile.pageContainer.off(".navigationManager");
                    });
                }
            });
        });
    }
    NavigationManager.waitForNavigation = function () {
        var deferred = jQuery.Deferred();
        jQuery.mobile.pageContainer.on("pagecontainershow.navigationManager", function (evt) {
            deferred.resolve();
        });
        jQuery.mobile.pageContainer.on("pagecontainerchangefailed.navigationManager", function (evt) {
            deferred.reject();
        });
        return deferred.promise();
    };
    NavigationManager.showMessageWhenReady = function (title, messages, options, defaultOption) {
        var _this = this;
        if (options === void 0) { options = [Enum.ConfirmationOption.Ok]; }
        if (defaultOption === void 0) { defaultOption = Enum.ConfirmationOption.Ok; }
        var deferred = jQuery.Deferred();
        var nonEmptyMessages = messages.filter(function (message) {
            return !Utils.isEmpty(message);
        });
        if (nonEmptyMessages.length === 0) {
            return deferred.resolve().promise();
        }
        var $taskPopup = jQuery(".taskPopup.ui-popup");
        var taskPopup = $taskPopup.data(TaskPopup.TSF_TASK_KEY);
        //Hide an existing taskPopup temporarily because messages have priority (because of validation messages)
        if (taskPopup) {
            taskPopup.closeWithoutRemove();
        }
        PopupManager.queuePopup(deferred, title + messages.join()).then(function () {
            messages.reduce(function (prev, obj, index) {
                return prev.then(function () {
                    if (NavigationManager.currentNavigationAction) {
                        return NavigationManager.currentNavigationAction.then(function () {
                            return _this.showMessage(title, obj, options, defaultOption);
                        });
                    }
                    else {
                        return _this.showMessage(title, obj, options, defaultOption);
                    }
                });
            }, jQuery.Deferred().resolve().promise()).always(function (result) {
                //Show the taskPopup again, with a manual rebind to popupafterclose to resolve the deferred.
                if (taskPopup && !taskPopup.dontReopen) {
                    var deferred1_1 = jQuery.Deferred();
                    PopupManager.queuePopup(deferred1_1, null, true).then(function () {
                        $taskPopup.on("popupafterclose", function () {
                            deferred1_1.resolve();
                        });
                        taskPopup.open();
                    });
                }
                deferred.resolve(result);
            });
        });
        return deferred.promise();
    };
    NavigationManager.showMessage = function (title, message, options, defaultOption) {
        // Hide the loader if there is one.
        Utils.hideLoader();
        var popup = new ConfirmationPopup(options, defaultOption);
        return popup.initialize(title, message);
    };
    NavigationManager.navigate = function (url, options) {
        jQuery.mobile.pageContainer.pagecontainer("change", url, options);
        // Sometimes the slide menu stops working this is easy to reproduce by holding the backbutton on a abstractscreen
        // An internal property of jQueryMobile doesn't reset properly; This is actually just a hack to fix this issue.
        // I couldn't find an easy repro to report though but it happens because context.stop (line 4083) isn't called
        jQuery.event.special.swipe.eventInProgress = false;
    };
    Object.defineProperty(NavigationManager, "isNotNavigating", {
        get: function () {
            return NavigationManager.currentNavigationAction == null;
        },
        enumerable: true,
        configurable: true
    });
    NavigationManager.navigationManager = new NavigationManager();
    return NavigationManager;
}());
