var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var TSFBookmark = /** @class */ (function () {
    function TSFBookmark(obj) {
        this.obj = obj;
        this.conditions = [];
    }
    TSFBookmark.prototype.addCondition = function (condition) {
        this.conditions[condition.objProp.objPropId] = condition;
        this.conditions.push(condition);
    };
    TSFBookmark.prototype.toObject = function () {
        var object = {};
        for (var i = 0; i < this.conditions.length; i++) {
            var condition = this.conditions[i];
            if (!Utils.isEmpty(condition.value)) {
                var name_1 = condition.objProp.objPropId;
                object[name_1] = condition.value;
            }
        }
        return object;
    };
    return TSFBookmark;
}());
var TSFRowBookmark = /** @class */ (function (_super) {
    __extends(TSFRowBookmark, _super);
    function TSFRowBookmark(obj) {
        var _this = _super.call(this, obj) || this;
        _this.obj = obj;
        return _this;
    }
    /**
     * Used in offline code by Borgh but the code was a copy from fromReader
     * Remove after changing the Borgh code
     */
    TSFRowBookmark.create = function (obj, row) {
        return TSFRowBookmark.fromReader(obj, row);
    };
    TSFRowBookmark.fromReader = function (obj, row) {
        var result = new TSFRowBookmark(obj);
        if (!row)
            return result;
        var objProps = obj.objProps.array;
        for (var i = 0; i < objProps.length; i++) {
            var prop = objProps[i];
            var condition = new TSFBookmarkCondition(prop, row.getValueForObjProp(prop));
            result.addCondition(condition);
        }
        return result;
    };
    TSFRowBookmark.fromObject = function (obj, object) {
        var bookmark = new TSFRowBookmark(obj);
        var objProps = obj.objProps.array;
        for (var i = 0; i < objProps.length; i++) {
            var objProp = objProps[i];
            var value = object[objProp.objPropId];
            var condition = void 0;
            // Empty file the same filename means an unmodified file
            // Service tier doesn't check hashcodes in that case
            if (objProp.isFileColumn && value != null && value.File === null) {
                condition = new TSFBookmarkCondition(objProp, FileContract.CreateFilePathOnlyContract(value.FilePath));
            }
            if (!condition) {
                condition = new TSFBookmarkCondition(objProp, object[objProp.objPropId]);
            }
            bookmark.addCondition(condition);
        }
        return bookmark;
    };
    TSFRowBookmark.prototype.matchRowWithBookmark = function (row) {
        if (this.conditions.length == 0 || !row)
            return false;
        for (var i = 0; i < this.conditions.length; i++) {
            var condition = this.conditions[i];
            var objProp = condition.objProp;
            if (objProp.pk) {
                var value = condition.value;
                var rowValue = row.getValueForObjProp(objProp);
                if (!Utils.valuesEqual(value, rowValue)) {
                    return false;
                }
            }
        }
        return true;
    };
    TSFRowBookmark.prototype.toPkBookmark = function () {
        var result = new TSFPkBookmark(this.obj);
        for (var i = 0; i < this.conditions.length; i++) {
            if (this.conditions[i].objProp.pk) {
                result.addCondition(this.conditions[i]);
            }
        }
        return result;
    };
    return TSFRowBookmark;
}(TSFBookmark));
var TSFPkBookmark = /** @class */ (function (_super) {
    __extends(TSFPkBookmark, _super);
    function TSFPkBookmark(obj) {
        var _this = _super.call(this, obj) || this;
        _this.obj = obj;
        _this.isPkBookmark = true;
        return _this;
    }
    TSFPkBookmark.fromReader = function (obj, row) {
        var result = new TSFPkBookmark(obj);
        if (!row)
            return result;
        for (var i = 0; i < obj.pkProps.length; i++) {
            var prop = obj.pkProps[i];
            var condition = new TSFBookmarkCondition(prop, row.getValueForObjProp(prop));
            result.addCondition(condition);
        }
        return result;
    };
    TSFPkBookmark.fromObject = function (obj, object) {
        var bookmark = new TSFPkBookmark(obj);
        var pkProps = obj.pkProps;
        for (var _i = 0, pkProps_1 = pkProps; _i < pkProps_1.length; _i++) {
            var pkProp = pkProps_1[_i];
            if (object[pkProp.objPropId] == undefined) {
                // DvdB: This is a problem, should we error here? we have a PK with null values
                // The old code didn't throw an error here but I suspect it will crash eventually if this problem occurs
                // Might be more of an error for offline code though
            }
            var condition = new TSFBookmarkCondition(pkProp, object[pkProp.objPropId]);
            bookmark.addCondition(condition);
        }
        return bookmark;
    };
    TSFPkBookmark.prototype.matchRowWithBookmark = function (row) {
        if (this.conditions.length == 0 || !row)
            return false;
        for (var i = 0; i < this.conditions.length; i++) {
            var condition = this.conditions[i];
            var value = condition.value;
            var objProp = condition.objProp;
            var rowValue = row.getValueForObjProp(objProp);
            if (!Utils.valuesEqual(value, rowValue)) {
                return false;
            }
        }
        return true;
    };
    return TSFPkBookmark;
}(TSFRowBookmark));
var TSFBookmarkCondition = /** @class */ (function () {
    function TSFBookmarkCondition(objProp, value) {
        this.objProp = objProp;
        this.value = value;
        // All values should be converted to the data type of the corresponding ObjProp,
        // with the exception of identity ObjProp's which are given 'tsf_unique_X' values.
        // These are temporary placeholder values for records that are inserted offline and
        // have a yet to be determined identity value. These values will never be sent to Indicium
        // so the type mismatch is not an issue.
        if (typeof value === "string" && value.indexOf(IdentityReplacer.prefix) === 0) {
            return;
        }
        this.value = Utils.convertToDataType(this.value, this.objProp.objPropId, this.objProp.dttpId);
    }
    Object.defineProperty(TSFBookmarkCondition.prototype, "oDataUrlName", {
        get: function () {
            if (this.objProp.isFileColumn) {
                // TODO: Once file columns can be used in resource paths in Indicium,
                // this will probably have to be something other than the objPropId.
                // This property is already used in all of the right locations.
                return this.objProp.objPropId;
            }
            return this.objProp.objPropId;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TSFBookmarkCondition.prototype, "oDataUrlValue", {
        get: function () {
            if (this.objProp.isFileColumn && this.value instanceof FileContract) {
                return this.value.FilePath;
            }
            return this.value;
        },
        enumerable: true,
        configurable: true
    });
    return TSFBookmarkCondition;
}());
