var TSFCache = /** @class */ (function () {
    function TSFCache() {
        this.disposed = false;
    }
    TSFCache.prototype.initializeDatabase = function () {
        var _this = this;
        if (this.database) {
            return jQuery.Deferred().resolve(false);
        }
        var deferred = jQuery.Deferred(), dbName = TSFCache.cacheDatabase, dbVersion = 3;
        var syncRequired = false;
        var request = IndexedDb.open(dbName, dbVersion);
        request.onupgradeneeded = function (event) {
            var db = event.target.result;
            if (event.oldVersion == 0) {
                db.createObjectStore(TSFCache.modelCacheTable, { keyPath: 'get_gui' });
            }
            if (event.oldVersion < 2) {
                var localSyncStore = db.createObjectStore(TSFCache.localSyncTable, { keyPath: 'tsf_id', autoIncrement: true });
                localSyncStore.createIndex(TSFCache.localSyncIndexName, "ApplId");
                localSyncStore.createIndex(TSFCache.localSyncObjName, "ApplIdObj");
            }
            else if (event.oldVersion == 2) {
                var localSyncStore = request.transaction.objectStore(TSFCache.localSyncTable);
                localSyncStore.createIndex(TSFCache.localSyncObjName, "ApplIdObj");
            }
            syncRequired = true;
        };
        request.onsuccess = function (event) {
            _this.database = request.result;
            deferred.resolve(syncRequired);
        };
        request.onerror = function (event) {
            deferred.reject(Model.TranslKeys.gui_already_running);
        };
        return deferred.promise();
    };
    TSFCache.prototype.dispose = function () {
        this.disposed = true;
        if (this.database) {
            this.database.close();
            this.database = null;
        }
        this.disposed = true;
    };
    TSFCache.cacheDatabase = "tsf_cache";
    TSFCache.modelCacheTable = "tsf_model_cache";
    TSFCache.localSyncTable = "localSync";
    TSFCache.localSyncIndexName = "ApplId";
    TSFCache.localSyncObjName = "ApplIdObj";
    return TSFCache;
}());
