var TSFDataLookupContainer = /** @class */ (function () {
    function TSFDataLookupContainer(application, obj, viewProp) {
        this.application = application;
        this.obj = obj;
        this.viewProp = viewProp;
        this.selectionObjView = viewProp.lookup.targetObjView;
        this.selectionFormattedListField = viewProp.lookup.listProp;
        this.selectionFormattedDataField = viewProp.lookup.dataProp;
        // The getters handle the Continued Joining situation.
        this.translationObjView = viewProp.lookup.getTranslTargetObjView();
        this.translationFormattedListField = viewProp.lookup.getTranslListProp();
        this.translationFormattedDataField = viewProp.lookup.getTranslDataProp();
        this.translationSet = new TSFRecordSet(application, this.translationObjView);
        var translationQuery = this.translationSet.query = new TSFQuery(this.translationObjView.objId);
        translationQuery.selectClause.selectParts = [this.translationFormattedListField.objPropId, this.translationFormattedDataField.objPropId];
        this.selectionSet = new TSFRecordSet(application, this.selectionObjView);
        var selectionQuery = this.selectionSet.query = new TSFQuery(this.selectionObjView.objId);
        var listProp = this.viewProp.lookup.getTranslListProp().objProp;
        // Use a the navigation path when the lookup uses continued joining.
        if (this.viewProp.lookup.hasContinuedJoining()) {
            var targetViewProp = this.selectionFormattedDataField;
            selectionQuery.currentSorting.push({
                name: targetViewProp.objLinkLookupView.objLink.refId + "/" + listProp.objPropId,
                direction: Enum.SortDirection.Ascending,
                sortOrder: 0
            });
        }
        else {
            selectionQuery.currentSorting.push({
                name: listProp.objPropId,
                direction: Enum.SortDirection.Ascending,
                sortOrder: 0
            });
        }
        selectionQuery.selectClause.selectParts = [this.selectionFormattedListField.objPropId, this.selectionFormattedDataField.objPropId];
        for (var j = 0; j < this.viewProp.lookup.filters.length; j++) {
            selectionQuery.selectClause.selectParts.push(this.viewProp.lookup.filters[j].sourceProp.objPropId);
            if (!viewProp.lookup.hasContinuedJoining()) {
                translationQuery.selectClause.selectParts.push(this.viewProp.lookup.filters[j].sourceProp.objPropId);
            }
        }
        selectionQuery.whereClause.prefilterState = viewProp.lookup.targetObjView.getPrefilterState(2 /* Lookup */);
        this.init();
    }
    TSFDataLookupContainer.prototype.init = function () {
        var lookup = this.viewProp.lookup, filters = lookup.filters;
        // When this condition is true the filters are for a different table
        // for example with test_dvdbrink table uur we have a field project_activiteit_id_2. This in turn is a lookup for
        // project_activiteit which in turn references activiteit
        if (filters.length > 0 && !this.viewProp.lookup.hasContinuedJoining()) {
            var translationIndex = IndexedDb.IndexedDbWrapper.lfPrefix + filters.map(function (filter) { return filter.sourceProp.objPropId; }).join("");
            if (this.translationObjView.obj.lookupFilterIndexes[translationIndex]) {
                this.translationSet.query.preferredIndex = translationIndex;
            }
        }
        else {
            var translationIndex = IndexedDb.IndexedDbWrapper.lfPrefix + this.translationFormattedDataField.objPropId;
            if (this.translationObjView.obj.lookupFilterIndexes[translationIndex]) {
                this.translationSet.query.preferredIndex = translationIndex;
            }
        }
        var selectionSetIndex;
        if (filters.length > 0) {
            // Add an index when there are filters, this way the index will filter most rows out of the dataset.
            // case: project->subproject->activity
            selectionSetIndex = IndexedDb.IndexedDbWrapper.lfPrefix + filters.map(function (filter) { return filter.sourceProp.objPropId; }).join("");
        }
        else {
            // A lookup with no filters, this might be bad for performance on large datasets
            // In case the datatype is string, sorting can be done with an index.
            // Because this index is sorted, we can page on the db (we need to sort first or we
            // might load the wrong rows).
            selectionSetIndex = IndexedDb.IndexedDbWrapper.lfPrefix + this.selectionFormattedListField.objPropId;
        }
        if (this.selectionObjView.obj.lookupFilterIndexes[selectionSetIndex]) {
            this.selectionSet.query.preferredIndex = selectionSetIndex;
        }
        // For autocompletion we enable paging
        if (this.viewProp.objLinkLookupView.controlType != 1 /* Combo */) {
            this.selectionSet.query.usePagination = true;
            this.selectionSet.query.pageIndex = 0;
            // Load one extra item, because we need to know if we have more items..
            this.selectionSet.query.pageSize = TSFDataLookupContainer.selectionSetLength + 1;
        }
    };
    TSFDataLookupContainer.prototype.getClonedPrefilterState = function () {
        // create a clone, not a reference
        var state = [];
        var toClone = this.selectionSet.query.whereClause.prefilterState;
        for (var i = 0; i < toClone.length; i++) {
            state.push(toClone[i].clone());
        }
        return state;
    };
    TSFDataLookupContainer.prototype.setPrefilterState = function (prefilter, selected) {
        var prefilterState = this.selectionSet.query.whereClause.prefilterState;
        for (var i = 0; i < prefilterState.length; i++) {
            if (prefilterState[i].prefilter == prefilter) {
                prefilterState[i].selected = selected;
                break;
            }
        }
    };
    TSFDataLookupContainer.prototype.resetPrefilterState = function () {
        var modelState = this.viewProp.lookup.targetObjView.getPrefilterState(2 /* Lookup */);
        var currentState = this.selectionSet.query.whereClause.prefilterState;
        if (modelState.length == currentState.length) {
            for (var i = 0; i < modelState.length; i++) {
                if (modelState[i].selected != currentState[i].selected) {
                    currentState[i].selected = modelState[i].selected;
                    this.selectionSet.invalidate();
                    return true;
                }
            }
        }
        else {
            this.selectionSet.query.whereClause.prefilterState = modelState;
            this.selectionSet.invalidate();
            return true;
        }
        return false;
    };
    TSFDataLookupContainer.prototype.getTranslFromTranslatedReader = function (value, row) {
        var data = this.translatedReader;
        if (!row)
            row = this.contextRow;
        return TSFDataLookupContainer.getTranslFromTranslatedReaderUtil(data, [value], row, [this.translationFormattedDataField.objPropId], this.translationFormattedListField.objPropId);
    };
    TSFDataLookupContainer.getTranslFromTranslatedReaderUtil = function (data, values, row, translationFormattedDataFieldAliases, translationFormattedListField) {
        if (!data || values.length === 0 || !row) {
            return "";
        }
        for (var i = 0; i < data.length; i++) {
            var dataItem = data.item(i);
            var match = true;
            for (var j = 0; j < translationFormattedDataFieldAliases.length; j++) {
                var currDataFieldAlias = translationFormattedDataFieldAliases[j];
                // TODO this doesn't apply on dates.
                if (dataItem.getString(currDataFieldAlias) != values[j]) {
                    match = false;
                }
            }
            if (match) {
                return dataItem.getString(translationFormattedListField);
            }
        }
        return ""; // No translation found
    };
    TSFDataLookupContainer.prototype.getDataFieldProp = function (type) {
        return (type == 0 /* TableReader */ ? this.viewProp : this.selectionFormattedDataField);
    };
    TSFDataLookupContainer.prototype.getListFieldAlias = function (type) {
        if (type == 0 /* TableReader */) {
            return this.translationFormattedListField.objProp.obj.objId + "." + this.translationFormattedListField.objPropId;
        }
        else {
            return this.translationFormattedListField.objPropId;
        }
    };
    /**
    * Translates the given reader, note that the reader isn't always the selection set.
    * It could also be the list of keys in the grid.
    */
    TSFDataLookupContainer.prototype.translateReaderToDisplayValues = function (reader, type) {
        var _this = this;
        var deferred = jQuery.Deferred();
        var that = this;
        if (!reader) {
            return deferred.resolve(null).promise();
        }
        var viewPropFilters = this.viewProp.lookup.filters;
        // Keep track of already added keys
        var keyMap = {};
        // Because this code is used for both lookup readers and the normal table the names might be different
        var propId = this.getDataFieldProp(type).objPropId;
        // Get a list of keys that should be translated.
        var keysList = [];
        for (var i = 0; i < reader.length; i++) {
            var filter = [];
            var item = reader.item(i);
            for (var j = 0; j < viewPropFilters.length; j++) {
                var filterId = type == 1 /* LookupReader */ ? viewPropFilters[j].sourceProp.objPropId : viewPropFilters[j].targetProp.objPropId;
                var value_1 = item.getString(filterId);
                if (value_1 == null) {
                    break;
                }
                filter.push(value_1);
            }
            var value = item.getString(propId);
            // Filter contains empty items which isn't supported
            if (value == null || filter.length != viewPropFilters.length) {
                continue;
            }
            filter.push(value);
            // add a - so 11-0 is different from 1-10
            var key = filter.join("-");
            if (key == null || keyMap[key]) {
                continue;
            }
            keysList.push(filter);
            keyMap[key] = true;
        }
        // Translate the keys.
        this.translateDataToDisplayValues(keysList).done(function (translatedReader, status) {
            // Select the translation list from the translated reader.
            var selectParts = new Array();
            var selectPart = new SelectPart();
            selectPart.objProp = _this.translationFormattedListField.objProp;
            // Add the table alias so we don't overwrite the value
            selectPart.aliasWebServiceId = _this.getListFieldAlias(type);
            selectParts.push(selectPart);
            // Join the two readers on each others data props.
            var joinConditions = [];
            //Add the filters as joinConditions to support composite lookups.
            if (!that.viewProp.lookup.hasContinuedJoining()) {
                that.viewProp.lookup.filters.forEach(function (filter) {
                    var joinCondition = new JoinCondition();
                    joinCondition.sourceProp = that.obj.objProps.object[filter.targetProp.objPropId];
                    joinCondition.targetProp = that.viewProp.lookup.targetObjView.obj.objProps.object[filter.sourceProp.objPropId];
                    joinConditions.push(joinCondition);
                });
            }
            var joinCondition = new JoinCondition();
            // Because this code is used for both lookup readers and the normal table the names might be different
            joinCondition.sourceProp = (type == 0 /* TableReader */ ? _this.viewProp.objProp : _this.selectionFormattedDataField.objProp);
            joinCondition.targetProp = _this.translationFormattedDataField.objProp;
            joinConditions.push(joinCondition);
            // Make sure that the reader exists before joining it.
            if (reader) {
                reader.join(_this.translatedReader, selectParts, joinConditions);
            }
            deferred.resolve(reader, status);
        }).fail(function (status) {
            deferred.reject(status);
        });
        return deferred.promise();
    };
    /**
    * Gets the translation set filtered with the given values.
    */
    TSFDataLookupContainer.prototype.translateDataToDisplayValues = function (filterDataValues) {
        var _this = this;
        var filters = this.translationSet.query.whereClause;
        // Clear the list
        filters.orredConditions.splice(0, filters.orredConditions.length);
        var viewPropFilters = this.viewProp.lookup.filters;
        // Add the filter values in an orred condition list.
        for (var i = 0; i < filterDataValues.length; i++) {
            var filterValues = filterDataValues[i];
            var filterList = [];
            if (!this.viewProp.lookup.hasContinuedJoining()) {
                for (var j = 0; j < viewPropFilters.length; j++) {
                    var viewPropFilter = viewPropFilters[j];
                    filterList.push(new FilterContractCondition(viewPropFilter.sourceProp, Enum.FilterCondition.EqualTo, filterValues[j]));
                }
            }
            var key = filterValues[filterValues.length - 1];
            var dataField = this.translationFormattedDataField;
            filterList.push(new FilterContractCondition(dataField.objProp, Enum.FilterCondition.EqualTo, key));
            filters.orredConditions.push(filterList);
        }
        // When there are now rows, we shouldn't try to get the translation because there isn't a where clause
        // Therefore everything is loaded which isn't what we want.
        this.translationSet.query.whereClause.containsFalseCondtion = filterDataValues.length == 0;
        // Invalidate the translation set since the filters have changed.
        this.translationSet.invalidate();
        // Get the new translation set.
        var dataPromise = this.translationSet.fetchDataAsync();
        // Save the translation reader for future use.
        // Make sure that the reader is saved prior to other promise handlers.
        dataPromise.done(function (translatedReader) {
            _this.translatedReader = translatedReader;
        });
        return dataPromise;
    };
    TSFDataLookupContainer.prototype.updateContextRow = function (row) {
        this.contextRow = row;
    };
    TSFDataLookupContainer.prototype.addFiltersForSelectionSet = function (rowObject) {
        var changed = false;
        var query = this.selectionSet.query.whereClause.contextFilters;
        // No row given, use the contextrow instead.
        if (!rowObject)
            rowObject = Utils.normalizeRow(this.contextRow, this.obj.objProps.array);
        var filterCopy = [];
        for (var i = 0; i < query.length; i++) {
            filterCopy.push(query[i]);
        }
        this.selectionSet.query.whereClause.containsFalseCondtion = false;
        query.splice(0, query.length);
        var filters = this.viewProp.lookup.filters;
        for (var i_1 = 0; i_1 < filters.length; i_1++) {
            var value = rowObject[filters[i_1].targetProp.objPropId];
            if (Utils.isEmpty(value)) {
                this.selectionSet.query.whereClause.containsFalseCondtion = true;
                break;
            }
            query.push(new FilterContractCondition(filters[i_1].sourceProp, Enum.FilterCondition.EqualTo, value));
            changed =
                changed ||
                    !filterCopy[i_1] ||
                    query[i_1].targetPropId !== filterCopy[i_1].targetPropId ||
                    !Utils.valuesEqual(query[i_1].dbFilterValue, filterCopy[i_1].dbFilterValue);
        }
        if (filters.length > 0 && this.selectionSet.query.preferredIndex) {
            var obj = this.selectionObjView.obj;
            var filterValues = query.map(function (con) { return con.dbFilterValue; });
            this.selectionSet.query.keyRange = IndexedDb.createKeyRangeFromValues(obj, filterValues);
        }
        // TODO: Check if the filters are changed, if they are invalidate otherwise don't.
        this.invalidateSelectionSet();
        return changed;
    };
    TSFDataLookupContainer.prototype.invalidateSelectionSet = function () {
        this.selectionSet.invalidate();
    };
    /**
    * Performs the selection set request and translates it when its needed.
    */
    TSFDataLookupContainer.prototype.getTranslatedSelectionSet = function () {
        var _this = this;
        if (this.selectionSet.query.whereClause.containsFalseCondtion) {
            // We are not required to check for the falseCondition but it doesn't make much sense to do all the extra work
            // so we bail out early
            this.selectionSet.invalidate();
            return this.selectionSet.fetchDataAsync();
        }
        // Get the selection set.
        return this.selectionSet.fetchDataAsync().then(function (reader, status) {
            // Don't translate it when the obj id's and the listfields are the same.
            if (_this.selectionObjView.objId == _this.translationObjView.objId &&
                _this.selectionFormattedListField == _this.translationFormattedListField) {
                // Save the selection reader as translated reader since it already has al the data.
                _this.translatedReader = reader;
                return jQuery.Deferred().resolve(reader, new RequestStatus(true)).promise();
            }
            else {
                // Translate the selection reader.
                return _this.translateReaderToDisplayValues(reader, 1 /* LookupReader */);
            }
        });
    };
    /**
    * Adds filters to the selection query and executes it.
    */
    TSFDataLookupContainer.prototype.getFilteredTranslatedSelectionSet = function (value, comparison) {
        if (this.selectionSet.query.whereClause.containsFalseCondtion) {
            // We are not required to check for the falseCondition but it doesn't make much sense to do all the extra work
            // so we bail out early
            this.selectionSet.invalidate();
            return this.selectionSet.fetchDataAsync();
        }
        this.selectionSet.invalidate();
        var query = this.selectionSet.query;
        query.whereClause.userFilters.splice(0);
        // We can't use an index on listFields with a contains condition.
        // For startsWith we use a index to improve performance.
        // For example with 15000 items performance went from 5900ms to 15ms on the iPad
        if (comparison == Enum.FilterCondition.StartsWith) {
            var lookup = this.viewProp.lookup, filters = lookup.filters;
            var selectionSetIndex = void 0;
            if (filters.length > 0) {
                // TODO: WE should create a new index for this but fallback to the normal (but slow) behavior for now
                query.keyRange = null;
                query.preferredIndex = null;
                query.indexSorted = false;
            }
            else {
                selectionSetIndex = IndexedDb.IndexedDbWrapper.lfPrefix + "upper_" + this.selectionFormattedListField.objPropId;
            }
            if (this.selectionObjView.obj.lookupFilterIndexes[selectionSetIndex]) {
                this.selectionSet.query.preferredIndex = selectionSetIndex;
                query.indexSorted = true;
                var conditionValue = query.whereClause.contextFilters.map(function (condition) { return condition.dbFilterValue; })
                    .concat(value).join("-");
                // The ≤ is one of the last characters in the ascii range to make the starts startsWith/between filter work)
                this.selectionSet.query.keyRange = IDBKeyRange.bound(conditionValue.toUpperCase(), conditionValue.toUpperCase() + "≤", false, false);
            }
        }
        else {
            query.keyRange = null;
            query.preferredIndex = null;
            query.indexSorted = false;
        }
        if (!Utils.isEmpty(value)) {
            var condition = void 0;
            var listProp = this.viewProp.lookup.getTranslListProp().objProp;
            // Use a LookupContractCondition when the lookup uses continued joining.
            if (this.viewProp.lookup.hasContinuedJoining()) {
                var targetViewProp = this.selectionFormattedDataField;
                condition = new FilterLookupContractCondition(targetViewProp.objLinkLookupView.objLink.refId, listProp, targetViewProp.objProp, comparison, value);
            }
            else {
                condition = new FilterContractCondition(listProp, comparison, value);
            }
            this.selectionSet.query.whereClause.userFilters.push(condition);
        }
        return this.getTranslatedSelectionSet();
    };
    TSFDataLookupContainer.selectionSetLength = 5;
    return TSFDataLookupContainer;
}());
