var TSFMessage = /** @class */ (function () {
    function TSFMessage() {
    }
    // #endregion
    TSFMessage.translateTSFMessages = function (message, application) {
        if (!application) {
            return message.RawMessage;
        }
        if (!message.NamedParameters) {
            message.NamedParameters = [];
        }
        var newMessage;
        // Indicium 2017.1.1 uses: ApplicationMessage
        // Indicium > 2017.1.1 uses : ApplicationDbMessage
        if (message.MessageType === "ApplicationMessage" || message.MessageType === "ApplicationDbMessage") {
            newMessage = this.parseXmlMessage(application, message.RawMessage, message.NamedParameters);
        }
        else if (message.MessageID) {
            for (var i = 0; i < application.guiMsgList.length; i++) {
                if (application.guiMsgList[i].id == message.MessageID) {
                    newMessage = application.guiMsgList[i].transl;
                    break;
                }
            }
            if (newMessage == undefined) {
                return "[" + message.MessageID + "]";
            }
        }
        // Correct named parameters for database-generated messages.
        // Indicium 2017.1.1 uses: TechnicalMessage
        // Indicium > 2017.1.1 uses : TechnicalDbMessage
        if ((message.MessageType === "TechnicalMessage" || message.MessageType === "TechnicalDbMessage") && message.NamedParameters.length > 0) {
            var namedParameters = message.NamedParameters;
            var tabParameters = namedParameters.filter(function (namedParam) { return namedParam.Key == "table"; });
            if (tabParameters.length > 0) {
                var tabParameter = tabParameters[0];
                var tabId = tabParameter.Value.substring(tabParameter.Value.lastIndexOf(".") + 1);
                var obj = application.model.getObj(tabId);
                if (obj) {
                    tabParameter.Value = obj.defaultObjView.objTransl.translPlural;
                    var colParameters = namedParameters.filter(function (namedParam) { return namedParam.Key == "column"; });
                    if (colParameters.length > 0) {
                        var colParameter = colParameters[0];
                        var colId = colParameter.Value.substring(colParameter.Value.lastIndexOf(".") + 1);
                        var objProp = obj.objProps.object[colId];
                        if (objProp) {
                            colParameter.Value = objProp.objPropTransl.transl;
                        }
                    }
                }
            }
        }
        // Looks a bit weird but the Windows gui has the same check
        // This is required when an exception occurs from a trigger
        if (newMessage == "{0}" && message.NamedParameters.length == 0) {
            newMessage = message.RawMessage;
        }
        // Replace named parameters
        newMessage = this.replaceNamedParameters(newMessage, message.NamedParameters);
        return newMessage;
    };
    TSFMessage.parseXmlMessage = function (application, xmlInput, parameters) {
        // Remove unwanted ORA error, angle brackets and text outside the <msg> tags.
        xmlInput = TSFMessageParser.removeORAError(xmlInput);
        xmlInput = TSFMessageParser.removeAngleBrackets(xmlInput);
        xmlInput = TSFMessageParser.removeTextOutsideMsgTags(xmlInput);
        // Check if the xml is valid, else return original message.
        var parser;
        try {
            parser = TSFMessageParser.tryParseXML(xmlInput);
        }
        catch (exception) {
            return xmlInput;
        }
        var msg = parser.getElementsByTagName("msg");
        var item = msg.item(0);
        if (item && item.hasAttributes) {
            var msgElement = item;
            var id = msgElement.getAttribute("id");
            // Retrieve parameters from message.
            this.retrieveParameters(msgElement.childNodes, application, parameters);
            if (id) {
                // If the message is a clear_panel or add_separator message return an empty string, so the message is ignored.
                if (id.toLowerCase() == "clear_panel" || id.toLowerCase() == "add_separator") {
                    return "";
                }
                // Check if the message exists
                for (var i = 0; i < application.guiMsgList.length; i++) {
                    var msg_1 = application.guiMsgList[i];
                    if (msg_1.id == id) {
                        return msg_1.transl;
                    }
                }
                // Same as windows; return a translation from the objectmodel
                // when it doesn't exists it returns the id with brackets
                return application.language.transl(id);
            }
        }
        // Return complete message if no id is provided.
        return xmlInput;
    };
    TSFMessage.retrieveParameters = function (childNodes, application, parameters) {
        for (var i = 0; i < childNodes.length; i++) {
            var node = childNodes[i];
            // Don't use a default case to make sure unallowed tags will not be used.
            switch (node.nodeName.toLowerCase()) {
                case this.ReportParamTag:
                    // Reports are not implemented yet.
                    break;
                case this.ReportTag:
                    // Reports are not implemented yet.
                    break;
                case this.DomElementTag:
                    parameters.push({ Key: i, Value: this.parseDomElement(node, application) });
                    break;
                case this.ObjTag:
                    parameters.push({ Key: i, Value: this.parseObj(node, application) });
                    break;
                case this.ObjPropTag:
                    parameters.push({ Key: i, Value: this.parseObjProp(node, application) });
                    break;
                case this.TaskTag:
                    parameters.push({ Key: i, Value: this.parseTask(node, application) });
                    break;
                case this.TaskParamTag:
                    parameters.push({ Key: i, Value: this.parseTaskParameter(node, application) });
                    break;
                case this.TextParamTag:
                    parameters.push({ Key: i, Value: this.parseTextParam(node) });
                    break;
            }
        }
    };
    TSFMessage.replaceNamedParameters = function (message, namedParameters) {
        for (var i = 0; i < namedParameters.length; i++) {
            message = message.replace("{" + namedParameters[i].Key + "}", namedParameters[i].Value);
        }
        return message;
    };
    TSFMessage.getObjProp = function (application, obj, colId) {
        var viewProps = obj.objViewProps.array;
        for (var i = 0; i < viewProps.length; i++) {
            if (viewProps[i].objPropId == colId) {
                return viewProps[i];
            }
        }
        return null;
    };
    TSFMessage.getTaskParameters = function (application, task, taskParamId) {
        var objProps = task.userParamObj.objProps.array;
        for (var i = 0; i < objProps.length; i++) {
            if (objProps[i].objPropId == taskParamId) {
                return objProps[i];
            }
        }
        return null;
    };
    // #region parse parameters
    TSFMessage.parseDomElement = function (node, application) {
        var domId = node.getAttribute(this.DomIdAttr);
        var domElementId = node.getAttribute(this.DomElementIdAttr);
        if (!domId || !domElementId)
            return; // If there is no domId or domelementId, do nothing with the parameter.
        var paramValue;
        var domain = application.model.domains[domId];
        if (domain) {
            var domainElement = domain.elements.key(domElementId);
            if (domainElement) {
                paramValue = domainElement.elementTransl.transl;
            }
            else {
                // Try to get element from domain by using the given 'domElementId' as index to ensure backwards compatibility.
                var elementAsIndex = parseInt(domElementId);
                if (!isNaN(elementAsIndex) && (domainElement = domain.elements.item(elementAsIndex)) != null) {
                    paramValue = domainElement.elementTransl.transl;
                }
            }
        }
        if (paramValue == null) {
            paramValue = "[" + domElementId + "]";
        }
        return paramValue;
    };
    TSFMessage.parseObj = function (node, application) {
        var tabId = node.getAttribute(this.TabIdAttr);
        if (!tabId)
            return; // If there is no tabid, do nothing with the parameter.
        var obj = application.model.getObj(tabId);
        if (!obj)
            return "[" + tabId + "]"; // If there is no valid obj, just return the tab id (between brackets)
        var objView = obj.defaultObjView;
        var translAttr = node.getAttribute(this.TranslAttr);
        var paramValue;
        switch (translAttr) {
            case this.TranslPlural:
                paramValue = objView.objTransl.translPlural;
                break;
            case this.Transl:
            default:
                paramValue = objView.objTransl.transl;
                break;
        }
        return paramValue;
    };
    TSFMessage.parseObjProp = function (node, application) {
        var tabId = node.getAttribute(this.TabIdAttr);
        var colId = node.getAttribute(this.ColIdAttr);
        if (!tabId || !colId)
            return; // If there is no tabid, do nothing with the parameter.
        var obj = application.model.getObj(tabId);
        if (!obj)
            return; // If there is no valid obj, do nothing with the parameter.
        var objView = obj.defaultObjView;
        var objViewProp = this.getObjProp(application, objView, colId);
        if (!objViewProp)
            return "[" + colId + "]"; // If there is no valid objProp, return the colId with brackets.
        var translAttr = node.getAttribute(this.TranslAttr);
        var paramValue;
        switch (translAttr) {
            case this.TranslForm:
                paramValue = objViewProp.objProp.objPropTransl.translForm;
                break;
            case this.TranslGrid:
                paramValue = objViewProp.objProp.objPropTransl.translGrid;
                break;
            case this.Transl:
            default:
                paramValue = objViewProp.objProp.objPropTransl.transl;
                break;
        }
        return paramValue;
    };
    TSFMessage.parseTask = function (node, application) {
        var taskId = node.getAttribute(this.TaskIdAttr);
        if (!taskId)
            return; // If there is no valid taskId, do nothing with the parameter.
        var task = application.model.tasks.key(taskId);
        if (!task)
            return "[" + taskId + "]"; // If there is no valid task, return the taskId with brackets.
        return task.defaultTaskView.taskTransl.transl;
    };
    TSFMessage.parseTaskParameter = function (node, application) {
        var taskId = node.getAttribute(this.TaskIdAttr);
        var taskIdparam = node.getAttribute(this.TaskParamIdAttr);
        if (!taskId || !taskIdparam)
            return; // If there is no valid taskId or taskIdparam, do nothing with the parameter.
        var task = application.model.tasks.key(taskId);
        if (!task)
            return; // If there is no valid task, do nothing with the parameter.
        var objProp = this.getTaskParameters(application, task, taskIdparam);
        if (!objProp)
            return "[" + taskIdparam + "]"; // If there is no valid objProp, return the objProp id (with brackets)
        var translAttr = node.getAttribute(this.TranslAttr);
        var paramValue;
        switch (translAttr) {
            case this.TranslForm:
                paramValue = objProp.objPropTransl.translForm;
                break;
            case this.Transl:
            default:
                paramValue = objProp.objPropTransl.transl;
                break;
        }
        return paramValue;
    };
    TSFMessage.parseTextParam = function (node) {
        return node.textContent;
    };
    // #region Recognized xml
    // Tags
    TSFMessage.TextParamTag = "text";
    TSFMessage.ReportParamTag = "reportparam";
    TSFMessage.ReportTag = "report";
    TSFMessage.DomElementTag = "domelement";
    TSFMessage.ObjTag = "tab";
    TSFMessage.ObjPropTag = "col";
    TSFMessage.TaskTag = "task";
    TSFMessage.TaskParamTag = "taskparam";
    // Attributes
    TSFMessage.TaskIdAttr = "taskid";
    TSFMessage.TaskParamIdAttr = "taskparamid";
    TSFMessage.ReportIdAttr = "reportid";
    TSFMessage.ReportParamIdAttr = "reportparamid";
    TSFMessage.TabIdAttr = "tabid";
    TSFMessage.ColIdAttr = "colid";
    TSFMessage.DomIdAttr = "domid";
    TSFMessage.DomElementIdAttr = "elementid";
    TSFMessage.TranslAttr = "transl";
    // Attribute values
    TSFMessage.Transl = "transl";
    TSFMessage.TranslPlural = "plural";
    TSFMessage.TranslGrid = "grid";
    TSFMessage.TranslForm = "form";
    return TSFMessage;
}());
var TSFMessageParser = /** @class */ (function () {
    function TSFMessageParser() {
    }
    /**
     * Remove ORA error from TSFMessage but only when it contains << >>
     */
    TSFMessageParser.removeORAError = function (input) {
        var re = /<<(.+)>>/;
        var result = re.exec(input);
        if (result) {
            return result[1].trim();
        }
        return input;
    };
    /**
     * Remove the double `<` and `>` brackets from the start/end of the string
     */
    TSFMessageParser.removeAngleBrackets = function (input) {
        if (input.length > 5) {
            if (input.charAt(0) == "<" && input.charAt(1) == "<" &&
                input.charAt(input.length - 1) == ">" && input.charAt(input.length - 2) == ">") {
                return input.substring(2, input.length - 2).trim();
            }
        }
        return input;
    };
    /**
     * Remove the text outside <msg>...</msg> tags. The end tag can be handled like this because we enforce using the tsf_send_message stored procedure which always returns this structure.
     */
    TSFMessageParser.removeTextOutsideMsgTags = function (input) {
        var startIndex = input.indexOf("<msg");
        var endIndex = input.lastIndexOf("</msg>");
        return input.substr(startIndex, (endIndex - startIndex) + "</msg>".length);
    };
    /**
     * Try to parse the xml. If it contains incorrect node structures or text outside of the start and end node it will throw an exception.
     */
    TSFMessageParser.tryParseXML = function (xmlString) {
        var parser = new DOMParser();
        var parsererrorNS = parser.parseFromString('INVALID', 'text/xml').getElementsByTagName("parsererror")[0].namespaceURI;
        var dom = parser.parseFromString(xmlString, 'text/xml');
        if (dom.getElementsByTagNameNS(parsererrorNS, 'parsererror').length > 0) {
            throw new Error('Error parsing XML');
        }
        return dom;
    };
    return TSFMessageParser;
}());
