var TSFQuery = /** @class */ (function () {
    function TSFQuery(serviceId, original) {
        if (original === void 0) { original = null; }
        var _this = this;
        this.serviceId = serviceId;
        this.usePagination = false;
        this.pageSize = -1;
        this.pageIndex = -1;
        this.indexSorted = false;
        this.currentSorting = [];
        if (!original) {
            this.selectClause = new TSFSelectClause();
            this.whereClause = new TSFWhereClause();
        }
        else {
            this.cubeViewId = original.cubeViewId;
            this.usePagination = original.usePagination;
            this.pageSize = original.pageSize;
            this.pageIndex = original.pageIndex;
            this.preferredIndex = original.preferredIndex;
            this.indexSorted = original.indexSorted;
            this.keyRange = original.keyRange;
            this.selectClause = new TSFSelectClause(original.selectClause);
            this.whereClause = new TSFWhereClause(original.whereClause);
            if (original.currentSorting) {
                original.currentSorting.forEach(function (sorting) { return _this.currentSorting.push(sorting); });
            }
        }
    }
    TSFQuery.prototype.createBatches = function (orredConditionBatchSize) {
        var batches = [];
        if (orredConditionBatchSize < 0 || this.whereClause.orredConditions.length == 0) {
            batches.push(this);
        }
        else {
            while (this.whereClause.orredConditions.length > 0) {
                var newQuery = new TSFQuery(this.serviceId, this);
                newQuery.whereClause.orredConditions = this.whereClause.orredConditions.splice(0, orredConditionBatchSize);
                batches.push(newQuery);
            }
        }
        return batches;
    };
    return TSFQuery;
}());
var TSFSelectClause = /** @class */ (function () {
    function TSFSelectClause(original) {
        if (original === void 0) { original = null; }
        var _this = this;
        this.selectParts = [];
        if (original && original.selectParts) {
            original.selectParts.forEach(function (selectPart) { return _this.selectParts.push(selectPart); });
        }
    }
    return TSFSelectClause;
}());
var TSFWhereClause = /** @class */ (function () {
    function TSFWhereClause(original) {
        if (original === void 0) { original = null; }
        var _this = this;
        this.contextFilters = [];
        this.combinedFilterConditions = [];
        this.userFilters = [];
        this.orredConditions = [];
        this.containsFalseCondtion = false;
        this.startEmptyWithFilterActive = false;
        if (original) {
            if (original.contextFilters)
                original.contextFilters.forEach(function (contextFilter) { return _this.contextFilters.push(contextFilter); });
            if (original.combinedFilterConditions)
                original.combinedFilterConditions.forEach(function (combinedFilterCondition) { return _this.combinedFilterConditions.push(combinedFilterCondition); });
            if (original.userFilters)
                original.userFilters.forEach(function (userFilter) { return _this.userFilters.push(userFilter); });
            if (original.orredConditions)
                original.orredConditions.forEach(function (orredCondition) { return _this.orredConditions.push(orredCondition); });
            if (original.prefilterState)
                original.prefilterState.forEach(function (prefilterState) { return _this.prefilterState.push(prefilterState); });
            this.containsFalseCondtion = original.containsFalseCondtion;
            this.startEmptyWithFilterActive = original.startEmptyWithFilterActive;
        }
    }
    Object.defineProperty(TSFWhereClause.prototype, "isForcedEmpty", {
        get: function () { return this.containsFalseCondtion || this.startEmptyWithFilterActive; },
        enumerable: true,
        configurable: true
    });
    return TSFWhereClause;
}());
var TSFRecordSet = /** @class */ (function () {
    function TSFRecordSet(application, objView) {
        this.application = application;
        this.objView = objView;
        this.builder = null;
    }
    TSFRecordSet.prototype.invalidate = function () {
        this.data = null;
        // TODO: Should we reject this promise when this is called during a request?
        this.dataPromise = null;
    };
    TSFRecordSet.prototype.getData = function () {
        return this.data;
    };
    TSFRecordSet.prototype.isCurrentlyLoading = function () {
        return (this.dataPromise && this.dataPromise.state() == "pending");
    };
    TSFRecordSet.prototype.abortRequest = function () {
        if (this.isCurrentlyLoading() && this.builder !== null) {
            this.builder.abortRequest();
        }
    };
    TSFRecordSet.prototype.useOnlineBuilder = function () {
        return AppConfig.online && !this.objView.offlineAvailable;
    };
    TSFRecordSet.prototype.getRequestBatchSize = function () {
        if (this.useOnlineBuilder() && this.query.whereClause.orredConditions.length > 5) {
            // Assuming a batch size of 30 conditions for one column in each condition and scaling down linearly per extra column.
            return Math.floor(30 / this.query.whereClause.orredConditions[0].length);
        }
        return -1;
    };
    TSFRecordSet.prototype.fetchDataAsync = function () {
        var _this = this;
        if (this.dataPromise) {
            return this.dataPromise;
        }
        else {
            this.dataPromise = jQuery.Deferred();
        }
        var localDeferred = this.dataPromise;
        if (this.query.whereClause.isForcedEmpty) {
            // Resolve the promise since the builder initializes one.
            localDeferred.resolve(this.createBuilder(jQuery.Deferred(), this.query, false).getEmptyReader(), new RequestStatus(true));
        }
        else {
            var queryBatches = this.query.createBatches(this.getRequestBatchSize());
            queryBatches.reduce(function (prev, next, index) {
                return prev.then(function () {
                    var deferred = jQuery.Deferred();
                    _this.builder = _this.createBuilder(deferred, next, index > 0);
                    _this.builder.get();
                    return deferred.promise();
                });
            }, jQuery.Deferred().resolve().promise())
                .then(function (allData, status, lastData) {
                localDeferred.resolve(allData, status, lastData);
            }, function (status) { return localDeferred.reject(status); });
        }
        return localDeferred.promise();
    };
    TSFRecordSet.prototype.fetchNewPage = function () {
        var _this = this;
        if (this.isCurrentlyLoading()) {
            return this.dataPromise;
        }
        else {
            this.dataPromise = jQuery.Deferred();
        }
        var localDeferred = this.dataPromise;
        var queryBatches = this.query.createBatches(this.getRequestBatchSize());
        queryBatches.reduce(function (prev, next, index) {
            return prev.then(function () {
                var deferred = jQuery.Deferred();
                _this.createBuilder(deferred, next, true).get();
                return deferred.promise();
            });
        }, jQuery.Deferred().resolve().promise())
            .then(function (allData, status, lastBatch) {
            localDeferred.resolve(allData, status, lastBatch);
        }, function (status) { return localDeferred.reject(status); });
        return localDeferred.promise();
    };
    TSFRecordSet.prototype.createBuilder = function (deferred, query, importData) {
        if (importData === void 0) { importData = false; }
        if (this.useOnlineBuilder()) {
            return this.createOnlineBuilder(deferred, query, importData);
        }
        else {
            return this.createIndexedDbBuilder(deferred, query, importData);
        }
    };
    TSFRecordSet.prototype.createIndexedDbBuilder = function (deferred, query, importData) {
        var _this = this;
        var builder = IndexedDb.RequestBuilder.fromTSFQuery(this.application, this.objView, query);
        builder.setCallbackMethod(function (reader, status) {
            if (!status.success) {
                deferred.reject(status);
            }
            else if (importData) {
                _this.data.importData(reader);
                deferred.resolve(_this.data, status, reader);
            }
            else {
                _this.data = reader;
                deferred.resolve(reader, status);
            }
        });
        return builder;
    };
    TSFRecordSet.prototype.createOnlineBuilder = function (deferred, query, importData) {
        var _this = this;
        var builder = RequestBuilderFactory.createOnlineRequestBuilderFromQuery(this.application, query);
        builder.setCallbackMethod(function (reader, status) {
            if (!status.success) {
                deferred.reject(status);
            }
            else if (importData) {
                _this.data.importData(reader);
                deferred.resolve(_this.data, status, reader);
            }
            else if (query.cubeViewId) {
                // Reader got 3 sub contract lists, "Rows", "ColumnTotals" and "RowTotals"
                _this.data = reader;
                deferred.resolve(reader, status);
            }
            else {
                _this.data = reader;
                deferred.resolve(reader, status);
            }
        });
        return builder;
    };
    return TSFRecordSet;
}());
