var AbstractMenu = /** @class */ (function () {
    function AbstractMenu(application) {
        this.application = application;
        this.badgeInfo = {};
    }
    AbstractMenu.prototype.initialize = function () {
        throw new Error("Method not implemented.");
    };
    AbstractMenu.prototype.bindMenuEvents = function () {
        badgeHelper.clearTimers(this.badgeInfo);
        for (var key in this.badgeInfo) {
            var info = this.badgeInfo[key];
            info.execute();
        }
    };
    AbstractMenu.prototype.createBadge = function (item, anchorItem) {
        var that = this;
        var mapId = item.entity.entityType + "-" + item.entity.entityId;
        if (item.entity.configurationId) {
            mapId += "-" + item.entity.configurationId;
        }
        var badgeItem = this.badgeInfo[mapId];
        if (!badgeItem) {
            badgeItem = this.badgeInfo[mapId] = new badgeHelper.BadgeInfo(function () {
                return badgeHelper.executeBadge(that.application, this, {});
            }, this.getObjViewFromEntity(item.entity));
        }
        badgeItem.subscribe(function (value, error) {
            if (value != null) {
                var createBadgeSpan = function (root) {
                    var span = document.createElement("span");
                    span.className = "ui-li-count";
                    root.appendChild(span);
                    return span;
                };
                var span = jQuery(anchorItem).children(".ui-li-count")[0] || createBadgeSpan(anchorItem);
                span.textContent = value + "";
            }
            else {
                jQuery(anchorItem).children(".ui-li-count").remove();
            }
        });
    };
    AbstractMenu.prototype.getAnchorContent = function (item) {
        var content = {
            href: "",
            transl: ""
        };
        if (item.entity.entityType == 0 /* Obj */) {
            content.href = "abstractscreen.html?obj=" + item.entity.entityId;
            if (item.entity.configurationId) {
                content.href += "&config=" + item.entity.configurationId;
            }
            content.transl = item.itemTransl.translPlural;
        }
        else if (item.entity.entityType == 1 /* Task */) {
            content.transl = item.itemTransl.transl;
            content.href = "menu.html?task=" + item.entity.entityId;
            if (item.entity.configurationId) {
                content.href += "&taskConfig=" + item.entity.configurationId;
            }
        }
        content.href += "&comesFromMenu=true";
        return content;
    };
    AbstractMenu.prototype.getObjViewFromEntity = function (entity) {
        var model = Model.TSFApplicationManager.Current.activeAppl.model;
        switch (entity.entityType) {
            case 0 /* Obj */:
                var obj = model.getObj(entity.entityId);
                return entity.configurationId ? obj.objViews.key(entity.configurationId) : obj.defaultObjView;
            case 1 /* Task */:
                var task = model.tasks.key(entity.entityId);
                return entity.configurationId ? task.taskViews.key(entity.configurationId).paramObjView : task.defaultTaskView.paramObjView;
        }
        return null;
    };
    return AbstractMenu;
}());
