var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var ListbarMenu = /** @class */ (function (_super) {
    __extends(ListbarMenu, _super);
    function ListbarMenu(application, container, groups) {
        var _this = _super.call(this, application) || this;
        _this.container = container;
        _this.groups = groups;
        _this.panel = _this.container.children(".content-panel");
        if (!groups) {
            _this.groups = application.model.defaultMenu.groups;
        }
        return _this;
    }
    ListbarMenu.prototype.initialize = function () {
        var application = this.application;
        var listbarGroups = this.groups.array;
        var quickbarStyle = application.style && application.style.quickbar;
        var collapsiblesDiv = document.createElement("div");
        collapsiblesDiv.className = "collapsibles-div";
        for (var i = 0; i < listbarGroups.length; i++) {
            var group = listbarGroups[i], groupElement = this.createCollapsible(group), listView = document.createElement("ul");
            listView.setAttribute("data-role", "listview");
            listView.setAttribute("data-theme", "tsf");
            for (var j = 0; j < group.items.length; j++) {
                var item = group.items[j], itemElement = document.createElement("li"), anchorItem = document.createElement("a");
                if (quickbarStyle && quickbarStyle.showListBarItemIcons && item.icon) {
                    var img = document.createElement("img");
                    itemElement.className = "ui-li-has-icon";
                    img.className = "ui-li-icon";
                    img.src = Utils.createIconFromBase64(item.icon);
                    anchorItem.appendChild(img);
                }
                var anchorContent = _super.prototype.getAnchorContent.call(this, item);
                anchorItem.href = anchorContent.href;
                anchorItem.appendChild(document.createTextNode(anchorContent.transl));
                if (item.entity.showBadge) {
                    _super.prototype.createBadge.call(this, item, anchorItem);
                }
                itemElement.appendChild(anchorItem);
                listView.appendChild(itemElement);
            }
            if (listbarGroups.length == 1) {
                groupElement.setAttribute("data-collapsed", "false");
            }
            groupElement.appendChild(listView);
            collapsiblesDiv.appendChild(groupElement);
        }
        this.panel.prepend(collapsiblesDiv);
        this.setMenuIconColors();
        jQuery(collapsiblesDiv).on("click.menu", "ul li a", function (evt) {
            if (evt.currentTarget) {
                var ele = evt.currentTarget;
                if (ele.href.indexOf("task=") >= 0) {
                    if (window.matchMedia("(min-width: 800px)").matches) {
                        //Navigation is needed so a user can go back when he has navigated from a task popup to its lookup.
                        NavigationManager.navigate(ele.href, { transition: "none", allowSamePageTransition: true, reloadPage: true });
                    }
                    else {
                        //For small screens a fullscreen new page gets opened, so we don"t need to navigate to make going back possible.
                        Menu.openTaskPopup(ele.href.substring(ele.href.indexOf("?")));
                    }
                }
                else {
                    NavigationManager.navigate(ele.href);
                    // Prevent further clicks or it will break jQuery mobile (because it queues up navigation actions to the same page)
                    jQuery(collapsiblesDiv)
                        .off("click.menu", "ul li a")
                        .on("click.menu", "ul li a", function (evt) {
                        evt.preventDefault();
                    });
                }
                evt.preventDefault();
                return false;
            }
        });
    };
    ListbarMenu.prototype.createCollapsible = function (group) {
        var groupElement = document.createElement("div");
        groupElement.setAttribute("data-role", "collapsible");
        groupElement.setAttribute("data-theme", "tsf");
        groupElement.setAttribute("data-content-theme", "tsf");
        groupElement.setAttribute("data-group-id", group.groupId);
        if (group.defaultExpanded)
            groupElement.setAttribute("data-collapsed", "false");
        var groupHeader = document.createElement("h4");
        groupHeader.innerText = group.grpTransl.transl;
        if (group.icon) {
            var img = document.createElement("img");
            img.src = Utils.createIconFromBase64(group.icon);
            groupHeader.appendChild(img);
        }
        groupElement.appendChild(groupHeader);
        return groupElement;
    };
    ListbarMenu.prototype.bindMenuEvents = function () {
        var groups = this.groups;
        this.panel.on("collapsiblecollapse collapsibleexpand", "div:jqmData(role='collapsible')", function (evt) {
            var groupId = jQuery(this).data("group-id"), group = groupId ? groups.key(groupId) : null;
            if (group) {
                group.defaultExpanded = evt.type == "collapsibleexpand";
            }
        });
    };
    ListbarMenu.prototype.setMenuIconColors = function () {
        // Set menu header collapse/expand icons black or white.
        var menuHeader = jQuery(this.container).find("h4 > a");
        if (menuHeader.length > 0) {
            var menuHeaderColor = Utils.Color.hexToColor(menuHeader.first().css("color"));
            Utils.SetIconBlackOrWhite(jQuery(this.container).find("h4 > a"), menuHeaderColor);
        }
        // Set menu item icons black or white.
        var menuItems = jQuery(this.container).find(".ui-collapsible-content a");
        if (menuItems.length > 0) {
            var menuItemColor = Utils.Color.hexToColor(menuItems.first().css("color"));
            Utils.SetIconBlackOrWhite(menuItems, menuItemColor);
        }
    };
    return ListbarMenu;
}(AbstractMenu));
