var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var TilesMenu = /** @class */ (function (_super) {
    __extends(TilesMenu, _super);
    function TilesMenu(application, $container) {
        var _this = _super.call(this, application) || this;
        _this.$container = $container;
        _this.panel = _this.$container.children(".content-panel");
        _this.menu = application.model.defaultMenu;
        return _this;
    }
    TilesMenu.prototype.initialize = function () {
        this.$page = this.$container.parent("#menu[data-role='page']");
        var tileGroupId = Utils.parseUrlVars(jQuery.mobile.path.parseUrl(this.$page.data("url")).search).indexed["tileGroupId"];
        if (tileGroupId) {
            this.initializeGroup(tileGroupId);
            jQuery(".back-button").click(function (evt) {
                NavigationManager.navigate("menu.html", { reverse: true });
                return false;
            });
        }
        else {
            this.initializeGlobal();
        }
        this.fillLastRow();
        this.panel.addClass("content-panel-tiles-menu");
    };
    TilesMenu.prototype.initializeGlobal = function () {
        var application = this.application;
        var tileGroups = Utils.orderArrayByObjectProperty(this.menu.groups.array, "orderNo");
        var tilesMenuDiv = document.createElement("div");
        tilesMenuDiv.className = "tiles-menu";
        var fragment = document.createDocumentFragment();
        for (var i = 0; i < tileGroups.length; i++) {
            var group = tileGroups[i];
            //A group with only one item will just show the item tile
            if (group.items.length === 1) {
                fragment.appendChild(this.createItemTile(group.items[0]));
            }
            else {
                fragment.appendChild(this.createGroupTile(group));
            }
        }
        tilesMenuDiv.appendChild(fragment);
        this.panel.prepend(tilesMenuDiv);
    };
    TilesMenu.prototype.initializeGroup = function (tileGroupId) {
        var application = this.application;
        var group = this.menu.groups.object[tileGroupId];
        if (!group) {
            return;
        }
        var tileItems = Utils.orderArrayByObjectProperty(group.items, "orderNo");
        var tilesMenuDiv = document.createElement("div");
        tilesMenuDiv.className = "tiles-menu";
        for (var i = 0; i < tileItems.length; i++) {
            var item = tileItems[i];
            tilesMenuDiv.appendChild(this.createItemTile(item, tileGroupId));
        }
        this.panel.prepend(tilesMenuDiv);
        var $header = this.$page.children("div[data-role='header']");
        $header.empty();
        $header.prepend("<div class='ui-btn-left ui-mini' data-role='controlgroup' data-type='horizontal'>" +
            "<a href='#' class='ui-btn ui-icon-back ui-btn-icon-left ui-btn-icon-notext back-button disableSelection' data-rel='back'>back</a>" +
            "</div>");
        var $backButton = jQuery($header).find('a.back-button');
        Utils.SetIconBlackOrWhite($backButton, Utils.Color.hexToColor(jQuery($backButton).css("color")));
        $backButton.text(application.language.transl(Model.TranslKeys.back));
        $header.append("<h4 class='ui-title'>" + group.grpTransl.transl + "</h4>");
    };
    TilesMenu.prototype.createGroupTile = function (group) {
        var groupTile = document.createElement("a");
        groupTile.className = "group-tile tile";
        groupTile.href = "menu.html?tileGroupId=" + group.groupId;
        var tileText = document.createElement("span");
        tileText.className = "tile-name";
        tileText.innerText = group.grpTransl.transl;
        groupTile.appendChild(tileText);
        var firstItem = group.items[0];
        if (firstItem.icon) {
            groupTile.style.backgroundImage = "url(" + Utils.createIconFromBase64(firstItem.icon) + ")";
        }
        return groupTile;
    };
    TilesMenu.prototype.createItemTile = function (item, tileGroupId) {
        var application = this.application;
        var itemTile = document.createElement("a");
        itemTile.className = "item-tile tile";
        var content = _super.prototype.getAnchorContent.call(this, item);
        itemTile.href = tileGroupId ? content.href + "&tileGroupId=" + tileGroupId : content.href;
        itemTile.onclick = function (evt) {
            if (evt.currentTarget) {
                var ele = evt.currentTarget;
                if (ele.href.indexOf("task=") >= 0) {
                    if (window.matchMedia("(min-width: 800px)").matches) {
                        //Navigation is needed so a user can go back when he has navigated from a task popup to its lookup.
                        NavigationManager.navigate(ele.href, { transition: "none", allowSamePageTransition: true, reloadPage: true });
                    }
                    else {
                        //For small screens a fullscreen new page gets opened, so we don"t need to navigate to make going back possible.
                        Menu.openTaskPopup(ele.href.substring(ele.href.indexOf("?")));
                    }
                    evt.preventDefault();
                    return false;
                }
            }
        };
        var tileText = document.createElement("span");
        tileText.className = "tile-name";
        tileText.innerText = content.transl;
        itemTile.appendChild(tileText);
        if (item.icon) {
            itemTile.style.backgroundImage = "url(" + Utils.createIconFromBase64(item.icon) + ")";
        }
        if (item.entity.showBadge) {
            _super.prototype.createBadge.call(this, item, itemTile);
        }
        return itemTile;
    };
    /* Fills the last row so it's full of tiles, which makes it possible to left align the items in the last row. */
    TilesMenu.prototype.fillLastRow = function () {
        var $tilesMenu = this.$page.find(".tiles-menu");
        if ($tilesMenu.children(".tile").length === 0) {
            return;
        }
        //We hide the footerPicture while filling because else it'll jump up and down in the page
        var $footerPicture = this.$page.find(".pictureFooter").hide();
        for (var i = 0; i < 5; i++) {
            $tilesMenu.append("<div class='item-tile tile hide-keep-space'></div>");
        }
        var $lastRealTile = $tilesMenu.children(".tile:not(.hide-keep-space)").last();
        var $placeholderTiles = $tilesMenu.children(".hide-keep-space");
        //Wait 100ms to give the browser a chance to render the tiles
        setTimeout(hideExcessPlaceholderTiles, 100);
        jQuery(window).on("throttledresize", hideExcessPlaceholderTiles);
        //Hides placeholder tiles that aren't on the same row as the last real tile.
        function hideExcessPlaceholderTiles() {
            //All placeholder tiles need to be shown because else they don't have offsetTop
            $placeholderTiles.show();
            $placeholderTiles.each(function (i, elem) {
                //If the placeholder tile is not on the same row as the last real tile, hide it.
                if (elem.offsetTop !== $lastRealTile[0].offsetTop) {
                    elem.style.display = "none";
                }
                else {
                    elem.style.display = "block";
                }
            });
            $footerPicture.show();
        }
    };
    TilesMenu.prototype.bindMenuEvents = function () {
    };
    return TilesMenu;
}(AbstractMenu));
