jQuery(document).on('pagebeforecreate', "#abstractscreen", function () {
    var $this = jQuery(this), header = $this.children("div[data-role='header']"), footer = $this.children("div[data-role='footer']"), language = Model.TSFApplicationManager.Current.activeAppl.language;
    header.find("a.back-button").text(language.transl(Model.TranslKeys.back));
    header.find("a.add-button").text(language.transl(Model.TranslKeys.add));
    header.find("a.select-button").text(language.transl(Model.TranslKeys.select));
    footer.find("a.refresh").text(language.transl(Model.TranslKeys.refresh));
    footer.find("a.filter").text(language.transl(Model.TranslKeys.filters));
    footer.find("a.details").text(language.transl(Model.TranslKeys.details));
    footer.find("a.cubeview").text(language.transl(Model.TranslKeys.cubeViews));
    footer.find("a.edit").text(language.transl(Model.TranslKeys.edit));
    footer.find("a.delete").text(language.transl(Model.TranslKeys.Delete));
    footer.find("a.cancel").text(language.transl(Model.TranslKeys.cancel));
    footer.find("a.save").text(language.transl(Model.TranslKeys.save));
});
jQuery(document).on('pageinit', '#abstractscreen', function () {
    var page = jQuery(this);
    var comingFromMenu = false;
    if (page.data("url").indexOf("comesFromMenu") !== -1) {
        var newLocation = page.data("url").replace("&comesFromMenu=true", "");
        page.data("url", newLocation);
        comingFromMenu = true;
    }
    var abstractScreen = new AbstractScreen(page);
    var parsedUrl = Utils.parseUrlVars(abstractScreen.url.search);
    var controller = TSFController.getControllerFromQueryString(abstractScreen.application, parsedUrl.indexed['obj'], parsedUrl.indexed["config"], parsedUrl.raw);
    if (controller != null) {
        if (comingFromMenu) {
            controller.resetCombinedFilterConditions();
            if (controller.initialized) {
                controller.currentAction = controller.refreshData(false, true, null);
            }
        }
        // Check if the controller has a parent and if the parent controller has a currently executed action.
        // Disable the hide of the loader when the parents action is finished.
        // DvdB: I don't really know what this means and in which situations this occurs
        // we should remove this together with the controller.currentAction = controller.method code
        // as the controller should manage when action are available (and not the components itself)
        if (controller.parentController && controller.parentController.interfaceAvailable == false) {
            controller.parentController.cancelHideLoader = true;
        }
        // Retrieve the mode from the url.
        var rawMode = parsedUrl.indexed["mode"];
        // Build the screen tree before initializing the controller because the controller must apply
        // start empty with filter when a ListView is available.
        abstractScreen.controller = controller;
        abstractScreen.buildScreenTree(rawMode);
        if (!controller.initialized) {
            // The abstract screen supports start empty with filter when there is a ListView.
            var grid = abstractScreen.tryFindComponent(Grid);
            controller.currentAction = controller.initialize(grid != null && grid.allowStartEmptyWithFilter());
            //If coming from the menu, refreshData is already called, so in that case we need to prevent getting the data twice here.
        }
        else if (!comingFromMenu) {
            controller.currentAction = controller.getData();
        }
        abstractScreen.initialize();
        // Set black or white icon for the back / add button.
        var color = Utils.Color.hexToColor(jQuery("a.back-button").css("color"));
        Utils.SetIconBlackOrWhite(jQuery("a.back-button, a.select-button"), color);
        Utils.SetIconBlackOrWhite(jQuery("a.add-button"), color);
        if (controller.controllerContext.dataContextOption() == 2 /* Lookup */) {
            page.addClass("lookup-screen");
        }
        else {
            page.find("a.select-button").addClass("ui-screen-hidden");
        }
    }
    else {
        console.log("Abstractscreen couldn't be intialized, controller is undefined");
    }
});
var AbstractScreen = /** @class */ (function () {
    function AbstractScreen(page) {
        this.page = page;
        this.isAndroidTwo = false;
        this.screenComponents = [];
        this.header = this.page.children("div[data-role='header']");
        this.content = this.page.children("div[data-role='main']");
        this.footer = this.page.children("div[data-role='footer']");
        this.application = Model.TSFApplicationManager.Current.activeAppl;
        this.url = jQuery.mobile.path.parseUrl(this.page.data('url'));
        this.pageLink = "abstractscreen.html" + this.url.search;
        this.isAndroidTwo = Utils.isAndroidTwo();
        // Create a pageShown promise because the abstractscreen is build after the async initialization of the controller.
        var pageDeferred = jQuery.Deferred();
        var panelDeferred = jQuery.Deferred();
        this.page.one("pageshow", function () {
            // The updatePanelsPromise should be resoloved prior to the pageShowPromise.
            // Because the pageShowPromise be passed on to the components.
            panelDeferred.resolve();
            pageDeferred.resolve();
        });
        this.updatePanelsPromise = panelDeferred.promise();
        this.pageShowPromise = pageDeferred.promise();
    }
    AbstractScreen.prototype.getEditLink = function (mode) {
        return this.pageLink + "&mode=" + mode;
    };
    AbstractScreen.prototype.initialize = function () {
        var _this = this;
        var that = this;
        var footer = this.footer;
        // Build header
        var h1Element = this.header.children('h1');
        h1Element.text(this.controller.controllerContext.subjectTransl());
        if (this.controller.objView.icon) {
            var img = document.createElement("img");
            img.className = "subject-icon";
            img.src = Utils.createIconFromBase64(this.controller.objView.icon);
            h1Element.prepend(img);
        }
        else {
            h1Element.addClass("noIcon");
        }
        // No items will ever become visible so hide the button
        if (!this.controller.hasVisibleDetails() && !this.controller.hasVisibleTasks()) {
            this.footer.find("a.details").addClass("ui-screen-hidden");
        }
        if (this.controller.parentController) {
            this.header.addClass("hasBreadCrumb");
            var breadCrumb = document.createElement("h4");
            breadCrumb.className = "breadCrumb ui-title";
            jQuery(breadCrumb).insertAfter(h1Element);
            this.updateBreadCrumb();
        }
        this.sizePanelsHeight(this.panels, this.detailPanel);
        // Add event to addbutton to switch to the form when the form is not on the screen yet.
        if (this.tryFindComponent(TSFForm) == null) {
            var addButton = this.header.find('a.add-button');
            addButton.click(function (evt) {
                NavigationManager.navigate(that.getEditLink(1 /* Add */));
            });
            addButton.on("taphold", function () {
                var language = that.application.language;
                var options = [
                    new SelectMenuOption(1 /* Add */, language.transl(Model.TranslKeys.add), that.controller.addRowAccessType, function () { NavigationManager.navigate(that.getEditLink(1 /* Add */)); }),
                    new SelectMenuOption(2 /* Edit */, language.transl(Model.TranslKeys.edit), that.controller.editRowAccessType, function () { NavigationManager.navigate(that.getEditLink(2 /* Edit */)); }),
                    new SelectMenuOption(3 /* Copy */, language.transl(Model.TranslKeys.copy), that.controller.copyRowAccessType, function () { NavigationManager.navigate(that.getEditLink(3 /* Copy */)); })
                ];
                var selectMenu = new SelectMenuPopup(that.controller).initialize(options);
            });
        }
        var selectButton = this.header.find("a.select-button");
        this.updateSelectButton();
        selectButton.on("click", function () {
            var lookupController = _this.controller;
            var context = lookupController.controllerContext;
            var parentController = context.parentController;
            // select value when the parentLookupController is in editmode
            if (parentController != null && parentController.currentEditor) {
                var lookupContainer = context.parentController.getLookupSet(context.lookupProp);
                var lookupRow = lookupController.currentRow;
                var newState = lookupController.queryForTasks.whereClause.prefilterState;
                context.parentController.updateLookupContainerPrefilterState(lookupContainer, newState);
                if (lookupRow) {
                    parentController.currentEditor.editDataRow(context.lookupProp.objProp, lookupRow.getValueForObjProp(context.lookupProp.lookup.dataProp.objProp));
                }
                else {
                    parentController.currentEditor.editDataRow(context.lookupProp.objProp, null);
                }
            }
            if (context.parentController) {
                // Block editmode or it will try to save the row which you where editing
                context.parentController.blockEditMode();
                context.parentController.lookupDialogClosed();
            }
            history.go(context.currentHistoryIndex - jQuery.mobile.navigate.history.activeIndex);
        });
        var backButton = this.header.find('a.back-button');
        backButton.click(function (evt) {
            var controller = that.controller;
            if (controller.interfaceAvailable) {
                if (controller.parentController) {
                    controller.parentController.blockEditMode();
                }
                if (controller.controllerContext.dataContextOption() == 2 /* Lookup */) {
                    controller.controllerContext.parentController.blockEditMode();
                    var context = that.controller.controllerContext;
                    history.go(context.currentHistoryIndex - jQuery.mobile.navigate.history.activeIndex);
                    evt.preventDefault();
                }
                if (controller.isInEditMode) {
                    if (controller.currentEditor.allowActions() && !controller.currentEditor.isCreatingNewRow()) {
                        // We need to validate the editors to make sure everything is synced to the controller
                        controller.currentEditor.validateEditors().then(function () {
                            controller.clearControllerEditor();
                            history.back();
                        });
                    }
                    else {
                        if (controller.currentEditor.currentRowModified) {
                            controller.cancelEditmodeWithConfirm().done(function () {
                                history.back();
                            });
                        }
                        else {
                            controller.cancelEditMode().done(function () {
                                history.back();
                            });
                        }
                    }
                    evt.preventDefault();
                }
            }
            else {
                evt.preventDefault();
            }
        });
        backButton.on("taphold", function (evt) {
            // Disable fastclick, or it will click the button ontouchend.
            fastclickInstance.trackingClick = false;
            if (that.controller.interfaceAvailable) {
                // Disable longpress in editmode.
                if (!that.controller.isInEditMode) {
                    NavigationManager.navigate("menu.html");
                }
            }
        });
        if (!this.controller.hasVisiblePrefilters()) {
            footer.find("> span > :jqmData(role='controlgroup') a.filter").addClass('ui-state-disabled');
        }
        if (this.controller.objView.defaultEditableForm) {
            // replace edit with cancel
            footer.find("a.edit").replaceWith(footer.find("a.cancel").clone());
        }
        else if (this.controller.objView.formAutoSave) {
            footer.find("a.edit").before(footer.find("a.cancel").clone().addClass("ui-screen-hidden"));
        }
        // Handle footer button clicks.
        footer.find("> span > :jqmData(role='controlgroup'),> :jqmData(role='controlgroup')").on("click", "a", function (event) {
            if (that.controller.interfaceAvailable) {
                var $button = jQuery(this);
                if ($button.hasClass("filter")) {
                    that.controller.executeActionAfterSave(function () { return new PrefilterPopup(that.controller).initialize(); });
                }
                else if ($button.hasClass("details")) {
                    that.controller.executeActionAfterSave(function () { return new DetailsPopup(that.controller).initialize(); });
                }
                else if ($button.hasClass("cubeview")) {
                    new CubeViewPopup(that.controller).initialize().then(function (cubeViewId) {
                        if (cubeViewId != null) {
                            var cube = that.tryFindComponent(Cube.ScreenComponent);
                            cube.changeCubeView(cubeViewId);
                        }
                    });
                }
                else if ($button.hasClass("graph")) {
                    var cube = that.tryFindComponent(Cube.ScreenComponent);
                    cube.switchViewType();
                    that.updateCubeButtonTransl(cube);
                }
                else if ($button.hasClass("refresh")) {
                    if (that.controller.objView.offlineAvailable) {
                        if (that.controller.isInEditMode && that.controller.currentEditor.isDefaultEditable()) {
                            var form_1 = that.tryFindComponent(TSFForm);
                            var mode_1 = form_1 ? form_1.mode : 2 /* Edit */;
                            that.controller.saveEditmode().then(function () {
                                that.application.localSync.refreshSyncGroup(that.controller.objView)
                                    .always(function () {
                                    that.controller.currentAction = that.controller.refreshData(true).then(function () {
                                        if (form_1) {
                                            that.controller.currentAction = form_1.createFormEditor(mode_1);
                                        }
                                    });
                                });
                            });
                        }
                        else {
                            that.application.localSync.refreshSyncGroup(that.controller.objView)
                                .always(function () {
                                that.controller.currentAction = that.controller.refreshData(true);
                            });
                        }
                    }
                    else {
                        that.controller.currentAction = that.controller.refreshData(true);
                    }
                }
                else if ($button.hasClass("save")) {
                    that.controller.currentAction = that.saveEditmode();
                }
                else if ($button.hasClass("edit")) {
                    var form = that.tryFindComponent(TSFForm);
                    if (form) {
                        that.controller.currentAction = form.createFormEditor(2 /* Edit */);
                    }
                    else {
                        NavigationManager.navigate(that.getEditLink(2 /* Edit */));
                    }
                }
                else {
                    if ($button.hasClass("cancel")) {
                        var form_2 = that.tryFindComponent(TSFForm);
                        if (form_2 && that.controller.currentEditor && that.controller.currentEditor.isDefaultEditable()) {
                            if (that.controller.currentEditor.isCreatingNewRow()) {
                                that.controller.cancelEditmodeWithConfirm()
                                    .then(function () {
                                    that.updateBackAndAddText();
                                });
                            }
                            else {
                                // we need to wait for the synchronization to finish or the default might restore old values after we pressed cancel
                                form_2.waitForSynchronization().always(function () {
                                    if (that.controller.currentRow) {
                                        that.controller.currentEditor.createEditRow();
                                    }
                                    form_2.syncValuesFromController();
                                });
                            }
                        }
                        else {
                            if (that.controller.currentEditor.currentRowModified) {
                                // Don't store the promise in currentAction because that is done inside the cancel method.
                                // It's important that the question prompt is circumstanced because it behaves weird in combination with the loader.
                                that.controller.cancelEditmodeWithConfirm();
                            }
                            else {
                                that.controller.cancelEditMode();
                            }
                        }
                    }
                    else if ($button.hasClass("delete")) {
                        that.controller.deleteRowWithConfirm();
                    }
                    // Why are these needed?
                    event.preventDefault();
                    event.stopPropagation();
                }
            }
        });
        this.updateBackAndAddText();
        this.updateControlGroups();
        this.bindControllerEvents();
        // We can safely apply the layout state before the controller is finished initializing.
        // This is because the layouts are executed again after data retrieval.
        this.changeLayoutState();
        // This code is to re-enable editmode when returning from a lookup
        // When you have a table, and switch to a from with the 'add' button
        // newRow might still be empty here
        if (this.controller.isInEditMode && this.controller.currentEditor.newRow) {
            var form = this.tryFindComponent(TSFForm);
            if (form) {
                form.attachEditor();
            }
        }
        // Dispose the components and the remove events after this page is removed so we don't leak
        jQuery(document).one('pagebeforecreate', function (evt) {
            // Test if we are closing a lookup (by checking the target url page) so we can detect if we are going back or maybe switching to a detail of the lookup
            // Note: if we switch to the lookup we shouldn't remove the editor
            var newurl = jQuery.mobile.path.parseUrl(jQuery(evt.target).data("url")).search;
            var parsedUrl = Utils.parseUrlVars(newurl);
            var clearControllerEditor = false;
            // when switching to menu or maybe synchronize, then there isn't a controller
            if (parsedUrl.indexed['obj']) {
                var targetController = TSFController.getControllerFromQueryString(_this.application, parsedUrl.indexed['obj'], parsedUrl.indexed["config"], parsedUrl.raw);
                if (targetController) {
                    // Switching to a new child, remove the current editor (it is already saved when required)
                    for (var _i = 0, _a = _this.controller.childControllers; _i < _a.length; _i++) {
                        var childController = _a[_i];
                        if (childController == targetController) {
                            clearControllerEditor = true;
                            break;
                        }
                    }
                    if (targetController == _this.controller) {
                        // we are switching to a different page of the same subject (grid to form or form to grid)
                        // clear the editor now
                        clearControllerEditor = true;
                    }
                }
            }
            else {
                // going to menu or synchronize page - so we don't need the editor
                clearControllerEditor = true;
            }
            for (var i = 0; i < _this.screenComponents.length; i++) {
                _this.screenComponents[i].dispose();
            }
            that.controller.events.clearEvents();
            // Clear the editor after removing the events so we don't need to process the events
            // We can't clear the editor when moving to a lookup container because the user should be able to select an item
            if (clearControllerEditor) {
                that.controller.clearControllerEditor();
            }
        });
    };
    AbstractScreen.prototype.buildScreenTree = function (rawMode) {
        var _this = this;
        var multiPanelScreen = window.matchMedia("(min-width: 800px)").matches;
        var controller = this.controller;
        // Convert the mode and check if editmode should be blocked.
        var hasMode = (Utils.isEmpty(rawMode) == false);
        var mode = hasMode ? parseInt(rawMode, 10) : 0 /* View */;
        if (controller.tryBlockEditMode()) {
            mode = 0 /* View */;
        }
        // Do not allow multi-column grid on small screens.
        // We currently don't change the screentype when orientation changes and the
        // screen becomes to small
        var screenType = this.controller.controllerContext.getScreenType();
        if (screenType == null) {
            // Set master detail as default screentype when there is no screentype set.
            screenType = this.controller.application.model.screenTypes["master_detail"];
        }
        // Also show ListViewForm when there are 0 screen components OR when there are more than 2 components
        // we don't really know what to do with 3 at the moment
        if (multiPanelScreen && (screenType.actualScreenComponents.length == 0 || screenType.actualScreenComponents.length >= 2)) {
            this.panels = jQuery("<div class='panel panel1'></div><div class='panel panel2'></div>");
            this.content.append(this.panels);
            this.createListView(this.panels[0], screenType, true);
            this.createForm(mode, this.panels[1], false, screenType.hasTaskBar);
        }
        else {
            this.panels = jQuery("<div class='panel'></div>");
            this.content.append(this.panels);
            // Same check can be found in TSFController.initializeInternal; keep them in sync
            if (screenType.getActualScreenComponent("Cube") && this.controller.objView.objViewCube && this.controller.objView.objViewCube.cubeViews.length > 0) {
                this.createCube(this.panels[0]);
            }
            else if (hasMode || (screenType.getActualScreenComponent("Form") && !screenType.getActualScreenComponent("Grid"))) {
                // Show the form when there is a mode or when there's only a Form available.
                this.createForm(mode, this.panels[0], true, screenType.hasTaskBar);
            }
            else {
                // Only show the FlowTable when the screen size allows it.
                if (multiPanelScreen) {
                    this.createFlowTable(this.panels[0], screenType);
                }
                else {
                    this.createListView(this.panels[0], screenType);
                }
            }
        }
        var taskBar = this.initTaskBar(screenType);
        this.pageShowPromise.then(function () {
            if (taskBar) {
                _this.createTaskBar(taskBar);
            }
            _this.setScrollElementHeight();
        });
        this.controller.events.contextChanged(function () {
            // When taskbar elements are removed or the detail-panel we need to check the listviewheight again
            _this.setScrollElementHeight();
        });
        if (screenType.hasDetailButtons && this.controller.hasVisibleDetails() &&
            // If we have 1 visible component and it is a grid and we have a form (but on a tab), we don't show the detail panel because clicking on the table would switch to a
            // different row and to the form - the hasMode is for when the user clicked on it and switched to the form
            !(screenType.actualScreenComponents.length === 1 && screenType.getActualScreenComponent("Grid") && screenType.hasForm && !hasMode)) {
            this.detailPanel = this.createDetailButtons();
            this.footer.before(this.detailPanel);
        }
    };
    AbstractScreen.prototype.tryFindComponent = function (type) {
        for (var i = 0; i < this.screenComponents.length; i++) {
            var screenComponent = this.screenComponents[i];
            if (screenComponent instanceof type)
                return screenComponent;
        }
        return null;
    };
    AbstractScreen.prototype.sizePanelsHeight = function (panels, detailPanel) {
        var _this = this;
        if (detailPanel === void 0) { detailPanel = jQuery(); }
        // Use the pageshow event to allow the content to become visible. When they aren't visible the height can't be calculated correctly.
        var page = this.page;
        // Hide detailPanel and make it visible after sizing the panels. This way it won't jump from top to bottom on smaller screens.
        // This happens because this element is pushed down by setting the panels height
        detailPanel.css("visibility", "hidden");
        this.updatePanelsPromise.then(function () {
            Utils.updatePanelsHeight(panels, _this.header.outerHeight(), window.innerHeight, 0, _this.footer.outerHeight());
            detailPanel.css("visibility", "");
        });
        page.on("updatePanelsHeight", function (e, data) {
            Utils.updatePanelsHeight(panels, data.headerHeight, data.height, 0, data.footerHeight);
            _this.setScrollElementHeight();
            data.handled = true;
        });
    };
    AbstractScreen.prototype.addScreenComponent = function (component) {
        component.pageShown = this.pageShowPromise;
        this.screenComponents.push(component);
    };
    AbstractScreen.prototype.createListView = function (contentElement, screenType, omitTaskBar) {
        if (omitTaskBar === void 0) { omitTaskBar = false; }
        var listview = new ListView(this.application, this.controller, this.url, contentElement, screenType.hasForm);
        this.addScreenComponent(listview);
        listview.initialize();
    };
    AbstractScreen.prototype.createFlowTable = function (contentElement, screenType) {
        var flowTable = new FlowTable(this.application, this.controller, this.url);
        this.addScreenComponent(flowTable);
        flowTable.initialize(contentElement, screenType.hasForm);
    };
    AbstractScreen.prototype.initTaskBar = function (screenType) {
        if (!screenType.hasTaskBar || this.controller.objView.taskLinkViews.length === 0) {
            return;
        }
        var taskBarLinkedTo = "form";
        screenType.screenComponents.forEach(function (screenComponent) {
            if (screenComponent.screenComponentTypeId !== "TaskBar") {
                return;
            }
            if (!screenComponent.screenComponentProps.hasOwnProperty("LinkedTo")) {
                return;
            }
            var allowedLinkedToValues = ["grid", "form", "both"];
            var linkedToValue = screenComponent.screenComponentProps["LinkedTo"].toLowerCase();
            if (allowedLinkedToValues.indexOf(linkedToValue) !== -1) {
                taskBarLinkedTo = linkedToValue;
            }
            else {
                taskBarLinkedTo = "form";
            }
        });
        var taskBar = new TaskBar(this.application, this);
        taskBar.linkedTo = taskBarLinkedTo;
        return taskBar;
    };
    AbstractScreen.prototype.createTaskBar = function (taskBar) {
        var $form = jQuery(".form");
        var $listHeaderWrapper = jQuery(".list-header-wrapper");
        switch (taskBar.linkedTo) {
            case "form":
                taskBar.initialize($form);
                break;
            case "grid":
                taskBar.initialize($listHeaderWrapper);
                break;
            case "both":
                // It can be the case that the taskbar is set to both but the form is
                // not on the screen by either design or lack of space. The taskbar cannot
                // refresh if the element is not found so we don't want to initialize at all.
                if ($form.length > 0) {
                    taskBar.initialize($form);
                }
                var taskBar1 = new TaskBar(this.application, this);
                this.addScreenComponent(taskBar1);
                taskBar1.initialize($listHeaderWrapper);
                break;
        }
    };
    AbstractScreen.prototype.setScrollElementHeight = function () {
        // We will only apply this function when there is a scrollable element..
        if (!jQuery(".screen .panel.grid, .screen .panel.table").length) {
            return;
        }
        // The first time the height of the taskBar is incorrectly calculated. The 0 timer will fix this.
        window.setTimeout(function () {
            var scrollElement = jQuery(".listview-scroll, .table-scroll-wrapper");
            var panelHeight = jQuery(".screen .panel").height();
            var taskBarHeight = jQuery(".list-header-wrapper > .taskBar.ui-navbar").outerHeight(true);
            var filterHeight = jQuery(".list-header-wrapper > .ui-filterable > .ui-input-search").outerHeight(true);
            var footerHeight = 0;
            // The detail-panel should not be calculated when a form is visible
            if (!jQuery(".panel2.form").length) {
                footerHeight = jQuery(".details-panel").height();
            }
            var correctHeight = panelHeight - taskBarHeight - filterHeight - footerHeight;
            scrollElement.height(correctHeight);
        }, 0);
    };
    AbstractScreen.prototype.createCube = function (contentElement) {
        var cube = new Cube.ScreenComponent(this.controller, contentElement);
        this.addScreenComponent(cube);
        cube.initialize();
        this.updateCubeButtonTransl(cube);
    };
    AbstractScreen.prototype.updateCubeButtonTransl = function (cube) {
        if (cube.cubeViewType == 2 /* Chart */) {
            this.footer.find("a.graph").text(this.application.language.transl(Model.TranslKeys.cube));
        }
        else {
            this.footer.find("a.graph").text(this.application.language.transl(Model.TranslKeys.graph));
        }
    };
    AbstractScreen.prototype.createForm = function (mode, contentElement, fullScreenWidth, applyTaskBar) {
        var _this = this;
        // Add style class so we can decide in css when the form should show multiple columns.
        // Keep this in sync with the parameter page (the full-screen-width part), so the breakpoints are the same
        $(contentElement).addClass(fullScreenWidth ? "full-screen-width" : "split-screen-width");
        var form = new TSFForm(this.application, this.controller, mode);
        this.addScreenComponent(form);
        form.initialize(contentElement);
        if (mode == 0 /* View */ && this.controller.objView.defaultEditableForm) {
            mode = 2 /* Edit */;
        }
        form.createFormEditor(mode);
        var that = this;
        var addButton = this.header.find("a.add-button");
        var controller = this.controller;
        controller.rowModifiedCallbacks = [function (isModified) {
                if (isModified) {
                    if (controller.currentEditor &&
                        controller.currentEditor.allowActions() &&
                        !controller.currentEditor.isCreatingNewRow()) {
                        _this.changeAddButtonMode(addButton, 2 /* Edit */);
                        _this.changeButtonState(addButton, _this.controller.saveRowAccessType);
                    }
                }
                else {
                    _this.changeAddButtonMode(addButton, 1 /* Add */);
                    _this.changeButtonState(addButton, _this.controller.addRowAccessType);
                }
            }];
        addButton.click(function (evt) {
            evt.stopPropagation();
            evt.preventDefault();
            if (that.controller.interfaceAvailable) {
                var controller_1 = that.controller;
                if (controller_1.isInEditMode) {
                    if (controller_1.currentEditor.allowActions()) {
                        // On default editable forms while editing, save and go into editmode again
                        if (controller_1.currentEditor.currentRowModified || controller_1.currentEditor.isCreatingNewRow()) {
                            form.createFormEditor(2 /* Edit */);
                            that.changeAddButtonMode(addButton, 2 /* Edit */);
                        }
                        else {
                            // On default editable forms while not editing, go into add mode
                            form.createFormEditor(1 /* Add */);
                        }
                    }
                    else {
                        controller_1.currentAction = that.saveEditmode();
                    }
                }
                else {
                    controller_1.currentAction = form.createFormEditor(1 /* Add */);
                }
            }
        });
        addButton.on("taphold", function () {
            if (!that.controller.isInEditMode) {
                var language = that.application.language;
                var options = [
                    new SelectMenuOption(1 /* Add */, language.transl(Model.TranslKeys.add), that.controller.addRowAccessType, function () { that.controller.currentAction = form.createFormEditor(1 /* Add */); }),
                    new SelectMenuOption(2 /* Edit */, language.transl(Model.TranslKeys.edit), that.controller.editRowAccessType, function () { that.controller.currentAction = form.createFormEditor(2 /* Edit */); }),
                    new SelectMenuOption(3 /* Copy */, language.transl(Model.TranslKeys.copy), that.controller.copyRowAccessType, function () { that.controller.currentAction = form.createFormEditor(3 /* Copy */); })
                ];
                var selectMenu = new SelectMenuPopup(that.controller).initialize(options);
            }
        });
    };
    AbstractScreen.prototype.changeAddButtonMode = function (addButton, to) {
        var language = this.application.language;
        if (to === 1 /* Add */) {
            addButton.text(language.transl(Model.TranslKeys.add));
            addButton.removeClass("ui-icon-check").addClass("ui-icon-plus");
        }
        else if (to === 2 /* Edit */) {
            addButton.text(language.transl(Model.TranslKeys.save));
            addButton.removeClass("ui-icon-plus").addClass("ui-icon-check");
        }
    };
    AbstractScreen.prototype.createDetailButtons = function () {
        var $detailPanel = jQuery("<div class='panel details-panel'></div>");
        if (this.panels.length < 2) {
            $detailPanel.addClass("full-screen");
        }
        var detailButtons = new DetailButtons(this.controller);
        this.addScreenComponent(detailButtons);
        detailButtons.initialize($detailPanel[0], this.url);
        return $detailPanel;
    };
    AbstractScreen.prototype.updateControlGroups = function () {
        var controllerInEditmode = this.controller.isInEditMode && !this.controller.currentEditor.allowActions();
        var containsCube = this.tryFindComponent(Cube.ScreenComponent) != null;
        var controlGroups = this.footer.find("> span > :jqmData(role='controlgroup'), > :jqmData(role='controlgroup')");
        controlGroups.filter('.editmode').css('display', (controllerInEditmode && !containsCube) ? 'inline-block' : 'none');
        controlGroups.filter('.editmode-buttons').css('display', (controllerInEditmode || containsCube) ? 'none' : 'inline-block');
        var readonlyGroup = controlGroups.filter('.readonly').css('display', controllerInEditmode ? 'none' : 'inline-block');
        readonlyGroup.find(".cubeview, .graph").css('display', !containsCube ? 'none' : 'inline-block');
        var addButton = this.header.find('a.add-button');
        if (containsCube) {
            addButton.css('display', 'none');
        }
        if (controllerInEditmode) {
            addButton.removeClass("ui-icon-plus").addClass("ui-icon-check");
        }
        else {
            addButton.removeClass("ui-icon-check").addClass("ui-icon-plus");
        }
    };
    AbstractScreen.prototype.updateSelectButton = function () {
        var context = this.controller.controllerContext;
        if (context.dataContextOption() == 2 /* Lookup */) {
            var accesType = context.parentController.isInEditMode && !this.controller.isInEditMode ? Enum.AccessType.Regular : Enum.AccessType.ReadOnly;
            this.changeButtonState(this.header.find("a.select-button"), accesType);
        }
    };
    AbstractScreen.prototype.bindControllerEvents = function () {
        var _this = this;
        this.controller.events.editmodeChanged(function () {
            var controller = _this.controller;
            _this.updateBackAndAddText();
            _this.changeButtonState(_this.header.find("a.add-button"), (controller.isInEditMode && controller.currentEditor.allowActions() && !controller.currentEditor.isCreatingNewRow()) ? controller.addRowAccessType : controller.saveRowAccessType);
            _this.updateSelectButton();
            _this.updateControlGroups();
            var maxCancelType = Enum.AccessType.Regular;
            if (controller.objView.formAutoSave) {
                // make the buttons visible or hidden so the user can cancel when in editmode
                // with formautosave we toggle between cancel/edit buttons
                _this.footer.find("a.cancel").toggleClass("ui-screen-hidden", !controller.isInEditMode);
                _this.footer.find("a.edit").toggleClass("ui-screen-hidden", controller.isInEditMode);
                if (!controller.isInEditMode && controller.objView.formAutoSave) {
                    maxCancelType = Enum.AccessType.Hidden;
                }
            }
            // update the cancel button, because when there wasn't an editor it is disabled
            _this.changeButtonState(_this.footer.find("a.cancel"), Math.max(controller.cancelRowAccessType, maxCancelType));
        });
        this.controller.events.datasetInvalidated(function (args) {
            for (var i = 0; i < _this.screenComponents.length; i++) {
                var screenComponent = _this.screenComponents[i];
                screenComponent.invalidateData();
            }
            _this.controller.validateData();
        });
        this.controller.events.rowIndexChanged(function () {
            var controller = _this.controller;
            var group = _this.footer.find("> span > :jqmData(role='controlgroup').editmode-buttons");
            group.find(".delete, .edit").toggleClass('ui-state-disabled', controller.currentRow == null);
            badgeHelper.updateBadges(_this.controller.badgeInfo);
            // Restore default editable form - when the user has a filter and there aren't any rows left
            // we need to recreate an editor when data becomes available
            if (controller.initialized && controller.objView.defaultEditableForm && !controller.currentEditor && controller.currentRow) {
                var form = _this.tryFindComponent(TSFForm);
                if (form) {
                    form.createFormEditor(2 /* Edit */);
                }
            }
        });
        this.controller.events.layoutChanged(function (args) {
            _this.changeLayoutState();
        });
        this.controller.events.contextChanged(function (args) {
            _this.changeContextState(args.contextState);
        });
        this.controller.events.datasetChanged(function (args) {
            _this.updateBreadCrumb();
            badgeHelper.updateBadges(_this.controller.badgeInfo);
        });
        this.controller.events.becameDirtyBySync(function (args) {
            // Check if all parent rows still exist.
            _this.controller.validateParentContextFilters().fail(function () {
                // Prompt an invalid context message when the context doesn't exist anymore.
                var invalidContextMessage = _this.application.language.transl(Model.TranslKeys.invalidParentContext);
                var promptAndRefresh = function () {
                    NavigationManager.showMessageWhenReady(_this.controller.objView.objTransl.transl, [invalidContextMessage]).always(function () {
                        _this.controller.currentAction.always(function () {
                            _this.controller.currentAction = _this.controller.refreshData(true);
                        });
                    });
                };
                // Cancel editmode when the context disappeared.
                if (_this.controller.isInEditMode) {
                    _this.controller.cancelEditMode().always(function () {
                        promptAndRefresh();
                    });
                }
                else {
                    promptAndRefresh();
                }
            }).done(function () {
                // Validate the controllers own context when the parents context is valid.
                if (!_this.controller.isInEditMode) {
                    _this.controller.currentAction.always(function () {
                        _this.controller.validateContext().done(function () {
                            _this.controller.currentAction = _this.controller.refreshData(true);
                        });
                    });
                }
            });
        });
        this.controller.events.processActionCompleted(function (args) {
            switch (args.processAction.processActionType) {
                case 3 /* AddRow */:
                    {
                        var form = _this.tryFindComponent(TSFForm);
                        if (form) {
                            form.createFormEditor(1 /* Add */);
                        }
                    }
                    break;
                case 11 /* ActivateForm */:
                case 10 /* ActivateGrid */:
                    {
                        var controlType = args.processAction.processActionType == 11 /* ActivateForm */ ? TSFForm : Grid;
                        var control = _this.tryFindComponent(controlType);
                        if (control) {
                            _this.controller.fireProcessActionCompleted(args.processAction.processActionType, true);
                        }
                        else {
                            // Switch to the page
                            var addEvents = true;
                            if (args.processAction.processActionType == 10 /* ActivateGrid */) {
                                var screentype = _this.controller.controllerContext.getScreenType();
                                if (screentype && screentype.getActualScreenComponent("Grid")) {
                                    // Hack; we know the previous page is the grid
                                    addEvents = true;
                                    history.back();
                                }
                            }
                            else if (args.processAction.processActionType == 11 /* ActivateForm */) {
                                addEvents = true;
                                NavigationManager.navigate(_this.getEditLink(0 /* View */));
                            }
                            if (addEvents) {
                                // Add pagecontainer events and fire the process action
                                jQuery(document.body).one("pagecontainershow", function (event) {
                                    _this.controller.fireProcessActionCompleted(args.processAction.processActionType, true);
                                });
                            }
                            else {
                                _this.controller.fireProcessActionCompleted(args.processAction.processActionType, false);
                            }
                        }
                    }
                    break;
            }
        });
    };
    AbstractScreen.prototype.saveEditmode = function () {
        var _this = this;
        // Disable the button so the user can't click twice
        this.changeSaveState(true);
        return this.controller.saveEditmode().fail(function () {
            // only re-enable on fail, in the other cases it will be enabled by the editmode changed event
            _this.changeSaveState(false);
        });
    };
    AbstractScreen.prototype.changeSaveState = function (disable) {
        var saveButtons = this.footer.find("> :jqmData(role='controlgroup') .save").add(this.header.find('a.add-button'));
        saveButtons.toggleClass("ui-state-disabled", disable);
    };
    AbstractScreen.prototype.updateBreadCrumb = function () {
        // No parent, so no breadcrumb possible
        if (!this.controller.parentController)
            return;
        var breadCrumbText = this.controller.getBreadCrumb(), breadCrumb = this.header.children(".breadCrumb");
        breadCrumbText.done(function (value) {
            if (value !== "") {
                // prepend a space to fix title in viewer
                breadCrumb.text(" " + value); // Don't change the breadcrumb but leave the &nbsp;
            }
            else {
                breadCrumb.html("&nbsp;");
            }
        });
    };
    AbstractScreen.prototype.updateBackAndAddText = function () {
        var language = this.application.language;
        var addTransl, backTransl;
        var controller = this.controller;
        if (controller.isInEditMode) {
            addTransl = language.transl(Model.TranslKeys.save);
            if (controller.currentEditor.isDefaultEditable()) {
                backTransl = language.transl(Model.TranslKeys.back);
            }
            else {
                backTransl = language.transl(Model.TranslKeys.cancel);
            }
        }
        else {
            addTransl = language.transl(Model.TranslKeys.add);
            backTransl = language.transl(Model.TranslKeys.back);
        }
        this.header.find("a.add-button").text(addTransl);
        this.header.find("a.back-button").text(backTransl);
    };
    AbstractScreen.prototype.changeLayoutState = function () {
        var footer = this.footer, header = this.header;
        var controller = this.controller;
        var addButton = header.find("a.add-button");
        // Let the backbutton behave as the cancel button in editmode.
        if (this.controller.isInEditMode || this.controller.objView.defaultEditableForm) {
            this.changeButtonState(header.find("a.back-button"), controller.layoutState.cancelRowAccessType);
            //We can only check if a new row is being created if the currentEditor is set.
            if (this.controller.currentEditor) {
                if (this.controller.currentEditor.isCreatingNewRow() || this.controller.currentEditor.currentRowModified) {
                    this.changeButtonState(addButton, controller.saveRowAccessType);
                    this.changeAddButtonMode(addButton, 2 /* Edit */);
                }
                else {
                    this.changeButtonState(addButton, controller.addRowAccessType);
                    this.changeAddButtonMode(addButton, 1 /* Add */);
                }
            }
        }
        else {
            this.changeButtonState(header.find("a.back-button"), Enum.AccessType.Regular);
            this.changeButtonState(addButton, controller.addRowAccessType);
        }
        this.changeButtonState(footer.find("a.edit"), controller.editRowAccessType);
        this.changeButtonState(footer.find("a.delete"), controller.deleteRowAccessType);
        var maxCancelType = Enum.AccessType.Regular;
        if (!controller.isInEditMode && controller.objView.formAutoSave) {
            maxCancelType = Enum.AccessType.Hidden;
        }
        this.changeButtonState(footer.find("a.cancel"), Math.max(controller.cancelRowAccessType, maxCancelType));
        this.changeButtonState(footer.find("a.save"), controller.saveRowAccessType);
        // Refresh controlgroups, so when the last item is hidden, borders are refreshed.
        // Also refresh invisible ones, because this is also called during the transition
        footer.find(".ui-controlgroup").controlgroup('refresh');
    };
    AbstractScreen.prototype.changeContextState = function (contextState) {
        // Disable or enable the onscreen detail buttons
        var buttons = this.tryFindComponent(DetailButtons);
        var taskBars = this.tryFindComponent(TaskBar);
        if (buttons) {
            buttons.updateButtonPanel(false);
        }
        if (taskBars) {
            taskBars.updateTaskPanel(false);
        }
        if (buttons || taskBars) {
            var height = window.innerHeight;
            Utils.updatePanelsHeight(this.content.find("div.panel:not(div.details-panel)"), this.header.outerHeight(), height, this.content.find("div.details-panel").outerHeight(), this.footer.outerHeight());
        }
        // Disable or enable the popup detail button.
        var detailBtn = this.footer.find("> span > :jqmData(role='controlgroup') a.details");
        if (contextState.anyVisible()) {
            detailBtn.removeClass('ui-state-disabled');
        }
        else {
            detailBtn.addClass('ui-state-disabled');
        }
    };
    AbstractScreen.prototype.changeButtonState = function (button, accessType) {
        if (accessType == Enum.AccessType.ReadOnly) {
            button.addClass("ui-state-disabled");
            button.removeClass("ui-screen-hidden");
        }
        else if (accessType == Enum.AccessType.Hidden) {
            button.addClass("ui-screen-hidden");
            button.removeClass("ui-state-disabled");
        }
        else if (accessType == Enum.AccessType.Regular) {
            button.removeClass("ui-state-disabled");
            button.removeClass("ui-screen-hidden");
        }
    };
    return AbstractScreen;
}());
