var Cube;
(function (Cube) {
    var GraphRow = /** @class */ (function () {
        function GraphRow() {
            this.rowDimension = "";
        }
        return GraphRow;
    }());
    var Graph = /** @class */ (function () {
        function Graph(cube, controller, contentElement) {
            this.cube = cube;
            this.controller = controller;
            this.chart = document.createElement("div");
            this.chart.id = "chartContainer";
            this.contentElement = contentElement;
            this.contentElement.appendChild(this.chart);
            this.options = {
                commonSeriesSettings: {
                    visible: true,
                    argumentField: "rowDimension"
                },
                animation: {
                    enabled: false
                },
                argumentAxis: {
                    grid: {
                        visible: true
                    }
                },
                tooltip: {
                    enabled: true
                },
                legend: {
                    verticalAlignment: "bottom",
                    horizontalAlignment: "center"
                },
                commonPaneSettings: {
                    border: {
                        visible: true,
                        right: false
                    }
                },
                onSeriesClick: function (info) {
                    var clickedSeries = info.target;
                    clickedSeries.isVisible() ? clickedSeries.hide() : clickedSeries.show();
                }
            };
        }
        /**
         * Creates a graph object that holds all the data necessary for building the graph
         * after that it calls the necessary methods to create the cube
         * @param reader  consist all data from the webservice.
         */
        Graph.prototype.appendData = function (rows) {
            var _this = this;
            this.options.commonSeriesSettings.type = Graph.convertChartTypes(this.cube.cubeModel.chartType);
            for (var i = 0; i < rows.length; i++) {
                var item = rows.item(i);
                var graphRow = this.createGraphRow(item);
                if (graphRow != null) {
                    var cubeRow = this.cube.addCubeValue(item);
                    for (var j = 0; j < this.cube.lowLevelColDimensions.length; j++) {
                        var lowLevelCol = this.cube.lowLevelColDimensions[j];
                        if (cubeRow.colValues.object[lowLevelCol.id] != null) {
                            for (var k = 0; k < this.cube.values.length; k++) {
                                var value = this.cube.values[k];
                                if (value.id == cubeRow.valueId) {
                                    graphRow[lowLevelCol.id + "_" + value.id] = parseFloat(cubeRow.value);
                                }
                            }
                        }
                    }
                    this.options.dataSource.push(graphRow);
                }
            }
            for (var i = 0; i < this.cube.lowLevelColDimensions.length; i++) {
                var dimension = this.cube.lowLevelColDimensions[i];
                for (var j = 0; j < this.cube.values.length; j++) {
                    var dimensionName = void 0;
                    if (dimension.objProp.domain != null && dimension.objProp.domain.elements.length > 0) {
                        var domElements = dimension.objProp.domain.elements.array;
                        for (var k = 0; k < domElements.length; k++) {
                            if (domElements[k].value == dimension.value) {
                                dimensionName = domElements[k].elementTransl.transl;
                                break;
                            }
                        }
                    }
                    else {
                        dimensionName = dimension.value + " | " + this.cube.values[j].value;
                    }
                    this.options.series.push({
                        valueField: dimension.id + "_" + this.cube.values[j].id,
                        name: dimensionName
                    });
                }
            }
            this.cube.pageShown.then(function () {
                jQuery(_this.chart).dxChart(_this.options);
            });
        };
        Graph.prototype.createGraphRow = function (item) {
            var graphRow = new GraphRow();
            var rows = this.cube.cubeModel.rows.array;
            for (var j = 0; j < rows.length; j++) {
                var row = rows[j];
                var rowValue = item.getValueForObjProp(row);
                if (rowValue == null) {
                    return null;
                }
                graphRow.rowDimension += rowValue;
                if (j != (rows.length - 1)) {
                    graphRow.rowDimension += " | ";
                }
            }
            return graphRow;
        };
        /**
         * Puts a name on the number. The SF generates a number thats equal on a chartType.
         * This method converts the number to the chart type name.
         * @param chartType  the number of the chart type.
         * @return string the name of the chart type.
         */
        Graph.convertChartTypes = function (chartType) {
            // TODO add other types
            switch (chartType) {
                case Model.ChartType.Area2D:
                    return "area";
                case Model.ChartType.AreaStacked2D:
                    return "stackedArea";
                case Model.ChartType.BarSideBySide2D:
                    return "bar";
                case Model.ChartType.BarStacked2D:
                    return "stackedBar";
                case Model.ChartType.Line2D:
                    return "line";
                default:
                    return "bar";
            }
        };
        Graph.prototype.invalidateData = function () {
            this.options.dataSource = [];
            this.options.series = [];
        };
        return Graph;
    }());
    Cube.Graph = Graph;
})(Cube || (Cube = {}));
