var Cube;
(function (Cube) {
    var Table = /** @class */ (function () {
        function Table(cube, controller, contentElement) {
            this.cube = cube;
            this.controller = controller;
            // Create a table and add the appropriate css classes and html5 attributes.
            this.table = document.createElement("table");
            this.table.setAttribute("data-role", "table");
            this.table.id = "cubeTable";
            contentElement.appendChild(this.table);
            this.conditionalLayoutConsumer = new ConditionalLayoutConsumer();
            this.grandTotalTranslation = controller.application.language.transl(Model.TranslKeys.grand_total);
        }
        /**
         * Creates a table element and applies the data from the underlying datasource.
         * @param reader contains the data of the underlying datasource.
         */
        Table.prototype.appendData = function (rows) {
            // Create a table header and add it to the table.
            var tableHead = document.createElement("thead");
            this.buildColumnDimensions(tableHead);
            this.table.appendChild(tableHead);
            // Create a table body.
            var tableBody = document.createElement("tbody");
            this.buildRowDimensions(tableBody, null, this.cube.highLevelRowDimensions, 0);
            // Row Total row
            if (this.cube.cubeModel.cubeView.showRowGrandTotal) {
                var totalRow = document.createElement("tr");
                var totalDimCell = totalRow.insertCell(i);
                totalDimCell.id = "row_grand_total";
                totalDimCell.className = "dimensionCell borderCell";
                totalDimCell.colSpan = this.cube.rowDimensionAggregationDepth + 1;
                totalDimCell.textContent = this.grandTotalTranslation;
                var additionalCells = this.cube.lowLevelColDimensions.length * this.cube.values.length;
                if (this.cube.cubeModel.cubeView.showColGrandTotal) {
                    additionalCells += this.cube.values.length;
                }
                for (var i = 0; i < additionalCells; i++) {
                    var totalCell = totalRow.insertCell(1 + i);
                    totalCell.className = "borderCell totalCell";
                }
                tableBody.appendChild(totalRow);
            }
            // Add data to the table body.
            for (var i = 0; i < rows.length; i++) {
                var item = rows.item(i);
                // Determine the dimensions for every row.
                var cubeRow = this.cube.addCubeValue(item);
                // Add the cubeRow to the table.
                this.addOrUpdateRow(tableBody, cubeRow);
            }
            this.applyConditionalLayouts(tableBody);
            // Append the table body to the table.
            this.table.appendChild(tableBody);
        };
        Table.prototype.buildColumnDimensions = function (tableHead) {
            var columnDepth = this.cube.colDimensionAggregationDepth + (this.cube.values.length - 1);
            for (var i = 0; i <= columnDepth; i++) {
                // Create a table row for every aggregation level.
                var headerRow = document.createElement("tr");
                for (var j = 0; j <= this.cube.rowDimensionAggregationDepth; j++) {
                    var cell = headerRow.insertCell(j);
                    cell.className = "dimensionCell";
                    if (i == columnDepth) {
                        // Insert the row dimension translation for the lowest level in the aggregation.
                        cell.textContent = this.cube.cubeModel.rows.array[j].objPropTransl.transl;
                        cell.classList.add("rowDimensionHeader");
                        cell.classList.add("borderCell");
                    }
                }
                if (i <= this.cube.colDimensionAggregationDepth) {
                    // Select the column dimensions that apply to the handled aggregation level.
                    var colList = Array.prototype.filter.call(this.cube.colDimensions, function (col) {
                        return col.aggregationLevel == i;
                    });
                    // Sort the column dimension.
                    colList.sort(function (a, b) {
                        if (a.parentDimensionId != null && b.parentDimensionId != null) {
                            if (a.parentDimensionId < b.parentDimensionId)
                                return -1;
                            if (a.parentDimensionId > b.parentDimensionId)
                                return 1;
                        }
                        if (a.value < b.value)
                            return -1;
                        if (a.value > b.value)
                            return 1;
                        return 0;
                    });
                    for (var j = 0; j < colList.length; j++) {
                        var currentCol = colList[j];
                        var colAdditionalCellCount = this.getCellCountForDimension(currentCol, true);
                        // Give the cell the value of the dimension and the id of the dimension.
                        var cell = headerRow.insertCell(this.cube.rowDimensionAggregationDepth + 1 + j);
                        cell.className = "dimensionCell borderCell";
                        cell.textContent = currentCol.value;
                        cell.id = currentCol.id;
                        cell.colSpan = colAdditionalCellCount + 1;
                    }
                }
                else if (this.cube.values.length > 1) {
                    for (var j = 0; j < this.cube.lowLevelColDimensions.length; j++) {
                        for (var k = 0; k < this.cube.values.length; k++) {
                            var cell = headerRow.insertCell(this.cube.rowDimensionAggregationDepth + 1 + (j * this.cube.values.length) + k);
                            cell.className = "dimensionCell borderCell";
                            cell.textContent = this.cube.values[k].value;
                        }
                    }
                    if (this.cube.cubeModel.cubeView.showColGrandTotal) {
                        for (var l = 0; l < this.cube.values.length; l++) {
                            var totalCell = headerRow.insertCell(headerRow.cells.length);
                            totalCell.className = "dimensionCell dataCell borderCell";
                            totalCell.textContent = this.cube.values[l].value;
                        }
                    }
                }
                if (i == 0 && this.cube.cubeModel.cubeView.showColGrandTotal) {
                    var colTotalCell = headerRow.insertCell(headerRow.cells.length);
                    colTotalCell.className = "dimensionCell borderCell";
                    colTotalCell.textContent = this.grandTotalTranslation;
                    colTotalCell.colSpan = this.cube.values.length;
                    colTotalCell.rowSpan = this.cube.colDimensionAggregationDepth + 1;
                }
                // Append the header row to the table head.
                tableHead.appendChild(headerRow);
            }
        };
        Table.prototype.buildRowDimensions = function (tableBody, row, rowDimensions, offSet) {
            for (var i = 0; i < rowDimensions.length; i++) {
                var rowDim = rowDimensions[i];
                if (i > 0 || row == null) {
                    row = this.createDimensionRow(rowDim);
                    offSet = 0;
                }
                else {
                    var cell = row.cells[offSet];
                    cell.className = "dimensionCell borderCell";
                    cell.textContent = rowDim.value;
                    cell.id = rowDim.id;
                    cell.rowSpan = 1 + this.getCellCountForDimension(rowDim, false);
                }
                tableBody.appendChild(row);
                this.buildRowDimensions(tableBody, row, rowDim.childDimensions, offSet + 1);
            }
        };
        Table.prototype.createDimensionRow = function (rowDim) {
            var row = document.createElement("tr");
            for (var i = 0; i <= this.cube.rowDimensionAggregationDepth - rowDim.aggregationLevel; i++) {
                var cell = row.insertCell(i);
                if (i == 0) {
                    cell.className = "dimensionCell borderCell";
                    cell.textContent = rowDim.value;
                    cell.id = rowDim.id;
                    cell.rowSpan = 1 + this.getCellCountForDimension(rowDim, false);
                }
            }
            var additionalCells = this.cube.lowLevelColDimensions.length * this.cube.values.length;
            var offSet = row.cells.length;
            for (var j = 0; j < additionalCells; j++) {
                var cell = row.insertCell(offSet + j);
                cell.className = "dataCell borderCell";
            }
            if (this.cube.cubeModel.cubeView.showColGrandTotal) {
                for (var i = 0; i < this.cube.values.length; i++) {
                    var totalCell = row.insertCell(row.cells.length);
                    totalCell.className = "totalCell dataCell borderCell";
                }
            }
            return row;
        };
        Table.prototype.getCellCountForDimension = function (columnDimension, includeValues) {
            var count = 0;
            if (columnDimension.childDimensions.length > 0) {
                count = columnDimension.childDimensions.length - 1;
                for (var i = 0; i < columnDimension.childDimensions.length; i++) {
                    count += this.getCellCountForDimension(columnDimension.childDimensions[i], includeValues);
                }
            }
            else if (includeValues && columnDimension.childDimensions.length == 0) {
                count += (this.cube.values.length - 1);
            }
            return count;
        };
        /**
         * Adds a row to the table or updates an existing row with additional values.
         * @param tableBody the html table body element.
         * @param cubeRow the cubeRow that should be added to the table body.
         */
        Table.prototype.addOrUpdateRow = function (tableBody, cubeRow) {
            if (this.cube.cubeModel.cubeView.showColGrandTotal &&
                this.cube.cubeModel.cubeView.showRowGrandTotal &&
                cubeRow.rowValues.array.length == 0 &&
                cubeRow.colValues.array.length == 0) {
                // GRAND TOTAL
                var rowTotalRow = tableBody.rows[tableBody.rows.length - 1];
                for (var i = 0; i < this.cube.values.length; i++) {
                    if (this.cube.values[i].id == cubeRow.valueId) {
                        var grandTotalCell = rowTotalRow.cells[rowTotalRow.cells.length - this.cube.values.length + i];
                        grandTotalCell.textContent = cubeRow.value;
                    }
                }
            }
            else if (this.cube.cubeModel.cubeView.showColGrandTotal &&
                cubeRow.rowValues.array.length > 0 &&
                cubeRow.colValues.array.length == 0) {
                // COL GRAND TOTAL
                var colTotalRow = this.findRow(tableBody, cubeRow.rowValues.array);
                if (colTotalRow) {
                    for (var i = 0; i < this.cube.values.length; i++) {
                        if (this.cube.values[i].id == cubeRow.valueId) {
                            var colTotalCell = colTotalRow.cells[colTotalRow.cells.length - this.cube.values.length + i];
                            colTotalCell.textContent = cubeRow.value;
                        }
                    }
                }
            }
            else if (this.cube.cubeModel.cubeView.showRowGrandTotal &&
                cubeRow.rowValues.array.length == 0 &&
                cubeRow.colValues.array.length > 0) {
                // ROW GRAND TOTAL
                var rowTotalRow = tableBody.rows[tableBody.rows.length - 1];
                var rowTotalCellIndex = this.findCellIndex(cubeRow);
                jQuery(rowTotalRow).find(".totalCell").eq(rowTotalCellIndex - this.cube.rowDimensionAggregationDepth - 1).text(cubeRow.value);
            }
            else {
                // NORMAL DATA CELL
                var row = this.findRow(tableBody, cubeRow.rowValues.array);
                if (row) {
                    var cellIndex = this.findCellIndex(cubeRow);
                    jQuery(row).find(".dataCell").eq(cellIndex - this.cube.rowDimensionAggregationDepth - 1).text(cubeRow.value);
                }
            }
        };
        Table.prototype.findRow = function (tableBody, rowValues) {
            for (var i = 0; i < tableBody.rows.length; i++) {
                var row = tableBody.rows[i];
                var dimensions = jQuery(row).find(".dimensionCell");
                for (var j = 0; j < dimensions.length; j++) {
                    var cell = dimensions[j];
                    var checkRow = this.cube.rowDimensions[cell.id];
                    if (checkRow && checkRow.aggregationLevel == this.cube.rowDimensionAggregationDepth) {
                        var rowVal = rowValues[cell.id];
                        if (rowVal != null && rowVal == checkRow.value && this.checkParents(checkRow, this.cube.rowDimensions, rowValues)) {
                            return row;
                        }
                    }
                }
            }
            return null;
        };
        /**
         * Finds the cell index of the column that matches with the given column dimension values.
         * @param colValues a list of values indexed on column dimension id.
         */
        Table.prototype.findCellIndex = function (cubeRow) {
            var headerRowIndex = this.table.tHead.rows.length - 1;
            if (this.cube.values.length > 1) {
                headerRowIndex -= 1;
            }
            // Get the lowest header row which match the column dimensions at the lowest aggregation level.
            var headerRow = this.table.tHead.rows[headerRowIndex];
            // Check which cell matches with the value of the column dimension.
            for (var i = 0; i < headerRow.cells.length; i++) {
                var cell = headerRow.cells[i];
                var checkCol = this.cube.colDimensions[cell.id];
                var colVal = cubeRow.colValues.object[cell.id];
                // Also check if his parent match.
                if (colVal != null && colVal == checkCol.value && this.checkParents(checkCol, this.cube.colDimensions, cubeRow.colValues.array)) {
                    if (this.cube.values.length > 1) {
                        for (var j = 0; j < this.cube.values.length; j++) {
                            if (this.cube.values[j].id == cubeRow.valueId) {
                                return (this.cube.rowDimensionAggregationDepth + 1) + ((i - (this.cube.rowDimensionAggregationDepth + 1)) * this.cube.values.length) + j;
                            }
                        }
                    }
                    return i;
                }
            }
        };
        /**
         * recursive method that checks if a given column dimension his parents match with the given column dimension values.
         * @param colValues a list of values indexed on column dimension id.
         */
        Table.prototype.checkParents = function (dimension, dimensionList, values) {
            if (dimension.parentDimensionId != null) {
                // Check if the parent dimension his value matches the value present in the colValues list.
                var parentCol = dimensionList[dimension.parentDimensionId];
                var checkVal = values[parentCol.id];
                if (checkVal != null) {
                    if (parentCol.value == checkVal) {
                        return this.checkParents(parentCol, dimensionList, values);
                    }
                    else {
                        return false;
                    }
                }
            }
            return true;
        };
        Table.prototype.applyConditionalLayouts = function (tableBody) {
            var conditionalLayoutFields = [];
            var cubeViewFields = this.cube.cubeModel.cubeView.cubeViewFields.array;
            for (var j = 0; j < cubeViewFields.length; j++) {
                var field = cubeViewFields[j];
                if (field.cubeArea == Model.CubeArea.Data && field.conditionalLayouts.length > 0) {
                    conditionalLayoutFields.push(field);
                }
            }
            if (conditionalLayoutFields.length > 0) {
                for (var i = 0; i < tableBody.rows.length; i++) {
                    var row = tableBody.rows[i];
                    var dataCells = jQuery(row).find(".dataCell");
                    if (dataCells.length > 0) {
                        for (var j = 0; j < this.cube.lowLevelColDimensions.length; j++) {
                            for (var k = 0; k < this.cube.values.length; k++) {
                                var index = (j * this.cube.values.length) + k;
                                var cell = dataCells[index];
                                var cellValue = void 0;
                                if (!Utils.isEmpty(cell.textContent))
                                    cellValue = parseFloat(cell.textContent);
                                else
                                    cellValue = undefined;
                                for (var l = 0; l < conditionalLayoutFields.length; l++) {
                                    if (conditionalLayoutFields[l].cubeField.cubeFieldId == this.cube.values[k].id) {
                                        for (var m = 0; m < conditionalLayoutFields[l].conditionalLayouts.length; m++) {
                                            var cl = conditionalLayoutFields[l].conditionalLayouts[m];
                                            if (this.conditionalLayoutConsumer.evaluateCondition(cl.numericCondition, cellValue, cl.value, cl.untilValue)) {
                                                cell.classList.add(cl.conditionalLayoutID);
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        };
        /**
         * Clears the table since the data is invalid.
         */
        Table.prototype.invalidateData = function () {
            jQuery(this.table).empty();
        };
        return Table;
    }());
    Cube.Table = Table;
})(Cube || (Cube = {}));
