var DetailButtons = /** @class */ (function () {
    function DetailButtons(controller) {
        var _this = this;
        this.visibleBadges = [];
        this.detailButtonClick = function (evt) {
            var func = function () {
                var $this = jQuery(evt.currentTarget);
                if ($this.parent().hasClass("more")) {
                    new DetailsPopup(_this.controller).initialize();
                }
                else {
                    NavigationManager.navigate(evt.currentTarget.href);
                    // Prevent clicking multiple times, clicking multiple times on the same page queues it which breaks JQM
                    jQuery(_this.contentElement).off("click.detailButtons").on("click.prevent", function (evt) {
                        return evt.preventDefault();
                    });
                }
            };
            // when in editmode first save the row; when not in editmode open/navigate to the page directly
            // validate editors will sync everything to the controller and will try to save
            if (_this.controller.isInEditMode && _this.controller.currentEditor.allowActions()) {
                _this.controller.currentEditor.validateEditors()
                    .then(function () { return func(); }, function () { jQuery(_this.contentElement).off("click.prevent").on("click.detailButtons", "a", _this.detailButtonClick); });
            }
            else {
                func();
            }
            evt.preventDefault();
        };
        this.controller = controller;
        if (window.matchMedia("(min-width: 800px)").matches == true) {
            this.maxAmountOfButtons = 5;
        }
        else {
            this.maxAmountOfButtons = 3;
        }
    }
    DetailButtons.prototype.initialize = function (contentElement, url) {
        var _this = this;
        // Remove mode param, so it will switch to the grid instead of the form.
        this.urlParam = url.search.replace(/&mode=./g, "");
        this.contentElement = contentElement;
        this.createButtonPanel();
        jQuery(contentElement).on("click.detailButtons", "a", this.detailButtonClick);
        this.controller.events.editmodeChanged(function () {
            _this.updateButtonPanel(true);
        });
    };
    DetailButtons.prototype.updateButtonPanel = function (editmodeChanged) {
        var stringifiedState = JSON.stringify(this.controller.contextState.detailContextState);
        if (this.cachedContextState != stringifiedState || editmodeChanged) {
            jQuery(this.contentElement).empty();
            this.createButtonPanel();
        }
        this.cachedContextState = stringifiedState;
    };
    DetailButtons.prototype.createButtonPanel = function () {
        var _this = this;
        this.unsubscribeBadges();
        var that = this;
        var buttonPanel = document.createElement("div");
        buttonPanel.className = "detail-buttons";
        buttonPanel.setAttribute("data-role", "navbar");
        // Exclude hidden details
        var contextState = this.controller.contextState.detailContextState;
        var orderedChildObjs = this.controller.objView.getOrderedChildObjs();
        var visibleDetails = Array.prototype.filter.call(orderedChildObjs, function (value) {
            return contextState[value.objLink.refId] != Enum.AccessType.Hidden;
        });
        var buttonList = document.createElement("ul");
        var needsMoreItem = visibleDetails.length > this.maxAmountOfButtons;
        var maxButtons = needsMoreItem ? this.maxAmountOfButtons - 1 : visibleDetails.length;
        var controller = this.controller;
        var allowActions = !this.controller.isInEditMode || this.controller.currentEditor.allowActions();
        var _loop_1 = function (i) {
            var objViewLink = visibleDetails[i], url = "abstractscreen.html" + this_1.urlParam + "&link=" + objViewLink.objLink.refId;
            var refTransl = objViewLink.objLink.inheritance ? objViewLink.refTransl.transl : objViewLink.refTransl.translPlural;
            refTransl = objViewLink.refAddTransl ? refTransl + " (" + objViewLink.refAddTransl + ")" : refTransl;
            var item = this_1.createDetailButton(url, refTransl, objViewLink.objView.icon);
            if (contextState[objViewLink.objLink.refId] == Enum.AccessType.ReadOnly || !allowActions) {
                item.className = "ui-state-disabled";
            }
            if (objViewLink.objView.showBadge) {
                var badgeInfo = controller.badgeInfo[objViewLink.objLink.refId];
                var token = badgeInfo.subscribe(function (value, error) {
                    _this.updateBadge(value, item.firstElementChild);
                });
                this_1.visibleBadges.push({ token: token, badge: badgeInfo });
                this_1.updateBadge(badgeInfo.currentValue, item.firstElementChild);
            }
            buttonList.appendChild(item);
        };
        var this_1 = this;
        for (var i = 0; i < maxButtons; i++) {
            _loop_1(i);
        }
        // Show the "more button" when there are more details available than we can show.
        if (needsMoreItem) {
            var moreBtn = this.createDetailButton("#", "...", null);
            moreBtn.className = "more";
            buttonList.appendChild(moreBtn);
        }
        // When there are no items (maybe due to a contextprocedure), don't enhance this element or it will crash
        // When a detail becomes visible again, this code will fix it again.
        if (buttonList.children.length > 0) {
            buttonPanel.appendChild(buttonList);
            this.contentElement.appendChild(buttonPanel);
            jQuery(buttonPanel).navbar();
        }
    };
    DetailButtons.prototype.updateBadge = function (value, button) {
        if (value != null) {
            var createBadgeSpan = function (root) {
                var span = document.createElement("span");
                span.className = "ui-li-count";
                root.appendChild(span);
                return span;
            };
            var span = jQuery(button).children(".ui-li-count")[0] || createBadgeSpan(button);
            span.textContent = value + "";
        }
        else {
            jQuery(button).children(".ui-li-count").remove();
        }
    };
    DetailButtons.prototype.createDetailButton = function (url, transl, icon) {
        var listItem = document.createElement('li');
        var anchor = document.createElement('a');
        anchor.href = url;
        var span = document.createElement("span");
        if (icon) {
            anchor.className = "ui-li-has-icon";
            var image = document.createElement('img');
            image.src = Utils.createIconFromBase64(icon);
            anchor.appendChild(image);
            span.className = "icon-span";
        }
        else {
            span.className = "iconless-span";
        }
        span.textContent = transl;
        anchor.appendChild(span);
        listItem.appendChild(anchor);
        return listItem;
    };
    DetailButtons.prototype.unsubscribeBadges = function () {
        for (var _i = 0, _a = this.visibleBadges; _i < _a.length; _i++) {
            var info = _a[_i];
            info.badge.unsubscribe(info.token);
        }
        this.visibleBadges = [];
    };
    DetailButtons.prototype.invalidateData = function () {
    };
    DetailButtons.prototype.dispose = function () {
        this.unsubscribeBadges();
    };
    return DetailButtons;
}());
