var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var Popup = /** @class */ (function () {
    function Popup(className, dismissable) {
        if (dismissable === void 0) { dismissable = true; }
        this.className = className;
        this.dismissable = dismissable;
        this.removeOnClose = true;
        this.onDestroy = [];
        this.createScrollBar = true;
    }
    Popup.prototype.createPopupElement = function () {
        var popup = this.popupElement = document.createElement("div");
        popup.setAttribute("data-role", "popup");
        popup.style.minWidth = "200px";
        popup.className = this.className;
        return popup;
    };
    Popup.prototype.applyStyleAndHandleEvents = function () {
        var that = this;
        var popup = this.popupElement;
        var page = jQuery(popup).closest(".ui-page");
        jQuery(popup).popup({
            history: false,
            dismissible: this.dismissable
        }).enhanceWithin()
            .on("popupbeforeposition", function () {
            var $this = jQuery(this);
            var maxHeight = $(window).height() - 30; // some space for the popup.
            var fixedElementHeight = $this.children(":jqmData(role=header)").outerHeight() + $this.children(":jqmData(role=footer)").outerHeight();
            $this.css('max-height', maxHeight + 'px')
                .children(".scroll").css('max-height', (maxHeight - fixedElementHeight) + "px");
            if (that.scrollBar) {
                that.scrollBar.refresh();
            }
        }).on("popupafteropen", function () {
            var $this = jQuery(this);
            var data = $this.data("mobile-popup");
            if (data && data._ui) {
                // hack so jQuery mobile doesn't autofocus controls - which it still does after clicking on an empty space in the form
                // This is especially annoying when the first control on a taskpopup is suggestion control
                // I did this at the popupafteropen - so it does work the first time but not when clicking an empty space
                data._ui.focusElement = jQuery();
            }
            // Only enable handle click for Chrome on Android.
            if (that.createScrollBar) {
                var matches = /Android.*Chrome\/(\d+)/.exec(window.navigator.userAgent);
                // Added extra check for electron and ontouchstart - handleClick should be true or clicking a item won't work with touch
                var handleClick = matches && parseInt(matches[1], 10) >= 30 || (Utils.isElectron() && 'ontouchstart' in window);
                that.scrollBar = new iScroll(jQuery(this).children(".scroll")[0], { handleClick: handleClick });
            }
            // Save the initial height for resize purposes.
            var initialHeight = jQuery(popup).height();
            page.on("updatePanelsHeight.Popup", function () {
                that.resizeAndReposition(initialHeight);
            });
            // Execute a resize event after the popup opened. Because the initial popup might be to large.
            that.resizeAndReposition(initialHeight);
        }).popup('open')
            .on('popupafterclose', function () {
            if (that.removeOnClose) {
                that.destroyPopup();
            }
            // Always remove this event since it gets re-added on popupafteropen.
            page.off("updatePanelsHeight.Popup");
        });
        jQuery(popup).parent().addClass(this.className);
    };
    Popup.prototype.resizeAndReposition = function (initialHeight) {
        if (this.onResizeEvent) {
            this.onResizeEvent(initialHeight);
        }
        jQuery(this.popupElement).popup("reposition", {
            positionTo: "window"
        });
    };
    Popup.prototype.destroyPopup = function () {
        if (this.scrollBar) {
            this.scrollBar.destroy();
        }
        if (this.onDestroy.length > 0) {
            this.onDestroy.forEach(function (destroyFunc) {
                destroyFunc();
            });
            //Prevent the onDestroy callbacks from being called multiple times
            this.onDestroy = [];
        }
        if (this.popupElement && jQuery(this.popupElement).data("mobile-popup")) {
            jQuery(this.popupElement).popup('destroy').remove();
        }
    };
    return Popup;
}());
var ConfirmationPopup = /** @class */ (function (_super) {
    __extends(ConfirmationPopup, _super);
    function ConfirmationPopup(confirmationOptions, defaultConfirmationOption) {
        var _this = _super.call(this, "confirmationPopup") || this;
        _this.confirmationOptions = confirmationOptions;
        _this.result = defaultConfirmationOption;
        return _this;
    }
    ConfirmationPopup.CreateOkPopup = function () {
        return new ConfirmationPopup([Enum.ConfirmationOption.Ok], Enum.ConfirmationOption.Ok);
    };
    ConfirmationPopup.CreateYesNoPopup = function () {
        return new ConfirmationPopup([Enum.ConfirmationOption.No, Enum.ConfirmationOption.Yes], Enum.ConfirmationOption.No);
    };
    ConfirmationPopup.prototype.initialize = function (titleTransl, textTransl, buttonGenerator) {
        if (textTransl === void 0) { textTransl = null; }
        var deferred = jQuery.Deferred();
        var popup = this.createPopupElement();
        // Create header
        var header = document.createElement("div");
        header.setAttribute("data-role", "header");
        var title = document.createElement("h1");
        title.innerText = titleTransl;
        header.appendChild(title);
        popup.appendChild(header);
        // Create content and bind events
        var that = this;
        var footer = document.createElement("div");
        footer.setAttribute("data-role", "footer");
        if (!buttonGenerator) {
            buttonGenerator = ConfirmationPopup.defaultButtonGenerator;
        }
        for (var i = 0; i < this.confirmationOptions.length; i++) {
            var ele = buttonGenerator(this.confirmationOptions[i]);
            footer.appendChild(ele);
        }
        jQuery(popup).on('popupafterclose', function () {
            deferred.resolve(that.result);
        });
        // Create scroll div
        var scrollDiv = jQuery("<div class='scroll'></div>");
        if (textTransl) {
            var textSpan = document.createElement("span");
            textSpan.className = "popupText";
            textSpan.innerText = textTransl;
            scrollDiv.append(textSpan);
        }
        else {
            this.createScrollBar = false;
        }
        scrollDiv.appendTo(popup);
        popup.appendChild(footer);
        //If there's no transition happening we can append immediately, if there is, we wait for pagechange.
        if (jQuery("body.ui-mobile-viewport-transitioning").length === 0) {
            this.initializeInternal(popup, footer);
        }
        else {
            jQuery(document).one("pagechange", function () {
                that.initializeInternal(popup, footer);
            });
        }
        _super.prototype.resizeAndReposition.call(this, jQuery(popup).height());
        return deferred.promise();
    };
    ConfirmationPopup.prototype.initializeInternal = function (popup, footer) {
        var that = this;
        jQuery.mobile.activePage[0].appendChild(popup);
        // Bind default events
        this.applyStyleAndHandleEvents();
        jQuery(footer).on("click", "a", function () {
            var button = jQuery(this);
            var option = Enum.ConfirmationOption[button.attr("data-confirmoption")];
            that.result = option;
            jQuery(popup).popup('close');
        });
    };
    ConfirmationPopup.defaultButtonGenerator = function (option) {
        var language;
        if (Model.TSFApplicationManager.Current.activeAppl) {
            language = Model.TSFApplicationManager.Current.activeAppl.language;
        }
        else {
            language = AppConfig.language;
        }
        var transl;
        switch (option) {
            case Enum.ConfirmationOption.Ok:
                transl = language.transl(Model.TranslKeys.ok);
                break;
            case Enum.ConfirmationOption.Cancel:
                transl = language.transl(Model.TranslKeys.cancel);
                break;
            case Enum.ConfirmationOption.Yes:
                transl = language.transl(Model.TranslKeys.yes);
                break;
            case Enum.ConfirmationOption.No:
                transl = language.transl(Model.TranslKeys.no);
                break;
        }
        var element = document.createElement("a");
        element.href = "#";
        element.className = "popupBtn";
        element.setAttribute("data-role", "button");
        element.setAttribute("data-confirmoption", Enum.ConfirmationOption[option]);
        element.setAttribute("data-shadow", "false");
        element.innerText = transl;
        return element;
    };
    ConfirmationPopup.invalidGuiVersionButtonGenerator = function (option) {
        var manager = Model.TSFApplicationManager.Current;
        var language;
        if (manager.activeAppl) {
            language = manager.activeAppl.language;
        }
        else {
            language = AppConfig.language;
        }
        var matchesAndroid = Utils.isAndroid();
        var matchesIOS = Utils.getiOSVersion() != null;
        var btn = ConfirmationPopup.defaultButtonGenerator(option);
        if (matchesAndroid || matchesIOS) {
            var url = matchesAndroid ? manager.globalSettings.urlNextVersionGooglePlay : manager.globalSettings.urlNextVersionAppStore;
            btn.target = "_blank";
            btn.innerText = language.transl(Model.TranslKeys.download);
            // Make sure it starts with http:// or https://
            if (/^http[s]?:[\/\/]/.test(url)) {
                btn.href = url;
            }
            else {
                btn.href = "http://" + url;
            }
        }
        return btn;
    };
    return ConfirmationPopup;
}(Popup));
