var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var PrefilterPopup = /** @class */ (function (_super) {
    __extends(PrefilterPopup, _super);
    function PrefilterPopup(controller) {
        var _this = _super.call(this, "prefilterPopup") || this;
        _this.controller = controller;
        return _this;
    }
    PrefilterPopup.prototype.initialize = function () {
        var controller = this.controller;
        var prefilterstateList = this.createPrefilterCollection();
        var popup = this.createPopupElement();
        var popupHeader = this.createPopupHeader(this.controller.application.language);
        popup.appendChild(popupHeader);
        var scrollDiv = jQuery("<div class='scroll'></div>");
        var prefilterList = this.createPrefilterList(prefilterstateList.array);
        scrollDiv.append(prefilterList).appendTo(popup);
        jQuery.mobile.activePage[0].appendChild(popup);
        var popupFooter = this.createPopupFooter(this.controller.application.language);
        popup.appendChild(popupFooter);
        // Added extra check for electron and ontouchstart - we enable handleClick because of touch/mouse issues
        // but this is causing the click to fire twice; detect the iscroll click and cancel it
        if (Utils.isElectron() && 'ontouchstart' in window) {
            prefilterList.addEventListener("click", function (e) {
                if (e._fake) {
                    e.preventDefault();
                    e.stopImmediatePropagation();
                }
            });
        }
        var $prefilterList = jQuery(prefilterList).listview();
        $prefilterList.on("change", "input", function () {
            var item = jQuery(this);
            var groupId = item.data("prefiltergroup"), prefilterId = item.data("prefilter-id");
            if (groupId) {
                var group = prefilterstateList.key(prefilterId).prefilter.group;
                if (group.isExcluded) {
                    $prefilterList.find("input:jqmData(prefiltergroup='" + groupId + "')")
                        .not(this) // exclude clicked element
                        .prop('checked', false) // uncheck all other items in this group
                        .checkboxradio("refresh");
                }
            }
        });
        jQuery(popupFooter).children("a").click(function (evt) {
            var button = jQuery(this);
            if (button.hasClass('save')) {
                var refresh_1 = false;
                $prefilterList.find("input")
                    .each(function (index, element) {
                    var $element = jQuery(element);
                    var prefilterId = $element.data("prefilter-id"), isSelected = $element.prop('checked');
                    if (controller.setPrefilterState(prefilterstateList.key(prefilterId), isSelected)) {
                        refresh_1 = true;
                    }
                });
                if (refresh_1) {
                    controller.currentAction = controller.refreshData();
                }
            }
            //Prevent click being applied to listview underneath. TCP 37919
            evt.preventDefault();
            evt.stopPropagation();
            jQuery(popup).popup('close');
        });
        this.applyStyleAndHandleEvents();
    };
    PrefilterPopup.prototype.createPrefilterCollection = function () {
        var current = this.controller.currentPrefilterStates();
        var list = new Collections.List();
        for (var i = 0; i < current.length; i++) {
            var state = current[i];
            list.add(state.prefilter.prefilterId, state);
        }
        return list;
    };
    PrefilterPopup.prototype.createPrefilterList = function (prefilterstateList) {
        var prefilterView = document.createElement("ul");
        prefilterView.setAttribute("data-role", "listview");
        var previousGroup;
        for (var i = 0; i < prefilterstateList.length; i++) {
            var prefilterState = prefilterstateList[i], prefilter = prefilterState.prefilter;
            if (!prefilterState.isHidden) {
                if (prefilter.group) {
                    if (prefilter.group !== previousGroup) {
                        previousGroup = prefilter.group;
                        prefilterView.appendChild(this.createDivider(prefilter.group.grpTransl.transl)); // Create group divider
                    }
                }
                else if (previousGroup) {
                    // create new remaining group
                    previousGroup = null;
                    prefilterView.appendChild(this.createDivider("Misc"));
                }
                prefilterView.appendChild(this.createRow(prefilter, prefilterState.selected, prefilterState.locked));
            }
        }
        return prefilterView;
    };
    PrefilterPopup.prototype.createPopupHeader = function (language) {
        var header = document.createElement("div");
        header.setAttribute("data-role", "header");
        var title = document.createElement("h1");
        title.innerText = language.transl(Model.TranslKeys.prefilterSubmenu);
        header.appendChild(title);
        return header;
    };
    PrefilterPopup.prototype.createPopupFooter = function (language) {
        var footer = document.createElement("div");
        footer.setAttribute("data-role", "footer");
        var okButton = document.createElement("a");
        okButton.setAttribute("data-iconpos", "notext");
        var cancelButton = okButton.cloneNode(false);
        okButton.setAttribute("data-shadow", "false");
        okButton.innerText = language.transl(Model.TranslKeys.ok);
        okButton.className = "save";
        cancelButton.setAttribute("data-shadow", "false");
        cancelButton.innerText = language.transl(Model.TranslKeys.cancel);
        cancelButton.className = "cancel";
        var title = document.createElement("h1");
        title.innerText = language.transl(Model.TranslKeys.prefilterSubmenu);
        footer.appendChild(cancelButton);
        footer.appendChild(okButton);
        return footer;
    };
    PrefilterPopup.prototype.createDivider = function (text) {
        var divider = document.createElement("li");
        divider.setAttribute("data-role", "list-divider");
        divider.innerText = text;
        return divider;
    };
    PrefilterPopup.prototype.createRow = function (prefilter, selected, locked) {
        var element = document.createElement("li");
        element.className = "checkbox-item";
        var label = document.createElement("label");
        label.setAttribute("data-corners", "false");
        label.className = "checkbox-label";
        label.innerText = prefilter.prefilterTransl.transl;
        var checkbox = document.createElement("input");
        checkbox.type = "checkbox";
        checkbox.checked = selected;
        checkbox.disabled = locked;
        if (prefilter.group)
            checkbox.setAttribute("data-prefiltergroup", prefilter.group.groupId);
        checkbox.setAttribute("data-prefilter-id", prefilter.prefilterId);
        label.appendChild(checkbox);
        element.appendChild(label);
        return element;
    };
    return PrefilterPopup;
}(Popup));
