var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var TaskPopup = /** @class */ (function (_super) {
    __extends(TaskPopup, _super);
    function TaskPopup(taskView) {
        var _this = _super.call(this, "taskPopup", false) || this;
        _this.taskView = taskView;
        _this.dontReopen = false;
        _this.application = Model.TSFApplicationManager.Current.activeAppl;
        _this.createScrollBar = false;
        return _this;
    }
    TaskPopup.prototype.initialize = function (page, paramController, contextController) {
        var _this = this;
        var deferred = jQuery.Deferred();
        PopupManager.queuePopup(deferred, null).then(function () {
            var popup = _this.createPopupElement();
            var $popup = jQuery(popup);
            // The minimal width of the task popup is 50% of the windows width.
            popup.style.minWidth = (page.width() / 2) + "px";
            popup.appendChild(_this.createHeader());
            var scrollDiv = document.createElement("div");
            scrollDiv.className = "scroll taskPageContent";
            popup.appendChild(scrollDiv);
            popup.appendChild(_this.createFooter());
            // Update the height of the task popup after a resize event.
            _this.onResizeEvent = function (initialHeight) {
                _this.updatePopupHeight(initialHeight);
                // Trigger resize for form control
                jQuery(scrollDiv).trigger("tsf-updatelayout");
            };
            // We need to add the popup to the page here or else building the form won't work correctly with some controls
            // One of the controls which won't work is the signature control (it won't initialize)
            page[0].appendChild(popup);
            _this.paramPage = new ParameterPage($popup);
            _this.paramPage.build(_this.taskView, paramController, contextController, scrollDiv);
            _this.applyStyleAndHandleEvents();
            var taskPromise = null;
            $popup.data(TaskPopup.TSF_TASK_KEY, _this);
            var that = _this;
            _this.paramPage.footer.on("click", "a", function () {
                var $this = jQuery(this);
                if ($this.hasClass("execute")) {
                    // Close the popup without removing it. We might need to re-open the popup because we can't leave editor mode.
                    that.closeWithoutRemove();
                    that.dontReopen = true;
                    var timeoutDeferred_1 = jQuery.Deferred();
                    /* We need a small delay before trying to leave edit mode on handscanners because enter is triggered so
                       rapidly the editor is still in editmode. */
                    if (AppConfig.fixesForHandscanners) {
                        setTimeout(function () {
                            timeoutDeferred_1.resolve();
                        }, 100);
                    }
                    else {
                        timeoutDeferred_1.resolve();
                    }
                    timeoutDeferred_1.then(function () {
                        that.paramPage.form.controllerEditor.canLeaveEditorMode().done(function () {
                            taskPromise = taskHelper.tryExecuteTask(paramController, contextController, that.taskView);
                            taskPromise.always(function () {
                                that.close();
                            });
                        }).fail(function (objProp) {
                            $popup.popup("open");
                            that.paramPage.form.controllerEditor.suggestCursorIndex(objProp);
                        });
                    });
                }
                else {
                    that.close();
                }
            });
            if (AppConfig.fixesForHandscanners) {
                //Focus on the first empty text input if there is one.
                var $firstEmptyTextInput = $popup.find(".ui-input-text").filter(function () {
                    return jQuery(this).find("input").val() === "";
                });
                if ($firstEmptyTextInput.length > 0) {
                    $firstEmptyTextInput.find("input")[0].focus();
                }
            }
            $popup.on("popupafterclose", function () {
                deferred.resolve();
            });
            _this.onDestroy.push(function () {
                // Remove events, an error occors when events are fired while the component is already gone
                // For example form tries to refresh jQm controls which aren't enhanced
                // TCP #25862
                _this.paramPage.paramController.events.clearEvents();
                jQuery(_this.popupElement).removeData(TaskPopup.TSF_TASK_KEY);
                var taskSucceeded = taskPromise ? taskPromise.state() === "resolved" : false;
                if (contextController) {
                    contextController.taskActionPerformed(_this.taskView, taskSucceeded);
                }
                else {
                    _this.executeProcessActionWithoutContext(taskSucceeded);
                }
            });
        });
        return deferred.promise();
    };
    TaskPopup.prototype.executeProcessActionWithoutContext = function (success) {
        var properties = new ProcessEngine.CompletedProcessActionProperties(success);
        var processFlow = this.taskView.paramObjView.getTaskWithoutContextProcessFlowRoot(60 /* GotoTask */, this.taskView.task.taskId);
        if (!processFlow) {
            return;
        }
        ProcessEngine.postActionCompleted(new ProcessEngine.CompletedProcessAction(processFlow, properties), null);
    };
    TaskPopup.prototype.updatePopupHeight = function (initialHeight) {
        var taskPopup = jQuery(".taskPopup");
        var taskPageContent = taskPopup.children(".taskPageContent");
        var taskHeaderHeight = this.paramPage.header.outerHeight();
        var taskFooterHeight = this.paramPage.footer.outerHeight();
        var maxPopupHeight = window.innerHeight - 30;
        var currentContentHeight = taskPageContent.height();
        var currentPopupHeight = currentContentHeight + taskHeaderHeight + taskFooterHeight;
        // Use the maxPopupHeight when the currentPopupHeight exceeds to maximum.
        if (currentPopupHeight > maxPopupHeight) {
            var taskHeight = maxPopupHeight - taskHeaderHeight - taskFooterHeight;
            taskPageContent.css("height", taskHeight + "px");
            // Fire the tsf-updatelayout event to resize the form.
            taskPopup.trigger("tsf-updatelayout", [currentContentHeight, taskHeight, false]);
        }
        else if (currentPopupHeight < maxPopupHeight && currentPopupHeight < initialHeight) {
            var taskHeight = initialHeight - taskHeaderHeight - taskFooterHeight;
            taskPageContent.css("height", taskHeight + "px");
            // Fire the tsf-updatelayout event to resize the form.
            taskPopup.trigger("tsf-updatelayout", [currentContentHeight, taskHeight, false]);
        }
    };
    TaskPopup.prototype.createHeader = function () {
        var header = document.createElement("div");
        header.setAttribute("data-role", "header");
        var h1 = document.createElement("h1");
        h1.textContent = this.taskView.taskTransl.transl;
        header.appendChild(h1);
        return header;
    };
    TaskPopup.prototype.createFooter = function () {
        var footer = document.createElement("div");
        footer.setAttribute("data-role", "footer");
        var language = this.application.language;
        var cancelAnchor = this.createButton(language.transl(Model.TranslKeys.cancel));
        cancelAnchor.className = "cancel";
        cancelAnchor.setAttribute("data-shadow", "false");
        var executeAnchor = this.createButton(language.transl(Model.TranslKeys.execute));
        executeAnchor.className = "execute";
        executeAnchor.setAttribute("data-shadow", "false");
        //Execute is appended before cancel to make sure barscanners press enter on the execute button.
        footer.appendChild(executeAnchor);
        footer.appendChild(cancelAnchor);
        return footer;
    };
    TaskPopup.prototype.createButton = function (text) {
        var anchor = document.createElement("a");
        anchor.href = "#";
        anchor.innerText = text;
        return anchor;
    };
    TaskPopup.prototype.close = function () {
        //Needed so the task controller doesn't stay in editmode.
        this.paramPage.paramController.clearControllerEditor();
        this.removeOnClose = true;
        jQuery(this.popupElement).popup('close');
        this.destroyPopup();
    };
    TaskPopup.prototype.open = function () {
        jQuery(this.popupElement).popup('open');
    };
    TaskPopup.prototype.closeWithoutRemove = function () {
        this.removeOnClose = false;
        jQuery(this.popupElement).popup('close');
        this.removeOnClose = true;
    };
    TaskPopup.TSF_TASK_KEY = 'task-task';
    return TaskPopup;
}(Popup));
