var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
// tslint:disable:max-classes-per-file
var ControlHelper = /** @class */ (function () {
    function ControlHelper(viewProp, form) {
        this.viewProp = viewProp;
        this.form = form;
        this.mandatoryClass = "mandatory-field";
        this.readonlyClass = "readonly-field";
        this.hiddenClass = "hidden-field";
        this.hiddenKeepSpaceClass = "hidden-keep-space-field";
        this.objProp = this.viewProp.objProp;
        this.isNumeric = this.objProp.isNumeric();
    }
    ControlHelper.prototype.createControl = function (height) {
        var container = document.createElement("div");
        container.className = "ui-input-text ui-body-inherit";
        var control = document.createElement("input");
        control.setAttribute("data-enhanced", "true");
        control.id = this.objProp.objPropId + "_control";
        control.className = "preventFastclickFocus";
        control.type = "text";
        container.appendChild(control);
        return container;
    };
    ControlHelper.prototype.initializeControl = function () {
        this.__control = this.form.formBody.find('#' + this.objProp.objPropId + "_control");
        this.__field = this.form.formBody.find("[data-control-id=\"" + this.objProp.objPropId + "\"]");
        // Apply input validation
        if (this.objProp.uppercase) {
            this.__control.addClass("input-uppercase");
        }
        this.trySetNumeric();
        this.trySetMaxLength();
    };
    ControlHelper.prototype.validateValue = function () {
        return jQuery.Deferred().resolve().promise();
    };
    ControlHelper.prototype.setValue = function (value) {
        if (value != null && this.isNumeric && this.objProp.precision > 0) {
            // Formats the number so it can be used in a JavaScript input type=number and removes extra precision
            // which is sometimes provided by the services
            value = parseFloat(value).toFixed(this.objProp.precision);
        }
        this.findControl().val(value);
    };
    ControlHelper.prototype.setDefaultValue = function () {
        this.findControl().val(null);
    };
    ControlHelper.prototype.getValue = function () {
        var value = this.findControl().val();
        if (!Utils.isEmpty(value)) {
            // Numeric values should be parsed back to a float and then stringified.
            // This is because the DataSet contains the value "8" for Numeric(x,2) instead of "8.00" which is the value of the control.
            // Useless defaults will be executed if we do parse the value.
            if (this.isNumeric) {
                // && this.objProp.precision > 0 // we could use this check but inputvalidations.js do fail sometimes (input type number on cefclient)
                // and Electron because of some weird issues with selection on number inputs.
                // We should actually fix the inputvalidations but this is just an extra guard, so it won't go wrong ever
                value = parseFloat(value) + "";
            }
            if (this.objProp.uppercase && typeof value == "string") {
                value = value.toUpperCase();
            }
            return value;
        }
        return null;
    };
    ControlHelper.prototype.setMandatory = function (mandatory) {
        var control = this.findControl();
        if (mandatory) {
            control.parent().addClass(this.mandatoryClass);
        }
        else {
            control.parent().removeClass(this.mandatoryClass);
        }
    };
    ControlHelper.prototype.setReadonly = function (readonly) {
        var control = this.findControl();
        if (readonly) {
            control.parent().addClass(this.readonlyClass);
        }
        else {
            control.parent().removeClass(this.readonlyClass);
        }
        control.prop("readOnly", readonly);
    };
    ControlHelper.prototype.setHidden = function (hide, keepSpace) {
        if (hide) {
            if (keepSpace) {
                this.findField().addClass(this.hiddenKeepSpaceClass);
            }
            else {
                this.findField().addClass(this.hiddenClass);
            }
        }
        else {
            this.findField().removeClass(this.hiddenClass);
            this.findField().removeClass(this.hiddenKeepSpaceClass);
        }
    };
    ControlHelper.prototype.focus = function () {
        this.findControl().focus();
    };
    ControlHelper.prototype.trySetNumeric = function () {
        if (this.objProp.isNumeric()) {
            this.findControl().numeric(this.objProp.precision > 0, true);
        }
    };
    ControlHelper.prototype.trySetMaxLength = function () {
        if (this.objProp.dataLength) {
            if (this.objProp.isNumeric()) {
                this.findControl().maxLength(true, this.objProp.dataLength, this.objProp.precision);
            }
            else {
                this.findControl().maxLength(false, this.objProp.dataLength, undefined);
            }
        }
    };
    ControlHelper.prototype.findControl = function () {
        return this.__control;
    };
    ControlHelper.prototype.findField = function () {
        return this.__field;
    };
    ControlHelper.prototype.getLabelForControl = function () {
        return this.objProp.objPropId + "_control";
    };
    ControlHelper.prototype.dispose = function () {
    };
    return ControlHelper;
}());
var ClearableControlHelper = /** @class */ (function (_super) {
    __extends(ClearableControlHelper, _super);
    function ClearableControlHelper(viewProp, form) {
        return _super.call(this, viewProp, form) || this;
    }
    ClearableControlHelper.prototype.initializeControl = function () {
        var _this = this;
        _super.prototype.initializeControl.call(this);
        if (this.objProp.controlId === Enum.ControlType.SIGNATURE || this.form.controller.application.extendedProperties.showClearbuttons) {
            var control_1 = this.findInput();
            this.clearButton = document.createElement("a");
            this.clearButton.href = "#";
            this.clearButton.className = "clear-button ui-input-clear ui-btn ui-icon-delete ui-btn-icon-notext";
            this.clearButton.text = "Clear text";
            this.clearButton.tabIndex = -1;
            control_1.parent().append(this.clearButton);
            jQuery(this.clearButton).click(function (evt) {
                if (!control_1.is("[readonly]")) {
                    _this.setDefaultValue();
                    _this.form.synchronizeProp(_this.objProp);
                }
                evt.preventDefault();
                evt.stopPropagation();
                // Focus the control because when you clear a field you probably want to insert new data.
                _this.focus();
            });
        }
        else {
            this.hideClearButton = jQuery.noop;
        }
    };
    ClearableControlHelper.prototype.findInput = function () {
        return this.findControl();
    };
    ClearableControlHelper.prototype.setReadonly = function (readonly) {
        _super.prototype.setReadonly.call(this, readonly);
        this.hideClearButton(readonly);
    };
    ClearableControlHelper.prototype.hideClearButton = function (hide) {
        if (hide) {
            jQuery(this.clearButton).addClass(this.hiddenClass);
        }
        else {
            jQuery(this.clearButton).removeClass(this.hiddenClass);
        }
    };
    return ClearableControlHelper;
}(ControlHelper));
var NumericControlHelper = /** @class */ (function (_super) {
    __extends(NumericControlHelper, _super);
    function NumericControlHelper(viewProp, form) {
        var _this = _super.call(this, viewProp, form) || this;
        _this.language = form.controller.application.language;
        _this.showGroupSeparator = Utils.allowGroupSeparator(viewProp.objProp.controlId);
        return _this;
    }
    NumericControlHelper.prototype.createControl = function (height) {
        var container = document.createElement("div");
        container.className = "ui-input-text ui-body-inherit";
        var control = document.createElement("input");
        control.setAttribute("data-enhanced", "true");
        control.id = this.objProp.objPropId + "_control";
        control.className = "preventFastclickFocus";
        control.type = "number";
        control.step = "any";
        // Create a new input (which is always readonly) to show the group/decimal separators
        var control2 = document.createElement("input");
        control2.setAttribute("data-enhanced", "true");
        control2.readOnly = true;
        control2.className = "preventFastclickFocus";
        control2.type = "text";
        // Add the min and max value as attributes to the input.
        // The checkValidity method automatically checks the bounds.
        if (this.objProp.maxValue != null) {
            control.max = this.objProp.maxValue;
        }
        if (this.objProp.minValue != null) {
            control.min = this.objProp.minValue;
        }
        control.addEventListener("mousewheel", function (evt) {
            evt.preventDefault();
        });
        container.appendChild(control);
        container.appendChild(control2);
        return container;
    };
    NumericControlHelper.prototype.setValue = function (value) {
        _super.prototype.setValue.call(this, value);
        var controlReadonlyInput = this.findControl().next("input");
        if (isNaN(parseFloat(value))) {
            value = undefined;
        }
        if (value !== undefined) {
            value = Utils.formatNumber(parseFloat(value), this.objProp.precision, this.language, this.showGroupSeparator);
        }
        controlReadonlyInput.val(value);
    };
    NumericControlHelper.prototype.setDefaultValue = function () {
        _super.prototype.setDefaultValue.call(this);
        // also empty the second input (which did contain the text with group separators)
        this.findControl().next("input").val(null);
    };
    NumericControlHelper.prototype.validateValue = function () {
        var _this = this;
        var deferred = jQuery.Deferred(), input = this.findControl()[0];
        var message;
        // Mandatority validation is done when the user saves.
        if (!Utils.isEmpty(input.value)) {
            // We've created custom input validition functionality since iOS doesn't have support for input.validity.
            // Note that we still use the max/min attributes on the input element.
            var value = parseFloat(input.value);
            // Make sure that the value can be parsed to a float.
            if (isNaN(value)) {
                message = this.form.controller.application.language.transl(Model.TranslKeys.noNumericValue);
            }
            else if (input.max && value > parseFloat(input.max)) {
                message = this.form.controller.application.language.transl(Model.TranslKeys.rangeOverflow).replace("[maxValue]", this.objProp.maxValue);
            }
            else if (input.min && value < parseFloat(input.min)) {
                message = this.form.controller.application.language.transl(Model.TranslKeys.rangeUnderflow).replace("[minValue]", this.objProp.minValue);
            }
        }
        // Reject the promise when the input isn't valid.
        if (message) {
            var popup = ConfirmationPopup.CreateOkPopup();
            var promise = popup.initialize(this.form.controller.objView.objTransl.transl, message);
            promise.done(function (option) {
                // Don't null the value but set the defaultValue.
                _this.setDefaultValue();
                _this.form.controllerEditor.suggestCursorIndex(_this.objProp);
                deferred.reject();
            });
        }
        else {
            deferred.resolve();
        }
        return deferred.promise();
    };
    return NumericControlHelper;
}(ClearableControlHelper));
var HiddenControlHelper = /** @class */ (function (_super) {
    __extends(HiddenControlHelper, _super);
    function HiddenControlHelper(viewProp, form) {
        return _super.call(this, viewProp, form) || this;
    }
    HiddenControlHelper.prototype.setValue = function (value) {
        this.value = value;
    };
    HiddenControlHelper.prototype.setDefaultValue = function () {
        this.value = null;
    };
    HiddenControlHelper.prototype.getValue = function () {
        return this.value;
    };
    HiddenControlHelper.prototype.setMandatory = function (mandatory) { };
    HiddenControlHelper.prototype.setReadonly = function (readonly) { };
    HiddenControlHelper.prototype.setHidden = function (hide, keepSpace) { };
    HiddenControlHelper.prototype.focus = function () { };
    HiddenControlHelper.prototype.trySetNumeric = function () { };
    HiddenControlHelper.prototype.trySetMaxLength = function () { };
    HiddenControlHelper.prototype.findControl = function () {
        return jQuery();
    };
    HiddenControlHelper.prototype.findField = function () {
        return jQuery();
    };
    return HiddenControlHelper;
}(ControlHelper));
var MultilineControlHelper = /** @class */ (function (_super) {
    __extends(MultilineControlHelper, _super);
    function MultilineControlHelper(viewProp, form) {
        return _super.call(this, viewProp, form) || this;
    }
    MultilineControlHelper.prototype.createControl = function (height) {
        var control = document.createElement("textarea");
        control.id = this.objProp.objPropId + "_control";
        // Add padding height to height because this control contains the box-size=border-box property.
        control.setAttribute("style", "height:" + height + "px;");
        control.setAttribute("data-autogrow", "false");
        if (this.form.controller.application.extendedProperties.showClearbuttons) {
            if (Utils.isElectron()) {
                control.className = "has-clear-button";
            }
            else {
                control.className = "has-clear-button-large";
            }
        }
        return control;
    };
    MultilineControlHelper.prototype.setMandatory = function (mandatory) {
        var control = this.findControl();
        if (mandatory) {
            control.addClass(this.mandatoryClass);
        }
        else {
            control.removeClass(this.mandatoryClass);
        }
    };
    MultilineControlHelper.prototype.setReadonly = function (readonly) {
        var control = this.findControl();
        if (readonly) {
            control.addClass(this.readonlyClass);
        }
        else {
            control.removeClass(this.readonlyClass);
        }
        control.prop("readOnly", readonly);
        if (Utils.disableTextArea) {
            control.prop("disabled", readonly);
        }
        _super.prototype.hideClearButton.call(this, readonly);
    };
    return MultilineControlHelper;
}(ClearableControlHelper));
var UriControlHelper = /** @class */ (function (_super) {
    __extends(UriControlHelper, _super);
    function UriControlHelper(viewProp, form) {
        return _super.call(this, viewProp, form) || this;
    }
    UriControlHelper.prototype.createControl = function (height) {
        var control = document.createElement("input");
        control.id = this.objProp.objPropId + "_control";
        control.className = "preventFastclickFocus";
        if (this.objProp.controlId == Enum.ControlType.EMAIL) {
            control.type = "email";
        }
        else if (this.objProp.controlId == Enum.ControlType.PHONE_NUMBER) {
            control.type = "tel";
        }
        else if (this.objProp.controlId == Enum.ControlType.URL) {
            control.type = "url";
        }
        return control;
    };
    UriControlHelper.prototype.setValue = function (value) {
        _super.prototype.setValue.call(this, value);
        var anchorElement = this.findControl().siblings(".uri-anchor"), convertedValue = this.getAnchorElementValue(value);
        if (Utils.isEmpty(convertedValue.text)) {
            anchorElement.html("&nbsp;");
            anchorElement.css('visibility', 'hidden');
        }
        else {
            anchorElement.text(convertedValue.text);
            anchorElement.css('visibility', 'visible');
        }
        anchorElement.attr("href", convertedValue.anchorValue);
    };
    UriControlHelper.prototype.setDefaultValue = function () {
        _super.prototype.setDefaultValue.call(this);
        var anchorElement = this.findControl().siblings(".uri-anchor");
        this.findControl().val(null);
        anchorElement.text("").attr("href", "#");
    };
    UriControlHelper.prototype.setReadonly = function (readonly) {
        var inputContainer = this.findControl().parent();
        if (readonly) {
            var anchor = inputContainer.find(".uri-anchor");
            if (anchor.length == 0) {
                this.initializeAnchor(inputContainer);
            }
        }
        inputContainer.toggleClass("tsf-link", readonly);
        inputContainer.toggleClass("tsf-link-input", !readonly);
        _super.prototype.setReadonly.call(this, readonly);
    };
    UriControlHelper.prototype.initializeAnchor = function (container) {
        // Hacking an anchor tag in the input container is not a nice way, but jQuery mobile makes
        // it really hard to do this another way.
        var value = this.getAnchorElementValue(this.getValue());
        var element = document.createElement("a");
        element.className = "uri-anchor";
        element.href = value.anchorValue;
        element.target = "_blank";
        element.innerText = value.text;
        element.setAttribute("data-role", "none");
        container.append(element);
    };
    UriControlHelper.prototype.getAnchorElementValue = function (value) {
        var text = "", anchorValue = "#";
        if (value) {
            text = value;
            if (this.objProp.controlId == Enum.ControlType.EMAIL) {
                anchorValue = "mailto:" + value;
            }
            else if (this.objProp.controlId == Enum.ControlType.PHONE_NUMBER) {
                anchorValue = "tel:" + value;
            }
            else if (this.objProp.controlId == Enum.ControlType.GOOGLE_MAPS) {
                value = encodeURIComponent(value);
                if (typeof window.cordova === "undefined") {
                    anchorValue = "https://www.google.nl/maps/search/" + value;
                }
                else {
                    anchorValue = Utils.getiOSVersion() == null ? "geo:0,0?q=" + value : "https://maps.apple.com/?q=" + value;
                }
            }
            else {
                var success = "http://"; // Make sure it starts with http://
                anchorValue = value.substring(0, success.length) !== success ? success + value : value;
            }
        }
        return {
            text: text,
            anchorValue: anchorValue
        };
    };
    return UriControlHelper;
}(ClearableControlHelper));
var SharedFileControlHelper = /** @class */ (function (_super) {
    __extends(SharedFileControlHelper, _super);
    function SharedFileControlHelper() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.isReadonly = true;
        return _this;
    }
    SharedFileControlHelper.prototype.initializeControl = function () {
        var _this = this;
        _super.prototype.initializeControl.call(this);
        this.attachUploadEvent();
        this.createDeleteButton();
        this.deleteButton.click(function () {
            _this.form.controllerEditor.editDataRow(_this.objProp, null);
            _this.setValue(null);
        });
    };
    SharedFileControlHelper.prototype.createDeleteButton = function () {
        var deleteButton = document.createElement("a");
        this.deleteButton = jQuery(deleteButton);
        deleteButton.href = "#";
        deleteButton.className = "popup-button ui-btn ui-icon-delete ui-btn-icon-notext";
        this.findControl().closest(".ui-field-contain").append(deleteButton);
    };
    SharedFileControlHelper.prototype.createUploadInput = function () {
        var uploadInput = document.createElement("input");
        uploadInput.id = this.objProp.objPropId + "_control";
        uploadInput.type = "file";
        return uploadInput;
    };
    SharedFileControlHelper.prototype.attachUploadEvent = function () {
        var _this = this;
        var useFilePlugin = (window["cordova"] && window["cordova"].plugins && window["cordova"].plugins.tsf_file_chooser);
        if (!useFilePlugin) {
            this.form.formBody.on("change", "#" + this.objProp.objPropId + "_control", function (evt) {
                var files = evt.target.files;
                var file = files.length > 0 ? files[0] : null;
                if (file) {
                    var reader = new FileReader();
                    reader.onload = function (e) {
                        var result = e.target.result;
                        //Prevents obscure "Base64 sequence length" error from being given
                        //when trying to save the form with an empty file uploaded.
                        if (result === "data:") {
                            var appl = Model.TSFApplicationManager.Current.activeAppl;
                            var transl = appl.language.transl(Model.TranslKeys.emptyFileError);
                            ConfirmationPopup.CreateOkPopup().initialize("", transl);
                            return;
                        }
                        if (AppConfig.compressImages && result.indexOf("data:image") === 0) {
                            _this.compressImage(result, file.name, undefined);
                            return;
                        }
                        _this.setValueAsFileContract(result, file.name, undefined);
                    };
                    reader.onerror = function (err) {
                        throw err;
                    };
                    reader.readAsDataURL(file);
                    // Reset the value because the change event will not go off when the same file gets chosen.
                    jQuery(_this).val("");
                }
            });
        }
        else {
            var control = this.findControl();
            // attach on the parent, because on IE the input will be removed sometimes.
            control.parent().on("click", function (evt) {
                evt.preventDefault();
                var permissions = window["cordova"].plugins.permissions;
                permissions.hasPermission(permissions.READ_EXTERNAL_MEDIA, permissions.requestPermission(permissions.READ_EXTERNAL_MEDIA, openChooser));
                function openChooser() {
                    var _this = this;
                    window["cordova"].plugins.tsf_file_chooser.openChooser(function (result, filename) {
                        if (!filename) {
                            filename = moment().format("YYYY-MM-DD-HH-mm-ss") + "-photo.jpg";
                        }
                        else if (filename.indexOf("/") === 0) {
                            //Strip forward slash because it results in an error when uploading the file.
                            filename = filename.substring(1);
                        }
                        if (AppConfig.compressImages &&
                            (Utils.stringEndsWith(filename, ".png") ||
                                Utils.stringEndsWith(filename, ".jpg") ||
                                Utils.stringEndsWith(filename, ".jpeg"))) {
                            _this.compressImage(result, filename, -1);
                            return;
                        }
                        _this.setValueAsFileContract(result, filename, -1);
                    }, null, { destinationType: 0 });
                }
            });
        }
    };
    SharedFileControlHelper.prototype.compressImage = function (result, fileName, hashCode) {
        var _this = this;
        var split = result.split(",");
        var contentType;
        if (split.length === 2) {
            result = split[1]; // strip out the "data:image/jpeg;base64"
            contentType = split[0].substring(split[0].lastIndexOf(":") + 1, split[0].lastIndexOf(";"));
        }
        else {
            if (Utils.stringEndsWith(fileName, ".png")) {
                contentType = "image/png";
            }
            else {
                contentType = "image/jpeg";
            }
        }
        //iOS needs more compression because else the files are still pretty big.
        var isIOS = Utils.getiOSVersion() !== null;
        var quality = isIOS ? 0.6 : 0.78;
        var maxDimensions = 2000;
        Utils.showLoader();
        var compressedImage = new ImageCompressor(Utils.b64toBlob(result, contentType), {
            mimeType: contentType,
            quality: quality,
            maxWidth: maxDimensions,
            maxHeight: maxDimensions,
            success: function (compressedBlobResult) {
                var compressedImageReader = new FileReader();
                compressedImageReader.onload = function (readerEvent) {
                    var compressedResult = readerEvent.target.result;
                    _this.setValueAsFileContract(compressedResult, fileName, hashCode);
                    Utils.hideLoader();
                };
                compressedImageReader.readAsDataURL(compressedBlobResult);
            },
            error: function (err) {
                Utils.hideLoader();
                throw err;
            }
        });
    };
    SharedFileControlHelper.prototype.setValueAsFileContract = function (result, fileName, hashCode) {
        var resultSplit = result.split(",");
        if (resultSplit.length === 2) {
            result = resultSplit[1]; // strip out the "data:image/jpeg;base64"
        }
        var fileContract = FileContract.CreateFromFileName(this.objProp.fileStorageFolder, fileName, result, undefined, hashCode);
        this.form.controllerEditor.editDataRow(this.objProp, fileContract);
        this.setValue(fileContract);
    };
    SharedFileControlHelper.prototype.switchControlState = function () {
        var field = this.findField(), control = this.findControl(), controlContainer = control.parent();
        if (this.isHidden) {
            if (this.hiddenKeepSpace) {
                field.addClass(this.hiddenKeepSpaceClass);
            }
            else {
                field.addClass(this.hiddenClass);
            }
        }
        else {
            // Make field back visible again when it was hidden before.
            field.removeClass(this.hiddenClass);
            field.removeClass(this.hiddenKeepSpaceClass);
            if (this.isReadonly) {
                controlContainer.hide();
                this.deleteButton.addClass("hide");
                control.prop("disabled", true);
            }
            else {
                if (this.filename) {
                    controlContainer.hide();
                    this.deleteButton.removeClass("hide");
                }
                else {
                    controlContainer.show();
                    this.deleteButton.addClass("hide");
                }
                // Convert to boolean
                control.prop("disabled", !!this.filename);
            }
        }
    };
    SharedFileControlHelper.prototype.setValue = function (value) {
        if (!Utils.isEmpty(value)) {
            var fileContract = value;
            // Only show filename plus extension in control
            var displayValue = this.getDisplayValue(fileContract.FileName);
            this.filepath = fileContract.FilePath;
            this.filename = fileContract.FileName;
            this.hashcode = fileContract.HashCode;
        }
        else {
            this.setDefaultValue();
        }
        this.switchControlState();
    };
    SharedFileControlHelper.prototype.setDefaultValue = function () {
        this.filepath = null;
        this.filename = null;
        this.hashcode = null;
        // Clear the file upload control. Special code for IE because they don't allow us to manipulate the value.
        // http://stackoverflow.com/questions/1043957/clearing-input-type-file-using-jquery
        var fileUpload = this.findControl();
        fileUpload.val('');
    };
    SharedFileControlHelper.prototype.getDisplayValue = function (value) {
        if (value) {
            var sep = value.indexOf("/") >= 0 ? "/" : "\\";
            if (value.indexOf(sep) >= 0) {
                var split = value.split(sep);
                var file = split[split.length - 1];
                if (file.indexOf(".") >= 0) {
                    return file;
                }
            }
            else
                return value;
        }
        return null;
    };
    SharedFileControlHelper.prototype.getValue = function () {
        if (this.filepath) {
            return FileContract.CreateFromFilePath(this.filepath, null, undefined, this.hashcode);
        }
        return null;
    };
    SharedFileControlHelper.prototype.setReadonly = function (readonly) {
        if (this.isReadonly != readonly) {
            this.isReadonly = readonly;
            this.switchControlState();
        }
    };
    SharedFileControlHelper.prototype.setHidden = function (hide, keepSpace) {
        if (this.isHidden != hide || this.hiddenKeepSpace != keepSpace) {
            this.isHidden = hide;
            this.hiddenKeepSpace = keepSpace;
            this.switchControlState();
        }
    };
    SharedFileControlHelper.prototype.trySetNumeric = function () { };
    SharedFileControlHelper.prototype.trySetMaxLength = function () { };
    return SharedFileControlHelper;
}(ControlHelper));
var ImageControlHelper = /** @class */ (function (_super) {
    __extends(ImageControlHelper, _super);
    function ImageControlHelper(viewProp, form) {
        return _super.call(this, viewProp, form) || this;
    }
    ImageControlHelper.prototype.initializeControl = function () {
        this.__image = this.form.formBody.find('#' + this.objProp.objPropId + "_image");
        _super.prototype.initializeControl.call(this);
    };
    ImageControlHelper.prototype.switchControlState = function () {
        _super.prototype.switchControlState.call(this);
        if (!this.isHidden) {
            if (this.filename) {
                this.findImage().removeClass("hide");
            }
            else {
                this.findImage().addClass("hide");
            }
        }
    };
    ImageControlHelper.prototype.createControl = function (height) {
        var control = document.createElement("div");
        control.className = "image-control-div";
        var image = document.createElement("img");
        image.id = this.objProp.objPropId + "_image";
        image.style.cursor = "zoom-in";
        image.src = "img/photo.png";
        image.removeAttribute("width");
        image.removeAttribute("height");
        var imageHeight = height + "px";
        // Do not set the width, but let the image decide what to do based on the height"width: " + prop.fieldWidth + "px;
        control.style.height = imageHeight;
        image.style.maxHeight = imageHeight;
        control.appendChild(image);
        control.appendChild(this.createUploadInput());
        return control;
    };
    ImageControlHelper.prototype.setValue = function (value) {
        var _this = this;
        if (value && (value.HashCode !== this.visibleHashCode || Utils.isEmpty(value.HashCode))) {
            // Reset the file promise, because when a setvalue is called multiple times it should use the latest image.
            // (eg loading the images may take alot of time and the user can switch again.
            this.filePromise = null;
            // Store the hashcode the visibleHashCode because this.hashCode is also set when the editor isn't readonly.
            this.visibleHashCode = value.HashCode;
            var control_2 = this.findImage();
            // Default changed the file
            if ((this.form.controller.isInEditMode || value.HashCode == null || value.HashCode == 0) && value.File) {
                var icon = Utils.createIconFromBase64(value.File);
                this.createZoomableIcon(this, control_2, icon);
            }
            else {
                var that_1 = this;
                // Math.round is required, sometimes it calculates the height as a decimal causing the service tier to crash
                // http://bugs.jquery.com/ticket/12215
                var setValueInner_1 = function () {
                    var parent = control_2.parent();
                    var imageHeight = Math.round(parent.height()), imageWidth = Math.round(parent.width());
                    if (typeof window.devicePixelRatio !== "undefined" && window.devicePixelRatio !== 1) {
                        imageHeight = Math.round(imageHeight * window.devicePixelRatio);
                        imageWidth = Math.round(imageWidth * window.devicePixelRatio);
                    }
                    _this.filePromise = _this.form.controller.getImage(null, _this.objProp, imageWidth, imageHeight, false);
                    // NOTE: Do not chain the done function, because in offline mode the promise is resolved synchronous causing that.filePromise == this to fail
                    _this.filePromise.done(function (base64Icon) {
                        if (that_1.filePromise == this && that_1.visibleHashCode == value.HashCode) {
                            if (base64Icon) {
                                var icon = Utils.createIconFromBase64(base64Icon);
                                that_1.createZoomableIcon(that_1, control_2, icon);
                            }
                            else {
                                that_1.setDefaultValue();
                            }
                        }
                    });
                };
                // Delay the setValueInner call until the page is shown. The width can't be calculated correctly otherwise.
                this.form.pageShown.then(function () {
                    setValueInner_1();
                });
            }
        }
        _super.prototype.setValue.call(this, value);
    };
    ImageControlHelper.prototype.createZoomableIcon = function (that, control, icon) {
        var iOSVersion = Utils.getiOSVersion();
        control.attr("src", icon);
        // Resetting the click event handler.
        control.off("click.tsfzoom");
        control.on("click.tsfzoom", function () {
            Utils.showLoader();
            that.form.controller.getFile(null, that.objProp).done(function (fullImage) {
                if (fullImage !== null) {
                    fullImage = Utils.createIconFromBase64(fullImage);
                    var windowOpenParam = "location=no,resizable=yes";
                    if (iOSVersion === null || iOSVersion >= 11) {
                        windowOpenParam += ",enableviewportscale=true";
                    }
                    window.open(fullImage, "_blank", windowOpenParam);
                }
                Utils.hideLoader();
            });
        });
    };
    ImageControlHelper.prototype.setDefaultValue = function () {
        _super.prototype.setDefaultValue.call(this);
        // Reset the file promise because a setValue can be called prior to the setDefaultValue.
        this.filePromise = null;
        // Also reset the visible hashcode since no image is visible anymore.
        this.visibleHashCode = null;
        var img = this.findImage();
        img.attr('src', "img/photo.png");
        img.removeAttr("width").removeAttr("height");
    };
    ImageControlHelper.prototype.findImage = function () {
        return this.__image;
    };
    return ImageControlHelper;
}(SharedFileControlHelper));
var FileControlHelper = /** @class */ (function (_super) {
    __extends(FileControlHelper, _super);
    function FileControlHelper(viewProp, form) {
        return _super.call(this, viewProp, form) || this;
    }
    FileControlHelper.prototype.createControl = function (height) {
        // In non editmode we use a button as download button
        var control = document.createElement("div");
        control.className = "file-download-control-div";
        var downloadButton = document.createElement("a");
        downloadButton.id = this.objProp.objPropId + "_download";
        downloadButton.className = "file-control-button hide ui-link ui-btn ui-icon-arrow-d ui-btn-icon-right";
        control.appendChild(downloadButton);
        control.appendChild(this.createUploadInput());
        return control;
    };
    FileControlHelper.prototype.initializeControl = function () {
        var _this = this;
        this.downloadButton = this.form.formBody.find('#' + this.objProp.objPropId + "_download");
        _super.prototype.initializeControl.call(this);
        this.downloadButton.click(function () {
            _this.loadFile();
        });
    };
    FileControlHelper.prototype.switchControlState = function () {
        _super.prototype.switchControlState.call(this);
        var fileDiv = this.findControl().closest(".file-download-control-div");
        if (!this.isHidden) {
            if (this.filename || this.isReadonly) {
                this.downloadButton.removeClass("hide");
            }
            else {
                this.downloadButton.addClass("hide");
            }
            if (this.filename && !this.isReadonly) {
                fileDiv.addClass("delete-visible");
            }
            else {
                fileDiv.removeClass("delete-visible");
            }
        }
    };
    FileControlHelper.prototype.setValue = function (value) {
        if (!Utils.isEmpty(value)) {
            var fileContract = value;
            // Only show filename plus extension in control
            var displayValue = this.getDisplayValue(fileContract.FileName);
            this.downloadButton.text(displayValue);
        }
        _super.prototype.setValue.call(this, value);
    };
    FileControlHelper.prototype.setDefaultValue = function () {
        _super.prototype.setDefaultValue.call(this);
        this.downloadButton.html("&nbsp;");
    };
    FileControlHelper.prototype.loadFile = function () {
        var _this = this;
        if (!this.filename)
            return;
        // When the user is in editmode - we might need to open the new selected file or download the contents
        var fileContract = this.form.controller.isInEditMode && this.form.controller.currentEditor.newRow[this.objProp.objPropId];
        if (fileContract && fileContract.File) {
            this.openFile(fileContract.File);
        }
        else {
            Utils.showLoader();
            this.form.controller.getFile(null, this.objProp).done(function (base64File) {
                Utils.hideLoader();
                if (base64File) {
                    _this.openFile(base64File);
                }
                else {
                    var language = _this.form.controller.application.language;
                    var msgPopup = ConfirmationPopup.CreateOkPopup();
                    msgPopup.initialize(_this.objProp.objPropTransl.translForm, language.transl(Model.TranslKeys.cantFindFile));
                }
            });
        }
    };
    FileControlHelper.prototype.openFile = function (base64File) {
        var _this = this;
        var requestFileSystem = window.requestFileSystem || window.webkitRequestFileSystem; // API should be the same
        if (requestFileSystem) {
            // Save file to disk. E.g. in browsers download folder to cache the file also locally.
            // http://docs.phonegap.com/en/2.4.0/cordova_file_file.md.html
            try {
                var store = typeof LocalFileSystem !== "undefined" ? LocalFileSystem.TEMPORARY : window["TEMPORARY"];
                var target_1 = (Utils.isAndroid() ? "_system" : "_blank");
                var filename_1 = this.filename;
                var fail_1 = function (evt) {
                    // TODO Create transls for FileError and remove enum.
                    var errorCode = Enum.FileError[evt.code];
                    alert('Save file failed. Code: ' + (errorCode ? errorCode : evt.code)); // TODO translate
                    // Try to open without save it.
                    window.open('data:' + Utils.getMimeType(filename_1) + ';base64,' + base64File);
                };
                requestFileSystem(store, 0, function (fileSystem) {
                    fileSystem.root.getFile(_this.getDisplayValue(_this.filename), { create: true }, function (fileEntry) {
                        var that = _this;
                        fileEntry.createWriter(function (writer) {
                            writer.onwriteend = function (evt) {
                                // We shouldn't do this, but when using _blank in the viewer it creates a new window and a additional one for the pdf.
                                // We need to use the inappbrowser on for example the iPhone because it doesn't have a backbutton and without this plugin you would get stuck.
                                if (typeof window.cordova !== "undefined") {
                                    try {
                                        var offset = that.downloadButton.offset();
                                        offset.top += (that.downloadButton.outerHeight() / 2);
                                        var openExternal = window.cordova.require("com.thinkwise.openexternal.openExternal");
                                        openExternal.openDocument(decodeURI(fileEntry.toURL()), offset.left, offset.top, window.innerHeight, window.innerWidth, null, function () {
                                            // Plugin couldn't find alternative apps, fallback to default
                                            window.open(fileEntry.toURL(), target_1, "location=no");
                                        });
                                    }
                                    catch (error) {
                                        // We are in cordova (but no openExternal plugin), open it in the inappbrowser with some options
                                        window.open(fileEntry.toURL(), target_1, "location=no");
                                    }
                                }
                                else {
                                    var image = Utils.createIconFromBase64(base64File);
                                    if (image) {
                                        window.open(image, "_blank", "location=no,resizable=yes");
                                    }
                                    else {
                                        // Viewer or browser
                                        var BrowserWindow = require('electron').remote.BrowserWindow;
                                        var PDFWindow = require('electron-pdf-window');
                                        var win = new BrowserWindow({ width: 800, height: 600 });
                                        PDFWindow.addSupport(win);
                                        win.loadURL(fileEntry.toURL());
                                    }
                                }
                            };
                            function dataURItoBlob(dataURI, fileName) {
                                // convert base64 to raw binary data held in a string
                                var byteString = atob(dataURI);
                                // write the bytes of the string to an ArrayBuffer
                                var ab = new ArrayBuffer(byteString.length);
                                var ia = new Uint8Array(ab);
                                for (var i = 0; i < byteString.length; i++) {
                                    ia[i] = byteString.charCodeAt(i);
                                }
                                try {
                                    try {
                                        return new Blob([ab]);
                                    }
                                    catch (error) {
                                        window.BlobBuilder = window.BlobBuilder || window.WebKitBlobBuilder || window.MozBlobBuilder || window.MSBlobBuilder;
                                        // TypeError old chrome and FF
                                        if (error.name == 'TypeError' && window.BlobBuilder) {
                                            var bb = new BlobBuilder();
                                            bb.append(ab);
                                            return bb.getBlob(Utils.getMimeType(fileName));
                                        }
                                    }
                                }
                                catch (err) {
                                    // TODO: Remove alert call
                                    alert(err);
                                }
                            }
                            var blob = dataURItoBlob(base64File, _this.filename);
                            if (blob) {
                                writer.write(blob);
                                if (blob.close)
                                    blob.close();
                            }
                        }, fail_1);
                    }, fail_1);
                }, fail_1);
            }
            catch (error) {
                alert(error);
            }
        }
        else {
            console.log("Open directly (no filesystem)");
            window.open('data:' + Utils.getMimeType(this.filename) + ';base64,' + base64File);
        }
    };
    return FileControlHelper;
}(SharedFileControlHelper));
var DateControlHelper = /** @class */ (function (_super) {
    __extends(DateControlHelper, _super);
    function DateControlHelper(viewProp, form) {
        return _super.call(this, viewProp, form) || this;
    }
    DateControlHelper.prototype.createControl = function (height) {
        var control = document.createElement("input");
        control.id = this.objProp.objPropId + "_control";
        control.className = "preventFastclickFocus";
        control.type = "date";
        return control;
    };
    DateControlHelper.prototype.setValue = function (value) {
        if (value) {
            // First format the value to date format. Format: yyyy-MM-dd
            var formattedDate = moment(value).format("YYYY-MM-DD");
            this.findControl().val(formattedDate);
        }
        else {
            this.findControl().val(value);
        }
    };
    DateControlHelper.prototype.getValue = function () {
        var dateInput = this.findControl()[0];
        if (!dateInput)
            return null;
        if (typeof dateInput.valueAsDate === "undefined") {
            return new Date(dateInput.value);
        }
        else {
            var date = dateInput.valueAsDate;
            if (date) {
                date.setHours(0); // Remove timezone
            }
            return date;
        }
    };
    return DateControlHelper;
}(ControlHelper));
var TimeControlHelper = /** @class */ (function (_super) {
    __extends(TimeControlHelper, _super);
    function TimeControlHelper(viewProp, form) {
        return _super.call(this, viewProp, form) || this;
    }
    TimeControlHelper.prototype.createControl = function (height) {
        var control = document.createElement("input");
        control.id = this.objProp.objPropId + "_control";
        control.className = "preventFastclickFocus";
        control.type = "time";
        return control;
    };
    TimeControlHelper.prototype.getValue = function () {
        var timeControl = this.findControl()[0];
        if (!timeControl)
            return null;
        var date = moment(timeControl.value, "HH:mm:ss").year(1970).month(0).date(1);
        if (date.isValid()) {
            return date.toDate();
        }
        return null;
    };
    TimeControlHelper.prototype.setValue = function (value) {
        var timeControl = this.findControl()[0];
        if (!timeControl)
            return;
        if (value) {
            var date = new Date(value);
            // Format time for time control. Format: HH:mm
            var format = "HH:mm:ss";
            var formattedTime = moment(date).format(format);
            timeControl.value = formattedTime;
        }
        else {
            timeControl.value = value;
        }
    };
    TimeControlHelper.prototype.trySetNumeric = function () { };
    TimeControlHelper.prototype.trySetMaxLength = function () { };
    return TimeControlHelper;
}(ControlHelper));
var DateTimeControlHelper = /** @class */ (function (_super) {
    __extends(DateTimeControlHelper, _super);
    function DateTimeControlHelper(viewProp, form) {
        var _this = _super.call(this, viewProp, form) || this;
        _this.dateControlId = _this.objProp.objPropId + "_c_date";
        _this.timeControlId = _this.objProp.objPropId + "_c_time";
        return _this;
    }
    DateTimeControlHelper.prototype.createControl = function (height) {
        var control = document.createElement("div");
        control.id = this.objProp.objPropId + "_control";
        control.className = "ui-grid-a datetime";
        //Date control
        var dateDiv = document.createElement("div");
        dateDiv.className = "ui-block-a date";
        control.appendChild(dateDiv);
        var dateInput = document.createElement("input");
        dateInput.type = "date";
        dateInput.id = this.objProp.objPropId + "_c_date";
        dateInput.className = "preventFastclickFocus";
        dateDiv.appendChild(dateInput);
        // Time control
        var timeDiv = document.createElement("div");
        timeDiv.className = "ui-block-b time";
        control.appendChild(timeDiv);
        var timeInput = document.createElement("input");
        timeInput.type = "time";
        timeInput.id = this.objProp.objPropId + "_c_time";
        timeInput.className = "preventFastclickFocus";
        timeDiv.appendChild(timeInput);
        return control;
    };
    DateTimeControlHelper.prototype.getValue = function () {
        var field = this.findField(), dateControl = field.find("#" + this.dateControlId)[0], timeControl = field.find("#" + this.timeControlId)[0];
        if (!dateControl || !timeControl)
            return null;
        // ValueAsDate function not supported.
        if (typeof dateControl.valueAsDate === "undefined" || typeof timeControl.valueAsDate === "undefined") {
            var d = new Date(dateControl.value + " " + timeControl.value);
            if (moment(d).isValid()) {
                return d;
            }
        }
        else {
            // both should be valid
            if (dateControl.valueAsDate && timeControl.valueAsDate) {
                var dateValue = dateControl.valueAsDate;
                var date = new Date(dateValue.setHours(0) + timeControl.valueAsDate.getTime());
                return date;
            }
        }
        return null;
    };
    DateTimeControlHelper.prototype.setValue = function (value) {
        var field = this.findField(), dateControl = field.find("#" + this.dateControlId), timeControl = field.find("#" + this.timeControlId);
        if (value) {
            // First format the value to date format. Format: yyyy-MM-dd
            var date = new Date(value);
            var formattedDate = moment(date).format("YYYY-MM-DD");
            dateControl.val(formattedDate);
            // Format time for datetime control. Format: HH-mm
            var formattedTime = moment(date).format("HH:mm:ss");
            timeControl.val(formattedTime);
        }
        else {
            dateControl.val(value);
            timeControl.val(value);
        }
    };
    DateTimeControlHelper.prototype.setDefaultValue = function () {
        var field = this.findField(), dateControl = field.find("#" + this.dateControlId), timeControl = field.find("#" + this.timeControlId);
        dateControl.val(null);
        timeControl.val(null);
    };
    DateTimeControlHelper.prototype.setMandatory = function (mandatory) {
        var field = this.findField(), dateControl = field.find("#" + this.dateControlId), timeControl = field.find("#" + this.timeControlId);
        if (mandatory) {
            dateControl.parent().addClass(this.mandatoryClass);
            timeControl.parent().addClass(this.mandatoryClass);
        }
        else {
            dateControl.parent().removeClass(this.mandatoryClass);
            timeControl.parent().removeClass(this.mandatoryClass);
        }
    };
    DateTimeControlHelper.prototype.setReadonly = function (readonly) {
        var field = this.findField(), dateControl = field.find("#" + this.dateControlId), timeControl = field.find("#" + this.timeControlId);
        if (readonly) {
            dateControl.parent().addClass(this.readonlyClass);
            timeControl.parent().addClass(this.readonlyClass);
        }
        else {
            dateControl.parent().removeClass(this.readonlyClass);
            timeControl.parent().removeClass(this.readonlyClass);
        }
        dateControl.prop("readOnly", readonly);
        timeControl.prop("readOnly", readonly);
    };
    DateTimeControlHelper.prototype.focus = function () {
        var field = this.findField(), dateControl = field.find("#" + this.dateControlId);
        dateControl.focus();
    };
    DateTimeControlHelper.prototype.getLabelForControl = function () {
        return this.dateControlId;
    };
    return DateTimeControlHelper;
}(ControlHelper));
var CheckboxControlHelper = /** @class */ (function (_super) {
    __extends(CheckboxControlHelper, _super);
    function CheckboxControlHelper(viewProp, form) {
        var _this = _super.call(this, viewProp, form) || this;
        // TODO: What does the service tier return for an empty checkbox?
        _this.emptyValue = "-1";
        _this.falseValue = "false";
        _this.trueValue = "true";
        _this.emptyTransl = "-";
        _this.falseTransl = "0";
        _this.trueTransl = "1";
        var domain = _this.objProp.domain;
        _this.bindOnDomain = domain != null && domain.elements.length == 2;
        if (_this.bindOnDomain) {
            var firstDomain = domain.elements.item(0);
            var secondDomain = domain.elements.item(1);
            var firstElementIsTrue = Utils.convertToBoolean(firstDomain.value, false);
            var secondElementIsFalse = (Utils.convertToBoolean(secondDomain.value, true) == false);
            if (firstElementIsTrue && secondElementIsFalse) {
                _this.trueTransl = firstDomain.elementTransl.transl;
                _this.falseTransl = secondDomain.elementTransl.transl;
            }
            else if (!firstElementIsTrue && !secondElementIsFalse) {
                _this.trueTransl = secondDomain.elementTransl.transl;
                _this.falseTransl = firstDomain.elementTransl.transl;
            }
            // Else domains do not matches opposing boolean values, so keep the 0 and 1 default transl.
        }
        return _this;
        // TODO decide database values here using dttp and domainElement values.
    }
    CheckboxControlHelper.prototype.createControl = function (height) {
        var control;
        if (!this.viewProp.mandatory) {
            control = document.createElement("fieldset");
            control.id = this.objProp.objPropId + "_control";
            control.setAttribute("data-role", "controlgroup");
            control.setAttribute("data-type", "horizontal");
            var falseItem = document.createElement("input");
            falseItem.type = "radio";
            falseItem.id = this.objProp.objPropId + "_falseRadio";
            falseItem.value = this.falseValue;
            var falseLabel = document.createElement("label");
            falseLabel.setAttribute("for", this.objProp.objPropId + "_falseRadio");
            falseLabel.innerText = this.falseTransl;
            control.appendChild(falseItem);
            control.appendChild(falseLabel);
            var nullItem = document.createElement("input");
            nullItem.type = "radio";
            nullItem.id = this.objProp.objPropId + "_nullRadio";
            nullItem.value = this.emptyValue;
            var nullLabel = document.createElement("label");
            nullLabel.setAttribute("for", this.objProp.objPropId + "_nullRadio");
            nullLabel.innerText = this.emptyTransl;
            control.appendChild(nullItem);
            control.appendChild(nullLabel);
            var trueItem = document.createElement("input");
            trueItem.type = "radio";
            trueItem.id = this.objProp.objPropId + "_trueRadio";
            trueItem.value = this.trueValue;
            var trueLabel = document.createElement("label");
            trueLabel.setAttribute("for", this.objProp.objPropId + "_trueRadio");
            trueLabel.innerText = this.trueTransl;
            control.appendChild(trueItem);
            control.appendChild(trueLabel);
        }
        else {
            control = document.createElement("select");
            control.id = this.objProp.objPropId + "_control";
            control.setAttribute("data-role", "flipswitch");
            var option = document.createElement("option");
            option.value = this.falseValue;
            option.innerText = this.falseTransl;
            control.appendChild(option);
            option = document.createElement("option");
            option.value = this.trueValue;
            option.innerText = this.trueTransl;
            control.appendChild(option);
        }
        return control;
    };
    CheckboxControlHelper.prototype.setValue = function (value) {
        if (value != null) {
            if (value == true || value == "true" || value == "1") {
                this.findControl().val(this.trueValue);
            }
            else if (value == false || value == "false" || value == "0") {
                this.findControl().val(this.falseValue);
            }
            else {
                this.setDefaultValue();
            }
        }
        else {
            this.setDefaultValue();
        }
        if (this.viewProp.mandatory) {
            if (this.findControl().flipswitch() === undefined) {
                this.findControl().flipswitch();
            }
            this.findControl().flipswitch("refresh");
        }
    };
    CheckboxControlHelper.prototype.getValue = function () {
        var value = this.findControl().val();
        // On DB2 dttp is short and not a boolean so we should return it as 1 or 0
        // Same applies for oracle
        var domainDttp = this.objProp.xmlDttp;
        if (domainDttp === "byte" || domainDttp === "short" || domainDttp === "int") {
            return (Utils.convertToBoolean(value, false) ? 1 : 0);
        }
        if (domainDttp === "boolean") {
            //Null and the empty value won't be processed.
            if (value == "null" || value == this.emptyValue) {
                return value;
            }
            //We'll make sure booleans are actually booleans instead of strings.
            return Utils.convertToBoolean(value, false);
        }
        return value;
    };
    CheckboxControlHelper.prototype.setDefaultValue = function () {
        if (this.viewProp.mandatory) {
            this.findControl().val(this.falseValue);
            this.findControl().flipswitch("refresh");
        }
        else {
            this.findControl().val(this.emptyValue);
        }
    };
    CheckboxControlHelper.prototype.setMandatory = function (mandatory) { };
    CheckboxControlHelper.prototype.setReadonly = function (readonly) {
        var control = this.findControl();
        if (this.viewProp.mandatory) {
            control.flipswitch(readonly ? 'disable' : 'enable');
        }
        else {
            control.children().prop("disabled", readonly);
        }
    };
    // Disable label click for flipswitches.
    CheckboxControlHelper.prototype.getLabelForControl = function () {
        if (this.viewProp.mandatory) {
            return null;
        }
        _super.prototype.getLabelForControl.call(this);
    };
    CheckboxControlHelper.prototype.focus = function () { };
    CheckboxControlHelper.prototype.trySetNumeric = function () { };
    CheckboxControlHelper.prototype.trySetMaxLength = function () { };
    return CheckboxControlHelper;
}(ControlHelper));
var ComboControlHelper = /** @class */ (function (_super) {
    __extends(ComboControlHelper, _super);
    function ComboControlHelper(viewProp, form) {
        return _super.call(this, viewProp, form) || this;
    }
    ComboControlHelper.prototype.createControl = function (height) {
        var control = document.createElement("select");
        control.id = this.objProp.objPropId + "_control";
        if (this.objProp.domain != null) {
            // Add empty item to make the combo empty.
            var optionElement = document.createElement("option");
            optionElement.value = "";
            optionElement.innerHTML = "&nbsp;";
            control.appendChild(optionElement);
            var elements = this.objProp.domain.elements.array;
            for (var i = 0; i < elements.length; i++) {
                var element = elements[i];
                var option = document.createElement("option");
                option.value = element.value;
                option.innerText = element.elementTransl.transl;
                control.appendChild(option);
            }
        }
        return control;
    };
    ComboControlHelper.prototype.setMandatory = function (mandatory) {
        var control = this.findControl();
        if (mandatory) {
            control.parent().addClass(this.mandatoryClass);
        }
        else {
            control.parent().removeClass(this.mandatoryClass);
        }
    };
    ComboControlHelper.prototype.setReadonly = function (readonly) {
        var control = this.findControl();
        if (readonly) {
            control.parent().addClass(this.readonlyClass);
        }
        else {
            control.parent().removeClass(this.readonlyClass);
        }
        control.prop('disabled', readonly).trigger('change');
    };
    ComboControlHelper.prototype.setValue = function (value) {
        _super.prototype.setValue.call(this, value);
        // Do this to let the dynamic created combo show his value.
        this.findControl().trigger("change");
    };
    ComboControlHelper.prototype.setDefaultValue = function () {
        _super.prototype.setDefaultValue.call(this);
        // Do this to let the dynamic created combo show his value.
        this.findControl().trigger("change");
    };
    ComboControlHelper.prototype.trySetNumeric = function () { };
    ComboControlHelper.prototype.trySetMaxLength = function () { };
    return ComboControlHelper;
}(ControlHelper));
var SuggestionControlHelper = /** @class */ (function (_super) {
    __extends(SuggestionControlHelper, _super);
    function SuggestionControlHelper(viewProp, form) {
        var _this = _super.call(this, viewProp, form) || this;
        var controlType = viewProp.objLinkLookupView.controlType;
        switch (controlType) {
            case 0 /* TextAutoComplete */:
            case 3 /* TextSuggestionStartsWith */:
                _this.filterComparison = Enum.FilterCondition.StartsWith;
                break;
            case 2 /* TextSuggestionContains */:
            default:
                _this.filterComparison = Enum.FilterCondition.Contains;
                break;
        }
        return _this;
    }
    SuggestionControlHelper.prototype.createControl = function (height) {
        var container = document.createElement("div");
        container.className = "suggestion-control";
        var searchForm = document.createElement("form"), searchInput = document.createElement("input"), searchInputId = this.objProp.objPropId + "SuggestionSearchInput";
        searchInput.type = "text";
        searchInput.id = searchInputId;
        searchInput.className = "preventFastclickFocus suggestion";
        this.input = jQuery(searchInput);
        searchForm.appendChild(searchInput);
        var control = document.createElement("ul");
        control.id = this.objProp.objPropId + "_control";
        control.setAttribute("data-role", "listview");
        // Inset draws a tiny gray border around the items.
        control.setAttribute("data-inset", "true");
        control.setAttribute("data-input", "#" + searchInputId);
        container.appendChild(searchForm);
        container.appendChild(control);
        return container;
    };
    SuggestionControlHelper.prototype.findInput = function () {
        return this.input;
    };
    SuggestionControlHelper.prototype.initializeControl = function () {
        _super.prototype.initializeControl.call(this);
        var that = this;
        var control = this.findControl().listview().filterable();
        if (this.viewProp.objLinkLookupView.hasZoomButton && this.form.allowFormbuttons()) {
            var showPopupButton = document.createElement("a");
            showPopupButton.href = "#";
            showPopupButton.className = "popup-button lookup-popup-button ui-btn ui-icon-search ui-btn-icon-notext";
            control.closest(".ui-field-contain").append(showPopupButton);
        }
        control.filterable("option", "filterCallback", function () { });
        var input = this.input;
        input.textinput().typing({
            delay: 500,
            stop: function (event, element) {
                that.doLookupFilter(that, control);
            }
        });
        input.on("keydown", function (e) {
            // Hide suggestions after pressing tab
            if (e.which === 9) {
                input.trigger("hideControl");
            }
        });
        input.on("focus", function () {
            if (!input.prop("readOnly")) {
                that.doLookupFilter(that, control);
                control.css('display', "block");
            }
            jQuery(document).off("tap.form." + that.objProp.objPropId).on("tap.form." + that.objProp.objPropId, function (evt) {
                var $target = jQuery(evt.target);
                // Not input and not the delete button (also check if it is the delete for this control)
                if (evt.target !== input[0] &&
                    !($target.is(".clear-button") && $target.is(input.next())) &&
                    !($target.is(".lookup-popup-button") && $target.prev(".suggestion-control").find("input").is(input))) {
                    input.trigger("hideControl");
                }
            });
        });
        control.on("tap", "li", function (evt) {
            var $li = jQuery(this);
            if ($li.hasClass("noItems")) {
                that.setValue(null);
            }
            else if ($li.hasClass("more-items")) {
                // prevent closing the control
                evt.preventDefault();
                evt.stopPropagation();
                return;
            }
            else {
                // Make sure that the key is a string. We don't expect int values yet.
                that.selectedKey = "" + $li.data('value');
                input.val($li.text());
            }
            that.form.synchronizeProp(that.objProp);
            input.trigger("hideControl");
            //Prevent clicks on SuggestionControl items bubbling down to HTML elements underneath.
            //TCP 53117
            evt.preventDefault();
            evt.stopPropagation();
        });
        input.on('hideControl', function (evt) {
            if (that.selectedKey == null) {
                input.val("");
                that.form.synchronizeProp(that.objProp);
            }
            control.css('display', "none").empty();
            jQuery(document).off("tap.form." + that.objProp.objPropId);
        });
    };
    SuggestionControlHelper.prototype.doLookupFilter = function (that, $ul) {
        var _this = this;
        var value = that.input.val();
        if (value === "") {
            $ul.empty();
            return;
        }
        // First get the filter selection set and then apply to translations.
        that.lookupContainer.getFilteredTranslatedSelectionSet(value, this.filterComparison).always(function (translatedReader, status) {
            if (that.input.prop("readOnly"))
                return;
            that.selectedKey = null;
            $ul.empty();
            var fragment = document.createDocumentFragment();
            // There is no status when the lookupset is empty
            if (status && status.success && translatedReader.length > 0) {
                // The selection set is joined with the translation set, but translationFormattedDataField is wrong when you have a lookup on a lookup
                var dataField = _this.lookupContainer.selectionFormattedDataField.objPropId, listField = _this.lookupContainer.translationFormattedListField.objPropId;
                var length_1 = translatedReader.length > TSFDataLookupContainer.selectionSetLength ? TSFDataLookupContainer.selectionSetLength : translatedReader.length;
                for (var i = 0; i < length_1; i++) {
                    var item = translatedReader.item(i);
                    var listElement = document.createElement("li");
                    listElement.setAttribute("data-value", item.getString(dataField));
                    listElement.innerText = item.getString(listField);
                    fragment.appendChild(listElement);
                }
                // Select the key when there is one matching list item, otherwise clear the key
                if (length_1 == 1 && value == fragment.childNodes[0].innerText) {
                    that.selectedKey = fragment.childNodes[0].getAttribute("data-value");
                    that.form.synchronizeProp(that.objProp);
                }
                else {
                    that.selectedKey = null;
                }
            }
            else {
                // No results
                var listElement = document.createElement("li");
                listElement.className = "noItems";
                var lang = that.form.controller.application.language;
                listElement.innerText = lang.transl(Model.TranslKeys.noResultsAvailable);
                fragment.appendChild(listElement);
            }
            $ul.append(fragment);
            // Check if the element is still an enhanced listview
            if ($ul.data("mobile-listview")) {
                $ul.listview("refresh");
            }
        });
    };
    SuggestionControlHelper.prototype.setValue = function (value) {
        var _this = this;
        if (this.selectedKey != value) {
            this.selectedKey = value;
            if (!value) {
                this.setDefaultValue();
                return;
            }
            var reader_1;
            if (this.form.controller.currentEditor) {
                reader_1 = new IndexedDb.IndexedDbDataReader(this.form.controller.currentEditor.toWebserviceObject());
            }
            else {
                reader_1 = this.form.controller.currentRow;
            }
            var filters = [];
            var viewPropFilters = this.lookupContainer.viewProp.lookup.filters;
            for (var j = 0; j < viewPropFilters.length; j++) {
                filters.push(reader_1.getString(viewPropFilters[j].targetProp.objPropId));
            }
            filters.push(value);
            this.lookupContainer.translateDataToDisplayValues([filters]).done(function () {
                if (_this.selectedKey == value) {
                    _this.input.val(_this.lookupContainer.getTranslFromTranslatedReader(value, reader_1));
                }
            });
        }
    };
    SuggestionControlHelper.prototype.getValue = function () {
        //var control = this.findControl().prev().find("input");
        return this.selectedKey;
    };
    SuggestionControlHelper.prototype.getDisplayValue = function () {
        return this.input.val();
    };
    SuggestionControlHelper.prototype.setDefaultValue = function () {
        this.selectedKey = null;
        this.input.val("");
    };
    SuggestionControlHelper.prototype.setLookupContainer = function (lookupset) {
        this.lookupContainer = lookupset;
    };
    SuggestionControlHelper.prototype.update = function () {
        if (this.form.controller.currentEditor && this.form.controller.currentEditor.newRow[this.objProp.objPropId] == null) {
            this.setDefaultValue();
        }
    };
    SuggestionControlHelper.prototype.setMandatory = function (mandatory) {
        var control = this.input.parent();
        if (mandatory) {
            control.addClass(this.mandatoryClass);
        }
        else {
            control.removeClass(this.mandatoryClass);
        }
    };
    SuggestionControlHelper.prototype.setReadonly = function (readonly) {
        var control = this.input;
        if (readonly) {
            control.parent().addClass(this.readonlyClass);
        }
        else {
            control.parent().removeClass(this.readonlyClass);
        }
        control.prop("readOnly", readonly);
        this.hideClearButton(readonly);
    };
    SuggestionControlHelper.prototype.focus = function () {
        this.input.focus();
    };
    SuggestionControlHelper.prototype.trySetNumeric = function () { };
    SuggestionControlHelper.prototype.trySetMaxLength = function () { };
    SuggestionControlHelper.prototype.dispose = function () {
        jQuery(document).off("tap.form." + this.objProp.objPropId);
    };
    return SuggestionControlHelper;
}(ClearableControlHelper));
var LookupComboControlHelper = /** @class */ (function (_super) {
    __extends(LookupComboControlHelper, _super);
    function LookupComboControlHelper(viewProp, form) {
        return _super.call(this, viewProp, form) || this;
    }
    LookupComboControlHelper.prototype.createControl = function (height) {
        var control = document.createElement("select");
        control.id = this.objProp.objPropId + "_control";
        return control;
    };
    LookupComboControlHelper.prototype.initializeControl = function () {
        _super.prototype.initializeControl.call(this);
        if (this.viewProp.objLinkLookupView.hasZoomButton && this.form.allowFormbuttons()) {
            var showPopupButton = document.createElement("a");
            showPopupButton.href = "#";
            showPopupButton.className = "popup-button lookup-popup-button ui-btn ui-icon-search ui-btn-icon-notext";
            this.findControl().closest(".ui-field-contain").append(showPopupButton);
        }
        this.createItems();
    };
    LookupComboControlHelper.prototype.createItems = function () {
        var _this = this;
        var control = this.findControl();
        control.empty();
        var deferred = jQuery.Deferred();
        this.isLoadingLookup = deferred.promise();
        var optionElement = document.createElement("option");
        optionElement.value = "";
        optionElement.innerHTML = "&nbsp;";
        control.append(optionElement);
        this.updateControl();
        this.lookupContainer.getTranslatedSelectionSet().done(function (translatedReader, status) {
            if (translatedReader && translatedReader.length > 0) {
                _this.createItemsFromReader(control, translatedReader);
            }
            deferred.resolve();
            _this.isLoadingLookup = null;
        });
    };
    // overriden in LookupComboDateControlHelper
    LookupComboControlHelper.prototype.createItemsFromReader = function (control, reader) {
        var selectionDataField = this.lookupContainer.selectionFormattedDataField.objPropId;
        var listField = this.lookupContainer.getListFieldAlias(1 /* LookupReader */);
        var fragment = document.createDocumentFragment();
        for (var i = 0; i < reader.length; i++) {
            var item = reader.item(i);
            var optionElement = document.createElement("option");
            optionElement.value = item.getString(selectionDataField);
            optionElement.innerText = item.getString(listField);
            fragment.appendChild(optionElement);
        }
        control.append(fragment);
        // Reopen the popup when there is new data - for iOS this required us to enable the KeyboardDisplayRequiresUserAction option
        // This dismmisses adn reopens the popup but without this you wouldn't have data (and can't ever use the next/prev buttons)
        // from the keyboard
        if (control.parent().is(".ui-focus")) {
            control.blur().focus();
        }
    };
    LookupComboControlHelper.prototype.setMandatory = function (mandatory) {
        var control = this.findControl();
        if (mandatory) {
            control.parent().addClass(this.mandatoryClass);
        }
        else {
            control.parent().removeClass(this.mandatoryClass);
        }
    };
    LookupComboControlHelper.prototype.setReadonly = function (readonly) {
        var control = this.findControl();
        if (readonly) {
            control.parent().addClass(this.readonlyClass);
        }
        else {
            control.parent().removeClass(this.readonlyClass);
        }
        control.prop('disabled', readonly);
    };
    LookupComboControlHelper.prototype.setValue = function (value) {
        var _this = this;
        var func = function () {
            _super.prototype.setValue.call(_this, value);
            _this.updateControl();
        };
        if (this.isLoadingLookup) {
            // Wait for lookup loading to finish, regardless if the refresh failed or not.
            this.isLoadingLookup.always(func);
        }
        else {
            func();
        }
    };
    LookupComboControlHelper.prototype.setDefaultValue = function () {
        _super.prototype.setDefaultValue.call(this);
        this.updateControl();
    };
    LookupComboControlHelper.prototype.updateControl = function () {
        var control = this.findControl();
        // we could have used trigger('change') on the control but that also removes the focus which we don't want
        var selectedItem = control.find("option").filter(":selected");
        if (selectedItem.length) {
            control.prev("span").text(selectedItem.text());
        }
        else {
            // From jQueryMobile -  Set the contents to &nbsp; which we write as &#160; to be XHTML compliant - see gh-6699
            control.prev("span").html("&#160;");
        }
    };
    LookupComboControlHelper.prototype.setLookupContainer = function (lookupset) {
        this.lookupContainer = lookupset;
    };
    LookupComboControlHelper.prototype.update = function () {
        this.createItems();
    };
    LookupComboControlHelper.prototype.trySetNumeric = function () { };
    LookupComboControlHelper.prototype.trySetMaxLength = function () { };
    return LookupComboControlHelper;
}(ControlHelper));
var LookupComboDateControlHelper = /** @class */ (function (_super) {
    __extends(LookupComboDateControlHelper, _super);
    function LookupComboDateControlHelper(viewProp, form) {
        var _this = _super.call(this, viewProp, form) || this;
        _this.dateFormatString = "YYYY-MM-DDTHH:mm:ss";
        return _this;
    }
    LookupComboDateControlHelper.prototype.setValue = function (value) {
        // format dates to the same format as the value of the option element
        if (value && value instanceof Date) {
            value = moment(value).format(this.dateFormatString);
        }
        _super.prototype.setValue.call(this, value);
    };
    LookupComboDateControlHelper.prototype.createItemsFromReader = function (control, reader) {
        var fragment = document.createDocumentFragment();
        var selectionDataField = this.lookupContainer.translationFormattedDataField.objProp;
        var listField = this.lookupContainer.getListFieldAlias(1 /* LookupReader */);
        for (var i = 0; i < reader.length; i++) {
            var item = reader.item(i);
            var optionElement = document.createElement("option");
            // Convert to a vaid dateformat string
            optionElement.value = moment(item.getValueForObjProp(selectionDataField)).format(this.dateFormatString);
            optionElement.innerText = item.getString(listField);
            fragment.appendChild(optionElement);
        }
        control.append(fragment).trigger("change");
    };
    return LookupComboDateControlHelper;
}(LookupComboControlHelper));
var SignatureControlHelper = /** @class */ (function (_super) {
    __extends(SignatureControlHelper, _super);
    function SignatureControlHelper(viewProp, form) {
        var _this = _super.call(this, viewProp, form) || this;
        _this.value = null;
        return _this;
    }
    SignatureControlHelper.prototype.createControl = function (height) {
        var control = document.createElement("div");
        control.id = this.objProp.objPropId + "_control";
        control.setAttribute("style", "height: " + (height - this.form.controlBorderSize) + "px;");
        control.className = "signature-control ui-body-inherit";
        return control;
    };
    SignatureControlHelper.prototype.initializeControl = function () {
        var _this = this;
        _super.prototype.initializeControl.call(this);
        // This might seem a little bit weird but we need to make sure the setData is called after the control height/width is corrected.
        // Or else the image might become invisible or broken
        var deferred = jQuery.Deferred();
        this.initializedPromise = deferred.promise();
        var control = this.findControl();
        // Explicitly set the line width to 3, if you don't do it the lineWidth will change when the canvas is cleared.
        control.jSignature("init", { lineWidth: 3, 'decor-color': null });
        // Disable the auto resize code of jSignature, since its buggy.
        jQuery(window).off("resize.jSignature");
        // Adjust the height and width of the canvas when the control becomes visible.
        this.form.pageShown.then(function () {
            _this.resizeCanvas();
            var resizeTimer, that = _this;
            jQuery(window).on("resize." + that.objProp.objPropId, function (evt) {
                clearTimeout(resizeTimer);
                resizeTimer = window.setTimeout(function () {
                    that.resizeCanvas();
                    if (that.value) {
                        that.findControl().jSignature("setData", that.value);
                    }
                }, 500);
            });
            control.off("change." + that.objProp.objPropId).on("change." + that.objProp.objPropId, function (e) {
                that.value = control.jSignature("getData", "default");
            });
            // Save empty value, not in a static because it is different when the height/width is different.
            _this.emptyValue = control.jSignature("getData", "default");
            deferred.resolve();
        });
    };
    SignatureControlHelper.prototype.resizeCanvas = function () {
        var control = this.findControl();
        var canvas = control.find("canvas");
        canvas.css({
            'width': control.width() + 'px',
            'height': control.height() + 'px'
        });
        canvas.prop({
            'width': control.width(),
            'height': control.height()
        });
    };
    SignatureControlHelper.prototype.setMandatory = function (mandatory) {
        var control = this.findControl();
        if (mandatory) {
            control.addClass(this.mandatoryClass);
        }
        else {
            control.removeClass(this.mandatoryClass);
        }
    };
    SignatureControlHelper.prototype.setReadonly = function (readonly) {
        _super.prototype.hideClearButton.call(this, readonly);
        var control = this.findControl();
        if (readonly) {
            control.addClass(this.readonlyClass);
            control.jSignature("disable");
        }
        else {
            control.removeClass(this.readonlyClass);
            control.jSignature("enable");
        }
    };
    SignatureControlHelper.prototype.setHidden = function (hide, keepSpace) {
        var _this = this;
        var control = this.findControl();
        if (!keepSpace && hide) {
            this.value = control.jSignature("getData", "default");
        }
        _super.prototype.setHidden.call(this, hide, keepSpace);
        if (!keepSpace && !hide) {
            this.resizeCanvas();
            if (!Utils.isEmpty(this.value)) {
                this.initializedPromise.done(function () {
                    control.jSignature("setData", _this.value);
                });
            }
        }
    };
    SignatureControlHelper.prototype.setValue = function (value) {
        var _this = this;
        if (!Utils.isEmpty(value)) {
            this.filePromise = null;
            var control_3 = this.findControl();
            try {
                if (value.File) {
                    var file_1 = Utils.createIconFromBase64(value.File);
                    if (file_1 != control_3.jSignature("getData", "default")) {
                        this.clearValuePreventChangeEvent();
                        this.initializedPromise.done(function () {
                            control_3.jSignature("setData", file_1);
                            _this.value = file_1;
                        });
                    }
                }
                else if (value.FileName) {
                    // Make sure that the control is visible when the image is loaded
                    this.form.pageShown.then(function () {
                        _this.loadImage(_this, control_3);
                    });
                }
            }
            catch (error) {
                console.log("Error initializing signature: " + error);
                this.setDefaultValue();
            }
        }
        else {
            this.clearValuePreventChangeEvent();
        }
    };
    SignatureControlHelper.prototype.loadImage = function (that, control) {
        that.clearValuePreventChangeEvent();
        var controlPromise = this.initializedPromise;
        that.filePromise = that.form.controller.getFile(null, that.objProp).done(function (base64Icon) {
            if (that.filePromise == this && base64Icon) {
                var file_2 = Utils.createIconFromBase64(base64Icon);
                if (file_2 != control.jSignature("getData", "default")) {
                    controlPromise.done(function () {
                        control.jSignature("setData", file_2);
                        that.value = file_2;
                    });
                }
            }
        });
    };
    SignatureControlHelper.prototype.clearValuePreventChangeEvent = function () {
        var _this = this;
        // Prevent change event, so it won't update the controller, and as a
        // side effect (executing layouts/defaults multiple times)
        this.form.pageShown.then(function () {
            var control = _this.findControl();
            control.one("change", function (evt) {
                evt.stopPropagation();
            });
            _this.setDefaultValue();
        });
    };
    SignatureControlHelper.prototype.setDefaultValue = function () {
        var _this = this;
        this.form.pageShown.then(function () {
            var control = _this.findControl();
            control.jSignature("clear");
            _this.value = null;
        });
    };
    SignatureControlHelper.prototype.getValue = function () {
        // TODO only fill in the file in the contract when changed.
        var value = this.findControl().jSignature("getData", "default");
        if (value && value != this.emptyValue) {
            // TODO generate a decent filename, current format is: yyyy-mm-dd-hh-mm-ss-signature.png
            var fileName = moment().format("YYYY-MM-DD-HH-mm-ss") + "-signature.png";
            var fileContract = FileContract.CreateFromFileName(this.objProp.fileStorageFolder, fileName, value.substring("data:image/png;base64,".length), undefined, undefined);
            return fileContract;
        }
        return null;
    };
    SignatureControlHelper.prototype.dispose = function () {
        jQuery(window).off("resize." + this.objProp.objPropId);
    };
    return SignatureControlHelper;
}(ClearableControlHelper));
var RadioButtonControlHelper = /** @class */ (function (_super) {
    __extends(RadioButtonControlHelper, _super);
    function RadioButtonControlHelper(viewProp, form) {
        return _super.call(this, viewProp, form) || this;
    }
    RadioButtonControlHelper.prototype.createControl = function (height) {
        var control = document.createElement("fieldset"), prop = this.objProp;
        control.id = this.objProp.objPropId + "_control";
        control.className = "radio-button-fieldset";
        control.setAttribute("data-role", "controlgroup");
        /*  There's no RADIO_VERTICAL and RADIO_HORIZONTAL in 2018.1 and up anymore, they have been replaced by
            RADIO_BUTTON combined with fieldHeightInPositions. */
        if (Model.TSFApplicationManager.Current.globalSettings.currentSFVersion >= 2 /* V2018_1 */) {
            //fieldHeightPositions 2 and up means it's a vertical control.
            if (this.viewProp.fieldHeightInPositions < 2) {
                control.setAttribute("data-type", "horizontal");
            }
        }
        else if (prop.controlId === Enum.ControlType.RADIO_HORIZONTAL) {
            control.setAttribute("data-type", "horizontal");
        }
        // Create an option for each element
        var elements = prop.domain.elements.array;
        for (var i = 0; i < elements.length; i++) {
            var element = elements[i], id = this.objProp.objPropId + i;
            var uiCheckbox = document.createElement("div");
            uiCheckbox.className = "ui-radio";
            var label = document.createElement("label");
            label.className = "ui-btn ui-btn-inherit ui-radio-off";
            label.htmlFor = id;
            label.innerText = element.elementTransl.transl;
            var radio = document.createElement("input");
            radio.type = "radio";
            radio.name = this.objProp.objPropId;
            radio.id = id;
            radio.value = element.value;
            radio.setAttribute("data-enhanced", "true");
            uiCheckbox.appendChild(label);
            uiCheckbox.appendChild(radio);
            control.appendChild(uiCheckbox);
        }
        return control;
    };
    RadioButtonControlHelper.prototype.getValue = function () {
        // There should always be a checked radio button, but return null if there is none.
        var radio = this.findControl().find('input:radio:checked');
        if (radio.length > 0) {
            return radio.val();
        }
        return null;
    };
    RadioButtonControlHelper.prototype.setValue = function (value) {
        var allRadios = this.findControl().find('input:radio');
        // Filter on the corresponding value
        var input = allRadios.filter('[value="' + value + '"]');
        if (input.length > 0) {
            input.prop('checked', true);
            // Refresh all radio buttons since one of them is unchecked.
            allRadios.checkboxradio('refresh');
        }
        else {
            this.setDefaultValue();
        }
    };
    RadioButtonControlHelper.prototype.setDefaultValue = function () {
        // All radio buttons should default be unchecked.
        var allRadios = this.findControl().find('input:radio');
        for (var i = 0; i < allRadios.length; i++) {
            allRadios[i].checked = false;
        }
        allRadios.checkboxradio('refresh');
    };
    RadioButtonControlHelper.prototype.setReadonly = function (readonly) {
        var inputs = this.findControl().find('input:radio');
        if (readonly) {
            inputs.checkboxradio('disable');
        }
        else {
            inputs.checkboxradio('enable');
        }
    };
    RadioButtonControlHelper.prototype.setMandatory = function (mandatory) {
        // Do nothing since a radio control is always mandatory.
    };
    RadioButtonControlHelper.prototype.trySetNumeric = function () { };
    RadioButtonControlHelper.prototype.trySetMaxLength = function () { };
    return RadioButtonControlHelper;
}(ControlHelper));
var PictureComboControlHelper = /** @class */ (function (_super) {
    __extends(PictureComboControlHelper, _super);
    function PictureComboControlHelper(viewProp, form) {
        return _super.call(this, viewProp, form) || this;
    }
    PictureComboControlHelper.prototype.createControl = function (height) {
        var control = document.createElement("div");
        control.className = "image-control-div";
        var image = document.createElement("img");
        image.id = this.objProp.objPropId + "_image";
        image.src = "img/photo.png";
        image.removeAttribute("width");
        image.removeAttribute("height");
        var imageHeight = height + "px";
        // Do not set the width, but let the image decide what to do based on the height"width: " + prop.fieldWidth + "px;
        control.style.height = imageHeight;
        image.style.maxHeight = imageHeight;
        control.appendChild(image);
        var select = document.createElement("select");
        select.id = this.objProp.objPropId + "_control";
        select.setAttribute("data-native-menu", "false");
        // Guard against a image_combo without domain elements
        var domain = this.objProp.domain;
        var length = domain ? domain.elements.length : 0;
        for (var i = 0; i < length; i++) {
            var element = domain.elements.item(i);
            var option = document.createElement("option");
            option.value = element.value;
            option.innerText = element.elementTransl.transl;
            if (element.picture) {
                option.setAttribute("data-image", Utils.createIconFromBase64(element.picture));
            }
            select.appendChild(option);
        }
        control.appendChild(select);
        return control;
    };
    PictureComboControlHelper.prototype.initializeControl = function () {
        var _this = this;
        _super.prototype.initializeControl.call(this);
        var controlId = "#" + this.objProp.objPropId + "_control";
        var opts = jQuery(controlId + " option");
        // Cancel the click when the control is readonly.
        jQuery(controlId + "-button").click(function () {
            if (_this.findControl().prev().hasClass(_this.readonlyClass)) {
                return false;
            }
        });
        // Add the images to the popup.
        jQuery(controlId + "-listbox-popup").on("popupafteropen", function (event, ui) {
            jQuery(controlId + "-menu li").each(function (index) {
                jQuery(this).addClass("ui-li-has-icon");
                if (jQuery(this).find("img").length == 0) {
                    var imageURL = opts.eq(index).data("image");
                    var imgElem = document.createElement("img");
                    imgElem.src = imageURL || "img/photo.png";
                    imgElem.className = "ui-li-icon";
                    jQuery(this).find("a").prepend(imgElem);
                }
            });
        });
    };
    PictureComboControlHelper.prototype.setMandatory = function (mandatory) {
        var control = this.findControl();
        if (mandatory) {
            control.prev().addClass(this.mandatoryClass);
        }
        else {
            control.prev().removeClass(this.mandatoryClass);
        }
    };
    PictureComboControlHelper.prototype.setReadonly = function (readonly) {
        var control = this.findControl();
        if (readonly) {
            control.prev().addClass(this.readonlyClass);
        }
        else {
            control.prev().removeClass(this.readonlyClass);
        }
        control.parent().prev("img").toggleClass("hide", !readonly);
        control.parent().toggleClass("hide", readonly);
        control.prop('disabled', readonly).selectmenu('refresh');
    };
    PictureComboControlHelper.prototype.setValue = function (value) {
        _super.prototype.setValue.call(this, value);
        var control = this.findControl();
        var img = control.parent().prev("img")[0];
        var domain = this.objProp.domain;
        if (img && domain) {
            var domElements = domain.elements.array;
            for (var i = 0; i < domElements.length; i++) {
                if (domElements[i].value == value) {
                    img.src = (domElements[i].picture ? Utils.createIconFromBase64(domElements[i].picture) : "img/photo.png");
                    break;
                }
            }
        }
        // Do this to let the dynamic created combo show his value.
        control.selectmenu("refresh");
    };
    PictureComboControlHelper.prototype.setDefaultValue = function () {
        _super.prototype.setDefaultValue.call(this);
        var control = this.findControl();
        var img = control.parent().prev("img")[0];
        if (img) {
            img.src = "img/photo.png";
        }
        // Do this to let the dynamic created combo show his value.
        control.selectmenu("refresh");
    };
    PictureComboControlHelper.prototype.getValue = function () {
        var value = this.findControl().val();
        return Utils.isEmpty(value) ? null : value;
    };
    return PictureComboControlHelper;
}(ControlHelper));
var HtmlControlHelper = /** @class */ (function (_super) {
    __extends(HtmlControlHelper, _super);
    function HtmlControlHelper(viewProp, form) {
        return _super.call(this, viewProp, form) || this;
    }
    HtmlControlHelper.prototype.createControl = function (height) {
        var control = document.createElement("textarea");
        control.id = this.objProp.objPropId + "_control";
        control.setAttribute("data-role", "none");
        // Add padding height to height because this control contains the box-size=border-box property.
        control.setAttribute("style", "height:" + height + "px;");
        return control;
    };
    HtmlControlHelper.prototype.initializeControl = function () {
        var _this = this;
        _super.prototype.initializeControl.call(this);
        System.import("css.kendo.default.min");
        System.import("css.kendo.common.min");
        this.systemJsImport = System.import("kendo.editor.min").then(function () {
            _this.editor = _this.findControl().kendoEditor({
                resizable: {
                    content: true
                    //toolbar: true, disabled because it doesn't seem to work correctly; after adding a table
                    // the popups isn't visible but you can't click there anymore
                },
                tools: ["bold",
                    "italic",
                    "underline",
                    "insertUnorderedList",
                    "insertOrderedList"]
            }).data("kendoEditor");
            // We require this class on the toolbaar so it doesn't close the toolbar when clicking on it
            // For example the paragraph and the create table shouldn't close the popup
            (_this.editor.toolbar.element).addClass("preventBlur");
            //Open the HTML editor iframe links in a new window. If we don't do this,
            //the Mobile GUI will go to the clicked link, which forces the user to restart.
            _this.findControl().parent().children("iframe").load(function () {
                $(this).contents().find("a").each(function (index) {
                    $(this).on("click", function (event) {
                        event.preventDefault();
                        event.stopPropagation();
                        window.open($(this).attr("href"));
                    });
                });
            });
        });
    };
    HtmlControlHelper.prototype.setDefaultValue = function () {
        var _this = this;
        if (this.editor) {
            this.editor.value(null);
        }
        else {
            this.systemJsImport.then(function () {
                _this.setDefaultValue();
            });
        }
    };
    HtmlControlHelper.prototype.setValue = function (value) {
        var _this = this;
        if (this.editor) {
            // when passing undefined nothing happens
            this.editor.value(value || null);
        }
        else {
            this.systemJsImport.then(function () {
                _this.setValue(value);
            });
        }
    };
    HtmlControlHelper.prototype.getValue = function () {
        if (this.editor) {
            return this.editor.value();
        }
        return null;
    };
    HtmlControlHelper.prototype.setReadonly = function (readonly) {
        var _this = this;
        if (this.editor) {
            this.editor.body.contentEditable = (readonly ? "false" : "true");
            var toolbar_1 = this.editor.toolbar.element;
            var selector = jQuery(toolbar_1).add(jQuery(this.__control).closest(".k-editor"));
            selector.toggleClass(this.readonlyClass, readonly);
        }
        else {
            this.systemJsImport.then(function () {
                _this.setReadonly(readonly);
            });
        }
    };
    HtmlControlHelper.prototype.setMandatory = function (mandatory) {
        var control = this.findControl();
        if (mandatory) {
            control.addClass(this.mandatoryClass);
        }
        else {
            control.removeClass(this.mandatoryClass);
        }
    };
    HtmlControlHelper.prototype.dispose = function () {
        this.editor.destroy();
    };
    return HtmlControlHelper;
}(ControlHelper));
var LabelControlHelper = /** @class */ (function (_super) {
    __extends(LabelControlHelper, _super);
    function LabelControlHelper(viewProp, form) {
        return _super.call(this, viewProp, form) || this;
    }
    LabelControlHelper.prototype.createControl = function (height) {
        var control = document.createElement("label");
        control.id = this.objProp.objPropId + "_control";
        control.className = "label-control";
        return control;
    };
    LabelControlHelper.prototype.setValue = function (value) {
        this.value = value;
        this.findControl().html(value);
    };
    LabelControlHelper.prototype.getValue = function () {
        return this.value;
    };
    LabelControlHelper.prototype.setMandatory = function (mandatory) {
    };
    LabelControlHelper.prototype.setReadonly = function (readonly) {
    };
    return LabelControlHelper;
}(ControlHelper));
var GroupHeaderControlHelper = /** @class */ (function (_super) {
    __extends(GroupHeaderControlHelper, _super);
    function GroupHeaderControlHelper(viewProp, form) {
        return _super.call(this, viewProp, form) || this;
    }
    GroupHeaderControlHelper.prototype.createControl = function (height) {
        var control = document.createElement("div");
        control.id = this.objProp.objPropId + "_control";
        control.className = "group group-control";
        return control;
    };
    GroupHeaderControlHelper.prototype.setValue = function (value) {
        this.value = value;
        if (value == null) {
            this.findControl().html("&nbsp;");
        }
        else {
            this.findControl().text(value);
        }
    };
    GroupHeaderControlHelper.prototype.getValue = function () {
        return this.value;
    };
    GroupHeaderControlHelper.prototype.setMandatory = function (mandatory) {
    };
    GroupHeaderControlHelper.prototype.setReadonly = function (readonly) {
    };
    return GroupHeaderControlHelper;
}(ControlHelper));
var GroupHeaderIconControlHelper = /** @class */ (function (_super) {
    __extends(GroupHeaderIconControlHelper, _super);
    function GroupHeaderIconControlHelper() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return GroupHeaderIconControlHelper;
}(HiddenControlHelper));
var BarcodeScanControlHelper = /** @class */ (function (_super) {
    __extends(BarcodeScanControlHelper, _super);
    function BarcodeScanControlHelper() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    BarcodeScanControlHelper.prototype.initializeControl = function () {
        var _this = this;
        _super.prototype.initializeControl.call(this);
        if (!Utils.isCordova() || window["cordova"].plugins.barcodeScanner === undefined) {
            return;
        }
        this.barcodeScanButton = document.createElement("img");
        this.barcodeScanButton.className = "barcode-scan-button";
        this.barcodeScanButton.src = "./img/barcode-scan.svg";
        var control = this.findControl();
        control.parent().append(this.barcodeScanButton);
        control.addClass("barcode-scan-button-input");
        var noNumericValueTransl = this.form.controller.application.language.transl(Model.TranslKeys.noNumericValue1);
        var valueTooLongTransl = this.form.controller.application.language.transl(Model.TranslKeys.valueTooLong);
        this.barcodeScanButton.onclick = function () {
            Utils.showLoader();
            window["cordova"].plugins.barcodeScanner.scan(function (result) {
                Utils.hideLoader();
                var dataLength = _this.objProp.dataLength;
                if (dataLength && result.text.length > dataLength) {
                    var popup = ConfirmationPopup.CreateOkPopup();
                    popup.initialize("", valueTooLongTransl.replace("[value]", result.text).replace("[length]", "" + dataLength));
                    return;
                }
                if (_this.objProp.isNumeric()) {
                    var parsedAsFloat = parseFloat(result.text);
                    if (isNaN(parsedAsFloat) || parsedAsFloat != result.text) {
                        var popup = ConfirmationPopup.CreateOkPopup();
                        popup.initialize("", noNumericValueTransl.replace("[value]", result.text));
                        return;
                    }
                }
                _this.setValue(result.text);
                _this.form.synchronizeProp(_this.objProp);
            }, function (error) {
                Utils.hideLoader();
                throw new Error("Scanning failed: " + error);
            }, {
                preferFrontCamera: false,
                showFlipCameraButton: false,
                showTorchButton: true,
                torchOn: false,
                saveHistory: false,
                prompt: "Place a barcode inside the scan area",
                resultDisplayDuration: 0,
                disableAnimations: true,
                disableSuccessBeep: false // iOS and Android
            });
        };
    };
    BarcodeScanControlHelper.prototype.setReadonly = function (readonly) {
        _super.prototype.setReadonly.call(this, readonly);
        if (!Utils.isCordova()) {
            return;
        }
        jQuery(this.barcodeScanButton).toggle(!readonly);
        this.findControl().toggleClass("barcode-scan-button-input", !readonly);
    };
    return BarcodeScanControlHelper;
}(ControlHelper));
