var TSFForm = /** @class */ (function () {
    function TSFForm(application, controller, mode) {
        this.mode = mode;
        // List of ControlHelper which are displayed on the form.
        this.controlList = new Collections.List();
        // Indicates the rowheight within a form. Also set as default rowHeight in.
        this.rowHeight = 35;
        // Indicates the border size of a control (2 * 2px).
        this.controlBorderSize = 2;
        // Height between fields of jQuery Mobile (1em of the fontsize of the .form-control-column).
        this.controlHeightBetweenFields = 16;
        this.synchronizePropPromise = jQuery.Deferred().resolve().promise();
        if (mode == null || mode == undefined)
            alert('Incorrect form mode given!');
        this.controller = controller;
        this.obj = controller.obj;
        this.objView = controller.objView;
        this.sortedProps = this.objView.getFormOrderedProps();
    }
    TSFForm.prototype.initialize = function (contentElement) {
        var _this = this;
        jQuery(contentElement).addClass('form');
        this.formBody = jQuery(document.createElement('form'));
        this.formBody.appendTo(contentElement);
        this.build();
        this.initializeControls();
        if (!this.controller.initialized) {
            this.controller.events.controllerInitialized(function (args) {
                _this.bindControllerEvents();
            });
        }
        else {
            this.bindControllerEvents();
        }
        // Note: this one is outside the bindControllerEvents or it won't update the lookups correctly
        // Open a detail with a filtered lookup. The lookupset started empty because there is no row. After a row is selected during initialize
        // It changed the filters but update would be called (when the controller isn't initialized)
        this.controller.events.lookupsetChanged(function (args) {
            var helper = _this.controlList.key(args.lookupSet.viewProp.objPropId);
            // Check for update method, because it might actually be a hidden controlhelper.
            if (helper && helper.update)
                helper.update();
        });
        Utils.fixResizeOnFocus(this.formBody);
        this.formBody.on("submit", function (evt) {
            return false;
        });
        this.formBody.on("click", "a.lookup-popup-button", function (evt) {
            var helper = _this.searchControl(evt.currentTarget);
            //  open lookup popup/screen
            var displayValue = helper.getDisplayValue && helper.getDisplayValue();
            if (displayValue == "")
                displayValue = null;
            _this.controller.prepareLookupControl(helper.objProp, displayValue, helper.getValue());
            var page = jQuery(evt.currentTarget).closest(".ui-page");
            var url = "abstractscreen.html" + jQuery.mobile.path.parseUrl(page.data('url')).search.replace(/&mode=./g, "");
            var getSeparator = url.indexOf("?") !== -1 ? "&" : "?";
            url += getSeparator + "lookup=" + helper.objProp.objPropId;
            //Needed to fix opening lookup from a menu task popup.
            if (url.indexOf("task=") !== -1) {
                if (url.indexOf("obj=") === -1) {
                    url += "&obj=" + _this.controller.obj.objId;
                }
                if (url.indexOf("config=") === -1) {
                    if (_this.controller.objView.objViewId !== null) {
                        url += "&config=" + _this.controller.objView.objViewId;
                    }
                }
            }
            NavigationManager.navigate(url);
        });
    };
    TSFForm.prototype.bindControllerEvents = function () {
        var _this = this;
        var controller = this.controller;
        controller.events.layoutChanged(function (args) {
            _this.layoutsChanged(args.layoutState);
        });
        controller.events.rowIndexChanged(function () {
            _this.setValues(_this.controller.currentRow);
        });
        controller.events.datasetChanged(function (args) {
            _this.setValues(_this.controller.currentRow);
        });
        controller.events.dataRowValuesChanged(function (args) {
            _this.setValues(args.values);
        });
        // Abstractscreen calls getData - which might refresh the data when the form was dirty
        // Because of that, we don't need t oset the values when the controller is dirty; would be overwritten anyway
        // Also fixes issue where you changed a parent record and you would see the old values in the form (for a short period)
        // when navigating to it.
        if (!controller.isMarkedDirty) {
            this.syncValuesFromController();
        }
    };
    TSFForm.prototype.build = function () {
        var builder = new FormBuilder(this);
        var fragment = document.createDocumentFragment();
        var colList = [[]];
        var colNr = 0;
        var controlNr = 0;
        var numOfPositions = 0;
        for (var colListIndex = 0; colListIndex < this.sortedProps.length; colListIndex++) {
            var prop = this.sortedProps[colListIndex];
            if (prop.fieldInNextColumn || prop.fieldOnNextTab) {
                colNr++;
                controlNr = 0;
                numOfPositions = 0;
            }
            if (!colList[colNr])
                colList[colNr] = [];
            var placeHolder = new FormControlPlaceHolder(prop, numOfPositions);
            colList[colNr][controlNr] = placeHolder;
            numOfPositions += placeHolder.positionsOfControl;
            controlNr++;
        }
        var forcedSingleColumn = this.objView.numberOfColsInForm === 1;
        var numOfCols = forcedSingleColumn ? 1 : 2;
        var colCountOnTab = 0;
        if (forcedSingleColumn) {
            this.formBody.addClass("singleColumn");
        }
        var htmlGrid;
        // variable for the current group; this way we know when we have a group;
        // we don't render empty groups so we wait until we create a control
        var currentGroup;
        for (var colListIndex_1 = 0; colListIndex_1 < colList.length; colListIndex_1++) {
            var colPropList = colList[colListIndex_1];
            if (colPropList.length == 0)
                continue;
            var column = void 0;
            var firstColProp = colPropList[0].prop;
            // Create a new grid/tab
            if (colCountOnTab == 0 || firstColProp.fieldOnNextTab) {
                if ((colCountOnTab > 0 || firstColProp.tabTransl)
                    // Check if all props are hidden
                    && !colPropList.every(function (element, index, array) {
                        return (element.prop.formMaxAccessType == Enum.AccessType.Hidden);
                    })) {
                    fragment.appendChild(builder.createTab(firstColProp));
                }
                htmlGrid = document.createElement("div");
                htmlGrid.className = "form-tab ui-responsive ui-grid-a";
                fragment.appendChild(htmlGrid);
                // Check if controls of the tab should be spread over multiple columns.
                var spreadOverCols = false;
                if (colCountOnTab == 0 && this.obj.isTask == false) {
                    if (colList.length - 1 <= colListIndex_1) {
                        spreadOverCols = true;
                    }
                    else {
                        var col = colList[colListIndex_1 + 1], firstPropOnNextCol = col.length > 0 ? col[0] : null;
                        if (!firstPropOnNextCol || firstPropOnNextCol.prop.fieldOnNextTab) {
                            spreadOverCols = true;
                        }
                    }
                }
                if (spreadOverCols) {
                    // Look at the last in the list to know the number of positions of this column.
                    var colNumOfPositions = colPropList[colPropList.length - 1].currentPositionInCol;
                    var maxPositionsPerCol = Math.ceil(colNumOfPositions / numOfCols);
                    var startIndex = 0;
                    while (startIndex < colPropList.length) {
                        colCountOnTab++;
                        column = this.createColumn(colCountOnTab);
                        htmlGrid.appendChild(column);
                        var firstControlOnColumn = true;
                        for (var propIndex_1 = startIndex; propIndex_1 < colPropList.length; propIndex_1++) {
                            var placeHolder = colPropList[propIndex_1];
                            // End current column and continue with the next one. Always show first visible control in same column.
                            if (placeHolder.currentPositionInCol > colCountOnTab * maxPositionsPerCol && !firstControlOnColumn)
                                break;
                            // Add a group label to the content when given.
                            if (placeHolder.prop.groupTransl) {
                                currentGroup = builder.createGroup(placeHolder.prop);
                            }
                            if (currentGroup && placeHolder.prop.formMaxAccessType != Enum.AccessType.Hidden) {
                                column.appendChild(currentGroup);
                                currentGroup = null; // Null the group so we know it is visible (and added)
                            }
                            this.createControl(placeHolder.prop, column, builder);
                            // First visible control is shown now.
                            if (placeHolder.currentPositionInCol > 0)
                                firstControlOnColumn = false;
                            startIndex++;
                        }
                    }
                    // Reset the current column/grid and get through with the next one.
                    colCountOnTab = 0;
                    continue;
                }
            }
            colCountOnTab++;
            column = this.createColumn(colCountOnTab);
            htmlGrid.appendChild(column);
            // Reset when number of columns is reached.
            if (colCountOnTab == numOfCols) {
                colCountOnTab = 0; // Reset to begin with a new tab
            }
            for (var propIndex = 0; propIndex < colPropList.length; propIndex++) {
                var prop = colPropList[propIndex].prop;
                if (prop.groupTransl) {
                    currentGroup = builder.createGroup(prop);
                }
                if (currentGroup && prop.formMaxAccessType != Enum.AccessType.Hidden) {
                    column.appendChild(currentGroup);
                    currentGroup = null; // Null the group so we know it is visible (and added)
                }
                this.createControl(prop, column, builder);
            }
        }
        // Trigger the form create event to enhance the new form controls.
        this.formBody.append(fragment);
        this.formBody.enhanceWithin();
        // Add and remove focus class for labels
        this.formBody.on("focus", "input, textarea, select", function (e) {
            if (jQuery(this).is("textarea")) {
                jQuery(this).siblings('label').addClass("label_control_focused");
            }
            else if (jQuery(this).is("select")) {
                jQuery(this).parent().parent().siblings('label').addClass("label_control_focused");
            }
            else {
                jQuery(this).parent().siblings('label').addClass("label_control_focused");
                // Suggestion control and date/time control
                jQuery(this).parent().parent().parent().siblings('label').addClass("label_control_focused");
            }
        }).on("focusout", "input, textarea, select", function (e) {
            if (jQuery(this).is("textarea")) {
                jQuery(this).siblings('label').removeClass("label_control_focused");
            }
            else if (jQuery(this).is("select")) {
                jQuery(this).parent().parent().siblings('label').removeClass("label_control_focused");
            }
            else {
                jQuery(this).parent().siblings('label').removeClass("label_control_focused");
                // Suggestion control and date/time control
                jQuery(this).parent().parent().parent().siblings('label').removeClass("label_control_focused");
            }
        });
    };
    TSFForm.prototype.createColumn = function (colCount) {
        // Start with "a" when colcount is 1
        var uiBlockType = String.fromCharCode(96 + colCount), htmlColumn = document.createElement("div");
        htmlColumn.className = "form-control-column ui-block-" + uiBlockType;
        return htmlColumn;
    };
    TSFForm.prototype.createControl = function (prop, column, builder) {
        // Create and add ControlHelper
        var controlHelper = this.createControlHelper(prop);
        this.controlList.add(prop.objPropId, controlHelper);
        // Do not render a control when it never becomes visible.
        if (prop.formMaxAccessType != Enum.AccessType.Hidden) {
            column.appendChild(builder.renderControl(prop, controlHelper));
        }
    };
    TSFForm.prototype.createControlHelper = function (viewProp) {
        var controlHelper;
        var prop = viewProp.objProp;
        if (viewProp.formMaxAccessType == Enum.AccessType.Hidden) {
            controlHelper = new HiddenControlHelper(viewProp, this);
        }
        else if (viewProp.hasLookup()) {
            if (viewProp.objLinkLookupView.controlType == 1 /* Combo */) {
                if (prop.xmlDttp == "date") {
                    controlHelper = new LookupComboDateControlHelper(viewProp, this);
                }
                else {
                    controlHelper = new LookupComboControlHelper(viewProp, this);
                }
            }
            else {
                controlHelper = new SuggestionControlHelper(viewProp, this);
            }
            controlHelper.setLookupContainer(this.controller.getLookupSet(viewProp));
        }
        else {
            switch (prop.controlId) {
                case Enum.ControlType.TIME:
                    if (Modernizr.inputtypes.time)
                        controlHelper = new TimeControlHelper(viewProp, this);
                    break;
                case Enum.ControlType.DATE:
                    if (Modernizr.inputtypes.date)
                        controlHelper = new DateControlHelper(viewProp, this);
                    break;
                case Enum.ControlType.DATETIME:
                    if (Modernizr.inputtypes.date && Modernizr.inputtypes.time)
                        controlHelper = new DateTimeControlHelper(viewProp, this);
                    break;
                case Enum.ControlType.CHECKBOX:
                    controlHelper = new CheckboxControlHelper(viewProp, this);
                    break;
                case Enum.ControlType.COMBO:
                    controlHelper = new ComboControlHelper(viewProp, this);
                    break;
                case Enum.ControlType.IMAGE_COMBO:
                    controlHelper = new PictureComboControlHelper(viewProp, this);
                    break;
                case Enum.ControlType.RADIO_HORIZONTAL:
                case Enum.ControlType.RADIO_VERTICAL:
                case Enum.ControlType.RADIO_BUTTON:
                    controlHelper = new RadioButtonControlHelper(viewProp, this);
                    break;
                case Enum.ControlType.MULTILINE:
                    controlHelper = new MultilineControlHelper(viewProp, this);
                    break;
                case Enum.ControlType.IMAGE_LINK:
                case Enum.ControlType.IMAGE_BLOB:
                case Enum.ControlType.IMAGE_UPLOAD:
                    controlHelper = new ImageControlHelper(viewProp, this);
                    break;
                case Enum.ControlType.SIGNATURE:
                    controlHelper = new SignatureControlHelper(viewProp, this);
                    break;
                case Enum.ControlType.FILE:
                case Enum.ControlType.VIDEO_LINK:
                case Enum.ControlType.FILE_UPLOAD:
                    controlHelper = new FileControlHelper(viewProp, this);
                    break;
                case Enum.ControlType.EMAIL:
                case Enum.ControlType.URL:
                case Enum.ControlType.PHONE_NUMBER:
                case Enum.ControlType.GOOGLE_MAPS:
                    controlHelper = new UriControlHelper(viewProp, this);
                    break;
                case Enum.ControlType.HTML:
                    controlHelper = new HtmlControlHelper(viewProp, this);
                    break;
                case Enum.ControlType.LABEL:
                    controlHelper = new LabelControlHelper(viewProp, this);
                    break;
                case Enum.ControlType.GROUP_HEADER_LABEL:
                    controlHelper = new GroupHeaderControlHelper(viewProp, this);
                    break;
                case Enum.ControlType.GROUP_HEADER_ICON:
                    controlHelper = new GroupHeaderIconControlHelper(viewProp, this);
                    break;
                case Enum.ControlType.BARCODE_SCANNER:
                    controlHelper = new BarcodeScanControlHelper(viewProp, this);
                    break;
                default:
                    if (viewProp.objProp.isNumeric()) {
                        controlHelper = new NumericControlHelper(viewProp, this);
                    }
                    else {
                        controlHelper = new ClearableControlHelper(viewProp, this);
                    }
                    break;
            }
        }
        return controlHelper;
    };
    TSFForm.prototype.syncValuesFromController = function () {
        var currentRow = this.controller.currentRow, editor = this.controllerEditor;
        for (var i = 0; i < this.sortedProps.length; i++) {
            var prop = this.sortedProps[i].objProp;
            var control = this.controlList.key(prop.objPropId);
            // Set value
            if (this.mode == 0 /* View */ && currentRow) {
                // TODO get value in the correct datatype.
                control.setValue(currentRow.getValueForObjProp(prop));
            }
            else if (editor && (this.mode == 1 /* Add */ || this.mode == 2 /* Edit */ || this.mode == 3 /* Copy */)) {
                control.setValue(Utils.convertToDataType(editor.newRow[prop.objPropId], prop.objPropId, prop.dttpId));
            }
            else {
                control.setValue(null);
            }
        }
    };
    TSFForm.prototype.initializeControls = function () {
        var _this = this;
        var controller = this.controller;
        for (var i = 0; i < this.sortedProps.length; i++) {
            var prop = this.sortedProps[i].objProp;
            var control = this.controlList.key(prop.objPropId);
            control.initializeControl();
        }
        if (this.controller.initialized) {
            this.layoutsChanged(this.controller.layoutState);
        }
        else {
            controller.events.controllerInitialized(function (args) {
                _this.layoutsChanged(_this.controller.layoutState);
            });
        }
        // labels are next to controls but exclude tasks because on large screens they are opened in a small popup
        var allowLabelWidth = window.matchMedia("(min-width: 28em)").matches && !this.obj.isTask;
        // This looks a bit weird but we only implement label/fieldwidth when the table defaults are 100%
        // we change the widths after the enhance (because some jquery mobile controls create a new wrapper div and after the initializeControl call because we use the findField function.
        // Also, this might be better to do in the controlHelpers but they don't know the label.
        if (allowLabelWidth && (this.objView.formProps.defaultLabelWidth + this.objView.formProps.defaultFieldWidth) == 100) {
            for (var i = 0; i < this.controlList.length; i++) {
                var controlHelper = this.controlList.item(i);
                var prop = controlHelper.objProp;
                var field = controlHelper.findField();
                var labelWidth = controlHelper.viewProp.labelWidth, fieldWidth = controlHelper.viewProp.fieldWidth;
                var total = labelWidth + fieldWidth;
                if (total > 100) {
                    labelWidth = Math.round(labelWidth / total * 100);
                    fieldWidth = 100 - labelWidth;
                }
                if (controlHelper.viewProp.labelWidth !== 0) {
                    field.children("label").css("width", (labelWidth - 1) + "%");
                }
                // Not supported on checkboxes
                if (prop.controlId !== Enum.ControlType.CHECKBOX &&
                    !(prop.controlId === Enum.ControlType.SIGNATURE && controlHelper.viewProp.labelWidth === 0)) {
                    field.children(":eq(1)").css("width", (fieldWidth - 1) + "%");
                }
            }
        }
    };
    TSFForm.prototype.attachEditor = function () {
        if (this.mode == 0 /* View */) {
            this.mode = 2 /* Edit */;
        }
        this.controllerEditor = this.controller.currentEditor;
        this.controllerEditor.initializeComponentEditor(this);
        this.syncValuesFromController();
        this.layoutsChanged(this.controller.layoutState);
    };
    TSFForm.prototype.createFormEditor = function (mode) {
        var _this = this;
        // Invalid mode for editor
        if (mode == 0 /* View */) {
            return jQuery.Deferred().resolve().promise();
        }
        var oldMode = this.mode;
        if (this.mode != mode) {
            this.mode = mode;
        }
        if (this.controller.initialized) {
            return this.createEditorInternal(mode).fail(function () {
                _this.mode = oldMode;
                // reset form, because we need to go back the readonly
                _this.layoutsChanged(_this.controller.layoutState);
            });
        }
        else {
            var deferred_1 = jQuery.Deferred();
            this.controller.events.controllerInitialized(function () {
                _this.createEditorInternal(mode).then(function () {
                    deferred_1.resolve();
                }, function () {
                    _this.mode = oldMode;
                    // reset form, because we need to go back the readonly
                    _this.layoutsChanged(_this.controller.layoutState);
                    deferred_1.reject();
                });
            });
            return deferred_1.promise();
        }
    };
    TSFForm.prototype.createEditorInternal = function (mode) {
        var _this = this;
        var formEditor = new FormEditor(this.controller, this);
        // Controller will save automatically with the call to newRow when there was already an editor
        // TODO: change this, use some thing like an action structure.
        var controllerEditor = null;
        if (mode == 1 /* Add */) {
            controllerEditor = this.controller.newRow(formEditor);
        }
        else if (mode == 2 /* Edit */) {
            controllerEditor = this.controller.editRow(formEditor);
        }
        else {
            controllerEditor = this.controller.copyToNewRow(formEditor);
        }
        return controllerEditor.then(function (editor) {
            _this.controllerEditor = editor;
            _this.syncValuesFromController();
            if (!_this.obj.useLayouts) {
                _this.layoutsChanged(_this.controller.layoutState);
            }
            if (mode == 1 /* Add */) {
                // Scroll to the top of the form, but only when doing an add
                _this.formBody.parent().animate({ scrollTop: 0 });
            }
        });
    };
    TSFForm.prototype.invalidateData = function () {
        this.setValues(null);
    };
    TSFForm.prototype.setValues = function (row) {
        for (var i = 0; i < this.sortedProps.length; i++) {
            var prop = this.sortedProps[i].objProp;
            var control = this.controlList.key(prop.objPropId);
            if (row) {
                // TODO get value in the correct datatype.
                control.setValue(row.getValueForObjProp(prop));
            }
            else {
                control.setDefaultValue();
            }
        }
    };
    TSFForm.prototype.layoutsChanged = function (layoutState) {
        // We need to hide the group/tablabels when they are empty
        // So we create objects for them, we null it when we don't want to hide it
        var currentGroup = null;
        var currentTab = null;
        for (var i = 0; i < this.controlList.length; i++) {
            var control = this.controlList.item(i);
            var viewProp = control.viewProp;
            var newLayoutState = layoutState.fieldLayoutState[control.objProp.objPropId];
            if (this.mode == 0 /* View */ && newLayoutState > 2 /* ReadOnly */) {
                newLayoutState = 2 /* ReadOnly */;
            }
            else if (this.mode != 0 /* View */ && viewProp.formMaxAccessType != Enum.AccessType.Regular) {
                newLayoutState = viewProp.formMaxAccessType == Enum.AccessType.Hidden ? 0 /* Hidden */ : Math.min(2 /* ReadOnly */, newLayoutState);
            }
            if (viewProp.formMaxAccessType == Enum.AccessType.Hidden) {
                // When the maxAccess state is hidden we create special HiddenControlHelpers which are always invisible
                // but this still needs to have the right state here or the groups/tabs won't hide correctly
                // For example it might be a mandatory column but it might still be hidden in the form
                newLayoutState = 0 /* Hidden */;
            }
            if (viewProp.fieldOnNextTab && viewProp.tabTransl) {
                if (currentTab) {
                    currentTab.style.display = "none";
                }
                // Find the new tab
                currentTab = control.findField().closest(".form-tab").prev(".tab-separator")[0];
                if (currentTab && currentTab.style.display == "none") {
                    currentTab.style.display = "block";
                }
                // We started a new tab and there was still a group without items;
                // We should hide it
                if (currentGroup) {
                    currentGroup.style.display = "none";
                }
            }
            if (viewProp.fieldInNextGroup && viewProp.groupTransl) {
                if (currentGroup) {
                    currentGroup.style.display = "none";
                }
                // Find the new group
                currentGroup = control.findField().prev(".group")[0];
                if (currentGroup && currentGroup.style.display == "none") {
                    currentGroup.style.display = "block";
                }
            }
            if (newLayoutState !== 0 /* Hidden */) {
                // There are controls visible on the tab/group
                currentGroup = null;
                currentTab = null;
            }
            if (newLayoutState == 0 /* Hidden */) {
                control.setHidden(true, false);
            }
            else if (newLayoutState == 1 /* HiddenKeepSpace */) {
                control.setHidden(true, true);
            }
            else {
                control.setHidden(false, false);
                if (newLayoutState == 2 /* ReadOnly */) {
                    control.setReadonly(true);
                }
                else {
                    control.setReadonly(false);
                    control.setMandatory(newLayoutState == 4 /* Mandatory */);
                }
            }
        }
        // If the last column is an empty tab or group, we need to hide this too
        if (currentGroup) {
            currentGroup.style.display = "none";
        }
        if (currentTab) {
            currentTab.style.display = "none";
        }
    };
    TSFForm.prototype.synchronizeProp = function (prop) {
        var _this = this;
        var helper = this.controlList.key(prop.objPropId);
        if (helper && this.controllerEditor) {
            this.synchronizePropPromise = helper.validateValue().then(function () {
                var value = helper.getValue();
                // getValue doesn't return the File contents, but we need to make sure not to lose it
                // small note: the signature control does provide the file contents
                if (helper.objProp.isFileColumn && value && _this.controllerEditor.newRow[helper.objProp.objPropId]) {
                    var currentValue = _this.controllerEditor.newRow[helper.objProp.objPropId];
                    value.File = value.File || currentValue.File;
                    value.HashCode = currentValue.HashCode || -1;
                }
                return _this.controllerEditor.editDataRow(helper.objProp, value);
            });
            return this.synchronizePropPromise;
        }
        return jQuery.Deferred().resolve().promise();
    };
    TSFForm.prototype.waitForSynchronization = function () {
        return this.synchronizePropPromise;
    };
    TSFForm.prototype.allowFormbuttons = function () {
        // disable form buttons (lookup popup buttons) on task popups
        return !(this.obj.isTask && this.formBody.closest(".ui-popup").length > 0);
    };
    TSFForm.prototype.searchControl = function (element) {
        var controlId = jQuery(element).closest(".ui-field-contain").data('control-id');
        return (controlId ? this.controlList.key(controlId) : null);
    };
    TSFForm.prototype.dispose = function () {
        var controlList = this.controlList.array;
        for (var i = 0; i < controlList.length; i++) {
            controlList[i].dispose();
        }
    };
    return TSFForm;
}());
var FormControlPlaceHolder = /** @class */ (function () {
    function FormControlPlaceHolder(prop, currentPositionInCol) {
        this.prop = prop;
        this.positionsOfControl = 0;
        if (prop.formMaxAccessType != Enum.AccessType.Hidden)
            this.positionsOfControl = prop.fieldHeightInPositions;
        if (prop.groupTransl || prop.tabTransl) {
            this.positionsOfControl++;
        }
        this.currentPositionInCol = currentPositionInCol + this.positionsOfControl;
    }
    return FormControlPlaceHolder;
}());
var FormBuilder = /** @class */ (function () {
    function FormBuilder(form) {
        this.form = form;
    }
    FormBuilder.prototype.createTab = function (prop) {
        var div = document.createElement("div");
        div.className = "tab-separator";
        if (prop.tabIcon) {
            var img = document.createElement("img");
            img.src = Utils.createIconFromBase64(prop.tabIcon);
            div.appendChild(img);
        }
        if (prop.tabTransl) {
            div.appendChild(document.createTextNode(prop.tabTransl.transl));
        }
        return div;
    };
    FormBuilder.prototype.createGroup = function (prop) {
        var div = document.createElement("div");
        div.className = "group ui-field-contain";
        if (prop.groupIcon) {
            var img = document.createElement("img");
            img.src = Utils.createIconFromBase64(prop.groupIcon);
            div.appendChild(img);
        }
        div.appendChild(document.createTextNode(prop.groupTransl.transl));
        return div;
    };
    FormBuilder.prototype.renderControl = function (viewProp, controlHelper) {
        var controlField = document.createDocumentFragment();
        var fieldContain = document.createElement("div");
        fieldContain.setAttribute("data-control-id", viewProp.objPropId);
        fieldContain.className = "ui-field-contain";
        if (viewProp.hasLookup() && viewProp.objLinkLookupView.hasZoomButton && this.form.allowFormbuttons()) {
            fieldContain.className += " has-button";
        }
        // Create the label
        if (viewProp.labelWidth > 0 && viewProp.objProp.objPropTransl.translForm) {
            var label = document.createElement("label");
            var labelId = controlHelper.getLabelForControl();
            if (labelId) {
                label.setAttribute("for", labelId);
            }
            label.innerText = viewProp.objProp.objPropTransl.translForm.trim() + ":";
            fieldContain.appendChild(label);
        }
        else {
            // No label, we add a CSS class which will let it align with labels correctly.
            fieldContain.className += " no-label";
        }
        // Create the control.
        fieldContain.appendChild(this.createControl(viewProp, controlHelper));
        controlField.appendChild(fieldContain);
        return controlField;
    };
    FormBuilder.prototype.createControl = function (prop, controlHelper) {
        var rowSpan = prop.fieldHeightInPositions;
        var height = (this.form.rowHeight * rowSpan) + // Meta height
            (this.form.controlHeightBetweenFields * (rowSpan - 1)) + // For rowspan > 1: add control padding
            (this.form.controlBorderSize * (rowSpan)); // For rowspan > 1: add border size
        return controlHelper.createControl(height);
    };
    return FormBuilder;
}());
var FormEditor = /** @class */ (function () {
    function FormEditor(controller, form) {
        this.controller = controller;
        this.form = form;
    }
    FormEditor.prototype.suggestCursorIndex = function (objProp) {
        if (objProp === null) {
            return;
        }
        console.log("suggestCursorIndex: " + objProp.objPropTransl.transl);
        var helper = this.form.controlList.key(objProp.objPropId);
        if (helper) {
            helper.focus();
        }
    };
    FormEditor.prototype.removeControlEvents = function () {
        this.form.formBody
            .off("focus.formeditmode", "input, textarea, select, .k-editor")
            .off("blur.formeditmode change.formeditmode", "input[type!='file'], textarea, select, .signature-control, .k-editor")
            .off(".slider", "div.ui-flipswitch");
    };
    FormEditor.prototype.enter = function () {
        this.removeControlEvents();
        var that = this;
        this.form.formBody
            .on("focus.formeditmode", "input, textarea, select, .k-editor", function () {
            console.log('focus: ' + this.id);
            var helper = that.form.searchControl(this);
            if (helper) {
                that.currentFocusProp = helper.objProp;
            }
        }).on("blur.formeditmode change.formeditmode", "input[type!='file'], textarea, select, .signature-control, .k-editor", function (evt) {
            var helper = that.form.searchControl(this);
            // Check the isTrigger property, because setting the value with javascript also triggers this event.
            // We use a special case for flipswitch, so don't sync here.
            // Also combo controls are special because it is called multiple times, a change event with isTrigger == true and a normal change.
            // Make sure that combo's with not native menu's are not canceled.
            if ((this.tagName.toLowerCase() === "select" && evt.isTrigger !== undefined && !jQuery(this).is(":jqmData(native-menu=false)"))
                || jQuery(this).is(":jqmData(role=flipswitch)") || jQuery(this).hasClass("suggestion")) {
                return;
            }
            // The change event is fired a lot during typing for a date control causing weird problems with defaults
            // Only synchronize when blur is called
            if ((this.type == "date" || this.type == "time") && evt.type == "change") {
                return;
            }
            if (helper) {
                that.form.synchronizeProp(helper.objProp);
            }
        }).on("keydown.slider click.slider swipeleft.slider swiperight.slider", "div.ui-flipswitch", function (evt) {
            // Above events are used by the slider to change the value, they should always match!
            var helper = that.form.searchControl(this);
            if (helper) {
                that.form.synchronizeProp(helper.objProp);
            }
        });
    };
    FormEditor.prototype.leave = function () {
        this.removeControlEvents();
        this.form.mode = 0 /* View */;
        var controlList = this.form.controlList.array;
        for (var i = 0; i < controlList.length; i++) {
            controlList[i].setReadonly(true);
        }
        if (!this.form.obj.useLayouts) {
            this.form.layoutsChanged(this.controller.layoutState);
        }
    };
    Object.defineProperty(FormEditor.prototype, "isSynchronizingProp", {
        get: function () {
            return this.form.synchronizePropPromise.state() == "pending";
        },
        enumerable: true,
        configurable: true
    });
    FormEditor.prototype.lazyInitialize = function (form) {
        this.form = form;
        this.enter();
    };
    FormEditor.prototype.allowActions = function () {
        return this.controller.objView.formAutoSave;
    };
    FormEditor.prototype.isDefaultEditable = function () {
        return this.controller.objView.defaultEditableForm;
    };
    FormEditor.prototype.validateEditors = function () {
        var _this = this;
        var form = this.form;
        var controls = this.form.controlList;
        var lastPromise;
        // the synchronizeProp function makes sure that multiple changes are queued.
        // but when we are in validate editors there should only be one change actually
        for (var i = 0; i < controls.length; i++) {
            var helper = controls.item(i);
            // Don't sync identity columns (they can't change)
            if (!helper.objProp.identity) {
                lastPromise = form.synchronizeProp(helper.objProp);
            }
        }
        // When there aren't any controls there might not be a promise - but that is a bit of an edge case and having this wouldn't make any sense
        if (lastPromise) {
            var deferred_2 = jQuery.Deferred();
            lastPromise.then(function () {
                if (_this.controller.currentEditor && _this.controller.currentEditor.currentRowModified) {
                    _this.controller.saveCurrentRow().then(function () { return deferred_2.resolve(); }, function () { return deferred_2.reject(); });
                }
                else {
                    deferred_2.resolve();
                }
            }, function () { return deferred_2.reject(); });
            return deferred_2.promise();
        }
        else {
            return jQuery.Deferred().resolve().promise();
        }
    };
    return FormEditor;
}());
var TaskFormEditor = /** @class */ (function () {
    function TaskFormEditor(taskcontroller) {
        this.taskcontroller = taskcontroller;
    }
    Object.defineProperty(TaskFormEditor.prototype, "isSynchronizingProp", {
        get: function () {
            if (this.formEditor) {
                return this.formEditor.isSynchronizingProp;
            }
            return false;
        },
        enumerable: true,
        configurable: true
    });
    TaskFormEditor.prototype.suggestCursorIndex = function (objProp) {
        if (this.formEditor) {
            this.formEditor.suggestCursorIndex(objProp);
        }
    };
    TaskFormEditor.prototype.enter = function () {
        if (this.formEditor) {
            return this.formEditor.enter();
        }
    };
    TaskFormEditor.prototype.leave = function () {
        if (this.formEditor) {
            this.formEditor.leave();
        }
    };
    TaskFormEditor.prototype.lazyInitialize = function (form) {
        this.formEditor = new FormEditor(this.taskcontroller, form);
        this.formEditor.enter();
    };
    TaskFormEditor.prototype.allowActions = function () {
        return false;
    };
    TaskFormEditor.prototype.isDefaultEditable = function () {
        return false;
    };
    TaskFormEditor.prototype.validateEditors = function () {
        if (this.formEditor) {
            return this.formEditor.validateEditors();
        }
        return jQuery.Deferred().reject();
    };
    return TaskFormEditor;
}());
