var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var Grid = /** @class */ (function () {
    function Grid(controller, url) {
        this.controller = controller;
        this.columns = [];
        this.electronCombinedFilterDebounce = 300;
        this.mobileCombinedFilterDebounce = 500;
        this.screenLink = "abstractscreen.html" + url.search;
    }
    Grid.prototype.getEditLink = function (mode) {
        return this.screenLink + "&mode=" + mode;
    };
    Grid.prototype.setRowActive = function (parentElement, rowIndex) {
        jQuery(parentElement).children(".active-row").removeClass("active-row");
        if (rowIndex >= 0 && rowIndex < parentElement.childNodes.length) {
            jQuery(parentElement.childNodes[rowIndex]).addClass("active-row");
        }
    };
    Grid.prototype.allowDifferentWidthImages = function () {
        return false;
    };
    Grid.prototype.createColumn = function (viewProp, headerCol) {
        var objProp = viewProp.objProp;
        var controlId = objProp.controlId;
        if (viewProp.isTranslatedLookup()) {
            return new GridLookupColumn(this.controller, viewProp, headerCol);
        }
        else if (controlId == Enum.ControlType.DATE || controlId == Enum.ControlType.DATETIME || controlId == Enum.ControlType.TIME) {
            return new GridDateColumn(viewProp, headerCol);
        }
        else if (controlId == Enum.ControlType.IMAGE_LINK || controlId == Enum.ControlType.IMAGE_BLOB || controlId == Enum.ControlType.IMAGE_UPLOAD || controlId == Enum.ControlType.SIGNATURE) {
            var imageColumn = new GridImageColumn(viewProp);
            if (this.allowDifferentWidthImages()) {
                imageColumn.imageWidth = viewProp.gridColumnWidth || 60;
            }
            return imageColumn;
        }
        else if (controlId == Enum.ControlType.CHECKBOX) {
            return new GridCheckboxColumn(viewProp, headerCol);
        }
        else if (objProp.domain) {
            return new GridDomainColumn(viewProp, headerCol);
        }
        else if (controlId == Enum.ControlType.NUMERIC || objProp.xmlDttp === "double") {
            return new GridNumberColumn(this.controller.application.language, viewProp, headerCol);
        }
        else {
            return new GridColumn(viewProp, headerCol);
        }
    };
    Grid.prototype.invalidateData = function () {
    };
    Grid.prototype.preventClickOnMouseMove = function (element, eventSelector) {
        var startPostion;
        var ele = jQuery(element);
        ele.on("mousedown.preventClick", eventSelector, function (evt) {
            startPostion = { x: evt.pageX, y: evt.pageY };
        });
        ele.on("click.preventClick", eventSelector, function (evt) {
            if (!startPostion)
                return; // On touch devices mousedown might not fire.
            var margin = 10; // Allowed margin without preventing the click
            var x = evt.pageX - startPostion.x, y = evt.pageY - startPostion.y;
            if (Math.abs(x - margin) > margin || Math.abs(y - margin) > margin) {
                evt.preventDefault();
                evt.stopPropagation();
            }
            startPostion = null;
        });
    };
    Grid.prototype.getCombinedFilterSearchTerm = function () {
        return this.controller.getCombinedFilterSearchTerm();
    };
    Grid.prototype.createSearchForm = function () {
        var _this = this;
        var searchForm = document.createElement("form");
        searchForm.className = "ui-filterable";
        var searchDiv = document.createElement("div");
        searchDiv.className = "ui-input-search ui-body-inherit ui-input-has-clear";
        var searchInput = document.createElement("input");
        searchInput.type = "text";
        searchInput.setAttribute("data-type", "search");
        searchInput.setAttribute("data-enhanced", "true");
        //If there's combinedFilterConditions in the controller, use the first combinedFilterCondition
        //that's a contains filter value as searchInput value.
        var combinedFilterConditions = this.controller.queryForTasks.whereClause.combinedFilterConditions;
        if (combinedFilterConditions.length > 0) {
            var containsConditions = combinedFilterConditions.filter(function (condition) {
                return condition.comparision === Enum.FilterCondition.Contains;
            });
            if (containsConditions.length > 0) {
                searchInput.value = containsConditions[0].dbFilterValue;
            }
        }
        searchInput.placeholder = this.controller.application.language.transl(Model.TranslKeys.combinedFilter);
        searchDiv.appendChild(searchInput);
        var clearBtn = document.createElement("a");
        clearBtn.tabIndex = -1;
        clearBtn.className = "ui-input-clear ui-btn ui-icon-delete ui-btn-icon-notext";
        searchInput.value === "" ? clearBtn.className += " ui-input-clear-hidden" : null;
        clearBtn.onclick = function () {
            searchInput.value = "";
            onSearchInputChanged();
        };
        var debounceDelay = Utils.isElectron() ? this.electronCombinedFilterDebounce : this.mobileCombinedFilterDebounce;
        var debouncedOnSearchInputChanged = Utils.debounce(onSearchInputChanged, debounceDelay, false);
        $(searchInput).on("input", function () {
            debouncedOnSearchInputChanged();
        });
        searchForm.onsubmit = function (e) {
            onSearchInputChanged();
            e.preventDefault();
            return false;
        };
        var previousInput;
        var that = this;
        function onSearchInputChanged() {
            var searchInputValue = searchInput.value;
            if (searchInputValue === previousInput) {
                return;
            }
            that.controller.performCombinedFilter(searchInputValue, that.getAdditionalFilterColumns());
            //Hide or show the clear button depending on searchInputValue empty or not
            searchInputValue === "" ? clearBtn.style.display = "none" : clearBtn.style.display = "block";
            previousInput = searchInputValue;
        }
        searchDiv.appendChild(clearBtn);
        searchForm.appendChild(searchDiv);
        this.controller.whenInitialized(function () {
            // issue 42276
            if (_this.controller.isStartEmptyWithFilterActive) {
                if (typeof Keyboard !== 'undefined') {
                    Keyboard.show();
                }
                _this.pageShown.always(function () {
                    searchInput.focus();
                });
            }
        });
        return searchForm;
    };
    Grid.prototype.flushCombinedFilter = function () {
        var searchInput = this.getSearchInput();
        // When the combinedFilter input is the activeElement make sure to flush it filter
        // We need this because clicking on a row (or its form button) first clicks and blur is fired later
        // our buttons don't get focus so blur isn't because of focus
        // it is fired because jQuery mobile blurs the active element in a transition
        if (searchInput[0] == document.activeElement) {
            searchInput.blur();
        }
    };
    Grid.prototype.dispose = function () {
    };
    return Grid;
}());
var GridColumn = /** @class */ (function () {
    function GridColumn(viewProp, headerCol) {
        this.viewProp = viewProp;
        this.headerCol = headerCol;
        this.objProp = viewProp.objProp;
    }
    GridColumn.prototype.getDisplayValue = function (row) {
        if (this.objProp.isFileColumn) {
            var file = row.getFile(this.objProp.objPropId);
            if (file) {
                return file.FileName;
            }
        }
        else {
            // Quick exit for empty values.
            var value = row.getString(this.objProp.objPropId);
            if (Utils.isEmpty(value)) {
                return null;
            }
            // Display floats according to their precision
            if (this.objProp.dttpId === Model.DttpId.float) {
                var floatValue = parseFloat(value);
                if (!isNaN(floatValue)) {
                    return floatValue.toFixed(this.objProp.getPrecision());
                }
            }
            // Fall back to the string value.
            return value;
        }
        return null;
    };
    GridColumn.prototype.getElement = function (row, rowIndex, createLabel) {
        if (createLabel === void 0) { createLabel = false; }
        var displayValue = this.getDisplayValue(row);
        var text = Utils.isEmpty(displayValue) ? " " : displayValue;
        if (this.viewProp.cardListLabel && createLabel) {
            var objPropTransl = this.viewProp.objProp.objPropTransl.translForm;
            if (!Utils.isEmpty(objPropTransl)) {
                text = objPropTransl.trim() + ": " + text;
            }
        }
        if (this.headerCol) {
            return this.createElement("h2", text);
        }
        else {
            return this.createElement("p", text);
        }
    };
    GridColumn.prototype.createElement = function (elementName, value) {
        var item = document.createElement(elementName);
        if (!value) {
            item.innerHTML = "&nbsp;";
        }
        else {
            item.innerText = value;
        }
        return item;
    };
    return GridColumn;
}());
var GridDomainColumn = /** @class */ (function (_super) {
    __extends(GridDomainColumn, _super);
    function GridDomainColumn(viewProp, headerCol) {
        var _this = _super.call(this, viewProp, headerCol) || this;
        _this.elements = _this.objProp.domain.elements.array;
        return _this;
    }
    GridDomainColumn.prototype.getDisplayValue = function (row) {
        // Quick exit for empty values.
        var value = row.getString(this.objProp.objPropId);
        if (Utils.isEmpty(value)) {
            return null;
        }
        // Try to find a matching element and use its translation.
        for (var i = 0; i < this.elements.length; i++) {
            var ele = this.elements[i];
            if (ele.value === "" + value) {
                return ele.elementTransl.transl;
            }
        }
        // Fall back to the string value.
        return value;
    };
    GridDomainColumn.prototype.getElement = function (row, rowIndex) {
        if (this.objProp.controlId === Enum.ControlType.IMAGE_COMBO
            && this.objProp.domain) {
            var imageWrapper = document.createElement("div");
            imageWrapper.className = "cardlist-image-wrapper";
            var imageElement = document.createElement("img");
            var value = row.getString(this.objProp.objPropId);
            for (var i = 0; i < this.elements.length; i++) {
                var ele = this.elements[i];
                if (ele.value == value) {
                    var picture = ele.picture;
                    imageElement.src = (picture ? Utils.createIconFromBase64(picture) : "img/photo.png");
                    break;
                }
            }
            imageElement.setAttribute("data-row-index", rowIndex + "");
            imageElement.setAttribute("data-prop", this.objProp.objPropId);
            imageElement.className = "verticalAlignedImage";
            imageElement.removeAttribute("width");
            imageElement.removeAttribute("height");
            imageWrapper.appendChild(imageElement);
            return imageWrapper;
        }
        else {
            return _super.prototype.getElement.call(this, row, rowIndex);
        }
    };
    return GridDomainColumn;
}(GridColumn));
var GridCheckboxColumn = /** @class */ (function (_super) {
    __extends(GridCheckboxColumn, _super);
    function GridCheckboxColumn(viewProp, headerCol) {
        var _this = _super.call(this, viewProp, headerCol) || this;
        var domain = _this.objProp.domain;
        // check xmlDttp, domain.dttp isn't normalized
        _this.useTransls = domain != null && domain.elements.length == 2;
        if (_this.useTransls) {
            _this.elements = domain.elements.array;
        }
        return _this;
    }
    GridCheckboxColumn.prototype.getDisplayValue = function (row) {
        // Quick exit for empty values.
        var value = row.getString(this.objProp.objPropId);
        if (Utils.isEmpty(value)) {
            return null;
        }
        // If the string value cannot be converted to boolean, then return the string value.
        var boolValue = Utils.convertToBoolean(value, undefined);
        if (Utils.isEmpty(boolValue)) {
            return value;
        }
        // Try to find a matching element and use its translation.
        if (this.useTransls) {
            for (var i = 0; i < this.elements.length; i++) {
                if (Utils.convertToBoolean(this.elements[i].value, null) === boolValue) {
                    return this.elements[i].elementTransl.transl;
                }
            }
            // Use the language translations for yes and no.
        }
        else {
            var language = void 0;
            if (Model.TSFApplicationManager.Current.activeAppl) {
                language = Model.TSFApplicationManager.Current.activeAppl.language;
            }
            else {
                language = AppConfig.language;
            }
            if (boolValue === true) {
                return language.transl(Model.TranslKeys.yes);
            }
            else if (boolValue === false) {
                return language.transl(Model.TranslKeys.no);
            }
        }
        // Fall back to the string value.
        return value;
    };
    return GridCheckboxColumn;
}(GridColumn));
var GridDateColumn = /** @class */ (function (_super) {
    __extends(GridDateColumn, _super);
    function GridDateColumn(viewProp, headerCol) {
        var _this = _super.call(this, viewProp, headerCol) || this;
        switch (_this.objProp.controlId) {
            case Enum.ControlType.DATE:
                _this.dateFormatString = "DD-MM-YYYY";
                break;
            case Enum.ControlType.TIME:
                _this.dateFormatString = "HH:mm:ss";
                break;
            default:
                _this.dateFormatString = "DD-MM-YYYY HH:mm";
                break;
        }
        return _this;
    }
    GridDateColumn.prototype.getDisplayValue = function (row) {
        var value = row.getValueForObjProp(this.objProp);
        if (!Utils.isEmpty(value)) {
            return moment(value).format(this.dateFormatString);
        }
        return null;
    };
    return GridDateColumn;
}(GridColumn));
var GridImageColumn = /** @class */ (function (_super) {
    __extends(GridImageColumn, _super);
    function GridImageColumn(viewProp) {
        var _this = 
        // An image column should never have a h2 element and therefore is never marked as an HeaderColumn
        _super.call(this, viewProp, false) || this;
        _this.imageHeight = 80;
        _this.imageWidth = viewProp.gridColumnWidth !== null ? viewProp.gridColumnWidth : 60;
        // Correct height/width with the devicePixelRatio so the images are sharp again
        if (typeof window.devicePixelRatio !== "undefined" && window.devicePixelRatio !== 1) {
            _this.imageHeight = Math.round(_this.imageHeight * window.devicePixelRatio);
            _this.imageWidth = Math.round(_this.imageWidth * window.devicePixelRatio);
        }
        return _this;
    }
    GridImageColumn.prototype.getElement = function (row, rowIndex) {
        var imageWrapper = document.createElement("div");
        imageWrapper.className = "cardlist-image-wrapper";
        var imageElement = document.createElement("img");
        imageElement.src = "img/photo.png";
        imageElement.setAttribute("data-row-index", rowIndex + "");
        imageElement.setAttribute("data-prop", this.objProp.objPropId);
        imageElement.className = "verticalAlignedImage";
        imageElement.removeAttribute("width");
        imageElement.removeAttribute("height");
        imageWrapper.appendChild(imageElement);
        return imageWrapper;
    };
    return GridImageColumn;
}(GridColumn));
var GridNumberColumn = /** @class */ (function (_super) {
    __extends(GridNumberColumn, _super);
    function GridNumberColumn(language, viewProp, headerCol) {
        var _this = _super.call(this, viewProp, headerCol) || this;
        _this.language = language;
        _this.showGroupSeparator = Utils.allowGroupSeparator(viewProp.objProp.controlId);
        return _this;
    }
    GridNumberColumn.prototype.getDisplayValue = function (row) {
        // Quick exit for empty values.
        var value = row.getString(this.objProp.objPropId);
        if (Utils.isEmpty(value)) {
            return null;
        }
        // If the value is a valid float value, then it will be shown in the configured format.
        var floatValue = parseFloat(value);
        if (!isNaN(floatValue)) {
            return Utils.formatNumber(floatValue, this.objProp.precision, this.language, this.showGroupSeparator);
        }
        return null;
    };
    return GridNumberColumn;
}(GridColumn));
var GridLookupColumn = /** @class */ (function (_super) {
    __extends(GridLookupColumn, _super);
    function GridLookupColumn(controller, viewProp, headerCol) {
        var _this = _super.call(this, viewProp, headerCol) || this;
        _this.viewProp = viewProp;
        _this.headerCol = headerCol;
        _this.lookupContainer = new GridLookupContainer(controller, _this.viewProp);
        return _this;
    }
    GridLookupColumn.prototype.getDisplayValue = function (row) {
        var values = [];
        var filters = this.lookupContainer.viewProp.lookup.filters.map(function (filter) { return filter.targetProp.objPropId; });
        filters.push(this.lookupContainer.lookupContainer.getDataFieldProp(0 /* TableReader */).objPropId);
        filters.forEach(function (filter) { return values.push(row.getString(filter)); });
        return this.lookupContainer.getTranslation(values, filters, row);
    };
    return GridLookupColumn;
}(GridColumn));
var GridLookupContainer = /** @class */ (function () {
    function GridLookupContainer(controller, viewProp) {
        this.controller = controller;
        this.viewProp = viewProp;
        this.lookupContainer = controller.getLookupSet(viewProp);
    }
    /*
     * This function requests all lookup data (asynchronous)
     */
    GridLookupContainer.prototype.prepare = function (data) {
        var _this = this;
        return this.lookupContainer.translateReaderToDisplayValues(data, 0 /* TableReader */).always(function (data) {
            _this.data = data;
        });
    };
    GridLookupContainer.prototype.getTranslation = function (value, filters, contextRow) {
        if (this.data) {
            var lookupContainer = this.lookupContainer;
            return TSFDataLookupContainer.getTranslFromTranslatedReaderUtil(this.data, value, contextRow, filters, lookupContainer.getListFieldAlias(0 /* TableReader */));
        }
        return "";
    };
    return GridLookupContainer;
}());
