var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var ListView = /** @class */ (function (_super) {
    __extends(ListView, _super);
    function ListView(application, controller, url, contentElement, hasForm) {
        var _this = _super.call(this, controller, url) || this;
        _this.contentElement = contentElement;
        _this.hasForm = hasForm;
        _this.presentationCols = [];
        // Note: .slice creates a copy because sort changes the original array.
        // Note: for now we use .call, because we also index props on id, so some function might get overwritten.
        var props = controller.objView.objViewProps.array;
        props = Array.prototype.slice.call(props, 0, props.length);
        props = Array.prototype.filter.call(props, function (value, index, array) {
            // Exclude hidden columns
            return value.cardListTypeOfCol != Enum.AccessType.Hidden;
        });
        _this.sortedProps = Array.prototype.sort.call(props, function (a, b) { return a.cardListOrderNo - b.cardListOrderNo; });
        _this.createColumns();
        return _this;
    }
    ListView.prototype.initialize = function () {
        var _this = this;
        jQuery(this.contentElement).addClass('grid');
        this.createListView(this.contentElement);
        var $listview = jQuery(this.listview);
        $listview.listview();
        var that = this;
        this.preventClickOnMouseMove($listview[0], ">li a");
        //Only enable doubleclick if the screen contains only a listview. TCP 57997.
        if (!this.contentElement.classList.contains("panel1")) {
            $listview.on("dblclick doubletap", ">li a", function () {
                if (that.controller.interfaceAvailable) {
                    jQuery(this).parent().find(".goToForm").click();
                }
            });
        }
        $listview.on("click", ">li a", function (event) {
            var $this = jQuery(this);
            // Do not change the index when the click is prevented (by preventClickOnMouseMove)
            if (event.isDefaultPrevented()) {
                return false;
            }
            // Don't allow switching to the form when the interface shouldn't be available yet
            // As that might cause some weird issues - for example when you use the filter on the listview
            // and it is slow - you can click a row and switch to the form - but in the background the new data is loaded
            // which will in turn change the underlying data (and the user is now on a different row then he expected)
            that.flushCombinedFilter();
            if (!that.controller.interfaceAvailable && $this.hasClass("goToForm")) {
                return false;
            }
            if ($this.hasClass("moreDataAvailable")) {
                if (that.controller.interfaceAvailable) {
                    that.controller.currentAction = that.controller.nextPage();
                }
            }
            else {
                that.controller.currentAction = that.controller.setRowIndex($this.parent().index());
            }
            // On phones navigate to the page when clicked, prevent new clicks
            if (that.hasForm && $this.hasClass("goToForm")) {
                $listview.off("click", ">li a").on("click", ">li a", function (event) {
                    event.preventDefault();
                });
            }
            else {
                // Block navigation to a new page (listview-forms screen)
                return false;
            }
        });
        $listview.on('swiperight', "li", function (e) {
            // Disable delete in editmode or when the layoutstate doesn't allow it or when swiping over the more items avilable text.
            if (that.controller.isInEditMode || that.controller.layoutState.deleteRowAccessType != Enum.AccessType.Regular || e.swipestart.coords[0] < 50
                || jQuery(this).hasClass("moreDataAvailable"))
                return;
            that.controller.currentAction = that.controller.setRowIndex(jQuery(this).index());
            // Remove the old visible button
            $listview.children("li.showDeleteButton").removeClass('showDeleteButton')
                .children(".deleteWrapper").remove();
            var li = jQuery(this).addClass('showDeleteButton');
            // TODO: JQM 1.4, buttonMarkup is deprecated.
            var translation = that.controller.application.language.transl(Model.TranslKeys.Delete);
            var wrapper = jQuery("<div class='deleteWrapper'><button class='deleteButton'>Delete</button></div>"), button = wrapper.children('.deleteButton').text(translation).buttonMarkup({ inline: true });
            li.append(wrapper);
            window.setTimeout(function () {
                button.addClass('animated');
            }, 0);
            $listview.off("click", ".deleteButton").on("click", ".deleteButton", function () {
                that.controller.deleteRowWithConfirm();
            });
            var removeDeleteButton = function () {
                wrapper.remove();
                li.removeClass("showDeleteButton");
            };
            jQuery(document).one("tap", function () {
                removeDeleteButton();
            });
            $listview.one("click", ">li a", function () {
                removeDeleteButton();
            });
        });
        $listview.on('dragstart', "li a", function (evt) {
            evt.preventDefault(); // Prevent dragstart, so the swipe event also works on desktop browsers.
        });
        if (this.controller.initialized) {
            this.controller.getData().done(function (reader, status) {
                // It is possible that the user already moved away from this page
                // Don't rebuild the listview when that happens (or it will crash on the listview('refresh') call
                if (jQuery(_this.listview).data("mobile-listview")) {
                    _this.rebuild(reader);
                }
            }).fail(function (status) {
                $listview.empty();
            });
        }
        else {
            // When opening lookups the filter might be added by the controller
            this.controller.whenInitialized(function () {
                var searchTerm = _this.controller.getCombinedFilterSearchTerm();
                if (searchTerm) {
                    _this.getSearchInput().val(searchTerm)
                        .next("a.ui-input-clear-hidden").removeClass("ui-input-clear-hidden");
                }
            });
        }
        this.controller.events.rowSaved(function (args) {
            if (_this.controller.currentEditor && !_this.controller.currentEditor.isCreatingNewRow()) {
                // The user is editing a row, so update the value in the grid
                // Don't do that when creating a new row because the row isn't present in the listview
                var index = _this.controller.getRowIndex();
                _this.updateRow(index, args.values);
            }
        });
        this.controller.events.datasetChanged(function (args) {
            if (args.pageIndex !== -1) {
                var moreDataItem = _this.listview.lastChild;
                if (jQuery(moreDataItem).hasClass("moreDataAvailable")) {
                    _this.listview.removeChild(moreDataItem);
                }
                _this.appendGridData(args.dataset, args.pageIndex * _this.controller.pageSize);
            }
            else {
                _this.rebuild(args.dataset);
            }
        });
        this.controller.events.rowIndexChanged(function () {
            _this.setRowActive(_this.listview, _this.controller.getRowIndex());
        });
        this.controller.events.editmodeChanged(function (args) {
            var readOnly = _this.controller.isInEditMode && !_this.controller.currentEditor.allowActions();
            var input = _this.getSearchInput().prop("readonly", readOnly);
            input.parent().toggleClass("readonly-filter", readOnly);
        });
    };
    ListView.prototype.createListView = function (contentElement) {
        var _this = this;
        var fragment = document.createDocumentFragment();
        var searchForm = this.createSearchForm();
        var listview = this.listview = document.createElement("ul");
        listview.setAttribute("data-role", "listview");
        listview.setAttribute("data-split-icon", "bars");
        var scrollDiv = document.createElement("div");
        scrollDiv.className = "listview-scroll";
        // Use a scroll wrapper because listview creates a new element for the filterbox.
        scrollDiv.appendChild(listview);
        var headerWrapper = document.createElement("div");
        headerWrapper.className = "list-header-wrapper";
        contentElement.appendChild(headerWrapper);
        headerWrapper.appendChild(searchForm);
        fragment.appendChild(scrollDiv);
        contentElement.appendChild(fragment);
        var controller = this.controller;
        if (TSFScroll.useNativeScroll()) {
            this.scrollBar = new TSFNativeScroll(scrollDiv, {
                threshold: 250,
                onScrollMove: function () {
                    if ((scrollDiv.scrollHeight - scrollDiv.clientHeight) == scrollDiv.scrollTop
                        && controller.interfaceAvailable) {
                        controller.currentAction = controller.nextPage();
                    }
                }
            });
            jQuery(contentElement).on("tsf-updatelayout", function (evt, oldHeight, height, firstTime) {
                if (firstTime) {
                    // We return when it is the first time, the code causes some flickering on iOS and it isn't needed the first time
                    return;
                }
                if (scrollDiv.clientHeight < jQuery(scrollDiv).children().outerHeight()) {
                    var $scrollDiv_1 = jQuery(scrollDiv).css('overflow', 'hidden');
                    window.setTimeout(function () {
                        $scrollDiv_1.css('overflow', 'auto');
                    }, 0);
                }
            });
        }
        else {
            // Only enable handle click for Chrome on Android.
            // Added extra check for electron and ontouchstart - handleClick should be true or clicking a item won't work with touch
            var matches = /Android.*Chrome\/(\d+)/.exec(window.navigator.userAgent);
            var handleClick = matches && parseInt(matches[1], 10) >= 30 || (Utils.isElectron() && 'ontouchstart' in window);
            this.scrollBar = new iScroll(scrollDiv, {
                onScrollMove: function (event) {
                    var scrollElement = jQuery(this.wrapper);
                    if (this.y < (this.maxScrollY - 5) && !scrollElement.hasClass('nextPage')) {
                        scrollElement.addClass('nextPage');
                        this.maxScrollY = this.maxScrollY;
                    }
                    else if (this.y > (this.maxScrollY + 5) && scrollElement.hasClass('nextPage')) {
                        scrollElement.removeClass('nextPage');
                    }
                },
                onScrollEnd: function (event) {
                    var scrollElement = jQuery(this.wrapper);
                    if (scrollElement.hasClass('nextPage')) {
                        if (controller.interfaceAvailable) {
                            controller.currentAction = controller.nextPage();
                        }
                        scrollElement.removeClass('nextPage');
                    }
                },
                handleClick: handleClick
            });
            jQuery(contentElement).on("tsf-updatelayout", function () {
                _this.scrollBar.refresh();
            });
        }
    };
    ListView.prototype.allowStartEmptyWithFilter = function () {
        // we always have a search input here so this is always allowed
        return true;
    };
    ListView.prototype.getAdditionalFilterColumns = function () {
        return this.presentationCols;
    };
    ListView.prototype.getSearchInput = function () {
        return jQuery(this.listview).closest(".grid").find("> form input");
    };
    ListView.prototype.createColumns = function () {
        var objView = this.controller.objView;
        var titleViewProp;
        // Use the lookup display column when the card list title should be deduced from the lookup settings.
        if (objView.cardListTitle == 2 /* LookupDisplay */) {
            titleViewProp = objView.objViewProps.object[objView.obj.lookupDisplay.lookUpDisplayColId];
            if (titleViewProp) {
                this.columns.push(this.createColumn(titleViewProp, true));
                this.presentationCols.push(titleViewProp.objPropId);
            }
        }
        var imageColumn;
        for (var i = 0; i < this.sortedProps.length; i++) {
            var sortProp = this.sortedProps[i];
            var controlType = sortProp.objProp.controlId;
            if (controlType == Enum.ControlType.IMAGE_LINK ||
                controlType == Enum.ControlType.IMAGE_BLOB ||
                controlType == Enum.ControlType.IMAGE_UPLOAD ||
                controlType == Enum.ControlType.IMAGE_COMBO) {
                // Set the imageColumn if it isn't set yet.
                if (!imageColumn && objView.cardListImage == 1 /* FirstVisibleImage */) {
                    if (sortProp.objProp.domain) {
                        imageColumn = new GridDomainColumn(sortProp, false);
                    }
                    else {
                        imageColumn = new GridImageColumn(sortProp);
                    }
                }
            }
            else if (!titleViewProp || sortProp.objPropId !== titleViewProp.objPropId) {
                // Only use the column if it isn't used as title yet.
                this.columns.push(this.createColumn(sortProp, this.columns.length === 0));
                this.presentationCols.push(sortProp.objPropId);
            }
        }
        if (imageColumn) {
            this.columns.unshift(imageColumn);
        }
    };
    ListView.prototype.invalidateData = function () {
        var $listview = jQuery(this.listview);
        $listview.empty();
    };
    ListView.prototype.appendGridData = function (reader, rowBaseIndex) {
        var _this = this;
        if (rowBaseIndex === void 0) { rowBaseIndex = 0; }
        var buildObject = this.rebuildObject = new Object();
        var lookupList = [];
        for (var i = 0; i < this.columns.length; i++) {
            if (this.columns[i].lookupContainer) {
                lookupList.push(this.columns[i].lookupContainer.prepare(reader));
            }
        }
        Utils.whenAll(lookupList).then(function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            if (!jQuery(_this.listview).data("mobile-listview")) {
                // When a user presses refresh and quickly presses back, datasetchanged event might have been fired so lookups are getting loaded.
                // But the listview is already destroyed
                return;
            }
            if (buildObject == _this.rebuildObject) {
                _this.appendGridDataInternal(reader, rowBaseIndex);
                _this.getSearchInput().parent().removeClass("inprogress");
            }
        });
    };
    ListView.prototype.appendGridDataInternal = function (reader, rowBaseIndex) {
        var _this = this;
        var listview = jQuery(this.listview);
        var fragment = document.createDocumentFragment();
        var formlink = this.getEditLink(0 /* View */);
        var columns = this.columns;
        // According to jsperf, cloning the node is better for performance than recreating it
        // http://jsperf.com/clonenode-vs-createelement-performance/7s
        var gotoForm = document.createElement("a");
        gotoForm.href = formlink;
        gotoForm.className = "goToForm";
        gotoForm.setAttribute("data-icon", "carat-r");
        var containsImage = this.columns[0] instanceof GridImageColumn || this.columns[0] instanceof GridDomainColumn;
        for (var i = 0, length_1 = reader.length; i < length_1; i++) {
            var item = document.createElement("li");
            // JQM 1.4 adds ui-li-has-thumb automatically, but thats deprecated in JQM 1.5 for performance reasons.
            if (containsImage) {
                item.className = "ui-li-has-thumb";
            }
            var itemReader = reader.item(i);
            var anchor = document.createElement("a");
            anchor.href = "#";
            anchor.className = "listitem";
            var textWrapper = document.createElement("div");
            textWrapper.className = "ui-listview-textwrapper";
            for (var j = 0; j < columns.length; j++) {
                //Put the icon inside the anchor and the rest in the textwrapper
                if (j === 0 && containsImage) {
                    anchor.appendChild(columns[j].getElement(itemReader, rowBaseIndex + i, true));
                }
                else {
                    textWrapper.appendChild(columns[j].getElement(itemReader, rowBaseIndex + i, true));
                }
            }
            anchor.appendChild(textWrapper);
            item.appendChild(anchor);
            if (this.hasForm) {
                item.appendChild(gotoForm.cloneNode(false));
            }
            else {
                item.setAttribute("data-icon", "false");
            }
            fragment.appendChild(item);
        }
        if (this.controller.hasNextPage()) {
            var language = this.controller.application.language;
            var moreDataElement = this.createInformationNode(language.transl(Model.TranslKeys.moreItemsAvailable), "moreDataAvailable");
            fragment.appendChild(moreDataElement);
        }
        listview.append(fragment);
        listview.listview('refresh');
        this.loadImages(listview, reader, rowBaseIndex);
        var rowIndex = this.controller.getRowIndex();
        this.setRowActive(this.listview, rowIndex);
        // Wait with scrolling until the page is shown.
        this.pageShown.then(function () {
            _this.scrollBar.refresh();
            // Scroll to the active row.
            if (rowIndex >= 0 && rowIndex < _this.listview.childNodes.length) {
                var activeRowYPos = jQuery(_this.listview.childNodes[rowIndex]).offset().top;
                var scrollHeight = jQuery(".listview-scroll").height();
                if (activeRowYPos > scrollHeight) {
                    // Substract the ScrollHeight from the ActiveRowYPos to determine the "scroll distance".
                    // Negate it because iScroll wants it negative.
                    _this.scrollBar.scrollTo(0, -(activeRowYPos - scrollHeight), null, null);
                }
            }
        });
    };
    ListView.prototype.updateRow = function (rowIndex, row) {
        var anchor = jQuery(this.listview).find("li:eq(" + rowIndex + ")>a:eq(0)");
        var textWrapper = document.createElement("div");
        var containsImage = this.columns[0] instanceof GridImageColumn || this.columns[0] instanceof GridDomainColumn;
        textWrapper.className = "ui-listview-textwrapper";
        anchor.empty();
        for (var i = 0, length_2 = this.columns.length; i < length_2; i++) {
            //Put the icon inside the anchor and the rest in the textwrapper
            if (i === 0 && containsImage) {
                anchor.append(this.columns[i].getElement(row, rowIndex));
            }
            else {
                textWrapper.appendChild(this.columns[i].getElement(row, rowIndex));
            }
            anchor.append(textWrapper);
        }
        if (this.columns[0] instanceof GridImageColumn) {
            var imageElement = anchor.find(">img")[0];
            if (imageElement) {
                this.loadImage(imageElement, row, this.columns[0]);
            }
        }
    };
    ListView.prototype.createInformationNode = function (text, className) {
        var _this = this;
        var element = document.createElement("li");
        element.className = className;
        var elementHeader = document.createElement("h2");
        elementHeader.textContent = text;
        element.appendChild(elementHeader);
        element.onclick = function () {
            _this.controller.nextPage();
        };
        return element;
    };
    ListView.prototype.rebuild = function (reader) {
        var $listview = jQuery(this.listview);
        $listview.empty();
        if (this.controller.isStartEmptyWithFilterActive) {
            var language = this.controller.application.language;
            var filterMessageNode = this.createInformationNode(language.transl(Model.TranslKeys.enterSearchTerm), "filter-message");
            this.listview.appendChild(filterMessageNode);
            $listview.listview("refresh");
        }
        else {
            this.appendGridData(reader);
        }
    };
    ListView.prototype.loadImages = function (listview, reader, clientStartIndex) {
        var controller = this.controller;
        var firstColumn = this.columns[0];
        var imageWrappers = listview.find(".cardlist-image-wrapper");
        if (!firstColumn || !(firstColumn instanceof GridImageColumn)) {
            if (firstColumn instanceof GridDomainColumn &&
                firstColumn.objProp.controlId === Enum.ControlType.IMAGE_COMBO) {
                var width = firstColumn.viewProp.gridColumnWidth !== null ? firstColumn.viewProp.gridColumnWidth : 60;
                imageWrappers.css("min-width", width + "px");
            }
            return;
        }
        // The reader only contains the new records when used with paging, that's why we start at clientStartIndex.
        // Also we only need to load images for the new rows
        this.cancelImageScheduler();
        var scheduler = this.imageScheduler = new Scheduler(4);
        var images = listview.find("img");
        var imageColumn = firstColumn;
        var imageHeight = imageColumn.imageHeight, imageWidth = imageColumn.imageWidth;
        for (var i = clientStartIndex; i < images.length; i++) {
            var element = images[i];
            var rowIndex = parseInt(jQuery(element).data("row-index"), 10);
            this.loadImage(element, reader.item(rowIndex - clientStartIndex), imageColumn);
        }
        imageWrappers.css("min-width", imageWidth + "px");
    };
    ListView.prototype.loadImage = function (element, row, imageColumn) {
        var scheduler = this.imageScheduler;
        var controller = this.controller;
        scheduler.schedule(function () {
            return controller.getImage(row, imageColumn.objProp, imageColumn.imageWidth, imageColumn.imageHeight, true).then(function (base64Icon) {
                element.src = base64Icon ? Utils.createIconFromBase64(base64Icon) : "img/photo.png";
            });
        });
    };
    ListView.prototype.cancelImageScheduler = function () {
        if (this.imageScheduler) {
            this.imageScheduler.cancel();
        }
    };
    ListView.prototype.dispose = function () {
        this.cancelImageScheduler();
        if (this.scrollBar) {
            this.scrollBar.destroy();
        }
        _super.prototype.dispose.call(this);
    };
    return ListView;
}(Grid));
