jQuery(document).on("pageinit", "#parameterPage", function () {
    var page = jQuery(this), url = jQuery.mobile.path.parseUrl(page.data('url')), parsedUrl = Utils.parseUrlVars(url.search), taskId = parsedUrl.indexed['task'], taskConfigId = parsedUrl.indexed['taskConfig'], contextObjId = parsedUrl.indexed['firstObj'], contextConfigId = parsedUrl.indexed['contextConfig'];
    var paramPage = new ParameterPage(page);
    var contextController;
    if (contextObjId) {
        contextController = TSFController.getControllerFromQueryString(paramPage.application, contextObjId, contextConfigId, parsedUrl.raw);
    }
    var task = paramPage.application.model.tasks.key(taskId);
    var taskView;
    if (taskConfigId) {
        taskView = task.taskViews.key(taskConfigId);
    }
    else {
        taskView = task.defaultTaskView;
    }
    // The TaskController should already exist since it has been created in showPageOrPopup.
    var paramController = TSFController.getOrCreateTaskController(paramPage.application, taskView, contextController);
    // Create a context div which gets added to the dom after the initialization of the complete form.
    var contentPanel = document.createElement("div");
    // parameterpage is always full-screen, so we need to keep this in sync with abstractscreen
    contentPanel.className = "content-panel full-screen-width";
    var main = page.find("div[data-role='main']");
    main.append(contentPanel);
    // append the contentpanel before calling paramPage.build - or some controls can't initialize (like the signature)
    // because it isn't on the dom yet.
    paramPage.build(taskView, paramController, contextController, contentPanel);
    var taskPromise = null;
    paramPage.footer.on("click", "a", function () {
        var $this = jQuery(this);
        if ($this.hasClass("execute")) {
            paramPage.form.controllerEditor.canLeaveEditorMode().done(function () {
                taskPromise = taskHelper.tryExecuteTask(paramController, contextController, taskView).always(function () {
                    history.back();
                });
            }).fail(function (objProp) {
                paramPage.form.controllerEditor.suggestCursorIndex(objProp);
            });
        }
        else {
            history.back();
        }
    });
    page.on("pagebeforehide.page", function () {
        if (contextController) {
            if (taskPromise) {
                contextController.taskActionPerformed(taskView, taskPromise.state() === "resolved");
            }
            else {
                contextController.taskActionPerformed(taskView, false);
            }
        }
    });
    // Fix translations
    var language = paramPage.application.language;
    var footer = jQuery(this).children("div[data-role='footer']");
    var cancelAnchor = footer.find(".cancel").text(language.transl(Model.TranslKeys.cancel));
    var executeAnchor = footer.find(".execute").text(language.transl(Model.TranslKeys.execute));
    var backButton = jQuery(this).find("a.back-button");
    var color = Utils.Color.hexToColor(backButton.css("color"));
    Utils.SetIconBlackOrWhite(backButton, color);
});
var ParameterPage = /** @class */ (function () {
    function ParameterPage(page) {
        this.page = page;
        this.application = Model.TSFApplicationManager.Current.activeAppl;
        this.header = this.page.children("div[data-role='header']");
        this.footer = this.page.children("div[data-role='footer']");
    }
    ParameterPage.prototype.build = function (taskView, paramController, contextController, content) {
        var _this = this;
        this.contextController = contextController;
        this.paramController = paramController;
        // Header
        var h1Element = this.header.children('h1');
        h1Element.text(taskView.taskTransl.transl);
        if (taskView.icon) {
            var img = document.createElement("img");
            img.className = "subject-icon";
            img.src = Utils.createIconFromBase64(taskView.icon);
            h1Element.prepend(img);
        }
        else {
            h1Element.addClass("noIcon");
        }
        // Content
        this.paramController.events.layoutChanged(function (e) {
            _this.footer.find("a.cancel").toggleClass("ui-state-disabled", paramController.cancelRowAccessType !== Enum.AccessType.Regular);
            _this.footer.find("a.execute").toggleClass("ui-state-disabled", paramController.saveRowAccessType !== Enum.AccessType.Regular);
        });
        this.form = new TSFForm(this.application, this.paramController, 1 /* Add */);
        // TODO Refactor this because this is a bit error prone. The pageshow event and updatePanelsHeight isn't fired for taskpopups
        // Because this.page is actually the popup.
        // It always worked this way so might be nice to normalize this code
        if (this.page.is(".ui-page")) {
            this.page.on("updatePanelsHeight", function (e, data) {
                _this.updateContentPanelHeight(data.height, data.footerHeight);
                data.handled = true;
            });
            // Make sure that the height of the main div is set. Otherwise scrolling on iOS doesn't work.
            var pageShown_1 = jQuery.Deferred();
            this.page.one("pageshow", function () {
                var height = window.innerHeight;
                _this.updateContentPanelHeight(height, _this.footer.outerHeight());
                pageShown_1.resolve();
            });
            this.form.pageShown = pageShown_1.promise();
        }
        else {
            // This is a popup, pageShown is actually implemented in abstractscreen so we have to do this manually here
            var pageShown_2 = jQuery.Deferred();
            this.page.one("popupafteropen", function () {
                pageShown_2.resolve();
            });
            this.form.pageShown = pageShown_2.promise();
        }
        this.form.initialize(content);
        this.form.attachEditor();
        // Layouts are already executed, make sure everything has the right status
        this.footer.find("a.cancel").toggleClass("ui-state-disabled", this.paramController.cancelRowAccessType !== Enum.AccessType.Regular);
        this.footer.find("a.execute").toggleClass("ui-state-disabled", this.paramController.saveRowAccessType !== Enum.AccessType.Regular);
    };
    ParameterPage.prototype.updateContentPanelHeight = function (newHeight, footerHeight) {
        // Fixes scroll on Android2.* because we use the browser scroll instead of iScroll.
        if (!Utils.isAndroidTwo()) {
            var contentPanel_1 = this.page.find(".content-panel");
            newHeight -= this.header.outerHeight() + footerHeight;
            var oldHeight_1 = contentPanel_1.height();
            var firstTime_1 = !(contentPanel_1.length > 0 && contentPanel_1[0].style.height);
            contentPanel_1.css('height', newHeight + 'px');
            // Trigger a updateLayout so let components know there size changed.
            // This makes it possible to update for example scroll bars the right way, use a timer, so the browser can redraw first
            window.setTimeout(function () {
                contentPanel_1.trigger("tsf-updatelayout", [oldHeight_1, newHeight, firstTime_1]);
            }, 0);
        }
    };
    return ParameterPage;
}());
