jQuery(document).on("pagecreate", "#synchronize", function () {
    var appl = Model.TSFApplicationManager.Current.activeAppl, page = jQuery(this), header = page.children("div[data-role='header']"), footer = page.children("div[data-role='footer']");
    // Translations
    var save = footer.find(".synchronize");
    save.text(appl.language.transl(Model.TranslKeys.synchronize));
    var del = footer.find(".delete");
    del.text(appl.language.transl(Model.TranslKeys.Delete));
    var multiPanelScreen = window.matchMedia("(min-width: 800px)").matches;
    if (multiPanelScreen) {
        save.attr("data-iconpos", "left");
        del.attr("data-iconpos", "left");
    }
    // Make sure that the height of the main div is set. Otherwise scrolling on iOS doesn't work.
    page.one("pageshow", function () {
        var height = window.innerHeight;
        height -= header.outerHeight() + footer.outerHeight();
        page.find(".content-panel").css('height', height + 'px');
    });
    // Set black or white icon for the back button.
    var color = Utils.Color.hexToColor(jQuery("a.back-button").css("color"));
    Utils.SetIconBlackOrWhite(jQuery("a.back-button"), color);
});
jQuery(document).on('pageinit', '#synchronize', function () {
    var page = jQuery(this);
    var application = Model.TSFApplicationManager.Current.activeAppl;
    var synchronize = new SynchronizeForm(page, application);
    synchronize.appendData();
});
var Synchronizer = /** @class */ (function () {
    function Synchronizer(application, page) {
        this.application = application;
        this.page = page;
    }
    Synchronizer.prototype.synchronizeToWebService = function () {
        Logger.log("Synchronize offline data", LogType.LOG);
        if (AppConfig.online) {
            return this.application.localSync.fullSynchronize(true);
        }
        else {
            var deferred_1 = jQuery.Deferred();
            var errorPopup = ConfirmationPopup.CreateOkPopup();
            var promise = errorPopup.initialize(this.application.language.transl(Model.TranslKeys.synchronizeFailedTitle), this.application.language.transl(Model.TranslKeys.deviceOffline));
            promise.done(function (option) {
                deferred_1.reject();
            });
            return deferred_1.promise();
        }
    };
    Synchronizer.prototype.clearQueue = function () {
        var _this = this;
        Logger.log("Clear queue for appl " + this.application.guiApplId, LogType.LOG);
        var deferred = jQuery.Deferred();
        var title = this.application.language.transl(Model.TranslKeys.clearQueue) + "?";
        var message = this.application.language.transl(Model.TranslKeys.clearQueueMessage);
        var promise = ConfirmationPopup.CreateYesNoPopup().initialize(title, message);
        promise.done(function (confirmationOption) {
            if (confirmationOption == Enum.ConfirmationOption.Yes) {
                LocalSync.resetSyncTokens(_this.application.guiApplId);
                var storageContainer = _this.application.getStorageContainer();
                storageContainer.clearOfflineData();
                storageContainer.clearQueue().always(function () {
                    // Mark all controllers with allowExport dirty.
                    // Dont filter on obj.offlineDataAvailable because the markControllerDirty method filters on
                    // controllers with an objView with offlineDataAvailable is true.
                    var hitObjs = _this.application.model.objs.array.map(function (obj) { return obj.objId; });
                    _this.application.localSync.markControllersDirty(hitObjs);
                    deferred.resolve();
                });
            }
            else {
                deferred.reject();
            }
        });
        return deferred.promise();
    };
    return Synchronizer;
}());
var SynchronizeForm = /** @class */ (function () {
    function SynchronizeForm(page, application) {
        this.page = page;
        this.application = application;
        var header = page.children("div[data-role='header']"), panel = page.find(".content-panel");
        header.children('h1').text(this.application.language.transl(Model.TranslKeys.offlineQueue));
        this.list = document.createElement("ul");
        this.list.className = "synchronize";
        this.list.id = "synchronizationList";
        jQuery(this.list).appendTo(panel).listview();
        var that = this;
        jQuery(this.list).on("click", "a", function () {
            var $this = jQuery(this);
            if ($this.hasClass("moreInfo")) {
                $this.parent().next().slideToggle();
            }
            else if ($this.hasClass("deleteBtn")) {
                var $li = $this.closest("li");
                that.application.serverSync.deleteQueueItem($li.data("tsf-id"));
                $li.remove();
                // Remove synctoken because the data isn't uptodate anymore.
                // For example a delete failed on the database (so it is already gone in the local db) but localSync won't return it anymore
                var obj = that.application.model.getObj($li.data("tsf-obj"));
                Logger.log("Delete offline record [" + obj.objId + "]", LogType.LOG);
                if (that.application.localSync != null && obj) {
                    that.application.localSync.removeSyncToken(obj);
                }
                jQuery(that.list).listview("refresh");
            }
        });
        var synchronizer = new Synchronizer(application, page[0]);
        var controlGroup = page.children("div[data-role='footer']").find(":jqmData(role='controlgroup')");
        controlGroup.on("click", "a", function (event) {
            var $button = jQuery(this);
            if ($button.hasClass("synchronize")) {
                that.disableSynchronizedEvent();
                synchronizer.synchronizeToWebService().always(function () {
                    that.reappendData();
                    that.attachDataSynchronizedEvent();
                });
            }
            else if ($button.hasClass("delete")) {
                synchronizer.clearQueue().always(function () {
                    that.reappendData();
                    application.localSync.fullSynchronize(true);
                });
            }
        });
        // When this page is active, also listen to the datasynchronized event, so we can update this page
        this.attachDataSynchronizedEvent();
    }
    SynchronizeForm.prototype.disableSynchronizedEvent = function () {
        Logger.log("Disable syncrhonized event", LogType.LOG);
        jQuery(this.page).off("datasynchronized");
    };
    SynchronizeForm.prototype.attachDataSynchronizedEvent = function () {
        var _this = this;
        Logger.log("Enable syncrhonized event", LogType.LOG);
        jQuery(this.page).off("datasynchronized").on("datasynchronized", function () {
            Logger.log("Data synchronized, reappending data", LogType.LOG);
            _this.reappendData();
        });
    };
    SynchronizeForm.prototype.reappendData = function () {
        jQuery(this.list).empty();
        this.appendData();
    };
    SynchronizeForm.prototype.appendData = function () {
        var _this = this;
        this.application.serverSync.getQueue(false).done(function (queue) {
            var moreInfoTranslation = _this.application.language.transl(Model.TranslKeys.moreInfo);
            var fragment = document.createDocumentFragment();
            var model = _this.application.model;
            var containsConflicts = false;
            for (var i = queue.length - 1; i >= 0; i--) {
                var queueItem = queue[i], obj = model.getObj(queueItem.objId);
                // Skip VolatileInsert types, they are created by offline code and removed
                // after a sync. They are for updating views from offline code
                // LocalSync will return them again (if not, they didn't belong there anyway)
                if (queueItem.actionType == 4 /* VolatileInsert */) {
                    continue;
                }
                var item = document.createElement("li");
                var img = document.createElement("img");
                img.src = _this.getImageForActionType(queueItem.actionType);
                var header = document.createElement("h2");
                header.innerText = obj ? obj.defaultObjView.objTransl.transl : "[" + queueItem.objId;
                item.appendChild(img);
                item.appendChild(header);
                switch (queueItem.status) {
                    case 0 /* UnSynced */:
                        item.className = "ui-icon-synchronize-information";
                        break;
                    case 1 /* Synced */:
                        item.className = "ui-icon-synchronize-ok";
                        break;
                    case 2 /* Conflicted */:
                        containsConflicts = true;
                        item.className = "ui-icon-synchronize-warning";
                        var conflictedMessage = document.createElement("p");
                        conflictedMessage.className = "conflictedMessage";
                        conflictedMessage.innerText = queueItem.conflict + " ";
                        item.appendChild(conflictedMessage);
                        if (queueItem.serverError) {
                            var anchor = document.createElement("a");
                            anchor.className = "moreInfo";
                            anchor.href = "#";
                            anchor.textContent = moreInfoTranslation;
                            var serverError = document.createElement("p");
                            serverError.className = "serverErrorMessage";
                            serverError.style.display = "none";
                            serverError.innerText = queueItem.serverError;
                            conflictedMessage.appendChild(anchor);
                            item.appendChild(serverError);
                        }
                        item.setAttribute("data-tsf-id", queueItem.tsf_id + "");
                        item.setAttribute("data-tsf-obj", queueItem.objId);
                        item.appendChild(_this.createDeleteButton());
                        break;
                }
                fragment.appendChild(item);
            }
            if (containsConflicts) {
                var conflictInfoElement = document.createElement("li");
                conflictInfoElement.className = "conflictedInfoMessage";
                var conflictInfoText = document.createElement("p");
                conflictInfoText.innerText = _this.application.language.transl(Model.TranslKeys.syncErrorDuringSynchronization);
                conflictInfoElement.appendChild(conflictInfoText);
                fragment.insertBefore(conflictInfoElement, fragment.firstChild);
            }
            _this.list.appendChild(fragment);
            var $list = jQuery(_this.list);
            // Make sure the listview is still visible or the refresh function would fail.
            if ($list.data("mobile-listview")) {
                $list.listview("refresh");
            }
        });
    };
    SynchronizeForm.prototype.getImageForActionType = function (type) {
        switch (type) {
            case 0 /* Insert */:
                return "img/synchronize/add.png";
            case 1 /* Update */:
                return "img/synchronize/edit.png";
            case 2 /* Delete */:
                return "img/synchronize/delete.png";
            case 3 /* Task */:
                return "img/synchronize/task.png";
        }
        return null;
    };
    SynchronizeForm.prototype.createDeleteButton = function () {
        if (!this.deleteButtonCache) {
            var wrapper = this.deleteButtonCache = document.createElement("span");
            var button = document.createElement("a");
            button.className = "deleteBtn";
            button.href = "#";
            wrapper.appendChild(button);
        }
        return this.deleteButtonCache.cloneNode(true);
    };
    return SynchronizeForm;
}());
