jQuery(document).on("pagecreate", "#synclog", function () {
    var page = jQuery(this), header = page.children("div[data-role='header']");
    // Make sure that the height of the main div is set. Otherwise scrolling on iOS doesn't work.
    page.one("pageshow", function () {
        var height = window.innerHeight;
        height -= header.outerHeight();
        page.find(".content-panel").css('height', height + 'px');
    });
    // Set black or white icon for the back button.
    var color = Utils.Color.hexToColor(header.find("a.back-button").css("color"));
    Utils.SetIconBlackOrWhite(header.find("a.back-button"), color);
});
jQuery(document).on('pageinit', '#synclog', function () {
    var page = jQuery(this);
    var application = Model.TSFApplicationManager.Current.activeAppl;
    var synchronize = new SyncLog(page, application);
    synchronize.appendData();
});
var SyncLog = /** @class */ (function () {
    function SyncLog(page, application) {
        this.page = page;
        this.application = application;
        var header = page.children("div[data-role='header']"), panel = page.find(".content-panel");
        header.children('h1').text(this.application.language.transl(Model.TranslKeys.synclog));
        this.list = document.createElement("ul");
        this.list.className = "synchronize";
        jQuery(this.list).appendTo(panel).listview();
        jQuery(this.list).on("click", "a.moreInfo", function () {
            var $this = jQuery(this);
            $this.next().slideToggle();
        });
    }
    SyncLog.prototype.appendData = function () {
        var _this = this;
        var fragment = document.createDocumentFragment();
        var application = this.application;
        var store = application.localSync.openSyncGroupLastSyncStore();
        var req = store.then(function (syncStore) {
            return syncStore.openCursor();
        }, function () {
            return null;
        });
        var moreInfoTranslation = application.language.transl(Model.TranslKeys.moreInfo);
        var lastSyncSucessfulTranslation = application.language.transl(Model.TranslKeys.sync_finished);
        var syncObjectFailedTranslation = application.language.transl(Model.TranslKeys.sync_objects_failed);
        var syncGroupTablesTranslation = application.language.transl(Model.TranslKeys.sync_group_tables);
        if (req) {
            req.then(function (request) {
                request.onsuccess = function (e) {
                    var cursor = request.result;
                    if (cursor) {
                        var value = cursor.value;
                        var syncGroup = application.syncGroups.key(value.sync_group_id);
                        if (!syncGroup) {
                            // Syncgroup not found
                            cursor['continue']();
                            return;
                        }
                        var item = document.createElement("li");
                        var img = document.createElement("img");
                        img.src = "img/synchronize/sync.png";
                        var header = document.createElement("h2");
                        header.innerText = syncGroup.transl.transl;
                        item.appendChild(img);
                        item.appendChild(header);
                        var message = document.createElement("p");
                        if (value.error) {
                            item.className = "ui-icon-synchronize-warning";
                            message.className = "conflictedMessage";
                            message.innerText = moment(value.errorDate).format("DD-MM-YYYY HH:mm") + " " + syncObjectFailedTranslation + "\r\n " + value.error + " ";
                            item.appendChild(message);
                        }
                        else {
                            item.className = "ui-icon-synchronize-ok";
                            message.innerText = lastSyncSucessfulTranslation + " " + moment(value.lastSuccessfulSync).format("DD-MM-YYYY HH:mm") + " ";
                            item.appendChild(message);
                        }
                        var anchor = document.createElement("a");
                        anchor.className = "moreInfo";
                        anchor.href = "#";
                        anchor.textContent = moreInfoTranslation;
                        var groups = document.createElement("p");
                        groups.className = "syncedTables";
                        groups.style.display = "none";
                        groups.innerText = syncGroupTablesTranslation + ":\r\n " + syncGroup.objs.array.map(function (obj) { return obj.defaultObjView.objTransl.transl; }).join("\r\n");
                        message.appendChild(anchor);
                        message.appendChild(groups);
                        fragment.appendChild(item);
                        cursor['continue']();
                    }
                    else {
                        _this.list.appendChild(fragment);
                        var $list = jQuery(_this.list);
                        // Make sure the listview is still visible or the refresh function would fail.
                        if ($list.data("mobile-listview")) {
                            $list.listview("refresh");
                        }
                    }
                };
            });
        }
    };
    return SyncLog;
}());
