var TaskBar = /** @class */ (function () {
    function TaskBar(application, abstractscreen) {
        this.application = application;
        this.abstractscreen = abstractscreen;
        this.controller = abstractscreen.controller;
    }
    TaskBar.prototype.initialize = function (contentElement) {
        var _this = this;
        if (!contentElement) {
            return;
        }
        this.contentElement = contentElement;
        this.cachedTaskContextState = JSON.stringify(this.controller.contextState.taskContextState);
        this.createTaskPanel();
        this.controller.events.editmodeChanged(function () {
            _this.updateTaskPanel(true);
        });
        this.abstractscreen.addScreenComponent(this);
    };
    TaskBar.prototype.updateTaskPanel = function (editModeChanged) {
        var stringifiedState = JSON.stringify(this.controller.contextState.taskContextState);
        if (stringifiedState != this.cachedTaskContextState || editModeChanged) {
            this.contentElement.find(".taskBar").remove();
            this.createTaskPanel();
        }
        this.cachedTaskContextState = stringifiedState;
    };
    TaskBar.prototype.createTaskPanel = function () {
        var taskContextState = this.controller.contextState.taskContextState;
        var taskBar = document.createElement("div");
        taskBar.setAttribute("data-role", "navbar");
        taskBar.className = "taskBar";
        var ul = document.createElement("ul");
        var visibleTasks = Array.prototype.filter.call(this.controller.objView.taskLinkViews, function (value) {
            return taskContextState[value.taskView.task.taskId] != Enum.AccessType.Hidden;
        });
        for (var i = 0; i < visibleTasks.length; i++) {
            var taskView = visibleTasks[i].taskView;
            var li = this.createTaskIcon(taskView, taskContextState[taskView.task.taskId]);
            ul.appendChild(li);
        }
        // jQuery mobile crashes when enhancing a navbar without items
        if (ul.children.length > 0) {
            taskBar.appendChild(ul);
            this.contentElement.prepend(taskBar);
            jQuery(taskBar).navbar();
        }
    };
    TaskBar.prototype.createTaskIcon = function (taskView, accessType) {
        var _this = this;
        var li = document.createElement("li");
        li.className = "taskIcon";
        var anchor = document.createElement("a");
        anchor.href = "#";
        if (taskView.icon) {
            var img = document.createElement("img");
            img.src = Utils.createIconFromBase64(taskView.icon);
            anchor.appendChild(img);
        }
        else {
            var transl = taskView.taskTransl.transl;
            anchor.innerText = transl.charAt(0);
        }
        anchor.onclick = function () {
            if (_this.controller.interfaceAvailable) {
                //If the row is modified, save it before starting the task execution.
                if (_this.controller.currentEditor && _this.controller.currentEditor.currentRowModified) {
                    _this.controller.executeActionAfterSave(function () {
                        return taskHelper.showPageOrPopup(taskView, _this.controller);
                    });
                    return;
                }
                taskHelper.showPageOrPopup(taskView, _this.controller);
            }
        };
        if (accessType === Enum.AccessType.ReadOnly || this.controller.isInEditMode) {
            //Only make the task disabled if we're not in a default editable form.
            if (!this.controller.currentEditor || !this.controller.currentEditor.isDefaultEditable()) {
                anchor.className = "ui-state-disabled";
            }
        }
        li.appendChild(anchor);
        return li;
    };
    TaskBar.prototype.invalidateData = function () {
    };
    TaskBar.prototype.dispose = function () {
    };
    return TaskBar;
}());
