var IndexedDb;
(function (IndexedDb) {
    var IndexedDbModelCacher = /** @class */ (function () {
        function IndexedDbModelCacher(tsfCache) {
            this.tsfCache = tsfCache;
        }
        IndexedDbModelCacher.prototype.getModelCacheObjectStore = function (mode) {
            if (!this.tsfCache.database) {
                return null;
            }
            var transaction = this.tsfCache.database.transaction(TSFCache.modelCacheTable, mode);
            var objectStore = transaction.objectStore(TSFCache.modelCacheTable);
            return objectStore;
        };
        IndexedDbModelCacher.prototype.storeGetGui = function (key, response) {
            var deferred = jQuery.Deferred();
            var store = this.getModelCacheObjectStore("readwrite");
            if (!store) {
                // Database was closed, probably because another request failed.
                return deferred.reject().promise();
            }
            store.openCursor(IDBKeyRange.only(key), "next").onsuccess = function (event) {
                var cursor = event.target.result;
                if (cursor) {
                    var updateReq = cursor.update({ "get_gui": key, "response": response });
                    updateReq.onsuccess = function (event) {
                        deferred.resolve();
                    };
                    updateReq.onerror = function (event) {
                        deferred.reject("Error updating get gui " + key);
                    };
                }
                else {
                    var req = store.add({ "get_gui": key, "response": response });
                    req.onsuccess = function (event) {
                        deferred.resolve();
                    };
                    req.onerror = function (event) {
                        deferred.reject("Error inserting get gui " + key);
                    };
                }
            };
            return deferred.promise();
        };
        IndexedDbModelCacher.prototype.getGetGuiCache = function (key) {
            var deferred = jQuery.Deferred();
            if (!this.tsfCache.database || this.tsfCache.disposed) {
                return deferred.reject("TSFCache was already disposed.").promise();
            }
            var transaction = this.tsfCache.database.transaction(TSFCache.modelCacheTable, "readonly");
            transaction.onerror = function (event) {
                deferred.reject("Transaction error: " + transaction.error);
            };
            var store = transaction.objectStore(TSFCache.modelCacheTable);
            if (!store) {
                // Database was closed, probably because another request failed.
                return deferred.reject("Database was closed. Can't read get_gui from cache.").promise();
            }
            var getAction = store.get(key);
            getAction.onsuccess = function (event) {
                var result = event.target.result;
                if (result) {
                    deferred.resolve(result.response);
                }
                else {
                    deferred.reject("Get gui is not stored.");
                }
            };
            getAction.onerror = function (err) {
                deferred.reject("Store get error: " + getAction.error);
            };
            return deferred.promise();
        };
        IndexedDbModelCacher.prototype.removeGetGuiCache = function (key) {
            var deferred = jQuery.Deferred();
            var store = this.getModelCacheObjectStore("readwrite");
            if (!store) {
                // Database was closed, probably because another request failed.
                return deferred.reject().promise();
            }
            store.openCursor(IDBKeyRange.only(key), 'next').onsuccess = function (event) {
                var cursor = event.target.result;
                if (cursor) {
                    var req = cursor['delete']();
                    req.onsuccess = function (event) {
                        deferred.resolve();
                    };
                    req.onerror = function (event) {
                        deferred.reject("Error removing get gui " + key);
                    };
                }
                else {
                    // We didn't find the key but we resolve anyway because the caller functions didn't check if it was there.
                    deferred.resolve();
                }
            };
            return deferred.promise();
        };
        return IndexedDbModelCacher;
    }());
    IndexedDb.IndexedDbModelCacher = IndexedDbModelCacher;
})(IndexedDb || (IndexedDb = {}));
