var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var ServerSync = /** @class */ (function () {
    function ServerSync(application) {
        this.application = application;
        /**
         * The stale sync service doesn't support some actions so we need to change behavior when in staleSync mode
         */
        this.isStaleSync = false;
        this.disposed = false;
        this.db = application.getDatabase();
    }
    ServerSync.prototype.syncQueueToServer = function () {
        var _this = this;
        var deferred = jQuery.Deferred();
        if (!this.db || this.disposed) {
            deferred.reject("Tried to syncQueueToServer but the database was already disposed.");
            return deferred.promise();
        }
        // Store everything in a queue, because when processing the queue one by one with different async calls
        // (defaults/layouts, insert/update), browsers will mark the transaction inactive. Updating the cursor, or calling continue fails
        // when this happens
        this.getQueue(true).done(function (queue) {
            // This is actually the date when we got the queue. This is the date we should check so we know when
            // new changes are added, we didn't sync them yet.
            var startSyncDate = new Date();
            var queueLength = queue.length;
            var currentRow = 0, errorsDuringSynchronize = 0 /* None */;
            var changedGroups = {};
            var processNextRow = function (rowIndex) {
                deferred.notify((currentRow / queueLength * 100));
                var item = queue[currentRow];
                _this.processQueueRow(item, changedGroups).fail(function (errorReason) {
                    if (errorReason == 3 /* Network */) {
                        errorsDuringSynchronize = 1 /* Network */;
                    }
                    else {
                        errorsDuringSynchronize = 2 /* DatabaseError */;
                        //Log all unexpected errors to Sentry.
                        if (errorsDuringSynchronize !== 2 /* DatabaseError */) {
                            Utils.tryRavenCaptureException("Failed to processQueueRow. Error: " + errorReason);
                        }
                    }
                }).always(function () {
                    if (currentRow < queueLength - 1) {
                        // Stop on network errors, but continue for database errors
                        if (errorsDuringSynchronize != 1 /* Network */) {
                            processNextRow(++currentRow);
                        }
                        else {
                            // Don't perform a LocalSync when a network error occurs.
                            deferred.resolve(errorsDuringSynchronize, changedGroups);
                        }
                    }
                    else {
                        // Trigger synchronized event, to let the activePage know something happened.
                        jQuery.mobile.activePage.trigger("datasynchronized");
                        // We treat network errors als resolved with an additional parameter
                        // because connection might fail sometimes
                        // Database errors will cause a rejection state
                        if (errorsDuringSynchronize == 2 /* DatabaseError */) {
                            deferred.reject();
                        }
                        else {
                            _this.updateOfflineChangesInLocalstorage(startSyncDate);
                            deferred.resolve(errorsDuringSynchronize, changedGroups);
                        }
                    }
                });
            };
            if (queueLength > 0) {
                processNextRow(0);
            }
            else {
                deferred.resolve(errorsDuringSynchronize); // no items in the queue, just resolve the promise
            }
        }).fail(function (err) {
            deferred.reject(err);
        });
        return deferred.promise();
    };
    ServerSync.prototype.deleteQueueItem = function (id) {
        // No error handling, we expect this to succeed and don't really care about the result
        // Only used from the synchronisation page where we already remove this from the list so we don't need to reload the queue.
        var queueTableName = IndexedDb.IndexedDbWrapper.queueTableName;
        var transaction = this.db.transaction(queueTableName, "readwrite");
        var queueStore = transaction.objectStore(queueTableName);
        queueStore['delete'](id);
    };
    ServerSync.prototype.getQueue = function (skipSyncedItems) {
        var deferred = jQuery.Deferred();
        if (!this.db || this.disposed) {
            deferred.reject("Tried to getQueue but the database was already disposed.");
            return deferred.promise();
        }
        var queueTableName = IndexedDb.IndexedDbWrapper.queueTableName;
        var transaction = this.db.transaction(queueTableName, "readonly");
        var queueStore = transaction.objectStore(queueTableName);
        var result = [];
        queueStore.openCursor(null, 'next').onsuccess = function (event) {
            var cursor = event.target.result;
            if (cursor) {
                // Shim doesn't add autoincrement columns to the value object
                // But we use this key for index filtering
                cursor.value["tsf_id"] = cursor.key;
                if (!skipSyncedItems || cursor.value.status != 1 /* Synced */) {
                    result.push(cursor.value);
                }
                cursor['continue']();
            }
            else {
                // iOS sometimes returns an unsorted sync queue.
                result = Array.prototype.sort.call(result, function (a, b) {
                    return a.tsf_id - b.tsf_id;
                });
                deferred.resolve(result); // Finished, no more rows
            }
        };
        return deferred.promise();
    };
    ServerSync.prototype.getUnsyncedAmount = function () {
        return this.getQueue(true).then(function (queue) {
            return queue.filter(function (value, index, arr) {
                // Don't count volatileInserts but do count items skipped by network errors!
                return value.actionType !== 4 /* VolatileInsert */;
            }).length;
        });
    };
    ServerSync.prototype.processQueueRow = function (queueItem, changedGroups) {
        var _this = this;
        var deferred = jQuery.Deferred();
        // Skip synced items, only sync unsynced items and retry conflicted.
        if (queueItem.status === 1 /* Synced */) {
            return deferred.resolve().promise();
        }
        var obj = (queueItem.actionType === 3 /* Task */
            ? (this.application.model.tasks.key(queueItem.objId)).userParamObj
            : this.application.model.getObj(queueItem.objId));
        if (!obj) {
            return deferred.resolve().promise();
        }
        // VolatileInsert identity's should not be resolved
        if (queueItem.actionType == 4 /* VolatileInsert */) {
            this.deleteRowFromOfflineStorage(queueItem, deferred);
        }
        else {
            var resolveReq = this.resolveIdentities(queueItem, obj.defaultObjView);
            resolveReq.done(function () {
                if (queueItem.actionType == 2 /* Delete */) {
                    _this.deleteRow(queueItem, deferred);
                }
                else if (queueItem.actionType == 1 /* Update */) {
                    // Stale sync doesn't have a get method
                    if (_this.isStaleSync) {
                        // Create a copy so we don't break the queueItem (after an update)
                        var dataRow = {};
                        for (var x in queueItem.pk) {
                            dataRow[x] = queueItem.pk[x];
                        }
                        _this.synchronizeRow(queueItem, dataRow, deferred);
                    }
                    else {
                        _this.searchRowForUpdate(queueItem, deferred);
                    }
                }
                else if (queueItem.actionType == 0 /* Insert */) {
                    var row = {};
                    var objViewProps = obj.defaultObjView.objViewProps.array;
                    for (var i = 0; i < objViewProps.length; i++) {
                        var objViewProp = objViewProps[i];
                        if (objViewProp.defaultValue) {
                            row[objViewProp.objPropId] = objViewProp.defaultValue;
                        }
                    }
                    _this.synchronizeRow(queueItem, row, deferred);
                }
                else if (queueItem.actionType == 3 /* Task */) {
                    _this.executeTask(queueItem, deferred);
                }
                if (obj.syncGroupId && !changedGroups[obj.syncGroupId]) {
                    changedGroups[obj.syncGroupId] = _this.application.syncGroups.key(obj.syncGroupId);
                }
            });
            resolveReq.fail(function (msg) {
                queueItem.status = 2 /* Conflicted */;
                queueItem.conflict = msg;
                _this.updateQueue(queueItem, deferred, 0 /* Unknown */);
            });
        }
        return deferred.promise();
    };
    ServerSync.prototype.deleteRowFromOfflineStorage = function (queueItem, deferred) {
        var _this = this;
        var wrapper = this.application.getStorageContainer();
        var obj = this.application.model.getObj(queueItem.objId);
        wrapper.deleteRow(obj, TSFPkBookmark.fromObject(obj, queueItem.pk), false).always(function () {
            queueItem.status = 1 /* Synced */;
            _this.updateQueue(queueItem, deferred);
        });
    };
    ServerSync.prototype.updateQueueItemWithError = function (queueItem, translationKey, status) {
        queueItem.status = 2 /* Conflicted */;
        var transl = this.application.language.transl(translationKey);
        if (status.reason == 3 /* Network */) {
            // In this case we add the error with the network error text so we don't hide it on the synchronize page
            queueItem.conflict = transl + ": " + status.messages;
        }
        else {
            queueItem.conflict = transl;
            queueItem.serverError = status.messages[0];
        }
    };
    ServerSync.prototype.searchRowForUpdate = function (queueItem, deferred) {
        var _this = this;
        var obj = this.application.model.getObj(queueItem.objId);
        var builder = RequestBuilderFactory.createOnlineRequestBuilder(this.application, obj.defaultObjView);
        var filterContract = FilterContract.fromDataRow(obj, queueItem.oldPk);
        builder.addFilterContract(filterContract);
        builder.setCallbackMethod(function (reader, status) {
            if (!status.success || reader == null || reader.length !== 1) {
                _this.updateQueueItemWithError(queueItem, Model.TranslKeys.syncErrorRowNotFound, status);
                _this.updateQueue(queueItem, deferred, status.reason);
            }
            else {
                var itemReader = reader.item(0);
                var row = {};
                var objProps = obj.objProps.array;
                for (var i = 0; i < objProps.length; i++) {
                    var objProp = objProps[i];
                    row[objProp.objPropId] = itemReader.getValueForObjProp(objProp);
                }
                _this.synchronizeRow(queueItem, row, deferred);
            }
        });
        builder.get();
    };
    ServerSync.prototype.resolveIdentities = function (queueObject, objView) {
        var resolveList = [];
        var viewProps = objView.objViewProps.array;
        for (var i = 0; i < viewProps.length; i++) {
            var objViewProp = viewProps[i];
            // If the objProp is an identity or when its a lookup with a dataprop that is an identity
            if ((objViewProp.objProp.identity && queueObject.actionType != 0 /* Insert */) || (objViewProp.hasLookup() && objViewProp.lookup.dataProp.objProp.identity)) {
                var pkValue = queueObject.pk[objViewProp.objPropId];
                if (pkValue && (pkValue + "").indexOf(IdentityReplacer.prefix) != -1) {
                    resolveList.push(this.prepareResolveIdentity(pkValue, queueObject, objViewProp, false));
                }
                if (queueObject.oldPk) {
                    var oldPkValue = queueObject.oldPk[objViewProp.objPropId];
                    if (oldPkValue && (oldPkValue + "").indexOf(IdentityReplacer.prefix) != -1) {
                        resolveList.push(this.prepareResolveIdentity(oldPkValue, queueObject, objViewProp, true));
                    }
                }
            }
        }
        return jQuery.when.apply(undefined, resolveList);
    };
    ServerSync.prototype.prepareResolveIdentity = function (value, queueObject, objViewProp, oldPk) {
        return this.resolveIdentity(value).done(function (databaseId) {
            if (oldPk) {
                queueObject.oldPk[objViewProp.objPropId] = databaseId;
            }
            else {
                queueObject.pk[objViewProp.objPropId] = databaseId;
            }
        });
    };
    ServerSync.prototype.resolveIdentity = function (identity) {
        var deferred = jQuery.Deferred();
        if (!this.db || this.disposed) {
            return deferred.reject("Tried to resolveIdentity but the database was already disposed.").promise();
        }
        var identityTableName = IndexedDb.IndexedDbWrapper.identityTableName;
        var transaction = this.db.transaction(identityTableName, "readonly"), identityStore = transaction.objectStore(identityTableName);
        var req = identityStore.openCursor(IDBKeyRange.only(identity));
        req.onsuccess = function (e) {
            var cursor = e.target.result;
            if (cursor) {
                var item = cursor.value;
                if (item.databaseId != null) {
                    deferred.resolve(item.databaseId);
                }
                else {
                    deferred.reject("Error resolving identity, tsf_unique unresolved yet.");
                }
            }
            else {
                deferred.reject("Error resolving identity, tsf_unique not found.");
            }
        };
        req.onerror = function () {
            deferred.reject("Error opening the identity store");
        };
        return deferred.promise();
    };
    ServerSync.prototype.deleteRow = function (queueItem, deferred) {
        var _this = this;
        var obj = this.application.model.getObj(queueItem.objId);
        var builder = RequestBuilderFactory.createOnlineRequestBuilder(this.application, obj.defaultObjView);
        builder.setCallbackMethod(function (reader, status) {
            if (status.success) {
                queueItem.status = 1 /* Synced */;
            }
            else {
                _this.updateQueueItemWithError(queueItem, Model.TranslKeys.syncErrorDeleteRow, status);
            }
            _this.updateQueue(queueItem, deferred, status.reason);
        }).Delete(TSFRowBookmark.fromObject(obj, queueItem.pk));
    };
    ServerSync.prototype.executeTask = function (queueItem, deferred) {
        var _this = this;
        // The objId matches the taskId
        var task = this.application.model.tasks.key(queueItem.objId);
        var builder = RequestBuilderFactory.createOnlineRequestBuilder(this.application, task.userParamObj.defaultObjView);
        builder.setCallbackMethod(function (reader, status) {
            if (status.success) {
                queueItem.status = 1 /* Synced */;
            }
            else {
                queueItem.status = 2 /* Conflicted */;
                queueItem.conflict = "Error executing task: " + status.messages;
            }
            _this.updateQueue(queueItem, deferred, status.reason);
        }).executeTask(queueItem.pk);
    };
    ServerSync.prototype.synchronizeRow = function (queueItem, rowObject, deferred) {
        var _this = this;
        var deferredInternal = jQuery.Deferred();
        deferredInternal.done(function () {
            queueItem.status = 1 /* Synced */;
            _this.updateQueue(queueItem, deferred);
        }).fail(function (error, reason) {
            // Some fixes the queueItem itself so we can differentiate between network errors and server orrors and change the message accordingly
            // That code rejects without arguments
            if (error != undefined) {
                queueItem.status = 2 /* Conflicted */;
                queueItem.conflict = error;
            }
            _this.updateQueue(queueItem, deferred, reason);
        });
        this.synchronizeRowInternal(queueItem, rowObject, deferredInternal);
    };
    ServerSync.prototype.synchronizeRowInternal = function (queueItem, rowObject, deferred) {
        var _this = this;
        var obj = this.application.model.getObj(queueItem.objId);
        var layoutMode = queueItem.oldPk ? Enum.LayoutMode.Update : Enum.LayoutMode.Add;
        var saveFunc = function () {
            var bookmark = new TSFRowBookmark(obj);
            // TODO: maybe move this to TSFBookmark and create a fromObject method.
            var objProps = obj.objProps.array;
            for (var i = 0; i < objProps.length; i++) {
                var prop = objProps[i];
                var value = rowObject[prop.objPropId];
                if (prop.identity && !Utils.isEmpty(value) && (value + "").indexOf(IdentityReplacer.prefix) === 0) {
                    // Don't send the tsf_unique values for identity columns to Indicium. Indicium doesn't like
                    // values that don't match the data type of the column, and furthermore, Indicium has no purpose
                    // for the tsf_unique placeholders.
                }
                else if (prop.isFileColumn && value != null && value.File === null) {
                    // Make sure to not add the File when it is actually empty
                    var condition = new TSFBookmarkCondition(prop, FileContract.CreateFilePathOnlyContract(value.FilePath));
                    bookmark.addCondition(condition);
                }
                else {
                    var condition = new TSFBookmarkCondition(prop, value);
                    bookmark.addCondition(condition);
                }
            }
            var builder = RequestBuilderFactory.createOnlineRequestBuilder(_this.application, obj.defaultObjView);
            builder.setCallbackMethod(function (reader, status) {
                if (status.success) {
                    if (layoutMode == Enum.LayoutMode.Add) {
                        var item = reader.item(0);
                        for (var i = 0; i < obj.pkProps.length; i++) {
                            var pkProp = obj.pkProps[i];
                            if (pkProp.identity) {
                                // Insert identity and call resolve after it has been updated (async)
                                _this.updateMobileIdentityValue(queueItem.pk[pkProp.objPropId], item.getString(pkProp.objPropId), deferred);
                                // Call and forget, will finish eventually and we don't need to force a wait here.
                                // If another transactions tries to read the table it needs to wait for this transaction to finish
                                // It is really important to update this identity value or localSync will fail to locate this row and will add a new row
                                _this.updateIndexedDBIdentityValue(obj, pkProp, item.getString(pkProp.objPropId), queueItem);
                                return;
                            }
                        }
                    }
                    deferred.resolve();
                }
                else {
                    _this.updateQueueItemWithError(queueItem, Model.TranslKeys.syncErrorSaveRow, status);
                    deferred.reject(queueItem.conflict, status.reason);
                }
            });
            if (layoutMode == Enum.LayoutMode.Add) {
                builder.create(bookmark);
            }
            else {
                builder.put(TSFPkBookmark.fromObject(obj, queueItem.oldPk), bookmark);
            }
        };
        this.executeLayouts(obj, rowObject, layoutMode).then(function (layoutState) {
            var fieldLayoutState = layoutState.fieldLayoutState;
            var pk = TSFRowBookmark.fromObject(obj, queueItem.pk);
            var objProps = obj.objProps.array;
            for (var i = 0; i < objProps.length; i++) {
                var objProp = objProps[i];
                //const regular = fieldLayoutState[objProp.objPropId] >= Enum.FieldLayoutState.Regular;
                var condition = pk.conditions[objProp.objPropId];
                // Disable check because it doesn't copy the values from the newrow when it is disabled by the layout
                // But the change is actually causesd by one of the new values.
                //if (regular || (Utils.isEmpty(rowObject[objProp.objPropId]) && objProp.mandatory)) { // leeg en mandatory
                if (condition && !Utils.isEmpty(condition.value)) {
                    rowObject[objProp.objPropId] = condition.value;
                }
                else {
                    rowObject[objProp.objPropId] = null;
                }
                //}
            }
        }).then(function () {
            return _this.executeDefaults(obj, rowObject, layoutMode);
        }).then(function () {
            return _this.executeLayouts(obj, rowObject, layoutMode).then(function (layoutState) {
                var fieldLayoutState = layoutState.fieldLayoutState;
                var objProps = obj.objProps.array;
                for (var i = 0; i < objProps.length; i++) {
                    var objProp = objProps[i];
                    if (fieldLayoutState[objProp.objPropId] == 4 /* Mandatory */
                        && !objProp.identity && Utils.isEmpty(rowObject[objProp.objPropId])) {
                        // Not all mandatory fields are filled in.
                        var transl = _this.application.language.transl(Model.TranslKeys.emptyMandatoryField);
                        deferred.reject(transl.replace("[objProp]", objProp.objPropTransl.translForm));
                        return;
                    }
                }
                if (!obj.useLayouts || layoutState.saveRowAccessType == Enum.AccessType.Regular) {
                    saveFunc();
                }
                else {
                    deferred.reject(_this.application.language.transl(Model.TranslKeys.saveDisabledByLayout));
                }
            });
        }).fail(function (translationKey, status) {
            // This code is only executed when a default/layout fails to execute. We fix the queueItem object and reject without
            // giving additional info
            _this.updateQueueItemWithError(queueItem, translationKey, status);
            deferred.reject(queueItem.conflict, status.reason);
        });
    };
    ServerSync.prototype.executeDefaults = function (obj, row, layoutCause) {
        var deferred = jQuery.Deferred();
        if (obj.useDefaults) {
            RequestBuilderFactory.createOnlineRequestBuilder(this.application, obj.defaultObjView)
                .setCallbackMethod(function (reader, status) {
                if (!reader || !status.success) {
                    deferred.reject(Model.TranslKeys.syncErrorDefault, status);
                    return;
                }
                var item = reader.item(0);
                // Copy values to row
                var props = obj.objProps.array;
                for (var _i = 0, props_1 = props; _i < props_1.length; _i++) {
                    var prop = props_1[_i];
                    if (prop.useDefaultOutput) {
                        RowHelper.copyValue(prop, item.getValueForObjProp(prop), row);
                    }
                }
                deferred.resolve();
            })
                .executeDefault(this.convertToWebserviceContract(row, obj, true), layoutCause, Enum.ImportMode.Importing, null);
        }
        else {
            deferred.resolve();
        }
        return deferred.promise();
    };
    ServerSync.prototype.executeLayouts = function (obj, row, layoutCause) {
        var deferred = jQuery.Deferred();
        var layoutState = new LayoutState(obj.defaultObjView);
        if (obj.useLayouts) {
            RequestBuilderFactory.createOnlineRequestBuilder(this.application, obj.defaultObjView)
                .setCallbackMethod(function (reader, status) {
                if (!reader || !status.success) {
                    deferred.reject(Model.TranslKeys.syncErrorLayout, status);
                    return;
                }
                var item = reader.item(0);
                layoutState.translateLayoutState(item);
                deferred.resolve(layoutState);
            })
                .executeLayout(this.convertToWebserviceContract(row, obj, true), layoutCause, Enum.ImportMode.Importing);
        }
        else {
            deferred.resolve(layoutState);
        }
        return deferred.promise();
    };
    ServerSync.prototype.convertToWebserviceContract = function (row, obj, ignoreFileData) {
        return RowHelper.convertRowToWebserviceObject(obj, row, ignoreFileData, true);
    };
    ServerSync.prototype.updateIndexedDBIdentityValue = function (obj, pkIdentityProp, newPkValue, queueItem) {
        if (!this.db || this.disposed) {
            return;
        }
        var transaction = this.db.transaction(obj.objId, "readwrite"), store = transaction.objectStore(obj.objId);
        var oldPkValue = queueItem.pk[pkIdentityProp.objPropId];
        // Ai, we need to check every cursor here, use indexes when available in the shim
        var req = store.openCursor();
        req.onsuccess = function () {
            var cursor = req.result;
            if (cursor) {
                if (cursor.value[pkIdentityProp.objPropId] == oldPkValue) {
                    cursor.value[pkIdentityProp.objPropId] = newPkValue;
                    // Update the pk, because there might be a combined keyPath (when there are multiple pk's)
                    IndexedDb.addIndexes(obj, cursor.value);
                    ;
                    cursor.update(cursor.value);
                }
                else {
                    cursor["continue"]();
                }
            }
        };
    };
    ServerSync.prototype.updateMobileIdentityValue = function (oldValue, newValue, deferred) {
        if (!this.db || this.disposed) {
            deferred.reject("Tried to updateMobileIdentityValue but the database was already disposed.");
            return;
        }
        var identityTableName = IndexedDb.IndexedDbWrapper.identityTableName;
        var transaction = this.db.transaction(identityTableName, "readwrite"), identityStore = transaction.objectStore(identityTableName);
        var req = identityStore.openCursor(IDBKeyRange.only(oldValue));
        req.onsuccess = function (e) {
            var cursor = e.target.result;
            if (cursor) {
                var item = cursor.value;
                item.databaseId = newValue;
                cursor.update(item);
                deferred.resolve();
            }
            else {
                deferred.reject("Error updating identity: No cursor found");
            }
        };
        req.onerror = function (e) {
            deferred.reject("Error updating identity: " + e.message);
        };
    };
    ServerSync.prototype.updateQueue = function (queueItem, deferred, reason) {
        if (!this.db || this.disposed) {
            deferred.reject("Tried to updateQueue but the database was already disposed.");
            return;
        }
        var queueTableName = IndexedDb.IndexedDbWrapper.queueTableName;
        var transaction = this.db.transaction(queueTableName, "readwrite"), queueStore = transaction.objectStore(queueTableName);
        transaction.oncomplete = function () {
            if (queueItem.status == 2 /* Conflicted */) {
                deferred.reject(reason);
            }
            else {
                deferred.resolve();
            }
        };
        transaction.onerror = function () {
            // This is a problem, we couldn't update the cursor
            deferred.reject(transaction.error);
        };
        var req = queueStore.openCursor(IDBKeyRange.only(queueItem.tsf_id));
        req.onsuccess = function (e) {
            var cursor = e.target.result;
            if (cursor) {
                cursor.update(queueItem);
            }
            else {
                // This is really bad, updating the queue failed, this means we can't update the status and our queue will be corrupt
                deferred.reject("Couldn't find queue item " + queueItem.tsf_id);
            }
        };
    };
    ServerSync.prototype.dispose = function () {
        this.disposed = true;
        return jQuery.Deferred().resolve().promise();
    };
    ServerSync.prototype.newOfflineChangeAdded = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this.lastItemAdded = new Date();
                        return [4 /*yield*/, Utils.Preferences.setItem(ServerSync.OFFLINE_STORAGE_CONTAINS_CHANGES, true + "")];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    ServerSync.prototype.updateOfflineChangesInLocalstorage = function (syncDate) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!(this.lastItemAdded == null || moment(this.lastItemAdded).isBefore(syncDate))) return [3 /*break*/, 2];
                        return [4 /*yield*/, Utils.Preferences.setItem(ServerSync.OFFLINE_STORAGE_CONTAINS_CHANGES, false + "")];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2: return [2 /*return*/];
                }
            });
        });
    };
    // This isn't very nice but we can't open the database without loading the application first
    // or the upgrade path for the indexeddb will be very difficult with versioning
    // so keep track of offline changes in localstorage instead
    ServerSync.OFFLINE_STORAGE_CONTAINS_CHANGES = "offline_storage_contains_changes";
    return ServerSync;
}());
