///<reference path="ObjectModel.ts" />
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var BaseModelReader = /** @class */ (function () {
    function BaseModelReader(manager, tsfCache) {
        this.manager = manager;
        this.retriedAfterGetGuiCacheError = false;
        this.cachedProperties = ["model", "style", "codeFiles", "extendedProperties", "language", "syncGroups", "guiMsgList"];
        this.cachedGetGuiNames = [
            "get_gui_appl_lang", "get_gui_appl_transl", "get_gui_connection", "get_gui_global_settings",
            "get_gui_gui_appl", "get_gui_up_global"
        ];
        this.readGetGui = this.readGetGuiOnline;
        this.modelCacher = new IndexedDb.IndexedDbModelCacher(tsfCache);
    }
    BaseModelReader.prototype.initialize = function () {
        return __awaiter(this, void 0, void 0, function () {
            var loadedAppls;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, Utils.Preferences.getItem(BaseModelReader.savedObjectModelsKey)];
                    case 1:
                        loadedAppls = _a.sent();
                        this.cachedObjectModels = loadedAppls ? Utils.deserializeJSON(loadedAppls) : {};
                        return [2 /*return*/];
                }
            });
        });
    };
    BaseModelReader.prototype.artificiallySetGuidsChanged = function () {
        return __awaiter(this, void 0, void 0, function () {
            var guiApplId;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        for (guiApplId in this.cachedObjectModels) {
                            this.cachedObjectModels[guiApplId] += "-changed";
                        }
                        return [4 /*yield*/, Utils.Preferences.setItem(BaseModelReader.savedObjectModelsKey, JSON.stringify(this.cachedObjectModels))];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    BaseModelReader.prototype.readGetGuiOnline = function (getGuiName, params, readFunction) {
        var deferred = jQuery.Deferred();
        var that = this;
        this.manager.metaClient.invoke(getGuiName, "POST", params, function (reader, requestStatus) {
            if (requestStatus.success) {
                var applParam = params.get(BaseModelReader.guiApplParam);
                var key = applParam ? getGuiName + applParam : getGuiName;
                if (that.useApplCaching) {
                    if (that.cachedGetGuiNames.indexOf(getGuiName) === -1) {
                        readFunction(reader);
                        return deferred.resolve().promise();
                    }
                    that.modelCacher.storeGetGui(key, reader.serialize()).done(function () {
                        try {
                            readFunction(reader);
                            deferred.resolve();
                        }
                        catch (err) {
                            deferred.reject(err);
                        }
                    }).fail(function (err) {
                        deferred.reject("Saving Get GUI cache failed. " + err);
                    });
                }
                else {
                    try {
                        readFunction(reader);
                        deferred.resolve();
                    }
                    catch (err) {
                        deferred.reject(err);
                    }
                }
            }
            else {
                deferred.reject(requestStatus.messages);
            }
        });
        return deferred.promise();
    };
    BaseModelReader.prototype.readGetGuiFromStorage = function (getGuiName, params, readFunction) {
        var deferred = jQuery.Deferred();
        var applParam = params.get(BaseModelReader.guiApplParam);
        var key = applParam ? getGuiName + applParam : getGuiName;
        var req = this.modelCacher.getGetGuiCache(key);
        req.then(function (getGuiResult) {
            var items = IndexedDb.IndexedDbArrayReader.deserialize(getGuiResult);
            try {
                readFunction(items);
                deferred.resolve();
            }
            catch (err) {
                deferred.reject(err);
            }
        });
        req.fail(function (err) {
            deferred.reject("Error loading get_gui " + key + ", error: " + err);
        });
        return deferred.promise();
    };
    BaseModelReader.prototype.createUrlString = function (address) {
        //http://jsperf.com/js-startswith/6, substring seems to perform really well
        var success = "http";
        var url = "";
        if (address.substring(0, success.length) !== success) {
            url = "http://";
        }
        return url + address;
    };
    BaseModelReader.prototype.setApplicationCached = function (application) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!this.useApplCaching) return [3 /*break*/, 2];
                        this.cachedObjectModels[application.guiApplId] = application.guid;
                        return [4 /*yield*/, Utils.Preferences.setItem(BaseModelReader.savedObjectModelsKey, Utils.serializeJSON(this.cachedObjectModels))];
                    case 1:
                        _a.sent();
                        _a.label = 2;
                    case 2: return [2 /*return*/];
                }
            });
        });
    };
    BaseModelReader.prototype.removeUnusedLocalApplications = function () {
        var loadedModels = this.cachedObjectModels, appls = this.manager.appls, resolveList = [];
        for (var i in loadedModels) {
            var applFound = false;
            for (var j = 0; j < appls.length; j++) {
                if (appls[j].guiApplId == i) {
                    applFound = true;
                    break;
                }
            }
            if (!applFound) {
                resolveList.push(this.removeApplicationModelAndData(i));
            }
        }
        return jQuery.when.apply(undefined, resolveList);
    };
    BaseModelReader.prototype.removeApplicationModelAndData = function (applicationId) {
        var deferred = jQuery.Deferred();
        this.removeApplicationModel(applicationId).then(function () {
            IndexedDb.deleteDatabase(applicationId);
            LocalSync.resetSyncTokens(applicationId);
            deferred.resolve();
        }).catch(function () {
            deferred.reject("Error deleting application model.");
        });
        return deferred.promise();
    };
    BaseModelReader.getGuiIds = function () {
        return [
            "get_gui_layout",
            "get_gui_font",
            "get_gui_list_bar_item",
            //"get_gui_cube",
            //"get_gui_cube_field",
            //"get_gui_cube_view",
            //"get_gui_cube_view_field",
            //"get_gui_cube_view_field_conditional_layout",
            "get_gui_tab",
            "get_gui_col",
            "get_gui_tab_prefilter",
            "get_gui_tab_prefilter_col",
            "get_gui_ref",
            "get_gui_ref_col",
            "get_gui_elemnt",
            "get_gui_conditional_layout",
            "get_gui_conditional_layout_condition",
            "get_gui_task",
            "get_gui_task_parmtr",
            "get_gui_tab_task",
            "get_gui_tab_task_parmtr",
            "get_gui_tab_variant",
            "get_gui_tab_variant_col",
            "get_gui_tab_variant_conditional_layout",
            "get_gui_tab_variant_detail",
            "get_gui_tab_variant_look_up",
            "get_gui_tab_variant_prefilter",
            "get_gui_tab_variant_task",
            //"get_gui_tab_variant_cube",
            //"get_gui_tab_variant_cube_view",
            "get_gui_translations_full",
            "get_gui_translations_nav",
            "get_gui_offline_code_file",
            "get_gui_task_ref",
            "get_gui_task_ref_col",
            "get_gui_task_variant_look_up",
            "get_gui_process_flow",
            "get_gui_process_action",
            "get_gui_process_action_start_object",
            "get_gui_process_step"
        ];
    };
    //This removes the cached application model of the old way of caching.
    BaseModelReader.prototype.removeOldCachedApplicationModel = function (applicationId) {
        var resolveList = [];
        var getGuiIds = BaseModelReader.getGuiIds();
        for (var i = 0; i < getGuiIds.length; i++) {
            resolveList.push(this.modelCacher.removeGetGuiCache(getGuiIds[i] + applicationId));
        }
        return resolveList;
    };
    BaseModelReader.prototype.removeApplicationModel = function (applicationId) {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            return __generator(this, function (_a) {
                return [2 /*return*/, new Promise(function (resolve, reject) { return __awaiter(_this, void 0, void 0, function () {
                        var resolveList, getGuiIds, i;
                        return __generator(this, function (_a) {
                            switch (_a.label) {
                                case 0:
                                    resolveList = [];
                                    getGuiIds = this.cachedGetGuiNames.concat(this.cachedProperties);
                                    for (i = 0; i < getGuiIds.length; i++) {
                                        resolveList.push(this.modelCacher.removeGetGuiCache(getGuiIds[i] + applicationId));
                                    }
                                    if (!this.cachedObjectModels) return [3 /*break*/, 2];
                                    delete this.cachedObjectModels[applicationId];
                                    return [4 /*yield*/, Utils.Preferences.setItem(BaseModelReader.savedObjectModelsKey, Utils.serializeJSON(this.cachedObjectModels))];
                                case 1:
                                    _a.sent();
                                    _a.label = 2;
                                case 2:
                                    jQuery.when.apply(undefined, resolveList)
                                        .then(resolve)
                                        .fail(reject);
                                    return [2 /*return*/];
                            }
                        });
                    }); })];
            });
        });
    };
    BaseModelReader.prototype.LoadApplication = function (application, throwAwayExpiredModels) {
        var _this = this;
        if (throwAwayExpiredModels === void 0) { throwAwayExpiredModels = true; }
        var usingOnlineGetGui = (this.readGetGui === this.readGetGuiOnline);
        var deferred = jQuery.Deferred();
        this.checkUserMatchesCachedModel(application, throwAwayExpiredModels).then(function () {
            //Create another deferred so we can decide ourselves if we want to give an error
            //back or retry loading the application online.
            var deferred1 = _this.loadApplicationInternal(application, usingOnlineGetGui);
            deferred1.then(function () {
                deferred.resolve();
            }).fail(function (err) {
                //If loading from cache failed, the user is online and we haven't tried loading online yet,
                //we'll try loading the get_guis online.
                var wasReadingFromStorage = _this.readGetGui === _this.readGetGuiFromStorage;
                if (wasReadingFromStorage && !_this.retriedAfterGetGuiCacheError && AppConfig.online) {
                    _this.retriedAfterGetGuiCacheError = true;
                    _this.readGetGui = _this.readGetGuiOnline;
                    _this.removeApplicationModel(application.guiApplId).then(function () {
                        return _this.loadApplicationInternal(application, usingOnlineGetGui);
                    }).catch(function (err1) { return deferred.reject(err1); });
                }
                deferred.reject(err);
            });
        });
        return deferred.promise();
    };
    BaseModelReader.prototype.loadApplicationInternal = function (application, usingOnlineGetGui) {
        var _this = this;
        var deferred = jQuery.Deferred();
        var that = this;
        for (var key in Model.ConstructorMapping) {
            var namespaceElement = Model.ConstructorMapping[key];
            namespaceElement = Object.getPrototypeOf(namespaceElement).constructor;
        }
        this.resurrector = new Resurrect({
            resolver: new Resurrect.NamespaceResolver(Model.ConstructorMapping),
            cleanup: true
        });
        if (this.readGetGui === this.readGetGuiFromStorage) {
            var missingLocalStorageEntry = false;
            try {
                var resolveList = this.cachedProperties.map(function (cachedProp) {
                    var getGuiDeferred = jQuery.Deferred();
                    _this.modelCacher.getGetGuiCache(cachedProp + application.guiApplId).then(function (result) {
                        application[cachedProp] = that.resurrector.resurrect(result);
                        getGuiDeferred.resolve();
                    }).fail(function (err) {
                        //Properly reset the TSFApplication because else we get duplicated items in its properties.
                        application.model = new Model.TSFObjectModel(application);
                        delete application.style;
                        application.codeFiles = [];
                        application.extendedProperties = new Model.ExtendedProperties();
                        application.syncGroups = new Collections.List();
                        missingLocalStorageEntry = true;
                        getGuiDeferred.resolve();
                    });
                    return getGuiDeferred.promise();
                });
                return jQuery.when.apply(undefined, resolveList).then(function () {
                    if (!missingLocalStorageEntry) {
                        return application.initializeDatabase();
                    }
                    //this is supposed to be executed only the first time
                    //the new way of model caching is ran on a device (for migration).
                    if (AppConfig.online) {
                        that.readGetGui = that.readGetGuiOnline;
                    }
                    return _this.executeGetGuis(application, true);
                });
            }
            catch (err) {
                return deferred.reject(err).promise();
            }
        }
        else {
            return this.executeGetGuis(application, false);
        }
    };
    BaseModelReader.prototype.executeGetGuis = function (application, migrateFromOldWayOfCaching) {
        var _this = this;
        var that = this;
        var deferred = jQuery.Deferred();
        // Get GUI's to read from...
        // DP stands for dependency
        var fillHelper = new ModelFillHelper();
        fillHelper.translationList[4 /* Msg */] = application.guiMsgList;
        var defaultParams = this.getDefaultParameters(application);
        // get_gui_start_objects?(not yet)
        var resolveList = [];
        resolveList.push(this.loadFont(application, defaultParams, fillHelper));
        resolveList.push(this.loadTranslationsFull(application, defaultParams, fillHelper));
        resolveList.push(this.loadTranslationsNav(application, defaultParams, fillHelper));
        resolveList.push(this.loadScreenComponent(application, defaultParams));
        this.executeGetGuiGroup(resolveList).then(function () {
            var resolveList = [];
            resolveList.push(_this.loadListbarItem(application, defaultParams, fillHelper)); // DP: None
            resolveList.push(_this.loadTile(application, defaultParams, fillHelper)); // DP: None
            resolveList.push(_this.loadElement(application, defaultParams, fillHelper)); // DP: None
            resolveList.push(_this.loadTab(application, defaultParams, fillHelper)); // DP: None
            resolveList.push(_this.loadTask(application, defaultParams, fillHelper)); // DP: None
            resolveList.push(_this.loadScreenComponentProp(application, defaultParams)); // DP: ScreenComponent
            resolveList.push(_this.loadExtendedProperties(application, _this.getExtendedPropertiesParameters(application), fillHelper)); // DP: None
            return _this.executeGetGuiGroup(resolveList);
        }).then(function () {
            var resolveList = [];
            //resolveList.push(this.loadCube(application, defaultParams, fillHelper));
            resolveList.push(_this.loadOfflineCodeFile(application, defaultParams));
            resolveList.push(_this.loadTabVariant(application, defaultParams, fillHelper)); // DP: loadTab
            resolveList.push(_this.loadCol(application, defaultParams, fillHelper)); // DP: loadTab
            resolveList.push(_this.loadTabPrefilter(application, defaultParams, fillHelper)); // DP: loadTab
            resolveList.push(_this.loadTaskParameter(application, defaultParams, fillHelper)); // DP: loadTask
            resolveList.push(_this.loadPlatform(application, defaultParams, fillHelper)); //DP: loadListbarItem, loadTile
            return _this.executeGetGuiGroup(resolveList);
        }).then(function () {
            var resolveList = [];
            //resolveList.push(this.loadCubeField(application, defaultParams, fillHelper));
            resolveList.push(_this.loadGuiTheme(application, defaultParams, fillHelper)); // DP: Fonts, Platform
            resolveList.push(_this.loadRef(application, defaultParams, fillHelper)); // DP: loadTab
            resolveList.push(_this.loadTabPrefilterCol(application, defaultParams)); // DP: loadTabPrefilter
            resolveList.push(_this.loadTaskVariant(application, defaultParams, fillHelper)); // DP: loadTaskParameter
            resolveList.push(_this.loadConditionalLayout(application, defaultParams, fillHelper)); // DP: loadCol + fonts
            return _this.executeGetGuiGroup(resolveList);
        }).then(function () {
            var resolveList = [];
            //resolveList.push(this.loadCubeView(application, defaultParams, fillHelper));
            resolveList.push(_this.loadTabTask(application, defaultParams, fillHelper)); // DP: loadTaskParameter
            resolveList.push(_this.loadRefCol(application, defaultParams, fillHelper)); // DP: loadTab, loadRef
            resolveList.push(_this.loadConditionalLayoutCondition(application, defaultParams)); // DP: loadConditionalLayouts
            return _this.executeGetGuiGroup(resolveList);
        }).then(function () {
            var resolveList = [];
            resolveList.push(_this.loadTabVariantTask(application, defaultParams, fillHelper)); // DP: loadTabVariant
            //resolveList.push(this.loadCubeViewField(application, defaultParams, fillHelper));
            resolveList.push(_this.loadTabVariantDetail(application, defaultParams, fillHelper)); // DP: loadTabVariant
            return _this.executeGetGuiGroup(resolveList);
        }).then(function () {
            var resolveList = [];
            resolveList.push(_this.loadTabTaskParameter(application, defaultParams)); // DP: loadTabTask
            resolveList.push(_this.loadTabVariantCol(application, defaultParams, fillHelper)); // DP: loadTabVariant
            resolveList.push(_this.loadTaskVariantParmtr(application, defaultParams, fillHelper)); // DP: loadTabVariant
            resolveList.push(_this.loadTaskRef(application, defaultParams, fillHelper));
            return _this.executeGetGuiGroup(resolveList);
        }).then(function () {
            var resolveList = [];
            //resolveList.push(this.loadCubeVieWFieldConditionalLayout(application, defaultParams, fillHelper)); // DP: loadCubeViewField
            resolveList.push(_this.loadTabVariantConditionalLayout(application, defaultParams, fillHelper)); // DP: loadTabVariant
            resolveList.push(_this.loadTabVariantLookUp(application, defaultParams, fillHelper)); // DP: loadTabVariant
            resolveList.push(_this.loadTabVariantPrefilter(application, defaultParams, fillHelper)); // DP: loadTabVariant
            resolveList.push(_this.loadTaskRefCol(application, defaultParams, fillHelper)); // DP: loadTaskRef
            return _this.executeGetGuiGroup(resolveList);
        }).then(function () {
            var resolveList = [];
            resolveList.push(_this.loadTaskVariantLookUp(application, defaultParams, fillHelper)); // DP: loadTaskVariant
            resolveList.push(_this.loadProcessFlow(application, defaultParams, fillHelper));
            //resolveList.push(this.loadTabVariantCube(application, defaultParams, fillHelper));
            return _this.executeGetGuiGroup(resolveList);
        }).then(function () {
            var resolveList = [];
            resolveList.push(_this.loadProcessAction(application, defaultParams, fillHelper));
            //resolveList.push(this.loadTabVariantCubeView(application, defaultParams, fillHelper));
            return _this.executeGetGuiGroup(resolveList);
        }).then(function () {
            return _this.loadProcessActionStartObject(application, defaultParams, fillHelper);
        }).then(function () {
            return _this.loadProcessStep(application, defaultParams, fillHelper);
        }).then(function () {
            try {
                _this.resolveAllKeys(application, fillHelper);
            }
            catch (error) {
                deferred.reject(error);
                return deferred.promise();
            }
            // Resolve model after resolving all keys - because that one resolves lookups as well which we need to
            // know when resolving the model
            var resolver = new ModelResolver();
            resolver.resolve(application, fillHelper);
            // TODO clean Domains list in model to remove unused objects. also referenced by props.
            application.isLoaded = true;
            if (_this.readGetGui === _this.readGetGuiOnline || migrateFromOldWayOfCaching) {
                if (!_this.useApplCaching) {
                    return;
                }
                try {
                    //Remove the old cached model if we've migrated from the old way of caching.
                    var resolveList_1 = migrateFromOldWayOfCaching ?
                        that.removeOldCachedApplicationModel(application.guiApplId) : [];
                    _this.cachedProperties.forEach(function (cachedProp) {
                        resolveList_1.push(that.modelCacher.storeGetGui(cachedProp + application.guiApplId, that.resurrector.stringify(application[cachedProp])));
                    });
                    jQuery.when.apply(undefined, resolveList_1).then(function () {
                        _this.setApplicationCached(application);
                    });
                }
                catch (err) {
                    return deferred.reject(err).promise();
                }
            }
            if (_this.readGetGui === _this.readGetGuiFromStorage) {
                _this.readGetGui = _this.readGetGuiOnline;
            }
        }).then(function () {
            application.initializeDatabase().then(deferred.resolve).fail(deferred.reject);
        }).fail(function (err) {
            deferred.reject(err);
        });
        return deferred.promise();
    };
    BaseModelReader.prototype.switchToOffline = function () {
        var _this = this;
        var deferred = jQuery.Deferred();
        // We cant check the getGuiCache anymore because we append the applId to the get_gui name
        // So we check the cachedObjectModels instead
        //this.modelCacher.getGetGuiCache("objs").then((response) =>
        if (!AppConfig.online) {
            this.initialize().then(function () {
                var hasOfflineAppls = false;
                for (var a in _this.cachedObjectModels) {
                    hasOfflineAppls = true;
                    break;
                }
                if (hasOfflineAppls) {
                    _this.readGetGui = _this.readGetGuiFromStorage;
                    deferred.resolve();
                }
                else {
                    deferred.reject(Model.TranslKeys.noServerConnection);
                }
            }).catch(deferred.reject);
        }
        else {
            this.readGetGui = this.readGetGuiOnline;
            deferred.resolve();
        }
        return deferred.promise();
    };
    BaseModelReader.prototype.checkUserMatchesCachedModel = function (application, throwAwayExpiredModels) {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            return __generator(this, function (_a) {
                if (!throwAwayExpiredModels) {
                    return [2 /*return*/, Promise.resolve()];
                }
                return [2 /*return*/, new Promise(function (resolve, reject) { return __awaiter(_this, void 0, void 0, function () {
                        var cachedGuid, latestSignedInUsername;
                        return __generator(this, function (_a) {
                            switch (_a.label) {
                                case 0:
                                    cachedGuid = this.cachedObjectModels[application.guiApplId];
                                    if (!(this.useApplCaching && cachedGuid && application.guid === cachedGuid)) return [3 /*break*/, 2];
                                    return [4 /*yield*/, Utils.Preferences.getItem(BaseModelReader.latestSignedInUsernameKey)];
                                case 1:
                                    latestSignedInUsername = _a.sent();
                                    if (Utils.caseInsenstiveStringCompare(this.manager.user.username, latestSignedInUsername)) {
                                        this.readGetGui = this.readGetGuiFromStorage;
                                        resolve();
                                    }
                                    else {
                                        this.readGetGui = this.readGetGuiOnline;
                                        this.removeApplicationModelAndData(application.guiApplId).always(function () {
                                            resolve();
                                        });
                                    }
                                    return [3 /*break*/, 3];
                                case 2:
                                    this.readGetGui = this.readGetGuiOnline;
                                    // Remove the application as this new version might have new tables or indexes
                                    this.removeApplicationModelAndData(application.guiApplId).always(function () {
                                        resolve();
                                    });
                                    _a.label = 3;
                                case 3: return [2 /*return*/];
                            }
                        });
                    }); })];
            });
        });
    };
    BaseModelReader.prototype.executeGetGuiGroup = function (resolveList) {
        return jQuery.when.apply(undefined, resolveList);
    };
    BaseModelReader.prototype.loadGuiTheme = function (application, params, fillHelper) {
        return this.readGetGui("get_gui_theme", params, function (reader) {
            application.style = BaseModelReader.getTheme(reader, fillHelper.fonts);
        });
    };
    BaseModelReader.getTheme = function (reader, fonts) {
        // For Mobile, get_gui_theme always returns at most one result.
        // The only possible exception is a 2018.1 SF metasource which is missing hotfixes.
        if (reader && reader.length > 0) {
            var item = reader.item(0);
            var currentThemeId = item.getString("theme_id");
            return this.getStyle(currentThemeId, item, fonts || {});
        }
    };
    BaseModelReader.getStyle = function (themeId, item, fonts) {
        var style = new Model.ApplicationStyle(themeId);
        if (Model.TSFApplicationManager.Current.globalSettings.currentSFVersion >= 1 /* V2017_1 */) {
            style.mandatoryFieldGradient = BaseModelReader.readGradient(item, "form_mand_field_background_gradient_color_1", "form_mand_field_background_gradient_color_2", "form_mand_field_background_gradient_direction");
            style.readonlyFieldGradient = BaseModelReader.readGradient(item, "form_read_only_field_background_gradient_color_1", "form_read_only_field_background_gradient_color_2", "form_read_only_field_background_gradient_direction");
            style.regularFieldGradient = BaseModelReader.readGradient(item, "form_editable_field_background_gradient_color_1", "form_editable_field_background_gradient_color_2", "form_editable_field_background_gradient_direction");
        }
        else {
            style.mandatoryFieldGradient = BaseModelReader.readGradient(item, "document_mand_field_background_gradient_color_1", "document_mand_field_background_gradient_color_2", "document_mand_field_background_gradient_direction");
            style.readonlyFieldGradient = BaseModelReader.readGradient(item, "document_read_only_field_background_gradient_color_1", "document_read_only_field_background_gradient_color_2", "document_read_only_field_background_gradient_direction");
            style.regularFieldGradient = BaseModelReader.readGradient(item, "document_editable_field_background_gradient_color_1", "document_editable_field_background_gradient_color_2", "document_editable_field_background_gradient_direction");
        }
        style.pageHeader = new Model.PageBarLayout();
        style.pageHeader.backgroundGradient = BaseModelReader.readGradient(item, "header_background_gradient_color_1", "header_background_gradient_color_2", "header_background_gradient_direction");
        //Button background gets taken over by the page header background.
        var pageHeaderBackground = style.pageHeader.backgroundGradient;
        style.button.activeGradient = style.button.inActiveGradient = style.button.mouseOverGradient = pageHeaderBackground;
        var pageHeaderFontId = item.getString("header_font_id");
        if (pageHeaderFontId) {
            style.pageHeader.font = fonts[pageHeaderFontId];
            //Button font gets taken over by the page header font.
            style.button.font = style.pageHeader.font;
        }
        style.pageFooter = new Model.PageBarLayout();
        style.pageFooter.backgroundGradient = BaseModelReader.readGradient(item, "footer_background_gradient_color_1", "footer_background_gradient_color_2", "footer_background_gradient_direction");
        var pageFooterFontId = item.getString("footer_font_id");
        if (pageFooterFontId) {
            style.pageFooter.font = fonts[pageFooterFontId];
        }
        style.grid.backColor = item.getNullableNumber("grid_background_color");
        if (style.grid.backColor) {
            //style.grid.alternateBackColor = null;
        }
        style.grid.activeRowGradient = BaseModelReader.readGradient(item, "grid_row_active_background_gradient_color_1", "grid_row_active_background_gradient_color_2", "grid_row_active_background_gradient_direction");
        style.grid.rowMouseOverGradient = BaseModelReader.readGradient(item, "grid_row_mouse_over_background_gradient_color_1", "grid_row_mouse_over_background_gradient_color_2", "grid_row_mouse_over_background_gradient_direction");
        style.documentBackgroundColor = item.getNullableNumber("document_background_gradient_color_1");
        style.popupBackgroundColor = item.getNullableNumber("popup_background_gradient_color_1");
        var fontId = item.getString("list_bar_header_font_id");
        if (fontId) {
            style.quickbar.headerFont = fonts[fontId];
        }
        fontId = item.getString("list_bar_header_mouse_over_font_id");
        if (fontId) {
            style.quickbar.headerMouseoverFont = fonts[fontId];
        }
        fontId = item.getString("list_bar_item_font_id");
        if (fontId) {
            style.quickbar.itemFont = fonts[fontId];
        }
        fontId = item.getString("list_bar_item_mouse_over_font_id");
        if (fontId) {
            style.quickbar.itemMouseoverFont = fonts[fontId];
        }
        style.quickbar.headerGradient = BaseModelReader.readGradient(item, "list_bar_header_gradient_color_1", "list_bar_header_gradient_color_2", "list_bar_header_gradient_direction");
        style.quickbar.headerMouseoverGradient = BaseModelReader.readGradient(item, "list_bar_header_mouse_over_gradient_color_1", "list_bar_header_mouse_over_gradient_color_2", "list_bar_header_mouse_over_gradient_direction");
        style.quickbar.itemGradient = BaseModelReader.readGradient(item, "list_bar_item_gradient_color_1", "list_bar_item_gradient_color_2", "list_bar_item_gradient_direction");
        style.quickbar.itemMouseoverGradient = BaseModelReader.readGradient(item, "list_bar_item_mouse_over_gradient_color_1", "list_bar_item_mouse_over_gradient_color_2", "list_bar_item_mouse_over_gradient_direction");
        style.quickbar.pictureFooter = item.getString("list_bar_picture_footer");
        style.quickbar.showListBarItemIcons = item.getBool("list_bar_show_list_bar_item_icons");
        style.tiles.tileGroupFont = fonts[item.getString("tile_grp_font_id")];
        style.tiles.tileGroupBackgroundColor = item.getNullableNumber("tile_grp_background_color");
        style.tiles.tileGroupBorderColor = item.getNullableNumber("tile_grp_border_color");
        style.tiles.tileGroupMouseOverBackgroundColor = item.getNullableNumber("tile_grp_mouse_over_background_color");
        style.tiles.tileFont = fonts[item.getString("tile_font_id")];
        style.tiles.tileBorderColor = item.getNullableNumber("tile_border_color");
        style.tiles.tileGradient = BaseModelReader.readGradient(item, "tile_background_gradient_color_1", "tile_background_gradient_color_2", "tile_background_gradient_direction");
        style.tiles.tileMouseOverFont = fonts[item.getString("tile_mouse_over_font_id")];
        style.tiles.tileMouseOverBorderColor = item.getNullableNumber("tile_mouse_over_border_color");
        style.tiles.tileMouseOverGradient = BaseModelReader.readGradient(item, "tile_mouse_over_background_gradient_color_1", "tile_mouse_over_background_gradient_color_2", "tile_mouse_over_background_gradient_direction");
        style.tab.activeGradient = BaseModelReader.readGradient(item, "tab_active_background_gradient_color_1", "tab_active_background_gradient_color_2", "tab_active_background_gradient_direction");
        style.tab.inActiveGradient = BaseModelReader.readGradient(item, "tab_inactive_background_gradient_color_1", "tab_inactive_background_gradient_color_2", "tab_inactive_background_gradient_direction");
        style.tab.mouseOverGradient = BaseModelReader.readGradient(item, "tab_mouse_over_background_gradient_color_1", "tab_mouse_over_background_gradient_color_2", "tab_mouse_over_background_gradient_direction");
        style.tab.lineColor = item.getNullableNumber("tab_line_color");
        var labelFontId = item.getString("form_label_font_id");
        style.label.font = fonts[labelFontId];
        var labelActiveFontId = item.getString("form_label_active_font_id");
        style.labelActive.font = fonts[labelActiveFontId];
        style.badgeLayout.color = item.getNullableNumber("badge_background_color");
        var badgeFontId = item.getString("badge_font_id");
        style.badgeLayout.labelFont = fonts[badgeFontId];
        // Stop iterating when a layout is loaded.
        return style;
    };
    BaseModelReader.prototype.loadOfflineCodeFile = function (application, params) {
        return this.readGetGui("get_gui_offline_code_file", params, function (reader) {
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                var codeFileId = item.getString("code_file_id");
                var code = item.getString("code_file_generated_code");
                var codeFile = new Model.CodeFile(codeFileId, code);
                application.codeFiles.push(codeFile);
            }
        });
    };
    BaseModelReader.prototype.loadListbarItem = function (application, params, fillHelper) {
        return this.readGetGui("get_gui_list_bar_item", params, function (reader) {
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                var menuId = item.getString("menu_id");
                var menu = application.model.listbarMenus.key(menuId);
                if (!menu) {
                    menu = new Model.ListbarMenu(menuId);
                    application.model.listbarMenus.add(menuId, menu);
                }
                var tabId = item.getString("tab_id");
                var taskId = item.getString("task_id");
                var openEntity = void 0;
                var transl = void 0;
                if (tabId) {
                    var configId = item.getString("tab_variant_id");
                    openEntity = new Model.OpenEntity(tabId, configId, 0 /* Obj */);
                    if (configId && item.getBool("tab_variant_transl")) {
                        transl = fillHelper.findTranslation(configId, 281 /* TabVariant */);
                    }
                    else {
                        transl = fillHelper.findTranslation(tabId, 0 /* Tab */);
                    }
                }
                else if (taskId) {
                    var configId = item.getString("task_variant_id");
                    openEntity = new Model.OpenEntity(taskId, configId, 1 /* Task */);
                    if (configId && item.getBool("task_variant_transl")) {
                        transl = fillHelper.findTranslation(configId, 291 /* TaskVariant */);
                    }
                    else {
                        transl = fillHelper.findTranslation(taskId, 11 /* Task */);
                    }
                }
                else
                    continue;
                var listbarGroupId = item.getString("list_bar_grp_id");
                var group = menu.groups.key(listbarGroupId);
                if (!group) {
                    group = new Model.ListbarGroup(listbarGroupId);
                    group.grpTransl = fillHelper.findTranslation(listbarGroupId, 12 /* ListBar */);
                    group.defaultExpanded = item.getBool("default_expanded");
                    group.icon = item.getString("list_bar_grp_icon");
                    menu.groups.add(listbarGroupId, group);
                }
                var listbarItemId = item.getString("list_bar_item_id");
                var listBarItem = new Model.ListbarItem(listbarItemId);
                listBarItem.icon = item.getString("icon");
                listBarItem.itemTransl = transl;
                listBarItem.entity = openEntity;
                // TODO: Why is this part of the OpenEntity and not delegated to the underlying tab/task/variant
                listBarItem.entity.showBadge = item.getBool("show_badge");
                if (listBarItem) {
                    listBarItem.entity.badgeInterval = item.getNumber("badge_interval", 0);
                }
                group.items.push(listBarItem);
            }
        });
    };
    BaseModelReader.prototype.loadTile = function (application, params, fillHelper) {
        return this.readGetGui("get_gui_tile", params, function (reader) {
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                var menuId = item.getString("menu_id");
                var menu = application.model.tileMenus.key(menuId);
                if (!menu) {
                    menu = new Model.TilesMenu(menuId);
                    application.model.tileMenus.add(menuId, menu);
                }
                var tabId = item.getString("tab_id");
                var taskId = item.getString("task_id");
                var openEntity = void 0;
                var transl = void 0;
                if (tabId) {
                    var configId = item.getString("tab_variant_id");
                    openEntity = new Model.OpenEntity(tabId, configId, 0 /* Obj */);
                    if (configId && item.getBool("tab_variant_transl")) {
                        transl = fillHelper.findTranslation(configId, 281 /* TabVariant */);
                    }
                    else {
                        transl = fillHelper.findTranslation(tabId, 0 /* Tab */);
                    }
                }
                else if (taskId) {
                    var configId = item.getString("task_variant_id");
                    openEntity = new Model.OpenEntity(taskId, configId, 1 /* Task */);
                    if (configId && item.getBool("task_variant_transl")) {
                        transl = fillHelper.findTranslation(configId, 291 /* TaskVariant */);
                    }
                    else {
                        transl = fillHelper.findTranslation(taskId, 11 /* Task */);
                    }
                }
                else {
                    continue;
                }
                var tileGroupId = item.getString("tile_grp_id");
                var group = menu.groups.key(tileGroupId);
                if (!group) {
                    group = new Model.TileGroup(tileGroupId);
                    group.grpTransl = fillHelper.findTranslation(tileGroupId, 87 /* TileGroup */);
                    group.orderNo = item.getNumber("tile_grp_abs_order_no");
                    menu.groups.add(tileGroupId, group);
                }
                var tileId = item.getString("tile_id");
                var tile = new Model.Tile(tileId);
                tile.icon = item.getString("icon");
                tile.itemTransl = transl;
                tile.entity = openEntity;
                tile.orderNo = item.getNumber("abs_order_no");
                // TODO: Why is this part of the OpenEntity and not delegated to the underlying tab/task/variant
                tile.entity.showBadge = item.getBool("show_badge");
                if (tile) {
                    tile.entity.badgeInterval = item.getNumber("badge_interval", 0);
                }
                group.items.push(tile);
            }
        });
    };
    BaseModelReader.prototype.loadCube = function (application, params, fillHelper) {
        return this.readGetGui("get_gui_cube", params, function (reader) {
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                var cubeId = item.getString("cube_id");
                var obj = application.model.getObj(cubeId);
                if (!obj)
                    continue;
                var defaultCubeViewId = item.getString("def_cube_view_id");
                var cube = new Model.Cube(obj, defaultCubeViewId);
                cube.cubeTransl = fillHelper.findTranslation(cubeId, 24 /* Cube */);
                obj.cube = cube;
                obj.defaultObjView.objViewCube = new Model.ObjViewCube(obj.defaultObjView, cube);
            }
            fillHelper.containsCubes = reader.length > 0;
        });
    };
    BaseModelReader.prototype.loadCubeField = function (application, params, fillHelper) {
        if (!fillHelper.containsCubes) {
            return jQuery.Deferred().resolve().promise();
        }
        return this.readGetGui("get_gui_cube_field", params, function (reader) {
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                var cubeId = item.getString("cube_id");
                var obj = application.model.getObj(cubeId);
                if (!obj)
                    continue;
                var cube = obj.cube;
                if (!cube)
                    continue;
                var cubeFieldId = item.getString("cube_field_id");
                var cubeField = new Model.CubeField(cube, cubeFieldId);
                cubeField.fieldType = item.getNumber("cube_field_type");
                cubeField.colId = item.getString("col_id");
                cubeField.cubeFieldTransl = fillHelper.findTranslation(cubeId, 25 /* CubeField */);
                cube.cubeFields.add(cubeFieldId, cubeField);
            }
        });
    };
    BaseModelReader.prototype.loadCubeView = function (application, params, fillHelper) {
        if (!fillHelper.containsCubes) {
            return jQuery.Deferred().resolve().promise();
        }
        return this.readGetGui("get_gui_cube_view", params, function (reader) {
            var groupCache = {};
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                var cubeId = item.getString("cube_id");
                var obj = application.model.getObj(cubeId);
                if (!obj)
                    continue;
                var cube = obj.cube;
                if (!cube)
                    continue;
                var cubeViewId = item.getString("cube_view_id");
                var cubeView = new Model.CubeView(cube, cubeViewId);
                cubeView.chartType = item.getNumber("chart_type");
                cubeView.defaultView = item.getNumber("default_cube_view_type");
                cubeView.cubeViewTransl = fillHelper.findTranslation(cubeViewId, 26 /* CubeView */);
                cubeView.showColGrandTotal = item.getBool("show_col_grand_total");
                cubeView.showRowGrandTotal = item.getBool("show_row_grand_total");
                var groupId = item.getString("cube_view_grp_id");
                if (groupId) {
                    if (!groupCache[groupId]) {
                        var group = new Model.ActionGroup();
                        group.groupId = groupId;
                        group.groupIcon = item.getString("cube_view_grp_icon");
                        group.grpTransl = fillHelper.findTranslation(groupId, 27 /* CubeViewGrp */);
                        group.groupHasSubmenu = item.getBool("grp_has_sub_menu");
                        groupCache[groupId] = group;
                    }
                    cubeView.group = groupCache[groupId];
                }
                cube.allCubeViews.add(cubeViewId, cubeView);
                obj.defaultObjView.objViewCube.addCubeView(cubeView);
            }
        });
    };
    BaseModelReader.prototype.loadCubeViewField = function (application, params, fillHelper) {
        if (!fillHelper.containsCubes) {
            return jQuery.Deferred().resolve().promise();
        }
        return this.readGetGui("get_gui_cube_view_field", params, function (reader) {
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                var cubeId = item.getString("cube_id");
                var obj = application.model.getObj(cubeId);
                if (!obj)
                    continue;
                var cube = obj.cube;
                if (!cube)
                    continue;
                var cubeViewId = item.getString("cube_view_id");
                var cubeView = cube.allCubeViews.key(cubeViewId);
                if (!cubeView)
                    continue;
                var cubeViewFieldId = item.getString("cube_field_id");
                var cubeField = cube.cubeFields.object[cubeViewFieldId];
                if (!cubeField)
                    continue;
                var cubeViewField = new Model.CubeViewField(cube, cubeView, cubeField);
                cubeViewField.cubeArea = item.getNumber("cube_area");
                cubeView.cubeViewFields.add(cubeViewFieldId, cubeViewField);
            }
        });
    };
    BaseModelReader.prototype.loadCubeVieWFieldConditionalLayout = function (application, params, fillHelper) {
        if (!fillHelper.containsCubes) {
            return jQuery.Deferred().resolve().promise();
        }
        return this.readGetGui("get_gui_cube_view_field_conditional_layout", params, function (reader) {
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                var cubeId = item.getString("cube_id");
                var obj = application.model.getObj(cubeId);
                if (!obj)
                    continue;
                var cube = obj.cube;
                if (!cube)
                    continue;
                var cubeViewId = item.getString("cube_view_id");
                var cubeView = cube.allCubeViews.key(cubeViewId);
                if (!cubeView)
                    continue;
                var cubeFieldId = item.getString("cube_field_id");
                var cubeViewField = cubeView.cubeViewFields.object[cubeFieldId];
                if (!cubeViewField)
                    continue;
                var conditionalLayoutId = item.getString("conditional_layout_id");
                // Prefix the conditional layout id because it will be merged with the normal conditional layouts in the Stylesheet generator.
                var conditionalLayout = new Model.CubeViewFieldConditionalLayout("tsf_cl_cube_" + cubeViewId + "_" + cubeFieldId + "_" + conditionalLayoutId);
                conditionalLayout.numericCondition = item.getNumber("numeric_condition");
                conditionalLayout.value = item.getString("value");
                conditionalLayout.untilValue = item.getString("until_value");
                conditionalLayout.applyToCell = item.getBool("apply_to_cell");
                conditionalLayout.applyToTotalCell = item.getBool("apply_to_total_cell");
                conditionalLayout.applyToCustomTotalCell = item.getBool("apply_to_custom_total_cell");
                conditionalLayout.applyToGrandTotalCell = item.getBool("apply_to_grand_total_cell");
                var tmpBackColor = item.getNullableNumber("background_color_1");
                if (tmpBackColor) {
                    conditionalLayout.backGroundColor1 = Utils.Color.hexFromArgb(tmpBackColor);
                }
                var fontId = item.getString("font_id");
                if (fontId) {
                    conditionalLayout.font = fillHelper.fonts[fontId];
                }
                cubeViewField.conditionalLayouts.push(conditionalLayout);
            }
        });
    };
    BaseModelReader.prototype.loadTabVariantCube = function (application, params, fillHelper) {
        if (!fillHelper.containsCubes) {
            return jQuery.Deferred().resolve().promise();
        }
        return this.readGetGui("get_gui_tab_variant_cube", params, function (reader) {
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                var cubeId = item.getString("cube_id");
                var objViewId = item.getString("tab_variant_id");
                if (!fillHelper.objViews[cubeId])
                    continue;
                var objView = fillHelper.objViews[cubeId][objViewId];
                if (!objView)
                    continue;
                objView.objViewCube = new Model.ObjViewCube(objView, objView.obj.cube);
                objView.objViewCube.defaultCubeViewId = item.getString("def_cube_view_id");
            }
        });
    };
    BaseModelReader.prototype.loadTabVariantCubeView = function (application, params, fillHelper) {
        if (!fillHelper.containsCubes) {
            return jQuery.Deferred().resolve().promise();
        }
        return this.readGetGui("get_gui_tab_variant_cube_view", params, function (reader) {
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                var cubeId = item.getString("cube_id");
                var objViewId = item.getString("tab_variant_id");
                if (!fillHelper.objViews[cubeId])
                    continue;
                var objView = fillHelper.objViews[cubeId][objViewId];
                if (!objView || !objView.objViewCube)
                    continue;
                var allCubeViews = objView.objViewCube.cube.allCubeViews;
                var cubeView = allCubeViews.key(item.getString("cube_view_id"));
                objView.objViewCube.addCubeView(cubeView);
            }
        });
    };
    BaseModelReader.prototype.loadTab = function (application, params, fillHelper) {
        var _this = this;
        return this.readGetGui("get_gui_tab", params, function (reader) {
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                var objId = item.getString("tab_id");
                var obj = new Model.Obj(objId);
                obj.lookupDisplay.lookUpDisplayColId = item.getString("look_up_display_col_id");
                obj.useLayoutsAddButton = item.getBool("use_layouts_add_button_type");
                obj.useLayoutsCancelButton = item.getBool("use_layouts_cancel_button_type");
                obj.useLayoutsConfirmButton = item.getBool("use_layouts_confirm_button_type");
                obj.useLayoutsDeleteButton = item.getBool("use_layouts_delete_button_type");
                obj.useLayoutsUpdateButton = item.getBool("use_layouts_update_button_type");
                obj.useLayouts = item.getBool("use_layouts");
                obj.useDefaults = item.getBool("use_defaults");
                obj.useContexts = item.getBool("use_contexts");
                var groupId = void 0;
                if (Model.TSFApplicationManager.Current.globalSettings.currentSFVersion >= 2 /* V2018_1 */) {
                    groupId = item.getString("refresh_grp_id");
                    obj.offlineRefreshInterval = item.getNumber("refresh_grp_after_interval");
                }
                else {
                    groupId = item.getString("refresh_group_id");
                    obj.offlineRefreshInterval = item.getNumber("refresh_group_after_interval");
                }
                if (groupId) {
                    obj.syncGroupId = groupId;
                    var syncGroup = application.syncGroups.key(groupId);
                    if (!syncGroup) {
                        // Create the group, but don't add it here
                        // we do it later when we also know the objviews
                        syncGroup = new SyncGroup(groupId, obj.offlineRefreshInterval);
                        syncGroup.transl = fillHelper.findTranslation(groupId, 40 /* RefreshGroup */);
                        if (Model.TSFApplicationManager.Current.globalSettings.currentSFVersion >= 2 /* V2018_1 */) {
                            syncGroup.atomic = item.getBool("atomic_grp");
                        }
                        else {
                            syncGroup.atomic = item.getBool("atomic_group");
                        }
                        application.syncGroups.add(groupId, syncGroup);
                    }
                }
                _this.createObjView(application, item, obj.defaultObjView);
                obj.defaultObjView.objTransl = fillHelper.findTranslation(obj.objId, 0 /* Tab */);
                application.model.objs.add(objId, obj);
            }
        });
    };
    BaseModelReader.prototype.readXmlDttp = function (row, colId) {
        // previously we would always get date (for date, datetime and time); this changed recently
        // but breaks the checks so we normalize these back to date
        // This might seem incorrect (as time isn't a date) but the actual responses didn't change and so it is still always a full datetime
        var xmlDttp = row.getString(colId);
        if (xmlDttp) {
            switch (xmlDttp.toLowerCase()) {
                case 'time':
                case 'datetime':
                    return 'date';
            }
        }
        return xmlDttp;
    };
    BaseModelReader.prototype.loadCol = function (application, params, fillHelper) {
        var _this = this;
        return this.readGetGui("get_gui_col", params, function (reader) {
            var model = application.model;
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                var objId = item.getString("tab_id");
                var obj = model.getObj(objId);
                if (obj) {
                    var objPropId = item.getString("col_id");
                    var objProp = new Model.ObjProp(obj, objPropId);
                    objProp.controlId = Enum.ControlType[item.getString("control_id", "Default")];
                    objProp.isFileColumn = BaseModelReader.isFileColumn(objProp.controlId);
                    objProp.fileStorageFolder = item.getString("file_storage_folder");
                    // DttpId is used on the database
                    objProp.xmlDttp = _this.readXmlDttp(item, "xml_dttp_id");
                    // Set the JS DttpId mapped from the SQL DttpId
                    objProp.setDttpIdFromSqlDttp(item.getString("dttp_id"));
                    if (objProp.xmlDttp.indexOf("date") != -1 && objProp.controlId == Enum.ControlType.Default)
                        objProp.controlId = Enum.ControlType.DATETIME;
                    objProp.domId = item.getString("dom_id");
                    objProp.domain = model.domains[objProp.domId];
                    objProp.pk = item.getBool("primary_key");
                    objProp.identity = item.getBool("identity_col");
                    // TODO: There are more casing options.
                    objProp.uppercase = item.getNumber("case_type") == 1;
                    objProp.alignment = item.getNumber("alignment");
                    objProp.dataLength = item.getNumber("length");
                    objProp.precision = item.getNumber("prec");
                    // TODO: the min- and max value could also be applied on dates.
                    objProp.minValue = item.getString("min_value");
                    objProp.maxValue = item.getString("max_value");
                    // Disable defaults and layouts for Signature controls.
                    var objUsesLayout = obj.useLayouts;
                    objProp.useLayouts = objUsesLayout && item.getBool("layout_input");
                    objProp.useLayoutTypeOutput = objUsesLayout && item.getBool("layout_type_output");
                    objProp.useLayoutMandOutput = objUsesLayout && item.getBool("layout_mand_output");
                    var objUsesDefault = obj.useDefaults;
                    objProp.useDefaults = objUsesDefault && item.getBool("default_input");
                    objProp.useDefaultOutput = objUsesDefault && item.getBool("default_output");
                    objProp.useContext = obj.useContexts && item.getBool("context_input");
                    //UpSortNo
                    //UpSortOrderInd
                    objProp.maxAccessType = _this.getTypeOfCol(item.getNumber("type_of_col"));
                    objProp.objPropTransl = fillHelper.findTranslation(objPropId, 1 /* Col */);
                    objProp.includeInCopy = item.getBool("include_in_copy");
                    obj.objProps.add(objPropId, objProp);
                    if (objProp.pk) {
                        obj.pkProps.push(objProp);
                    }
                    var objViewProp = new Model.ObjViewProp(objProp);
                    _this.createObjViewProp(application, item, obj.defaultObjView, objViewProp, fillHelper);
                    obj.defaultObjView.objViewProps.add(objViewProp.objPropId, objViewProp);
                }
            }
        });
    };
    BaseModelReader.prototype.resolveAllKeys = function (application, fillHelper) {
        var _this = this;
        var resolveObj = function (obj) {
            // Resolve the pk indexes.
            var pkIndex = new Model.PrimaryKeyIndex();
            if (obj.pkProps.length == 1) {
                pkIndex.keyPath = obj.pkProps[0].objPropId;
            }
            else {
                pkIndex.keyPath = IndexedDb.IndexedDbWrapper.pkIndexName;
            }
            obj.pkIndex = pkIndex;
            // Sorting
            _this.createSorting(obj.defaultObjView);
            var objViews = fillHelper.objViews[obj.objId] || {};
            for (var viewId in objViews) {
                if (objViews.hasOwnProperty(viewId)) {
                    _this.createSorting(objViews[viewId]);
                }
            }
            // Resolve objlink indexes
            var childObjs = obj.childObjs.array;
            for (var j = 0; j < childObjs.length; j++) {
                var childObjLink = childObjs[j];
                childObjLink.fixSort();
                childObjLink.determineInheritance();
                var objLinkIndex = new Model.ObjLinkIndex();
                if (childObjLink.props.length == 1) {
                    // Create an index which points directly to the linking prop id.
                    objLinkIndex.indexName += childObjLink.props[0].targetProp.objPropId;
                    objLinkIndex.objLinkProps.push(childObjLink.props[0]);
                    objLinkIndex.keyPath = childObjLink.props[0].targetProp.objPropId;
                }
                else {
                    // Create an index which is a combination of the linking prop id's.
                    for (var g = 0; g < childObjLink.props.length; g++) {
                        objLinkIndex.indexName += childObjLink.props[g].targetProp.objPropId;
                        objLinkIndex.objLinkProps.push(childObjLink.props[g]);
                    }
                    objLinkIndex.keyPath = objLinkIndex.indexName;
                }
                childObjLink.childObj.objLinkIndexes[objLinkIndex.indexName] = objLinkIndex;
            }
            // Resolve lookups
            _this.createLookup(application, obj.defaultObjView);
            for (var viewId in objViews) {
                if (objViews.hasOwnProperty(viewId)) {
                    _this.createLookup(application, objViews[viewId]);
                }
            }
            // Resolve offline available
            var offlineAvailable = _this.isObjOfflineAvailable(obj, objViews);
            if (offlineAvailable) {
                application.model.containsOfflineObjs = true;
            }
            var syncGroup = application.syncGroups.key(obj.syncGroupId);
            if (syncGroup && offlineAvailable && obj.syncGroupId) {
                syncGroup.objs.add(obj.objId, obj);
            }
        };
        var objs = application.model.objs.array;
        for (var objIndex = 0; objIndex < objs.length; objIndex++) {
            var obj = objs[objIndex];
            resolveObj(obj);
        }
        // Resolve all tasks
        var tasks = application.model.tasks.array;
        for (var h = 0; h < tasks.length; h++) {
            var task = tasks[h];
            this.createLookup(application, task.defaultTaskView.paramObjView);
            // taskviews
            var taskViews = fillHelper.taskViews[task.taskId] || {};
            for (var viewId in taskViews) {
                if (taskViews.hasOwnProperty(viewId)) {
                    this.createLookup(application, taskViews[viewId].paramObjView);
                }
            }
        }
    };
    BaseModelReader.prototype.isObjOfflineAvailable = function (obj, objViews) {
        if (obj.defaultObjView.offlineAvailable) {
            return true;
        }
        for (var objViewId in objViews) {
            if (objViews.hasOwnProperty(objViewId) && objViews[objViewId].offlineAvailable) {
                return true;
            }
        }
        return false;
    };
    BaseModelReader.prototype.createSorting = function (objView) {
        objView.createDefaultSorting();
        if (objView.sorting && objView.sorting.sortProps.length > 0) {
            var sortProps = objView.sorting.sortProps;
            var sortIndex = new Model.SortIndex();
            if (objView.objViewId) {
                sortIndex.indexName += "_" + objView.objViewId;
            }
            // There is only one sortProp and it isn't numeric so we can use that column directly.
            // For numeric controls we do something funky with leading zero's in a different column.
            if (sortProps.length == 1 && !sortProps[0].viewProp.objProp.isNumeric() && sortProps[0].viewProp.mandatory) {
                sortIndex.keyPath = sortProps[0].viewProp.objPropId;
                sortIndex.sortingProps.push(sortProps[0].viewProp);
            }
            else {
                var keypath = "";
                for (var i = 0; i < sortProps.length; i++) {
                    var sortViewProp = sortProps[i].viewProp;
                    sortIndex.sortingProps.push(sortViewProp);
                    if (sortViewProp.objProp.isNumeric() || !sortViewProp.mandatory) {
                        keypath += "fake_" + sortViewProp.objPropId;
                    }
                    else {
                        keypath += sortViewProp.objPropId;
                    }
                }
                // sort the sorting props for the index, this happens directly in the array
                // The sort is done after creating the keypath to not break existing indexes..
                Array.prototype.sort.call(sortIndex.sortingProps, function (a, b) {
                    return a.sortNo - b.sortNo;
                });
                sortIndex.keyPath = keypath;
            }
            // Check if the created sortIndex matches with an existing sortIndex.
            for (var sortIndexName in objView.obj.sortIndexes) {
                var existingSortIndex = objView.obj.sortIndexes[sortIndexName];
                if (existingSortIndex.keyPath == sortIndex.keyPath) {
                    // Don't add the same sort index with another name.
                    objView.sortIndex = existingSortIndex;
                    return;
                }
            }
            objView.sortIndex = sortIndex;
            objView.obj.sortIndexes[sortIndex.indexName] = sortIndex;
        }
    };
    BaseModelReader.prototype.createLookup = function (application, objView) {
        var linkLookupViews = objView.objLinkLookupViews.array;
        for (var i = 0; i < linkLookupViews.length; i++) {
            var objLinkLookupView = linkLookupViews[i];
            objLinkLookupView.objLink.fixSort();
            objLinkLookupView.objLink.determineInheritance();
            var objLinkPropLength = objLinkLookupView.objLink.props.length;
            if (objLinkPropLength > 0) {
                var objLinkProp = objLinkLookupView.objLink.props[objLinkPropLength - 1];
                var lookupProps = new Model.LookupProps();
                var targetObjView = objLinkLookupView.objView;
                lookupProps.targetObjView = targetObjView;
                lookupProps.dataProp = lookupProps.targetObjView.objViewProps.object[objLinkProp.sourceProp.objPropId];
                if (objLinkLookupView.objLink.lookupDisplay.lookUpDisplayColId) {
                    lookupProps.listProp = targetObjView.objViewProps.object[objLinkLookupView.objLink.lookupDisplay.lookUpDisplayColId];
                }
                else if (targetObjView.obj.lookupDisplay.lookUpDisplayColId) {
                    lookupProps.listProp = targetObjView.objViewProps.object[targetObjView.obj.lookupDisplay.lookUpDisplayColId];
                }
                if (!lookupProps.listProp) {
                    lookupProps.listProp = lookupProps.dataProp;
                }
                for (var g = 0; g < objLinkLookupView.objLink.props.length - 1; g++) {
                    var objFilterProp = objLinkLookupView.objLink.props[g];
                    lookupProps.filters.push(objFilterProp);
                }
                var objViewProp = objView.objViewProps.object[objLinkProp.targetProp.objPropId];
                if (!objViewProp) {
                    throw new Error("Invalid reference \"" + objLinkLookupView.objLink.refId + "\": Column \"" + objLinkProp.targetProp.objPropId + "\" not found in table \"" + objView.objId + "\"");
                }
                objViewProp.lookup = lookupProps;
                objViewProp.objLinkLookupView = objLinkLookupView;
                this.createLookupIndexes(objViewProp);
            }
        }
    };
    // TODO: Check if the indexes match the pk index.
    BaseModelReader.prototype.createLookupIndexes = function (objViewProp) {
        // Lookup filter indexes
        var filters = objViewProp.lookup.filters;
        var lookupFilterIndex = new Model.LookupFilterIndex();
        var lookupTable = objViewProp.lookup.getTranslTargetObjView().obj;
        if (filters.length > 1) {
            // Create a composite filter index for multiple filters.
            for (var i = 0; i < filters.length; i++) {
                lookupFilterIndex.indexName += filters[i].targetProp.objPropId;
                lookupFilterIndex.lookupFilters.push(filters[i]);
            }
            lookupFilterIndex.keyPath = lookupFilterIndex.indexName;
        }
        else if (filters.length == 1) {
            // A single Lookup filter can point to the corresponding datafield.
            var propId = filters[0].targetProp.objPropId;
            lookupFilterIndex.indexName += propId;
            lookupFilterIndex.keyPath = propId;
        }
        else {
            // Lookup indexes with no filter use an index on the dataProp itself.
            var dataProp = objViewProp.lookup.getTranslDataProp();
            lookupFilterIndex.indexName += dataProp.objPropId;
            lookupFilterIndex.keyPath = dataProp.objPropId;
            // If there aren't any filters, we create an index on the description. This way we can page on the database because
            // we don't need to sort anymore (we need all data otherwise, which kills performance on large datasets)
            var listProp = objViewProp.lookup.getTranslListProp();
            if (listProp.objProp.xmlDttp == "string") {
                var fIndex = new Model.LookupFilterIndex();
                fIndex.indexName += listProp.objPropId;
                fIndex.keyPath = listProp.objPropId;
                lookupTable.lookupFilterIndexes[fIndex.indexName] = fIndex;
                // We also add a additional index so we can do some case insensitive filtering which we use for startsWith filtering
                var fIndex2 = new Model.LookupFilterIndex();
                fIndex2.indexName += "upper_" + listProp.objPropId;
                fIndex2.upperCase = true;
                fIndex2.keyPath = "upper_" + listProp.objPropId;
                lookupTable.lookupFilterIndexes[fIndex2.indexName] = fIndex2;
            }
        }
        lookupTable.lookupFilterIndexes[lookupFilterIndex.indexName] = lookupFilterIndex;
    };
    BaseModelReader.prototype.loadTabPrefilter = function (application, params, fillHelper) {
        var _this = this;
        return this.readGetGui("get_gui_tab_prefilter", params, function (reader) {
            var groupCache = {};
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                var objId = item.getString("tab_id");
                var obj = application.model.getObj(objId);
                if (obj) {
                    var prefilter = new Model.Prefilter();
                    prefilter.prefilterId = item.getString("tab_prefilter_id");
                    prefilter.icon = item.getString("tab_prefilter_icon");
                    prefilter.prefilterTransl = fillHelper.findTranslation(prefilter.prefilterId, 9 /* Prefilter */);
                    prefilter.orderno = item.getNumber("prefilter_order_no");
                    var groupId = item.getString("tab_prefilter_grp_id");
                    if (groupId) {
                        if (!groupCache[groupId]) {
                            var group = new Model.ActionGroup();
                            group.groupId = groupId;
                            group.groupIcon = item.getString("tab_prefilter_grp_icon");
                            group.grpTransl = fillHelper.findTranslation(groupId, 15 /* TabPrefilterGrp */);
                            group.isExcluded = item.getBool("grp_prefilter_exclude");
                            group.groupHasSubmenu = item.getBool("grp_has_sub_menu");
                            groupCache[groupId] = group;
                        }
                        prefilter.group = groupCache[groupId];
                    }
                    obj.prefilters.add(prefilter.prefilterId, prefilter);
                    _this.createObjViewPrefilter(item, obj.defaultObjView, prefilter);
                }
            }
        });
    };
    BaseModelReader.prototype.loadTabPrefilterCol = function (application, params) {
        return this.readGetGui("get_gui_tab_prefilter_col", params, function (reader) {
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                var objId = item.getString("tab_id");
                var obj = application.model.getObj(objId);
                if (obj) {
                    var prefilter = obj.prefilters.key(item.getString("tab_prefilter_id"));
                    if (!prefilter)
                        continue;
                    var prefilterProp = new Model.PrefilterProp();
                    prefilterProp.viewProp = obj.defaultObjView.objViewProps.object[item.getString("col_id")];
                    if (prefilterProp.viewProp.objProp.xmlDttp == "boolean") {
                        prefilterProp.filterFromValue = item.getBool("filter_value") + "";
                        var untilValue = item.getNullableBool("filter_value_until");
                        prefilterProp.filterUntilValue = (untilValue != null ? untilValue + "" : null);
                    }
                    else {
                        prefilterProp.filterFromValue = item.getString("filter_value");
                        prefilterProp.filterUntilValue = item.getString("filter_value_until");
                    }
                    prefilterProp.filterCondition = item.getNumber("filter_condition");
                    prefilter.prefilterProps.push(prefilterProp);
                }
            }
        });
    };
    BaseModelReader.prototype.loadRef = function (application, params, fillHelper) {
        var _this = this;
        return this.readGetGui("get_gui_ref", params, function (reader) {
            var model = application.model;
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                var sourceObjId = item.getString("source_tab_id");
                var sourceObj = model.getObj(sourceObjId);
                if (sourceObj) {
                    var targetObjId = item.getString("target_tab_id");
                    var targetObj = model.getObj(targetObjId);
                    if (targetObj) {
                        var refId = item.getString("ref_id");
                        var ref = new Model.ObjLink(sourceObj, refId, targetObj);
                        fillHelper.objLinks[refId] = ref;
                        ref.lookupDisplay.lookUpDisplayColId = item.getString("look_up_display_col_id");
                        ref.isDetail = item.getBool("is_detail");
                        ref.contextTypeOutput = item.getBool("context_type_output");
                        // Add detail to the source Obj.
                        sourceObj.childObjs.add(refId, ref);
                        // Only add the ObjLinkDetailView to the default ObjView when ShowTarget is enabled.
                        var showTarget = item.getBool("show_detail");
                        if (showTarget) {
                            var objLinkDetailView = new Model.ObjLinkDetailView(ref);
                            _this.createObjLinkDetailView(application, item, objLinkDetailView, "abs_order_no", fillHelper);
                            sourceObj.defaultObjView.objLinkDetailViews.add(refId, objLinkDetailView);
                            var refAdd = item.getString("ref_add");
                            var generateRefAddTransl = item.getBool("generate_ref_add_transl_object");
                            if (refAdd && generateRefAddTransl) {
                                var refAddTransl = fillHelper.findTranslation(refAdd, 7 /* RefAdd */);
                                if (refAddTransl) {
                                    objLinkDetailView.refAddTransl = refAddTransl.transl;
                                }
                            }
                        }
                        // Only add the ObjLinkLookupView to the default ObjView when ShowLookup is enabled.
                        var showLookup = item.getBool("show_look_up");
                        if (showLookup) {
                            var objLinkLookupview = new Model.ObjLinkLookupView(ref);
                            _this.createObjLinkLookupView(application, item, objLinkLookupview, fillHelper);
                            targetObj.defaultObjView.objLinkLookupViews.add(refId, objLinkLookupview);
                        }
                    }
                }
            }
        });
    };
    BaseModelReader.prototype.loadRefCol = function (application, params, fillHelper) {
        return this.readGetGui("get_gui_ref_col", params, function (reader) {
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                var refId = item.getString("ref_id");
                var ref = fillHelper.objLinks[refId];
                if (ref != null) {
                    var sourcePropId = item.getString("source_col_id");
                    var targetPropId = item.getString("target_col_id");
                    var refProp = new Model.ObjLinkProp(ref, sourcePropId, targetPropId);
                    refProp.orderNo = item.getNumber("abs_order_no");
                    ref.props.push(refProp);
                }
            }
        });
    };
    BaseModelReader.prototype.loadElement = function (application, params, fillHelper) {
        return this.readGetGui("get_gui_elemnt", params, function (reader) {
            var domains = application.model.domains;
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                var domId = item.getString("dom_id");
                var domain = domains[domId];
                if (!domain) {
                    domain = new Model.Domain(domId);
                    domains[domId] = domain;
                }
                var elementId = item.getString("elemnt_id");
                var element = new Model.DomainElement(elementId);
                element.value = item.getString("db_value");
                element.elementTransl = fillHelper.findTranslation(elementId, 2 /* DomElement */);
                element.orderNo = item.getNumber("abs_order_no");
                element.picture = item.getString("picture_file");
                domain.elements.add(elementId, element);
            }
        });
    };
    BaseModelReader.prototype.loadConditionalLayout = function (application, params, fillHelper) {
        return this.readGetGui("get_gui_conditional_layout", params, function (reader) {
            var color = Utils.Color;
            for (var i = 0; i < reader.length; i++) {
                var rawConditionalLayout = reader.item(i);
                var objId = rawConditionalLayout.getString("tab_id");
                var obj = void 0;
                if (objId && (obj = application.model.getObj(objId))) {
                    var conditionalLayoutId = rawConditionalLayout.getString("conditional_layout_id");
                    // Prefix the conditional layout id with the objId since its unique in the context of an obj.
                    var conditionalLayout = new Model.ConditionalLayout("tsf_cl_obj_" + objId + "_" + conditionalLayoutId);
                    var objPropId = rawConditionalLayout.getString("col_id");
                    conditionalLayout.objProp = obj.objProps.object[objPropId];
                    var tmpBackColor = rawConditionalLayout.getNullableNumber("background_color");
                    if (tmpBackColor) {
                        conditionalLayout.backGroundColor1 = color.hexFromArgb(tmpBackColor);
                    }
                    var fontId = rawConditionalLayout.getString("font_id");
                    if (fontId) {
                        conditionalLayout.font = fillHelper.fonts[fontId];
                    }
                    obj.conditionalLayouts[conditionalLayoutId] = conditionalLayout;
                    // Only add the conditonal layout to the default ObjView when ShowConditionalLayout is enabled.
                    var showConditionalLayout = rawConditionalLayout.getBool("show_conditional_layout");
                    if (showConditionalLayout) {
                        obj.defaultObjView.conditionalLayouts[conditionalLayoutId] = conditionalLayout;
                    }
                }
            }
        });
    };
    BaseModelReader.prototype.loadConditionalLayoutCondition = function (application, params) {
        return this.readGetGui("get_gui_conditional_layout_condition", params, function (reader) {
            var model = application.model;
            for (var i = 0; i < reader.length; i++) {
                var rawCondition = reader.item(i);
                var objId = rawCondition.getString("tab_id");
                var obj = (objId ? model.getObj(objId) : undefined);
                if (obj) {
                    var conditionalLayoutId = rawCondition.getString("conditional_layout_id");
                    if (conditionalLayoutId && obj.conditionalLayouts[conditionalLayoutId]) {
                        var conditionalLayout = obj.conditionalLayouts[conditionalLayoutId];
                        var condition = new Model.ConditionalLayoutCondition();
                        condition.objProp = obj.objProps.object[rawCondition.getString("col_id")];
                        condition.condition = rawCondition.getNumber("condition");
                        if (rawCondition.getBool("type_of_value")) {
                            condition.filterField = obj.objProps.object[rawCondition.getString("value_col_id")];
                        }
                        else {
                            condition.value = rawCondition.getString("value");
                        }
                        if (rawCondition.getBool("until_value_type")) {
                            condition.filterFieldUntil = obj.objProps.object[rawCondition.getString("until_value_col_id")];
                        }
                        else {
                            condition.valueUntil = rawCondition.getString("until_value");
                        }
                        if (condition.condition == Enum.FilterCondition.IsNull || condition.condition == Enum.FilterCondition.IsNotNull) {
                            condition.valueUntil = condition.value = null;
                        }
                        if (condition.objProp.xmlDttp == "boolean") {
                            condition.value = Utils.convertToBoolean(condition.value, false);
                            condition.valueUntil = condition.valueUntil != null && Utils.convertToBoolean(condition.valueUntil, false);
                        }
                        else if (condition.objProp.isNumeric()) {
                            condition.value = parseFloat(condition.value);
                            condition.valueUntil = condition.valueUntil != null && parseFloat(condition.valueUntil);
                        }
                        conditionalLayout.conditions.push(condition);
                    }
                }
            }
        });
    };
    BaseModelReader.prototype.loadFont = function (application, params, fillHelper) {
        return this.readGetGui("get_gui_font", params, function (reader) {
            BaseModelReader.importFonts(reader, fillHelper.fonts);
        });
    };
    BaseModelReader.importFonts = function (reader, fonts) {
        if (reader.length == 0) {
            return;
        }
        for (var i = 0; i < reader.length; i++) {
            var rawFont = reader.item(i);
            var font = {
                fontId: rawFont.getString("font_id"),
                //Fallback on sensible default fonts if the configured font is not available.
                fontFace: rawFont.getString("font_face_id") + ", Helvetica, Arial, sans-serif",
                // Font size is per definition in pixels.
                fontSize: rawFont.getNumber("font_size") + "px",
                color: rawFont.getNullableNumber("color"),
                italic: rawFont.getBool("italic"),
                bold: rawFont.getBool("bold"),
                underline: rawFont.getBool("underline")
            };
            fonts[font.fontId] = font;
        }
    };
    BaseModelReader.prototype.loadTask = function (application, params, fillHelper) {
        var _this = this;
        return this.readGetGui("get_gui_task", params, function (reader) {
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                var taskId = item.getString("task_id");
                var task = new Model.Task(taskId);
                task.taskType = item.getString("task_type_id");
                task.popupForEachRow = item.getBool("popup_for_each_row");
                task.userParamObj = new Model.Obj(taskId);
                task.userParamObj.isTask = true;
                task.userParamObj.useDefaults = item.getBool("use_defaults");
                task.userParamObj.useLayouts = item.getBool("use_layouts");
                task.userParamObj.useLayoutsAddButton = item.getBool("use_layouts_add_button_type");
                task.userParamObj.useLayoutsCancelButton = item.getBool("use_layouts_cancel_button_type");
                task.userParamObj.useLayoutsConfirmButton = item.getBool("use_layouts_confirm_button_type");
                task.userParamObj.useLayoutsDeleteButton = item.getBool("use_layouts_delete_button_type");
                task.userParamObj.useLayoutsUpdateButton = item.getBool("use_layouts_update_button_type");
                var taskView = new Model.TaskView(task, null);
                _this.createTaskView(application, item, taskView, fillHelper);
                taskView.paramObjView.objTransl = fillHelper.findTranslation(taskView.task.taskId, 11 /* Task */);
                task.defaultTaskView = taskView;
                application.model.tasks.add(taskId, task);
            }
        });
    };
    BaseModelReader.prototype.loadExtendedProperties = function (application, params, fillHelper) {
        return this.readGetGui("get_gui_extended_property", params, function (reader) {
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                application.extendedProperties.setExtendedProperty(item.getString("extended_property_id"), item.getString("property_value"));
            }
        });
    };
    BaseModelReader.prototype.loadTaskRef = function (application, params, fillHelper) {
        var _this = this;
        if (application.model.tasks.length == 0) {
            return jQuery.Deferred().resolve().promise();
        }
        return this.readGetGui("get_gui_task_ref", params, function (reader) {
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                var sourceObjId = item.getString("source_tab_id");
                var sourceObj = application.model.getObj(sourceObjId);
                if (!sourceObj)
                    continue;
                var targetTaskId = item.getString("target_task_id");
                var task = application.model.tasks.key(targetTaskId);
                if (!task)
                    continue;
                var taskRefId = item.getString("task_ref_id");
                var objLink = new Model.ObjLink(sourceObj, taskRefId, task.userParamObj);
                fillHelper.taskLinks[taskRefId] = objLink;
                objLink.lookupDisplay.lookUpDisplayColId = item.getString("look_up_display_col_id");
                objLink.inheritance = false;
                var objLinkLookupview = new Model.ObjLinkLookupView(objLink);
                _this.createObjLinkLookupView(application, item, objLinkLookupview, fillHelper);
                task.defaultTaskView.paramObjView.objLinkLookupViews.add(taskRefId, objLinkLookupview);
            }
        });
    };
    BaseModelReader.prototype.loadTaskRefCol = function (application, params, fillHelper) {
        if (application.model.tasks.length == 0) {
            return jQuery.Deferred().resolve().promise();
        }
        return this.readGetGui("get_gui_task_ref_col", params, function (reader) {
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                var taskRefId = item.getString("task_ref_id");
                var taskLink = fillHelper.taskLinks[taskRefId];
                if (taskLink) {
                    var sourcePropId = item.getString("source_col_id");
                    var targetPropId = item.getString("target_parmtr_id");
                    var refProp = new Model.ObjLinkProp(taskLink, sourcePropId, targetPropId);
                    refProp.orderNo = item.getNumber("abs_order_no");
                    taskLink.props.push(refProp);
                }
            }
        });
    };
    BaseModelReader.prototype.loadTaskParameter = function (application, params, fillHelper) {
        var _this = this;
        if (application.model.tasks.length == 0) {
            return jQuery.Deferred().resolve().promise();
        }
        return this.readGetGui("get_gui_task_parmtr", params, function (reader) {
            // Unused get gui columns:
            /*"order_no"
            "screen_output"
            "abs_order_no"
            "case_type"
            "dttp_type_ind"
            "transl_data_field"
            "transl_list_field"
            "look_up_transl_value_ind"
            "transl_short"*/
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                var taskId = item.getString("task_id");
                var taskParmtrId = item.getString("task_parmtr_id");
                var task = application.model.tasks.key(taskId);
                if (!task)
                    continue;
                var taskProp = new Model.ObjProp(task.userParamObj, taskParmtrId);
                taskProp.domId = item.getString("dom_id");
                taskProp.domain = application.model.domains[taskProp.domId];
                taskProp.maxAccessType = _this.getTypeOfCol(item.getNumber("type_of_col"));
                taskProp.objPropTransl = fillHelper.findTranslation(taskParmtrId, 32 /* TaskParmtr */);
                taskProp.alignment = item.getNumber("alignment");
                taskProp.controlId = Enum.ControlType[item.getString("control_id", "Default")];
                taskProp.isFileColumn = BaseModelReader.isFileColumn(taskProp.controlId);
                taskProp.fileStorageFolder = item.getString("file_storage_folder");
                taskProp.xmlDttp = _this.readXmlDttp(item, "xml_dttp_id");
                taskProp.dataLength = item.getNumber("length");
                taskProp.precision = item.getNumber("prec");
                var objUsesDefault = task.userParamObj.useDefaults;
                taskProp.useDefaults = objUsesDefault && item.getBool("default_input");
                taskProp.useDefaultOutput = objUsesDefault && item.getBool("default_output");
                var objUsesLayout = task.userParamObj.useLayouts;
                taskProp.useLayouts = objUsesLayout && item.getBool("layout_input");
                taskProp.useLayoutMandOutput = objUsesLayout && item.getBool("layout_mand_output");
                taskProp.useLayoutTypeOutput = objUsesLayout && item.getBool("layout_type_output");
                var objViewProp = new Model.ObjViewProp(taskProp);
                _this.readAbstractObjViewProp(application, item, task.defaultTaskView.paramObjView, objViewProp, fillHelper);
                objViewProp.formOrderNo = item.getNumber("abs_order_no") - 1; // Change to zero-based
                objViewProp.formMaxAccessType = _this.getTypeOfCol(item.getNumber("type_of_col"));
                task.defaultTaskView.paramObjView.objViewProps.add(taskProp.objPropId, objViewProp);
                task.userParamObj.objProps.add(taskParmtrId, taskProp);
            }
        });
    };
    BaseModelReader.prototype.loadPlatform = function (application, params, fillHelper) {
        return this.readGetGui("get_gui_platform", params, function (reader) {
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                var platform = item.getNumber("platform");
                if (platform !== 2) {
                    continue;
                }
                var defaultMenuId = item.getString("default_menu_id");
                if (application.model.listbarMenus.key(defaultMenuId)) {
                    application.model.defaultMenu = application.model.listbarMenus.key(defaultMenuId);
                }
                else if (application.model.tileMenus.key(defaultMenuId)) {
                    application.model.defaultMenu = application.model.tileMenus.key(defaultMenuId);
                }
                else {
                    if (application.model.listbarMenus.length > 0) {
                        application.model.defaultMenu = application.model.listbarMenus.item(0);
                    }
                    else if (application.model.tileMenus.length > 0) {
                        application.model.defaultMenu = application.model.tileMenus.item(0);
                    }
                }
            }
        });
    };
    BaseModelReader.prototype.loadTabTask = function (application, params, fillHelper) {
        if (application.model.tasks.length == 0) {
            return jQuery.Deferred().resolve().promise();
        }
        return this.readGetGui("get_gui_tab_task", params, function (reader) {
            // Unused get gui columns:
            //"abs_order_no"
            //"type_of_communication"
            var groupCache = {};
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                var objId = item.getString("tab_id");
                var obj = application.model.getObj(objId);
                if (!obj)
                    continue;
                var taskId = item.getString("task_id");
                var task = application.model.tasks.key(taskId);
                if (!task)
                    continue;
                var taskView = void 0;
                var taskVariantId = item.getString("task_variant_id");
                if (taskVariantId && fillHelper.taskViews[taskId][taskVariantId]) {
                    taskView = fillHelper.taskViews[taskId][taskVariantId];
                }
                else {
                    taskView = task.defaultTaskView;
                }
                var taskLink = new Model.TaskLink(task, obj);
                taskLink.refresh = item.getNumber("refresh_after_execute");
                taskLink.refreshOffline = item.getBool("refresh_offline_data_after_execute");
                taskLink.contextTypeOutput = item.getBool("context_type_output");
                var groupId = item.getString("tab_task_grp_id");
                if (groupId) {
                    if (groupCache[groupId]) {
                        taskLink.group = groupCache[groupId];
                    }
                    taskLink.group = new Model.ActionGroup();
                    taskLink.group.groupId = groupId;
                    taskLink.group.groupHasSubmenu = item.getBool("grp_has_sub_menu");
                    taskLink.group.grpTransl = fillHelper.findTranslation(taskLink.group.groupId, 16 /* TabTaskGrp */);
                }
                obj.tasks.add(taskId, taskLink);
                // Only add the task to the default ObjView when ShowTask is enabled.
                var showTask = item.getBool("show_tab_task");
                if (showTask) {
                    obj.defaultObjView.taskLinkViews.push(new Model.TaskLinkView(taskLink, taskView));
                }
            }
        });
    };
    BaseModelReader.prototype.loadTabTaskParameter = function (application, params) {
        if (application.model.tasks.length == 0) {
            return jQuery.Deferred().resolve().promise();
        }
        return this.readGetGui("get_gui_tab_task_parmtr", params, function (reader) {
            // Unused get gui columns:
            //"order_no"
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                var objId = item.getString("tab_id");
                var obj = application.model.getObj(objId);
                if (!obj)
                    continue;
                var taskId = item.getString("task_id");
                var task = application.model.tasks.key(taskId);
                if (!task)
                    continue;
                var taskLink = obj.tasks.object[taskId];
                var taskLinkId = item.getString("task_parmtr_id");
                var taskLinkProp = new Model.TaskLinkProp(taskLinkId);
                taskLinkProp.propId = item.getString("col_id");
                taskLinkProp.orderNo = item.getNumber("abs_order_no");
                taskLink.props.add(taskLinkId, taskLinkProp);
            }
        });
    };
    BaseModelReader.prototype.loadTabVariant = function (application, params, fillHelper) {
        var _this = this;
        return this.readGetGui("get_gui_tab_variant", params, function (reader) {
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                var objId = item.getString("tab_id");
                var obj = application.model.getObj(objId);
                if (!obj)
                    continue;
                var objViewId = item.getString("tab_variant_id");
                var objView = new Model.ObjView(obj, objViewId);
                _this.createObjView(application, item, objView);
                // Either look up a unique translation for this objView, or use the default for the whole obj, which is stored in obj.defaultObjView.
                if (item.getBool("generate_transl_object")) {
                    objView.objTransl = fillHelper.findTranslation(objView.objViewId, 281 /* TabVariant */);
                }
                else {
                    objView.objTransl = fillHelper.findTranslation(objView.objId, 0 /* Tab */);
                }
                if (!fillHelper.objViews[objId])
                    fillHelper.objViews[objId] = {};
                fillHelper.objViews[objId][objViewId] = objView;
            }
        });
    };
    BaseModelReader.prototype.createObjView = function (application, item, objView) {
        objView.nrOfFieldsLocked = item.getNumber("no_of_fields_locked");
        objView.icon = item.getString("icon");
        objView.allowAdd = item.getBool("allow_add");
        objView.allowCopy = item.getBool("allow_copy");
        objView.allowUpdate = item.getBool("allow_update");
        objView.allowDelete = item.getBool("allow_delete");
        if (Model.TSFApplicationManager.Current.globalSettings.currentSFVersion >= 2 /* V2018_1 */) {
            objView.allowNavigation = item.getBool("allow_nav");
        }
        else {
            objView.allowNavigation = item.getBool("allow_navigation");
        }
        objView.allowSort = item.getBool("allow_sort");
        objView.allowFilter = item.getBool("allow_filter");
        objView.allowSearch = item.getBool("allow_search");
        objView.offlineAvailable = item.getBool("offline_available");
        objView.offlineMutable = item.getBool("offline_mutable");
        objView.offlineRefreshAfterMutation = item.getBool("refresh_offline_data_after_mutation");
        objView.startEmptyWithFilter = item.getBool("start_empty_with_filter");
        objView.gridRowHeight = item.getNumber("grid_row_height");
        objView.numberOfColsInForm = item.getNullableNumber("no_of_cols_in_form");
        objView.defaultEditableForm = item.getBool("form_default_editable");
        // autoSave should always be true for a default editable form
        objView.formAutoSave = item.getBool("form_auto_save") || objView.defaultEditableForm;
        // TODO: Implement card_list_title_label
        //objView.cardListTitleLabel = item.getBool("card_list_title_label");
        objView.cardListTitle = item.getNumber("card_list_title");
        objView.cardListImage = item.getNumber("card_list_image");
        objView.showBadge = item.getBool("show_badge");
        if (objView.showBadge) {
            objView.badgeInterval = item.getNumber("badge_interval", 0);
        }
        objView.confirmDelete = item.getBool("confirm_delete");
        var mainScreenTypeValue = item.getString("main_screen_type_id");
        objView.mainScreenType = application.model.screenTypes[mainScreenTypeValue];
        var detailScreenTypeValue = item.getString("detail_screen_type_id");
        objView.defaultDetailSceenType = application.model.screenTypes[detailScreenTypeValue];
        var formProps = new Model.FormProps();
        formProps.defaultFieldWidth = item.getNumber("field_width");
        formProps.defaultLabelWidth = item.getNumber("label_width");
        objView.formProps = formProps;
    };
    BaseModelReader.prototype.loadTabVariantCol = function (application, params, fillHelper) {
        var _this = this;
        return this.readGetGui("get_gui_tab_variant_col", params, function (reader) {
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                var objId = item.getString("tab_id");
                var obj = application.model.getObj(objId);
                if (!obj)
                    continue;
                var objViewId = item.getString("tab_variant_id");
                var objView = fillHelper.objViews[objId][objViewId];
                if (!objView)
                    continue;
                var colId = item.getString("col_id");
                var objProp = obj.objProps.object[colId];
                if (!objProp)
                    continue;
                var objViewProp = new Model.ObjViewProp(objProp);
                _this.createObjViewProp(application, item, objView, objViewProp, fillHelper);
                objView.objViewProps.add(objViewProp.objPropId, objViewProp);
            }
        });
    };
    BaseModelReader.prototype.createObjViewProp = function (application, item, objView, objViewProp, fillHelper) {
        this.readAbstractObjViewProp(application, item, objView, objViewProp, fillHelper);
        var objProp = objViewProp.objProp;
        // TODO: Implement card_list_field_height_in_position
        objViewProp.cardListOrderNo = item.getNumber("card_list_order_no");
        objViewProp.cardListTypeOfCol = this.getTypeOfCol(item.getNumber("card_list_type_of_col"));
        objViewProp.cardListLabel = item.getBool("card_list_label");
        //objViewProp.cardListFieldHeightInPositions = item.getNumber("card_list_field_height_in_position");
        objViewProp.gridColumnWidth = item.getNullableNumber("grid_col_width");
        objViewProp.formOrderNo = item.getNumber("form_order_no") - 1; // Change to zero-based
        objViewProp.gridOrderNo = item.getNumber("grid_order_no") - 1; // Change to zero-based
        objViewProp.formMaxAccessType = this.getTypeOfCol(item.getNumber("form_type_of_col"));
        objViewProp.gridMaxAccessType = this.getTypeOfCol(item.getNumber("grid_type_of_col"));
        if (objViewProp.formMaxAccessType < objProp.maxAccessType)
            objViewProp.formMaxAccessType = objProp.maxAccessType;
        if (objViewProp.gridMaxAccessType < objProp.maxAccessType)
            objViewProp.gridMaxAccessType = objProp.maxAccessType;
        objViewProp.allowSort = item.getBool("allow_sort");
        objViewProp.sortNo = item.getNumber("sort_no");
        objViewProp.sortOrder = (item.getNumber("sort_order") == 1) ? Enum.SortDirection.Descending : Enum.SortDirection.Ascending;
        var includeInGlobalFilter = item.getBool("include_in_global_filter");
        objViewProp.includeInGlobalFilter = includeInGlobalFilter && objProp.maxAccessType != Enum.AccessType.Hidden;
    };
    BaseModelReader.prototype.loadTabVariantConditionalLayout = function (application, params, fillHelper) {
        return this.readGetGui("get_gui_tab_variant_conditional_layout", params, function (reader) {
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                var objId = item.getString("tab_id");
                var obj = application.model.getObj(objId);
                if (!obj)
                    continue;
                var objViewId = item.getString("tab_variant_id");
                var objView = fillHelper.objViews[objId][objViewId];
                if (!objView)
                    continue;
                var conditionalLayoutId = item.getString("conditional_layout_id");
                var conditionalLayout = obj.conditionalLayouts[conditionalLayoutId];
                objView.conditionalLayouts[conditionalLayoutId] = conditionalLayout;
            }
        });
    };
    BaseModelReader.prototype.loadTabVariantDetail = function (application, params, fillHelper) {
        var _this = this;
        return this.readGetGui("get_gui_tab_variant_detail", params, function (reader) {
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                var objId = item.getString("tab_id");
                var obj = application.model.getObj(objId);
                if (!obj)
                    continue;
                var objViewId = item.getString("tab_variant_id");
                var objView = fillHelper.objViews[objId][objViewId];
                if (!objView)
                    continue;
                var refId = item.getString("ref_id");
                var objLink = obj.childObjs.object[refId];
                if (!objLink)
                    continue;
                var objViewLink = new Model.ObjLinkDetailView(objLink);
                _this.createObjLinkDetailView(application, item, objViewLink, "order_no", fillHelper);
                objView.objLinkDetailViews.add(refId, objViewLink);
            }
        });
    };
    BaseModelReader.prototype.createObjLinkLookupView = function (application, item, objLinkLookupView, fillHelper) {
        objLinkLookupView.hasZoomButton = item.getBool("look_up_has_popup");
        objLinkLookupView.controlType = item.getNumber("look_up_control");
        var lookupConfigId = item.getString("look_up_tab_variant_id");
        if (lookupConfigId) {
            objLinkLookupView.objView = fillHelper.objViews[objLinkLookupView.objLink.parentObj.objId][lookupConfigId];
        }
        else {
            objLinkLookupView.objView = objLinkLookupView.objLink.parentObj.defaultObjView;
        }
    };
    BaseModelReader.prototype.createObjLinkDetailView = function (application, item, objLinkDetailView, orderNoCol, fillHelper) {
        var configId = item.getString("detail_tab_variant_id");
        if (configId) {
            objLinkDetailView.objView = fillHelper.objViews[objLinkDetailView.objLink.childObj.objId][configId];
            objLinkDetailView.refTransl = objLinkDetailView.objView.objTransl;
        }
        else {
            objLinkDetailView.objView = objLinkDetailView.objLink.childObj.defaultObjView;
            objLinkDetailView.refTransl = fillHelper.findTranslation(objLinkDetailView.objLink.childObj.objId, 0 /* Tab */);
        }
        objLinkDetailView.orderNo = item.getNumber(orderNoCol);
    };
    BaseModelReader.prototype.loadTabVariantLookUp = function (application, params, fillHelper) {
        var _this = this;
        return this.readGetGui("get_gui_tab_variant_look_up", params, function (reader) {
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                var objId = item.getString("tab_id");
                var obj = application.model.getObj(objId);
                if (!obj)
                    continue;
                var objViewId = item.getString("tab_variant_id");
                var objView = fillHelper.objViews[objId][objViewId];
                if (!objView)
                    continue;
                var refId = item.getString("ref_id");
                var objLink = fillHelper.objLinks[refId];
                if (!objLink)
                    continue;
                var objLinkLookupView = new Model.ObjLinkLookupView(objLink);
                _this.createObjLinkLookupView(application, item, objLinkLookupView, fillHelper);
                objView.objLinkLookupViews.add(refId, objLinkLookupView);
            }
        });
    };
    BaseModelReader.prototype.loadTabVariantPrefilter = function (application, params, fillHelper) {
        var _this = this;
        return this.readGetGui("get_gui_tab_variant_prefilter", params, function (reader) {
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                var objId = item.getString("tab_id");
                var obj = application.model.getObj(objId);
                if (!obj)
                    continue;
                var objViewId = item.getString("tab_variant_id");
                var objView = fillHelper.objViews[objId][objViewId];
                if (!objView)
                    continue;
                var prefilterId = item.getString("tab_prefilter_id");
                var prefilter = obj.prefilters.key(prefilterId);
                if (!prefilter)
                    continue;
                _this.createObjViewPrefilter(item, objView, prefilter);
            }
        });
    };
    BaseModelReader.prototype.createObjViewPrefilter = function (item, objView, prefilter) {
        var viewPrefilter = new Model.ObjViewPrefilter(prefilter);
        viewPrefilter.contextDefaultState[0 /* Master */] = item.getNumber("main_prefilter_state");
        viewPrefilter.contextDefaultState[1 /* Detail */] = item.getNumber("detail_prefilter_state");
        viewPrefilter.contextDefaultState[2 /* Lookup */] = item.getNumber("look_up_prefilter_state");
        objView.prefilters.add(prefilter.prefilterId, viewPrefilter);
    };
    BaseModelReader.prototype.loadTabVariantTask = function (application, params, fillHelper) {
        if (application.model.tasks.length == 0) {
            return jQuery.Deferred().resolve().promise();
        }
        return this.readGetGui("get_gui_tab_variant_task", params, function (reader) {
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                var objId = item.getString("tab_id");
                var obj = application.model.getObj(objId);
                if (!obj)
                    continue;
                var objViewId = item.getString("tab_variant_id");
                var objView = fillHelper.objViews[objId][objViewId];
                if (!objView)
                    continue;
                var taskId = item.getString("task_id");
                var taskLink = obj.tasks.object[taskId];
                if (!taskLink)
                    continue;
                var taskView = void 0;
                var taskVariantId = item.getString("task_variant_id");
                if (taskVariantId) {
                    taskView = fillHelper.taskViews[taskId][taskVariantId];
                }
                else {
                    taskView = taskLink.task.defaultTaskView;
                }
                objView.taskLinkViews.push(new Model.TaskLinkView(taskLink, taskView));
            }
        });
    };
    BaseModelReader.prototype.loadTaskVariant = function (application, params, fillHelper) {
        var _this = this;
        if (application.model.tasks.length == 0) {
            return jQuery.Deferred().resolve().promise();
        }
        return this.readGetGui("get_gui_task_variant", params, function (reader) {
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                var taskId = item.getString("task_id");
                var task = application.model.tasks.key(taskId);
                if (!task)
                    continue;
                var taskVariantId = item.getString("task_variant_id");
                var taskView = new Model.TaskView(task, taskVariantId);
                _this.createTaskView(application, item, taskView, fillHelper);
                // Either look up a unique translation for this TaskView, or use the default task translation.
                if (item.getBool("generate_transl_object")) {
                    taskView.paramObjView.objTransl = fillHelper.findTranslation(taskView.taskViewId, 291 /* TaskVariant */);
                }
                else {
                    taskView.paramObjView.objTransl = fillHelper.findTranslation(taskView.task.taskId, 11 /* Task */);
                }
                if (!fillHelper.taskViews[taskId])
                    fillHelper.taskViews[taskId] = {};
                fillHelper.taskViews[taskId][taskVariantId] = taskView;
            }
        });
    };
    BaseModelReader.prototype.createTaskView = function (application, item, taskView, fillHelper) {
        taskView.icon = item.getString("icon");
        taskView.askConfirmation = item.getBool("ask_confirmation");
        taskView.offlineExecutable = item.getBool("offline_executable");
        var taskConfigId = item.getString("task_variant_id");
        if (item.getBool("generate_transl_object")) {
            taskView.taskTransl = fillHelper.findTranslation(taskConfigId, 291 /* TaskVariant */);
        }
        else {
            taskView.taskTransl = fillHelper.findTranslation(taskView.task.taskId, 11 /* Task */);
        }
        taskView.paramObjView = new Model.ObjView(taskView.task.userParamObj, taskConfigId);
        taskView.paramObjView.formProps = application.defaultFormProps;
        taskView.paramObjView.allowAdd = taskView.paramObjView.allowUpdate = true;
        taskView.paramObjView.allowCopy = taskView.paramObjView.allowDelete = taskView.paramObjView.allowNavigation = false;
        taskView.paramObjView.allowSort = taskView.paramObjView.allowFilter = taskView.paramObjView.allowSearch = false;
        taskView.paramObjView.showBadge = item.getBool("show_badge");
        if (taskView.paramObjView.showBadge) {
            taskView.paramObjView.badgeInterval = item.getNumber("badge_interval", 0);
        }
    };
    BaseModelReader.prototype.loadTaskVariantParmtr = function (application, params, fillHelper) {
        var _this = this;
        if (application.model.tasks.length == 0) {
            return jQuery.Deferred().resolve().promise();
        }
        return this.readGetGui("get_gui_task_variant_parmtr", params, function (reader) {
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                var taskId = item.getString("task_id");
                var task = application.model.tasks.key(taskId);
                if (!task)
                    continue;
                var taskVariantId = item.getString("task_variant_id");
                var taskView = fillHelper.taskViews[taskId][taskVariantId];
                if (!taskView)
                    continue;
                var taskParameterId = item.getString("task_parmtr_id");
                var taskProp = task.userParamObj.objProps.object[taskParameterId];
                if (!taskProp)
                    continue;
                var objViewProp = new Model.ObjViewProp(taskProp);
                _this.readAbstractObjViewProp(application, item, taskView.paramObjView, objViewProp, fillHelper);
                objViewProp.formOrderNo = item.getNumber("form_order_no") - 1; // Change to zero-based
                objViewProp.formMaxAccessType = _this.getTypeOfCol(item.getNumber("form_type_of_col"));
                taskView.paramObjView.objViewProps.add(taskProp.objPropId, objViewProp);
            }
        });
    };
    BaseModelReader.prototype.loadTaskVariantLookUp = function (application, params, fillHelper) {
        var _this = this;
        if (application.model.tasks.length == 0) {
            return jQuery.Deferred().resolve().promise();
        }
        return this.readGetGui("get_gui_task_variant_look_up", params, function (reader) {
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                var taskId = item.getString("task_id");
                var task = application.model.tasks.key(taskId);
                if (!task)
                    continue;
                var taskVariantId = item.getString("task_variant_id");
                var taskView = fillHelper.taskViews[taskId][taskVariantId];
                if (!taskView)
                    continue;
                var taskRefId = item.getString("task_ref_id");
                var objLink = fillHelper.taskLinks[taskRefId];
                if (!objLink)
                    continue;
                var objLinkLookupview = new Model.ObjLinkLookupView(objLink);
                _this.createObjLinkLookupView(application, item, objLinkLookupview, fillHelper);
                taskView.paramObjView.objLinkLookupViews.add(taskRefId, objLinkLookupview);
            }
        });
    };
    BaseModelReader.prototype.readAbstractObjViewProp = function (application, item, objView, objViewProp, fillHelper) {
        var objProp = objViewProp.objProp;
        objViewProp.fieldWidth = item.getNumber("field_width");
        objViewProp.labelWidth = item.getNumber("label_width");
        objViewProp.fieldHeightInPositions = item.getNumber("field_height_in_positions", objProp.controlId == Enum.ControlType.MULTILINE ? 3 : 1);
        if (objViewProp.formMaxAccessType < objProp.maxAccessType)
            objViewProp.formMaxAccessType = objProp.maxAccessType;
        objViewProp.fieldInNextColumn = item.getBool("field_in_next_col");
        objViewProp.fieldInNextGroup = item.getBool("field_in_next_grp");
        if (objViewProp.fieldInNextGroup) {
            var groupKey = item.getString("next_grp_label");
            if (groupKey) {
                // DM: I have no idea why the enum is TabLabel but it is copied from the dotNet GUI.
                objViewProp.groupTransl = fillHelper.findTranslation(groupKey, 33 /* TabLabel */);
            }
            objViewProp.groupIcon = item.getString("next_grp_icon");
        }
        objViewProp.fieldOnNextTab = item.getBool("field_on_next_tab");
        if (objViewProp.fieldOnNextTab) {
            var tabKey = item.getString("next_tab_label");
            if (tabKey) {
                objViewProp.tabTransl = fillHelper.findTranslation(tabKey, 33 /* TabLabel */);
            }
            objViewProp.tabIcon = item.getString("next_tab_icon");
        }
        objViewProp.mandatory = item.getBool("mand");
        if (!objViewProp.objProp.isFileColumn) {
            if (Model.TSFApplicationManager.Current.globalSettings.currentSFVersion >= 2 /* V2018_1 */) {
                objViewProp.defaultValue = item.getString("default_value");
            }
            else {
                objViewProp.defaultValue = item.getString("def_value");
            }
        }
        // Value corrections, but only for booleans, for db2 it will be a short
        if (objProp.controlId == Enum.ControlType.CHECKBOX && objProp.xmlDttp.indexOf("boolean") >= 0) {
            if (objViewProp.mandatory && objViewProp.defaultValue == null) {
                objViewProp.defaultValue = "false";
            }
            else if (objViewProp.defaultValue != null) {
                objViewProp.defaultValue = "" + Utils.convertToBoolean(objViewProp.defaultValue, false);
            }
        }
    };
    BaseModelReader.prototype.loadTranslationsFull = function (application, params, fillHelper) {
        var _this = this;
        params = params.clone();
        params.add("appl_lang_id", application.language.code);
        var getGuiName = "get_gui_translations_full";
        if (Model.TSFApplicationManager.Current.globalSettings.currentSFVersion >= 2 /* V2018_1 */) {
            getGuiName = "get_gui_transl_full";
        }
        return this.readGetGui(getGuiName, params, function (reader) {
            _this.bindTranslations(application, reader, fillHelper);
        });
    };
    BaseModelReader.prototype.loadTranslationsNav = function (application, params, fillHelper) {
        var _this = this;
        params = params.clone();
        params.add("appl_lang_id", application.language.code);
        var getGuiName = "get_gui_translations_nav";
        if (Model.TSFApplicationManager.Current.globalSettings.currentSFVersion >= 2 /* V2018_1 */) {
            getGuiName = "get_gui_transl_nav";
        }
        return this.readGetGui(getGuiName, params, function (reader) {
            _this.bindTranslations(application, reader, fillHelper);
            // Only get_gui_translations_nav returns type 13 (gui_object)
            // We also store them here so they can overwrite the default mobile translations
            var list = fillHelper.translationList[13 /* GuiObject */] || [];
            for (var i = 0; i < list.length; i++) {
                var item = list[i];
                if (item.transl != null) {
                    application.language.translations[item.id] = item.transl;
                }
            }
        });
    };
    BaseModelReader.prototype.bindTranslations = function (application, reader, fillHelper) {
        for (var i = 0; i < reader.length; i++) {
            var item = reader.item(i);
            // Dont read the tooltip and help text because we don't support it.
            var id = item.getString("transl_object_id");
            var typeOfObject = item.getNumber("type_of_object");
            var transl = new UITranslation(id);
            transl.transl = item.getString("transl");
            transl.translForm = item.getString("transl_form");
            transl.translGrid = item.getString("transl_grid");
            transl.translPlural = item.getString("transl_plural");
            if (!fillHelper.translationList[typeOfObject]) {
                fillHelper.translationList[typeOfObject] = [];
            }
            fillHelper.translationList[typeOfObject].push(transl);
        }
    };
    BaseModelReader.prototype.loadProcessFlow = function (application, params, fillHelper) {
        return this.readGetGui("get_gui_process_flow", params, function (reader) {
            fillHelper.containsProcessFlow = reader.length > 0;
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                var processFlowId = item.getString("process_flow_id");
                fillHelper.processFlow[processFlowId] = new ProcessEngine.ProcessFlow(processFlowId);
            }
        });
    };
    BaseModelReader.prototype.loadProcessAction = function (application, params, fillHelper) {
        if (!fillHelper.containsProcessFlow) {
            return jQuery.Deferred().resolve().promise();
        }
        return this.readGetGui("get_gui_process_action", params, function (reader) {
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                var processFlowId = item.getString("process_flow_id");
                var processFlow = fillHelper.processFlow[processFlowId];
                if (!processFlow)
                    continue;
                var processActionId = item.getString("process_action_id");
                var processActionType = item.getNumber("process_action_type");
                var processAction = ProcessEngine.ProcessAction.fromProcessActionType(processFlow, processActionId, processActionType);
                if (Model.TSFApplicationManager.Current.globalSettings.currentSFVersion >= 2 /* V2018_1 */) {
                    processAction.mandatory = item.getBool("process_action_mand");
                }
                else {
                    processAction.mandatory = item.getBool("process_action_mandatory");
                }
                if (processAction.requiresObj()) {
                    var obj = application.model.getObj(item.getString("tab_id"));
                    if (!obj)
                        continue;
                    processAction.assignObj(obj);
                }
                else if (processAction.requiresObjView()) {
                    // For GoToDocument - but we are not going to implement that right away
                    //tab_id
                    //tab_variant_id
                }
                else if (processAction.requiresObjLink()) {
                    //ref_id
                }
                else if (processAction.requiresTaskLink()) {
                    var obj = application.model.getObj(item.getString("tab_id"));
                    var taskLink = obj && obj.tasks.object[item.getString("tab_task_id")];
                    if (!taskLink)
                        continue;
                    processAction.assignTaskLink(taskLink);
                }
                else if (processAction.requiresTaskView()) {
                    var taskId = item.getString("task_id");
                    var taskVariantId = item.getString("task_variant_id");
                    var taskView = void 0;
                    var task = application.model.tasks.object[taskId];
                    if (!task)
                        continue;
                    if (taskVariantId === null) {
                        taskView = task.defaultTaskView;
                    }
                    else {
                        taskView = task.taskViews.object[taskVariantId];
                    }
                    processAction.assignTaskView(taskView);
                }
                // TODO: Reports
                processAction.useProcessLogic = item.getBool("use_processes");
                processAction.confirmStart = item.getBool("confirm_start");
                processAction.transl = fillHelper.findTranslation(processActionId, 30 /* ProcessAction */);
                //auto_commit
                processFlow.processFlowActions[processActionId] = processAction;
            }
        });
    };
    BaseModelReader.prototype.loadProcessActionStartObject = function (application, params, fillHelper) {
        if (!fillHelper.containsProcessFlow) {
            return jQuery.Deferred().resolve().promise();
        }
        return this.readGetGui("get_gui_process_action_start_object", params, function (reader) {
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                var processFlowId = item.getString("process_flow_id");
                var processFlow = fillHelper.processFlow[processFlowId];
                var processAction = processFlow && processFlow.processFlowActions[item.getString("process_action_id")];
                if (!processAction)
                    continue;
                var obj = application.model.getObj(item.getString("tab_id"));
                if (obj) {
                    var tabVariantId = item.getString("tab_variant_id");
                    var objView = (tabVariantId ? fillHelper.objViews[obj.objId][tabVariantId] : obj.defaultObjView);
                    if (objView) {
                        objView.processFlowRoots.push(processAction);
                    }
                }
                var task = application.model.tasks.key(item.getString("task_id"));
                if (task) {
                    var taskVariantId = item.getString("task_variant_id");
                    var taskView = (taskVariantId ? fillHelper.taskViews[obj.objId][taskVariantId] : task.defaultTaskView);
                    if (taskView) {
                        taskView.paramObjView.processFlowRoots.push(processAction);
                    }
                }
                // TODO: Reports
            }
        });
    };
    BaseModelReader.prototype.loadProcessStep = function (application, params, fillHelper) {
        if (!fillHelper.containsProcessFlow) {
            return jQuery.Deferred().resolve().promise();
        }
        return this.readGetGui("get_gui_process_step", params, function (reader) {
            var successFulCache = [false, true, undefined];
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                var processFlowId = item.getString("process_flow_id");
                var processFlow = fillHelper.processFlow[processFlowId];
                if (!processFlow)
                    continue;
                var sourceNode = processFlow && processFlow.processFlowActions[item.getString("last_process_action_id")];
                var destinationNode = processFlow && processFlow.processFlowActions[item.getString("next_process_action_id")];
                if (!sourceNode || !destinationNode)
                    continue;
                var connector = new ProcessEngine.ProcessNodeConnector(item.getString("process_step_id"));
                connector.destinationNode = destinationNode;
                connector.HasProcessLogicInputParameter = item.getBool("process_order_input");
                connector.HasProcessLogicOutputParameter = item.getBool("process_order_output");
                connector.lastActionSuccessfull = successFulCache[item.getNumber("last_process_action_successful")];
                sourceNode.connectors.push(connector);
                //abs_order_no
            }
        });
    };
    BaseModelReader.prototype.loadScreenComponent = function (application, params) {
        var _this = this;
        return this.readGetGui("get_gui_screen_component", params, function (reader) {
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                var screenTypeId = item.getString("screen_type_id");
                var screenType = application.model.screenTypes[screenTypeId];
                if (!screenType) {
                    screenType = new Model.ScreenType(screenTypeId);
                    application.model.screenTypes[screenTypeId] = screenType;
                }
                var screenComponent = new Model.ScreenComponent(item.getString("screen_component_id"));
                screenComponent.parentScreenComponentId = item.getString("parent_screen_component_id");
                screenComponent.screenComponentTypeId = item.getString("screen_component_type_id");
                screenComponent.tabPageControlId = item.getString("tab_page_control_id");
                screenComponent.absOrderNo = item.getNumber("abs_order_no");
                screenType.screenComponents.push(screenComponent);
            }
            // Add the master detail screen tpye if it isn't added.
            if (!application.model.screenTypes["master_detail"]) {
                var masterDetailType = new Model.ScreenType("master_detail");
                var splitterScreenComponent = new Model.ScreenComponent("Splitter");
                splitterScreenComponent.screenComponentTypeId = "Splitter";
                splitterScreenComponent.absOrderNo = 0;
                var listViewScreenComponent = new Model.ScreenComponent("Grid");
                listViewScreenComponent.screenComponentTypeId = "Grid";
                listViewScreenComponent.absOrderNo = 10;
                listViewScreenComponent.parentScreenComponentId = "Splitter";
                var formScreenComponent = new Model.ScreenComponent("Form");
                formScreenComponent.screenComponentTypeId = "Form";
                formScreenComponent.absOrderNo = 20;
                formScreenComponent.parentScreenComponentId = "Splitter";
                masterDetailType.screenComponents.push(splitterScreenComponent);
                masterDetailType.screenComponents.push(listViewScreenComponent);
                masterDetailType.screenComponents.push(formScreenComponent);
                application.model.screenTypes["master_detail"] = masterDetailType;
            }
            // Resolve all screen types.
            for (var key in application.model.screenTypes) {
                var screenType = application.model.screenTypes[key];
                var rootComponent = screenType.screenComponents.filter(function (value) {
                    return value.parentScreenComponentId == null;
                })[0];
                _this.resolveScreenType(screenType, rootComponent, true);
            }
        });
    };
    BaseModelReader.prototype.loadScreenComponentProp = function (application, params) {
        var getGuiName = "get_gui_screen_component_prop";
        if (Model.TSFApplicationManager.Current.globalSettings.currentSFVersion >= 2 /* V2018_1 */) {
            getGuiName = "get_gui_screen_component_property";
        }
        return this.readGetGui(getGuiName, params, function (reader) {
            for (var i = 0; i < reader.length; i++) {
                var item = reader.item(i);
                var screenTypeId = item.getString("screen_type_id");
                var screenType = application.model.screenTypes[screenTypeId];
                if (!screenType) {
                    continue;
                }
                var screenComponentId = item.getString("screen_component_id");
                var screenComponent = screenType.getScreenComponent(screenComponentId);
                if (!screenComponent) {
                    return;
                }
                var screenComponentPropId = void 0;
                if (Model.TSFApplicationManager.Current.globalSettings.currentSFVersion >= 2 /* V2018_1 */) {
                    screenComponentPropId = item.getString("screen_component_property_id");
                }
                else {
                    screenComponentPropId = item.getString("screen_component_prop_id");
                }
                var screenComponentPropValue = item.getString("value");
                screenComponent.screenComponentProps[screenComponentPropId] = screenComponentPropValue;
            }
        });
    };
    BaseModelReader.prototype.resolveScreenType = function (screenType, parent, visible) {
        // Get the children and sort them.
        // If we only have one screen component, we don't have a parent show the filter would remove everything
        var sortedChildComponents = screenType.screenComponents.length == 1 ?
            screenType.screenComponents
            : screenType.screenComponents.filter(function (value) {
                return value.parentScreenComponentId == parent.screenComponentId;
            }).sort(function (a, b) {
                return a.absOrderNo - b.absOrderNo;
            });
        // Check for every child component if its a mobile screen type.
        for (var i = 0; i < sortedChildComponents.length; i++) {
            var component = sortedChildComponents[i];
            if (component.screenComponentTypeId === "Form") {
                if (visible) {
                    screenType.addActualScreenComponent(component);
                }
                screenType.hasForm = true;
            }
            else if (visible && component.screenComponentTypeId === "Grid" || component.screenComponentTypeId === "Cube") {
                screenType.addActualScreenComponent(component);
            }
            else if (visible && component.screenComponentTypeId === "TaskBar") {
                screenType.hasTaskBar = true;
            }
            else if (component.screenComponentTypeId === "CombinedFilter") {
                screenType.hasCombinedFilter = true;
            }
            else if (component.screenComponentTypeId === "DetailTabPage") {
                if (visible) {
                    screenType.hasDetailButtons = true;
                }
                this.resolveScreenType(screenType, component, i === 0 && visible);
            }
            else if (component.screenComponentTypeId === "TabPage") {
                if (visible && this.manager.globalSettings.currentSFVersion < 2 /* V2018_1 */ &&
                    component.tabPageControlId !== "grid_tab_page" &&
                    component.tabPageControlId !== "form_tab_page" &&
                    component.tabPageControlId !== null) {
                    screenType.hasDetailButtons = true;
                }
                this.resolveScreenType(screenType, component, i === 0 && visible);
            }
            else if (component.screenComponentTypeId === "Splitter" || component.screenComponentTypeId === "Tab") {
                this.resolveScreenType(screenType, component, visible);
            }
        }
    };
    BaseModelReader.prototype.getTypeOfCol = function (value) {
        return (value == 0) ? Enum.AccessType.Regular : ((value == 1) ? Enum.AccessType.ReadOnly : Enum.AccessType.Hidden);
    };
    BaseModelReader.readGradient = function (row, columnColor1, columnColor2, directionColumn) {
        var color1 = row.getNullableNumber(columnColor1), color2 = row.getNullableNumber(columnColor2), direction = row.getNumber(directionColumn);
        return {
            color1: color1,
            color2: color2,
            direction: direction
        };
    };
    BaseModelReader.isFileColumn = function (controlId) {
        return controlId == Enum.ControlType.IMAGE_LINK ||
            controlId == Enum.ControlType.IMAGE_BLOB ||
            controlId == Enum.ControlType.FILE ||
            controlId == Enum.ControlType.SIGNATURE ||
            controlId == Enum.ControlType.VIDEO_LINK ||
            controlId == Enum.ControlType.FILE_UPLOAD ||
            controlId == Enum.ControlType.IMAGE_UPLOAD;
    };
    BaseModelReader.guiApplParam = "gui_appl_id";
    BaseModelReader.userIdParam = "usr_id";
    BaseModelReader.projectIdParam = "project_id";
    BaseModelReader.projectVrsIdParam = "project_vrs_id";
    BaseModelReader.runtimeConfigurationIdParam = "runtime_configuration_id";
    BaseModelReader.savedObjectModelsKey = "cached_applications";
    BaseModelReader.latestSignedInUsernameKey = "latest-signed-in-username";
    BaseModelReader.latestSignedInPasswordKey = "latest-signed-in-password";
    return BaseModelReader;
}());
