var ModelResolver = /** @class */ (function () {
    function ModelResolver() {
        this.defaultTasksChecked = {};
    }
    ModelResolver.prototype.resolve = function (application, fillHelper) {
        var defaultObjsChecked = {};
        if (application.model.defaultMenu === null) {
            return;
        }
        var menuItems = application.model.defaultMenu.groups.array;
        // we add the objviews here, so unused variants are automatically removed
        for (var _i = 0, menuItems_1 = menuItems; _i < menuItems_1.length; _i++) {
            var group = menuItems_1[_i];
            for (var _a = 0, _b = group.items; _a < _b.length; _a++) {
                var item = _b[_a];
                switch (item.entity.entityType) {
                    case 0 /* Obj */:
                        var obj = application.model.getObj(item.entity.entityId);
                        // The same item can be added in multiple groups so we need to check if it was already resolved
                        if (item.entity.configurationId) {
                            var objView = fillHelper.objViews[obj.objId][item.entity.configurationId];
                            if (this.markTabAsResolved(objView, defaultObjsChecked)) {
                                this.resolveObjRecursive(objView, defaultObjsChecked);
                            }
                        }
                        else {
                            if (this.markTabAsResolved(obj.defaultObjView, defaultObjsChecked)) {
                                this.resolveObjRecursive(obj.defaultObjView, defaultObjsChecked);
                            }
                        }
                        break;
                    case 1 /* Task */:
                        var task = application.model.tasks.key(item.entity.entityId);
                        if (item.entity.configurationId) {
                            var taskView = fillHelper.taskViews[task.taskId][item.entity.configurationId];
                            this.resolveTask(taskView, defaultObjsChecked);
                        }
                        else {
                            this.resolveTask(task.defaultTaskView, defaultObjsChecked);
                        }
                        break;
                }
            }
        }
    };
    ModelResolver.prototype.resolveObjRecursive = function (objView, defaultObjsChecked) {
        for (var _i = 0, _a = objView.objLinkDetailViews.array; _i < _a.length; _i++) {
            var detailView = _a[_i];
            if (this.markTabAsResolved(detailView.objView, defaultObjsChecked)) {
                this.resolveObjRecursive(detailView.objView, defaultObjsChecked);
            }
        }
        for (var _b = 0, _c = objView.objLinkLookupViews.array; _b < _c.length; _b++) {
            var lookupView = _c[_b];
            if (this.markTabAsResolved(lookupView.objView, defaultObjsChecked)) {
                this.resolveObjRecursive(lookupView.objView, defaultObjsChecked);
            }
        }
        for (var _d = 0, _e = objView.objViewProps.array; _d < _e.length; _d++) {
            var col = _e[_d];
            if (col.hasLookup() && col.lookup.targetObjView) {
                var targetObjView = col.lookup.targetObjView;
                if (this.markTabAsResolved(targetObjView, defaultObjsChecked)) {
                    this.resolveObjRecursive(targetObjView, defaultObjsChecked);
                }
            }
        }
        for (var _f = 0, _g = objView.taskLinkViews; _f < _g.length; _f++) {
            var taskLinkView = _g[_f];
            this.resolveTask(taskLinkView.taskView, defaultObjsChecked);
        }
    };
    ModelResolver.prototype.resolveTask = function (taskView, defaultObjsChecked) {
        if (taskView.taskViewId) {
            if (!taskView.task.taskViews.key(taskView.taskViewId)) {
                taskView.task.taskViews.add(taskView.taskViewId, taskView);
                this.resolveObjRecursive(taskView.paramObjView, defaultObjsChecked);
            }
        }
        else {
            if (!this.defaultTasksChecked[taskView.task.taskId]) {
                this.defaultTasksChecked[taskView.task.taskId] = true;
                this.resolveObjRecursive(taskView.paramObjView, defaultObjsChecked);
            }
        }
    };
    ModelResolver.prototype.markTabAsResolved = function (objView, defaultObjsChecked) {
        if (objView.objViewId) {
            if (!objView.obj.objViews.key(objView.objViewId)) {
                //console.log(`Visible: ${objView.objId} - ${objView.objViewId}`);
                objView.obj.objViews.add(objView.objViewId, objView);
                return true;
            }
        }
        else {
            if (!defaultObjsChecked[objView.objId]) {
                //console.log(`Visible default objview: ${objView.objId}`);
                defaultObjsChecked[objView.objId] = true;
                return true;
            }
        }
        return false;
    };
    return ModelResolver;
}());
