var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var ProcessEngine;
(function (ProcessEngine) {
    function postActionCompleted(completedAction, controller) {
        var activeProcessFlow = completedAction.getActiveProcessFlow();
        if (activeProcessFlow) {
            activeProcessFlow.Continue(completedAction.processProperties);
        }
        else {
            if (completedAction.processAction) {
                activeProcessFlow = new ActiveProcessFlow(controller, completedAction.processAction);
                // Add a delay, this way when a task is the start action and is canceled it would first call abort and later set the activeProcessFlow
                window.setTimeout(function () {
                    activeProcessFlow.Continue(completedAction.processProperties);
                }, 0);
            }
        }
        return activeProcessFlow;
    }
    ProcessEngine.postActionCompleted = postActionCompleted;
    var ActiveProcessFlow = /** @class */ (function () {
        function ActiveProcessFlow(controller, processAction) {
            var _this = this;
            this.scheduledActions = [];
            this.controller = controller;
            this.processAction = processAction;
            this.nextAction = {
                CanContinue: true,
                Continue: function () {
                    return _this.pushActiveProcessProcResults(_this.nextAction.Parameter);
                },
                Parameter: null
            };
        }
        ActiveProcessFlow.prototype.Continue = function (processProperties) {
            this.nextAction.Parameter = processProperties;
            this.performAction();
        };
        ActiveProcessFlow.prototype.performAction = function () {
            var _this = this;
            var loop = function () {
                if (_this.nextAction &&
                    (typeof _this.nextAction.CanContinue === "function" && _this.nextAction.CanContinue() ||
                        typeof _this.nextAction.CanContinue === "boolean" && _this.nextAction.CanContinue)) {
                    var result = _this.nextAction.Continue();
                    if (result) {
                        result.always(function (nextAction) {
                            _this.nextAction = nextAction;
                            loop();
                        });
                    }
                }
            };
            loop();
        };
        ActiveProcessFlow.prototype.next = function () {
            if (this.scheduledActions.length > 0) {
                this.processAction = this.scheduledActions.pop();
                return this.checkConfirmStart();
            }
            else {
                return jQuery.Deferred().resolve(this.abort()).promise();
            }
        };
        ActiveProcessFlow.prototype.pushActiveProcessProcResults = function (completedProperties) {
            // TODO: Execute process stored procedure
            var success = completedProperties.success;
            if (this.processAction.connectors.length > 0) {
                var actions = [];
                //if (this.processAction.useProcessLogic) {
                //} else {
                for (var _i = 0, _a = this.processAction.connectors; _i < _a.length; _i++) {
                    var connector = _a[_i];
                    // undefined is always
                    if (connector.lastActionSuccessfull == success || connector.lastActionSuccessfull == undefined) {
                        if (connector.destinationNode.processActionType != 98 /* ProcessFlowStart */ &&
                            connector.destinationNode.processActionType != 99 /* ProcessFlowStop */) {
                            actions.push(connector.destinationNode);
                        }
                    }
                }
                //}
                for (var i = actions.length - 1; i >= 0; i--) {
                    this.scheduledActions.push(actions[i]);
                }
            }
            return this.next();
        };
        ActiveProcessFlow.prototype.checkConfirmStart = function () {
            var _this = this;
            if (this.processAction.confirmStart) {
                var confirmStart_1 = {
                    CanContinue: function () { return confirmStart_1.Parameter != null; },
                    Continue: function () {
                        var resultAction = {
                            CanContinue: function () { return resultAction.Parameter != null; },
                            Continue: function () {
                                if (resultAction.Parameter == Enum.ConfirmationOption.No) {
                                    return _this.next();
                                }
                                return _this.executeActiveProcessAction();
                            },
                            Parameter: null
                        };
                        var popup = ConfirmationPopup.CreateYesNoPopup();
                        var transl = _this.controller.application.language.transl(Model.TranslKeys.gui_process_action_confirm_start);
                        transl = transl.replace('{0}', _this.processAction.transl.transl);
                        popup.initialize('', transl).always(function (result) {
                            resultAction.Parameter = result;
                            _this.performAction();
                        });
                        return jQuery.Deferred().resolve(resultAction).promise();
                    },
                    Parameter: this.nextAction.Parameter
                };
                return jQuery.Deferred().resolve(confirmStart_1).promise();
            }
            else {
                return this.executeActiveProcessAction();
            }
        };
        ActiveProcessFlow.prototype.executeActiveProcessAction = function () {
            var _this = this;
            // Currently we only support actions on the same controller; but this might change in the future
            // if so, we should search for a different controller here
            var result = this.processAction.perform(this.processAction, this.controller);
            if (result && result instanceof CompletedProcessAction && result.processProperties) {
                return this.pushActiveProcessProcResults(result.processProperties);
            }
            else {
                var nextAction_1 = {
                    CanContinue: function () { return nextAction_1.Parameter != null; },
                    Continue: function () {
                        return _this.pushActiveProcessProcResults(nextAction_1.Parameter);
                    },
                    Parameter: null
                };
                return jQuery.Deferred().resolve(nextAction_1).promise();
            }
        };
        ActiveProcessFlow.prototype.abort = function () {
            var _this = this;
            return {
                CanContinue: true,
                Continue: function () {
                    if (_this.controller) {
                        _this.controller.activeProcessFlow = null;
                    }
                    return null;
                },
                Parameter: null
            };
        };
        return ActiveProcessFlow;
    }());
    ProcessEngine.ActiveProcessFlow = ActiveProcessFlow;
    var CompletedProcessAction = /** @class */ (function () {
        function CompletedProcessAction(processAction, processProperties) {
            this.processAction = processAction;
            this.processProperties = processProperties;
        }
        CompletedProcessAction.prototype.getActiveProcessFlow = function () { return null; };
        return CompletedProcessAction;
    }());
    ProcessEngine.CompletedProcessAction = CompletedProcessAction;
    // Class is still a bit empty but in the future we should also store rowBefore/after here.
    var CompletedProcessActionProperties = /** @class */ (function () {
        function CompletedProcessActionProperties(success) {
            this.success = success;
        }
        return CompletedProcessActionProperties;
    }());
    ProcessEngine.CompletedProcessActionProperties = CompletedProcessActionProperties;
    var ProcessFlow = /** @class */ (function () {
        function ProcessFlow(processFlowId) {
            this.processFlowActions = {};
            this.processFlowId = processFlowId;
        }
        return ProcessFlow;
    }());
    ProcessEngine.ProcessFlow = ProcessFlow;
    var ProcessNodeConnector = /** @class */ (function () {
        function ProcessNodeConnector(connectorId) {
            this.connectorId = connectorId;
        }
        return ProcessNodeConnector;
    }());
    ProcessEngine.ProcessNodeConnector = ProcessNodeConnector;
    var ProcessAction = /** @class */ (function () {
        function ProcessAction(processFlow, processActionId) {
            this.connectors = [];
            this.processFlow = processFlow;
            this.processActionId = processActionId;
        }
        ProcessAction.prototype.requiresObj = function () { return false; };
        ProcessAction.prototype.requiresObjView = function () { return false; };
        ProcessAction.prototype.requiresObjLink = function () { return false; };
        ProcessAction.prototype.requiresTaskLink = function () { return false; };
        ProcessAction.prototype.requiresTaskView = function () { return false; };
        ProcessAction.prototype.assignObj = function (obj) { };
        ProcessAction.prototype.assignObjView = function (objView) { };
        ProcessAction.prototype.assignObjLink = function (objLink) { };
        ProcessAction.prototype.assignTaskLink = function (taskLink) { };
        ProcessAction.prototype.assignTaskView = function (taskView) { };
        ProcessAction.fromProcessActionType = function (processFlow, processActionId, type) {
            switch (type) {
                case 3 /* AddRow */:
                    return new ObjContext.AddRow(processFlow, processActionId);
                case 6 /* ExcTask */:
                    return new TaskContext(processFlow, processActionId);
                case 60 /* GotoTask */:
                    return new TaskWithoutContext(processFlow, processActionId);
                case 98 /* ProcessFlowStart */:
                case 99 /* ProcessFlowStop */:
                    return new NoContext(processFlow, processActionId, type);
                default:
                    return new ObjContext.Default(processFlow, processActionId, type);
            }
        };
        return ProcessAction;
    }());
    ProcessEngine.ProcessAction = ProcessAction;
    var NoContext = /** @class */ (function (_super) {
        __extends(NoContext, _super);
        function NoContext(processFlow, processActionId, processActionType) {
            var _this = _super.call(this, processFlow, processActionId) || this;
            _this.processActionType = processActionType;
            return _this;
        }
        NoContext.prototype.perform = function (processAction, controller) {
            return new CompletedProcessAction(processAction, new CompletedProcessActionProperties(true));
        };
        return NoContext;
    }(ProcessAction));
    ProcessEngine.NoContext = NoContext;
    var ObjContext = /** @class */ (function (_super) {
        __extends(ObjContext, _super);
        function ObjContext() {
            return _super !== null && _super.apply(this, arguments) || this;
        }
        ObjContext.prototype.requiresObj = function () { return true; };
        ObjContext.prototype.assignObj = function (obj) { this.obj = obj; };
        ObjContext.prototype.perform = function (processAction, controller) {
            return controller.performObjProcessAction(this, processAction);
        };
        return ObjContext;
    }(ProcessAction));
    ProcessEngine.ObjContext = ObjContext;
    (function (ObjContext) {
        var Default = /** @class */ (function (_super) {
            __extends(Default, _super);
            function Default(processFlow, processActionId, processActionType) {
                var _this = _super.call(this, processFlow, processActionId) || this;
                _this.processActionType = processActionType;
                return _this;
            }
            return Default;
        }(ObjContext));
        ObjContext.Default = Default;
        var AddRow = /** @class */ (function (_super) {
            __extends(AddRow, _super);
            function AddRow(processFlow, processActionId) {
                var _this = _super.call(this, processFlow, processActionId) || this;
                _this.processActionType = 3 /* AddRow */;
                return _this;
            }
            return AddRow;
        }(ObjContext));
        ObjContext.AddRow = AddRow;
    })(ObjContext = ProcessEngine.ObjContext || (ProcessEngine.ObjContext = {}));
    var TaskContext = /** @class */ (function (_super) {
        __extends(TaskContext, _super);
        function TaskContext(processFlow, processActionId) {
            var _this = _super.call(this, processFlow, processActionId) || this;
            _this.processActionType = 6 /* ExcTask */;
            return _this;
        }
        TaskContext.prototype.requiresTaskLink = function () { return true; };
        TaskContext.prototype.assignTaskLink = function (tasklink) { this.taskLink = tasklink; };
        TaskContext.prototype.perform = function (processAction, controller) {
            // TODO: We might want to convert taskLinkViews to a Collections.List to make this easier
            for (var _i = 0, _a = controller.objView.taskLinkViews; _i < _a.length; _i++) {
                var tv = _a[_i];
                if (tv.taskView.task.taskId == this.taskLink.task.taskId) {
                    taskHelper.showPageOrPopup(tv.taskView, controller);
                    // the (context)controller will notify the process engine and continue
                    return null;
                }
            }
            // Task not found; return als failed so we don't get stuck in the processflow
            return new CompletedProcessAction(processAction, new CompletedProcessActionProperties(false));
        };
        return TaskContext;
    }(ProcessAction));
    ProcessEngine.TaskContext = TaskContext;
    var TaskWithoutContext = /** @class */ (function (_super) {
        __extends(TaskWithoutContext, _super);
        function TaskWithoutContext(processFlow, processActionId) {
            var _this = _super.call(this, processFlow, processActionId) || this;
            _this.processActionType = 60 /* GotoTask */;
            return _this;
        }
        TaskWithoutContext.prototype.requiresTaskView = function () { return true; };
        TaskWithoutContext.prototype.assignTaskView = function (taskview) { this.taskView = taskview; };
        TaskWithoutContext.prototype.perform = function (processAction, controller) {
            taskHelper.showPageOrPopup(this.taskView, null);
            return null;
        };
        return TaskWithoutContext;
    }(ProcessAction));
    ProcessEngine.TaskWithoutContext = TaskWithoutContext;
})(ProcessEngine || (ProcessEngine = {}));
