var Scheduler = /** @class */ (function () {
    /**
     * @param maxConcurrentTasks Maximum number of concurrent tasks
     */
    function Scheduler(maxConcurrentTasks, callback) {
        var _this = this;
        this.maxConcurrentTasks = maxConcurrentTasks;
        this.callback = callback;
        this.queue = [];
        this.currentRunningTasks = 0;
        this.canceled = false;
        this.itemCompleted = function () {
            _this.currentRunningTasks--;
            _this.doWork();
        };
    }
    /**
     * Schedule a task, this also starts the scheduler automatically.
     * @param func A function which should return a jQueryPromise
     */
    Scheduler.prototype.schedule = function (func) {
        this.queue.push(func);
        this.doWork();
    };
    Scheduler.prototype.doWork = function () {
        var queue = this.queue;
        if (this.currentRunningTasks < this.maxConcurrentTasks && queue.length > 0 && !this.canceled) {
            this.currentRunningTasks++;
            var task = queue.shift();
            task().always(this.itemCompleted);
        }
        else {
            if (this.canceled || this.currentRunningTasks == 0) {
                if (this.callback) {
                    this.callback(this.canceled);
                }
            }
        }
    };
    Scheduler.prototype.cancel = function () {
        this.canceled = true;
    };
    return Scheduler;
}());
