var TSFEventEmitter = /** @class */ (function () {
    function TSFEventEmitter() {
        this.eventList = {};
    }
    /*
     * Register an eventhandler for the specified type.
     * @param type The event name to register. To use the listener for multiple events separate the types by a space.
     * @param listener The listener which is called when the event is fired.
     */
    TSFEventEmitter.prototype.on = function (type, listener) {
        var events = this.eventList;
        type.split(" ").forEach(function (value, index, array) {
            (events[value]
                ? events[value]
                : events[value] = []).push(listener);
        });
    };
    /*
     * Emit al associated listeners based on the eventname.
     * @param eventname The event to emit.
     * @param args The EventArgs implementation which is passed to the associated listeners.
     */
    TSFEventEmitter.prototype.emit = function (eventname, args) {
        var listeners;
        if (this.eventList && (listeners = this.eventList[eventname])) {
            listeners.forEach(function (listener) {
                listener(args);
            });
        }
    };
    TSFEventEmitter.prototype.clearEvents = function () {
        // Clear all events by creating a new eventlist
        this.eventList = {};
    };
    return TSFEventEmitter;
}());
