var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var Utils;
(function (Utils) {
    /**
     * On Huawei devices, readonly textarea's still open the keyboard
     * Tested on Huawei Ascend P6 (Android 4.2.2)
     * Disabled does mean that it can't get focus, which is annoying on the windows 8 store apps for example
     * It was possible there to select text in readOnly mode
     *
     * Because of that we only apply the fix for Huawei and other Anroid 4.0-3 devices
     */
    Utils.disableTextArea = (/Huawei|Android 4.[0-3]/).test(navigator.userAgent);
    /**
     * Parse the query string and returns an array with the values.
     *
     * @param urlSearchPart The url query search including the "?"
     * @return Array Array with the url parameters, when the same name is found multiple times it will return it as an array.
     * for example, ?var1=foo&var1=bar will be parsed to [ 'var1': ["foo", "bar"] ]
     */
    function parseUrlVars(urlSearchPart) {
        // modified version from http://stackoverflow.com/questions/10609511/javascript-url-parameter-parsing
        var raw = [];
        var indexed = {};
        var hashes = urlSearchPart.substring(1).split('&');
        for (var i = 0; i < hashes.length; i++) {
            var hash = hashes[i].split('=');
            var key = hash[0], value = hash[1];
            raw.push({ key: key, value: value });
            if (indexed[key] != null) {
                var item = indexed[key];
                if (!Array.isArray(item)) {
                    indexed[key] = [];
                    indexed[key].push(item);
                }
                indexed[key].push(value);
            }
            else {
                indexed[key] = value;
            }
        }
        return { indexed: indexed, raw: raw };
    }
    Utils.parseUrlVars = parseUrlVars;
    //Source: https://stackoverflow.com/questions/1634748/how-can-i-delete-a-query-string-parameter-in-javascript#comment52631984_25214672
    function removeParameterFromUrl(url, parameter) {
        return url
            .replace(new RegExp("^([^#]*\?)(([^#]*)&)?" + parameter + "(\=[^&#]*)?(&|#|$)"), "$1$3$5")
            .replace(/^([^#]*)((\?)&|\?(#|$))/, "$1$3$4");
    }
    Utils.removeParameterFromUrl = removeParameterFromUrl;
    function whenAll(promises) {
        var lastResolved = 0;
        var wrappedDeferreds = [];
        for (var _i = 0, promises_1 = promises; _i < promises_1.length; _i++) {
            var promise = promises_1[_i];
            wrappedDeferreds.push(jQuery.Deferred());
            promise.always(wrappedDeferreds[lastResolved++].resolve);
        }
        return jQuery.when.apply(jQuery, wrappedDeferreds);
    }
    Utils.whenAll = whenAll;
    /**
     * Joins url by adding a slash ('/') between the parts in the string
     */
    function CombineUrl() {
        var parts = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            parts[_i] = arguments[_i];
        }
        var newUrl = '';
        for (var i = 0; i < parts.length - 1; i++) {
            var part = parts[i];
            if (part[part.length - 1] !== "/") {
                part += "/";
            }
            newUrl += part;
        }
        // Sometimes they fill in a / for the root (because it is mandatory in IAM)
        // causing the url to contain two slashes
        if (parts[parts.length - 1] != "/") {
            newUrl += parts[parts.length - 1];
        }
        return newUrl;
    }
    Utils.CombineUrl = CombineUrl;
    //Source: https://github.com/jeremyBanks/b64-to-blob/blob/master/b64toBlob.js
    function b64toBlob(b64Data, contentType, sliceSize) {
        if (contentType === void 0) { contentType = ""; }
        if (sliceSize === void 0) { sliceSize = 512; }
        var byteCharacters = atob(b64Data);
        var byteArrays = [];
        for (var offset = 0; offset < byteCharacters.length; offset += sliceSize) {
            var slice = byteCharacters.slice(offset, offset + sliceSize);
            var byteNumbers = [];
            for (var i = 0; i < slice.length; i++) {
                byteNumbers[i] = slice.charCodeAt(i);
            }
            var byteArray = new Uint8Array(byteNumbers);
            byteArrays.push(byteArray);
        }
        return new Blob(byteArrays, { type: contentType });
    }
    Utils.b64toBlob = b64toBlob;
    function createIconFromBase64(base64) {
        var types = {
            "iVBOR": "png",
            "/9j/4": "jpg",
            "Qk06A": "bmp",
            "R0lGO": "gif"
        };
        for (var type in types) {
            if (base64.substring(0, type.length) == type) {
                return "data:image/" + types[type] + ";base64," + base64;
            }
        }
        return "";
    }
    Utils.createIconFromBase64 = createIconFromBase64;
    function stringEndsWith(input, search) {
        return input.substring(input.length - search.length, input.length) === search;
    }
    Utils.stringEndsWith = stringEndsWith;
    var mimetypeMap = {
        'doc': 'application/msword',
        'docx': 'application/msword',
        'pdf': 'application/pdf',
        'ppt': 'application/vnd.ms-powerpoint',
        'pptx': 'application/vnd.ms-powerpoint',
        'xls': 'application/vnd.ms-excel',
        'xlsx': 'application/vnd.ms-excel',
        'rtf': 'application/rtf',
        'wav': 'audio/x-wav',
        'gif': 'image/gif',
        'jpeg': 'image/jpeg',
        'bmp': 'image/bmp',
        'png': 'image/png',
        'txt': 'text/plain',
        'mpg': 'video/*',
        'mpeg': 'video/*',
        'mpe': 'video/*',
        'mp4': 'video/*',
        'avi': 'video/*'
    };
    function getMimeType(fileName) {
        var index = fileName.lastIndexOf(".");
        if (index == -1) {
            return '*/*';
        }
        else {
            // +1 to remove the dot and toLowerCase so we make sure png is the same as PNG
            var extension = fileName.substr(index + 1).toLowerCase();
            var mimeType = mimetypeMap[extension];
            if (mimeType == undefined) {
                mimeType = '*/*';
            }
            return mimeType;
        }
    }
    Utils.getMimeType = getMimeType;
    function getOfflineLogicObject(application, obj) {
        var vrs = application.projectVersion.replace(/\./g, "");
        var namespace = application.projectName + vrs;
        var project = window[namespace];
        // TODO: We should provide a way for the offline functionality to return
        // a message and still continue with the values like TSF_SEND_MESSAGE.
        if (project) {
            return project[obj.objId];
        }
        return null;
    }
    Utils.getOfflineLogicObject = getOfflineLogicObject;
    /*
     * Returns if the given value represents an empty value.
     * @param value The value to check.
     * @return true if value is empty, otherwise false.
     */
    function isEmpty(value) {
        return value === null || value === undefined || value === "";
    }
    Utils.isEmpty = isEmpty;
    function sanitizeRow(rowObject, objProps, filter) {
        for (var i = 0; i < objProps.length; i++) {
            var objProp = objProps[i];
            if (!filter(objProp)) {
                delete rowObject[objProp.objPropId];
            }
        }
    }
    Utils.sanitizeRow = sanitizeRow;
    function normalizeRow(row, objProps) {
        var rowObject = {};
        if (!row)
            return rowObject;
        for (var i = 0; i < objProps.length; i++) {
            var value = row ? row.getValueForObjProp(objProps[i]) : null;
            if (value !== null) {
                rowObject[objProps[i].objPropId] = value;
            }
        }
        return rowObject;
    }
    Utils.normalizeRow = normalizeRow;
    function isFileContract(value) {
        //When coming from IndexedDB, FileContracts will be plain objects, so we have to check for FilePath property too.
        return value instanceof FileContract || value.hasOwnProperty("FilePath");
    }
    function convertToDataType(value, propId, dttpId) {
        var errorConverting = false;
        var newValue;
        //Null & undefined values, FileContracts and temporary identities don't need to be converted
        if (value === null || value === undefined || isFileContract(value) || (typeof value === "string" && value.indexOf(IdentityReplacer.prefix) === 0)) {
            return value;
        }
        switch (dttpId) {
            case Model.DttpId.boolean:
                newValue = Utils.convertToBoolean(value, null);
                if (newValue === null) {
                    errorConverting = true;
                }
                break;
            case Model.DttpId.integer:
                newValue = parseInt(value, 10);
                if (isNaN(newValue)) {
                    errorConverting = true;
                }
                break;
            case Model.DttpId.float:
                newValue = parseFloat(value);
                if (isNaN(value)) {
                    errorConverting = true;
                }
                break;
            case Model.DttpId.string:
                newValue = typeof value === "string" ? value : value.toString();
                break;
            case Model.DttpId.time:
                if (!moment(value, "hh:mm:ss.sss").isValid()) {
                    errorConverting = true;
                }
                else {
                    newValue = value;
                }
                break;
            case Model.DttpId.datetime:
                //Validate if given a date object
                if (value instanceof Date) {
                    var parsedDate = moment(value);
                    if (!parsedDate.isValid()) {
                        errorConverting = true;
                    }
                    else {
                        newValue = value;
                    }
                }
                else {
                    //Handle parsing from string
                    var parsedDate = moment(value.trim(), ['L', 'L LT'], true);
                    if (!parsedDate.isValid()) {
                        errorConverting = true;
                    }
                    newValue = parsedDate.toDate();
                }
                break;
            default:
                newValue = value;
                break;
        }
        if (errorConverting) {
            //    const language = Model.TSFApplicationManager.Current.activeAppl.language;
            //    let errorMsg = language.transl(Model.TranslKeys.errorConvertingDatatypeMessage);
            //    errorMsg = errorMsg.replace("[value]", value);
            //    errorMsg = errorMsg.replace("[column]", propId);
            //    errorMsg = errorMsg.replace("[datatype]", Model.DttpId[dttpId]);
            //    const errorPopup = ConfirmationPopup.CreateOkPopup();
            //    errorPopup.initialize(language.transl(Model.TranslKeys.errorConvertingDatatypeTitle), errorMsg);
            return;
        }
        return newValue;
    }
    Utils.convertToDataType = convertToDataType;
    function valuesEqual(value1, value2) {
        //If one side is undefined and the other null we consider them equal.
        if ((value1 === undefined || value1 === null) && (value2 === undefined || value2 === null)) {
            return true;
        }
        if (value1 instanceof Date && value2 instanceof Date) {
            return value1.getTime() === value2.getTime();
        }
        if (value1 instanceof FileContract && value2 instanceof FileContract) {
            return value1.FilePath === value2.FilePath;
        }
        return value1 === value2;
    }
    Utils.valuesEqual = valuesEqual;
    function convertToBoolean(value, defaultValue) {
        if (typeof value == "boolean")
            return value;
        if (value == 1)
            return true;
        if (value == 0)
            return false;
        if (value) {
            return value.indexOf("true") != -1 || value.indexOf("1") != -1;
        }
        return defaultValue;
    }
    Utils.convertToBoolean = convertToBoolean;
    function serializeJSON(input) {
        return JSON.stringify(input, function (key, value) {
            // All strings that strictly match the formats below are considered dates, all other strings are not.
            // Neither JSON.stringify nor Indicium's serializer will output dates in any other format than the two below.
            // YYYY-MM-DDTHH:mm:ssZ
            // YYYY-MM-DDTHH:mm:ss.SSSZ
            var matches = null;
            if (typeof value === "string" && (matches = value.match(/(\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}(?:[.]\d{1,3})?Z)/)) != null && matches.length == 2) {
                // Return the default serialized JSON datetime string without the Z.
                // This way there's no chance of timezone conversion taking place.
                return moment(matches[1]).format("YYYY-MM-DDTHH:mm:ss.SSS");
            }
            return value;
        });
    }
    Utils.serializeJSON = serializeJSON;
    function deserializeJSON(input) {
        return JSON.parse(input, function (key, value) {
            // All strings that strictly match the formats below are considered dates, all other strings are not.
            // Neither JSON.stringify nor Indicium's serializer will output dates in any other format than the two below.
            // YYYY-MM-DDTHH:mm:ssZ
            // YYYY-MM-DDTHH:mm:ss.SSSZ
            var matches = null;
            if (typeof value === "string" && (matches = value.match(/(\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}(?:[.]\d{1,3})?)Z/)) != null && matches.length == 2) {
                // Ignore the timezone (the Z is not captured in the group) when parsing dates to avoid conversions to the user's timezone.
                return moment(matches[1]).format("YYYY-MM-DDTHH:mm:ss.SSSZ");
            }
            return value;
        });
    }
    Utils.deserializeJSON = deserializeJSON;
    function addFooterPicture(style, container) {
        // Add a footer picture if there is one specified.
        if (style && style.quickbar.pictureFooter) {
            var img = document.createElement("img");
            img.src = Utils.createIconFromBase64(style.quickbar.pictureFooter);
            img.className = "pictureFooter";
            container.append(img);
        }
    }
    Utils.addFooterPicture = addFooterPicture;
    function editSyncImage(page, syncing) {
        var h1Element = page.children("div[data-role='header']").children("h1");
        if (syncing) {
            // Guard against adding the element twice
            if (h1Element.find(".sync-icon").length > 0) {
                return;
            }
            var syncImage = document.createElement("img");
            syncImage.className = "sync-icon title-icon";
            syncImage.setAttribute("src", "img/network/sync.png");
            h1Element.append(syncImage);
        }
        else {
            h1Element.find(".sync-icon").remove();
        }
    }
    Utils.editSyncImage = editSyncImage;
    Utils.editOnlineImage = !AppConfig.showOfflineIndicator ? jQuery.noop : function (page) {
        var h1Element = page.children("div[data-role='header']").children("h1");
        // Always try to remove in case it was added already.
        h1Element.find(".offline-icon").remove();
        if (!AppConfig.online) {
            var offlineImage = document.createElement("img");
            offlineImage.className = "offline-icon title-icon";
            offlineImage.setAttribute("src", "img/network/offline.png");
            h1Element.append(offlineImage);
        }
    };
    function isElectron() {
        return (typeof window["process"] != "undefined" && !!window["process"].versions['electron']);
    }
    Utils.isElectron = isElectron;
    function isCordova() {
        return typeof window["cordova"] !== "undefined";
    }
    Utils.isCordova = isCordova;
    function getiOSVersion() {
        var matches = navigator.userAgent.match(/(iPad|iPhone);.*OS (\d+)_(\d+)/);
        if (matches && matches.length == 4) {
            return parseFloat(matches[2] + "." + matches[3]);
        }
        return null;
    }
    Utils.getiOSVersion = getiOSVersion;
    function isAndroidTwo() {
        return /Android 2.[0-3]/.test(navigator.userAgent);
    }
    Utils.isAndroidTwo = isAndroidTwo;
    function isAndroidVersionOrUp(givenMinimumVersion) {
        var androidVersion = navigator.userAgent.toLowerCase().match(/android\s([0-9\.]*)/);
        if (!androidVersion) {
            return false;
        }
        return parseFloat(androidVersion[1]) >= givenMinimumVersion;
    }
    Utils.isAndroidVersionOrUp = isAndroidVersionOrUp;
    function isAndroid() {
        return /Android/.test(navigator.userAgent);
    }
    Utils.isAndroid = isAndroid;
    function documentToJSon(document) {
        var object = {};
        for (var i = 0; i < document.childNodes.length; i++) {
            object[document.childNodes[i].localName] = xmlToJSon(document.childNodes[i]);
        }
        return object;
    }
    Utils.documentToJSon = documentToJSon;
    function xmlToJSon(node) {
        // Return null when a node has no childs. An empty Object would suggest that the node has a value.
        if (node.childNodes.length == 0) {
            return null;
        }
        var object = {};
        for (var i = 0; i < node.childNodes.length; i++) {
            var childNode = node.childNodes[i];
            var value = void 0, key = void 0;
            if (childNode.nodeType === Node.TEXT_NODE) {
                if (node.childElementCount !== 0) {
                    continue;
                }
                return childNode.textContent;
            }
            else {
                key = childNode.localName;
                value = xmlToJSon(childNode);
            }
            if (object[childNode.localName] !== undefined) {
                if (Object.prototype.toString.call(object[key]) !== '[object Array]') {
                    var currentItem = object[key];
                    object[key] = [];
                    object[key].push(currentItem);
                }
                object[key].push(value);
            }
            else {
                object[key] = value;
            }
        }
        return object;
    }
    Utils.xmlToJSon = xmlToJSon;
    function isBuiltInError(errorMessage) {
        //Put the Model.TranslKeys property values in an array
        var translKeys = [];
        for (var key in Model.TranslKeys) {
            translKeys.push(Model.TranslKeys[key]);
        }
        return translKeys.indexOf(errorMessage) !== -1;
    }
    Utils.isBuiltInError = isBuiltInError;
    function replaceInErrorMessage(errorMessage, replaces) {
        for (var i = 0; i < replaces.length; i++) {
            errorMessage = errorMessage.replace(replaces[i][0], replaces[i][1]);
        }
        return errorMessage;
    }
    Utils.replaceInErrorMessage = replaceInErrorMessage;
    function chainPromise(chain, func) {
        return chain.then(func, func);
    }
    Utils.chainPromise = chainPromise;
    /*  Reports the error to Sentry if online and Sentry is loaded.
        Shows the user a popup with Sentry ID or error details (if Sentry is unavailable) */
    function reportError(error, msg, uri, lineno, columno) {
        var application = Model.TSFApplicationManager.Current.activeAppl;
        var language = application != null ? application.language : AppConfig.language;
        var confirmPopup = ConfirmationPopup.CreateOkPopup();
        // Do not show the Raven message when the user is offline, because the message tells the user 'the team has been notified'
        // which isn't the case
        if (Raven !== undefined && AppConfig.online) {
            confirmPopup.initialize(language.transl(Model.TranslKeys.errorDialogTitle), language.transl(Model.TranslKeys.errorDialogMessage) + Raven.lastEventId());
        }
        else {
            confirmPopup.initialize(language.transl(Model.TranslKeys.errorDialogTitle), msg + " " + uri + " " + lineno + ":" + columno);
        }
        Utils.tryRavenCaptureException(error, msg, uri, lineno, columno);
    }
    Utils.reportError = reportError;
    // Sends given error to Sentry
    function tryRavenCaptureException(error, msg, uri, lineno, columno) {
        if (Raven !== undefined) {
            // We had a few weird undefined errors in our sentry log
            // with this code we hopefully see what's wrong.
            if (!error) {
                if (msg) {
                    Raven.captureException(new Error(msg + " " + uri + " " + lineno + ":" + columno));
                }
            }
            else {
                //Assume it's an error when it has a stack property.
                if (error.hasOwnProperty("stack")) {
                    Raven.captureException(error);
                    return;
                }
                Raven.captureException(new Error(error));
            }
        }
    }
    Utils.tryRavenCaptureException = tryRavenCaptureException;
    function showLoaderWithDelay(delay, text) {
        return window.setTimeout(function () {
            showLoader(text);
        }, delay);
    }
    Utils.showLoaderWithDelay = showLoaderWithDelay;
    var loader = null;
    function showLoader(text) {
        if (text === void 0) { text = ""; }
        // Only show the loader when there is no popup visible.
        if (jQuery(".ui-popup-container:not(.ui-popup-hidden)").length == 0) {
            // Update the text of the loader if there's an existing one.
            if (loader !== null) {
                loader.children("h1").text(text);
                return;
            }
            loader = jQuery.mobile.loading('show', {
                text: text,
                textVisible: !Utils.isEmpty(text)
            });
        }
    }
    Utils.showLoader = showLoader;
    function hideLoader() {
        loader = null;
        jQuery.mobile.loading('hide');
    }
    Utils.hideLoader = hideLoader;
    function evaluateJoinConditions(joinConditions, sourceItem, targetItem) {
        for (var h = 0; h < joinConditions.length; h++) {
            var joinCondition = joinConditions[h];
            var itemValue = sourceItem.getValueForObjProp(joinCondition.sourceProp);
            var readerValue = targetItem.getValueForObjProp(joinCondition.targetProp);
            // TODO: Does this work for date's?
            if (itemValue != readerValue) {
                return false;
            }
        }
        return true;
    }
    Utils.evaluateJoinConditions = evaluateJoinConditions;
    function updatePanelsHeight(panels, headerHeight, newHeight, detailHeight, footerHeight) {
        // Fixes scroll on Android2.* because we use the browser scroll instead of iScroll.
        if (Utils.isAndroidTwo()) {
            panels = panels.not(".form");
        }
        newHeight -= headerHeight + footerHeight + detailHeight;
        var oldHeight = panels.height();
        var firstTime = !(panels.length > 0 && panels[0].style.height);
        panels.css('height', newHeight + 'px');
        // Trigger a updateLayout so let components know there size changed.
        // This makes it possible to update for example scroll bars the right way, use a timer, so the browser can redraw first
        window.setTimeout(function () {
            panels.trigger("tsf-updatelayout", [oldHeight, newHeight, firstTime]);
        }, 0);
    }
    Utils.updatePanelsHeight = updatePanelsHeight;
    function fixScrolling(page) {
        var header = page.children("div[data-role='header']");
        var footer = page.children("div[data-role='footer']");
        if (!Utils.isAndroidTwo()) {
            var contentPanel_1 = page.find(".content-panel");
            page.on("pageshow", function () {
                Utils.updatePanelsHeight(contentPanel_1, header.outerHeight(), window.innerHeight, 0, footer.outerHeight());
            });
            page.on("updatePanelsHeight", function (e, data) {
                Utils.updatePanelsHeight(contentPanel_1, data.headerHeight, data.height, 0, data.footerHeight);
                data.handled = true;
            });
        }
    }
    Utils.fixScrolling = fixScrolling;
    /**
     * Scrolls to the according input element when its out of sight.
     * Also adjusts the contentElements overflow property so that the user can scroll.
    */
    function fixResizeOnFocus(formBody) {
        var contentElement = formBody.parent();
        var touchStartX = 0;
        var touchStartY = 0;
        var trackForFocus = true;
        formBody.on("pointerdown touchstart", "input, textarea, select", function (evt) {
            var castedEvent = event;
            touchStartX = castedEvent.targetTouches ? castedEvent.targetTouches[0].pageX : castedEvent.pageX;
            touchStartY = castedEvent.targetTouches ? castedEvent.targetTouches[0].pageY : castedEvent.pageY;
            trackForFocus = true;
        });
        formBody.on("pointermove touchmove", "input, textarea, select", function (evt) {
            var castedEvent = event;
            var touchX = castedEvent.targetTouches ? castedEvent.targetTouches[0].pageX : castedEvent.pageX;
            var touchY = castedEvent.targetTouches ? castedEvent.targetTouches[0].pageY : castedEvent.pageY;
            if (trackForFocus && Math.abs(touchX - touchStartX) > 10 || Math.abs(touchY - touchStartY) > 10) {
                trackForFocus = false;
            }
        });
        formBody.on("pointerup touchend", "input, textarea, select, .k-editor", function (evt) {
            if (trackForFocus) {
                var target_1 = evt.currentTarget;
                jQuery.mobile.activePage.off("updatePanelsHeight.fixFocus").on("updatePanelsHeight.fixFocus", function () {
                    // Get to the closest label instead of the actual control when there is a label.
                    // The jQuery output demands that a control has only one parent .ui-field-contain.
                    var scrollToTarget;
                    var parentContainer = jQuery(target_1).closest(".ui-field-contain");
                    if (parentContainer.length == 1) {
                        scrollToTarget = parentContainer[0];
                    }
                    else {
                        scrollToTarget = target_1;
                    }
                    var requeueFocus = function () {
                        // requeue focus, so the browser can complete with what it is doing.
                        window.setTimeout(function () {
                            target_1.focus();
                            jQuery.mobile.activePage.off("updatePanelsHeight.fixFocus");
                        }, 0);
                    };
                    if (scrollToTarget["scrollIntoViewIfNeeded"]) {
                        scrollToTarget["scrollIntoViewIfNeeded"]();
                        requeueFocus();
                    }
                    else {
                        // Custom implementation of scrollIntoViewIfNeeded.
                        var targetRect = target_1.getBoundingClientRect();
                        // Use the contentElement because the size of the formbody might exceed the actual div that scrolls.
                        var formRect = contentElement[0].getBoundingClientRect();
                        if (targetRect.top < formRect.top || targetRect.bottom > formRect.bottom || targetRect.right > formRect.right || targetRect.left < formRect.left) {
                            scrollToTarget.scrollIntoView();
                            requeueFocus();
                        }
                        else {
                            window.setTimeout(function () {
                                jQuery.mobile.activePage.off("updatePanelsHeight.fixFocus");
                            }, 1000);
                        }
                    }
                });
            }
        });
        // Fix iOS scroll, scrolling doesn't work till after changing the overflow after a resize (keyboard)
        contentElement.on("tsf-updatelayout", function (evt, oldHeight, height, firstTime) {
            if (firstTime) {
                // We return when it is the first time, the code causes some flickering on iOS and it isn't needed the first time
                return;
            }
            var currentHeight = height ? height : parseInt(this.style.height, 10);
            // Count the total height of every children, since we can have a form and a taskbar.
            var childHeight = 0;
            jQuery(this).children().each(function () {
                childHeight += jQuery(this).outerHeight();
            });
            var needsScrollBar = currentHeight < childHeight;
            if (!oldHeight || (needsScrollBar && childHeight < oldHeight)) {
                var $this_1 = jQuery(this).css('overflow', 'hidden');
                window.setTimeout(function () {
                    $this_1.css('overflow', 'auto');
                }, 0);
            }
        });
    }
    Utils.fixResizeOnFocus = fixResizeOnFocus;
    function SetIconBlackOrWhite(element, color) {
        if (color && color.getBrightness() > 0.5)
            jQuery(element).addClass("ui-alt-icon"); // White
        else
            jQuery(element).removeClass("ui-alt-icon"); // Black, class is mostely not set, but it is e.g. when loging out on the options icon.
    }
    Utils.SetIconBlackOrWhite = SetIconBlackOrWhite;
    function allowGroupSeparator(controlId) {
        return controlId == Enum.ControlType.CALCULATOR ||
            controlId == Enum.ControlType.NUMERIC ||
            controlId == Enum.ControlType.CURRENCY;
    }
    Utils.allowGroupSeparator = allowGroupSeparator;
    function orderArrayByObjectProperty(arr, propName) {
        return arr.sort(function (a, b) {
            if (a[propName] < b[propName]) {
                return -1;
            }
            if (a[propName] > b[propName]) {
                return 1;
            }
            return 0;
        });
    }
    Utils.orderArrayByObjectProperty = orderArrayByObjectProperty;
    //Modified version from: http://stackoverflow.com/questions/6784894/add-commas-or-spaces-to-group-every-three-digits
    function formatNumber(value, precision, language, showGroupSeparator) {
        var globalization = language.globalization;
        if (precision > 0 || globalization.brokenToLocaleString) {
            var str = value.toFixed(precision).split('.');
            if (showGroupSeparator && str[0].length >= 4) {
                str[0] = str[0].replace(/(\d)(?=(\d{3})+$)/g, '$1' + globalization.numberGroupSeparator);
            }
            return str.join(globalization.numberDecimalSeparator);
        }
        else if (!showGroupSeparator) {
            // This case is for when there isn't a precision, but toLocaleString works correctly.
            // There might be cases where group separators shouldn't be added and toLocaleString would still do that
            // So we special case it - required for example for the electron viewer and possible for newer android versions
            return value.toFixed(0);
        }
        else {
            // This adds group separators by default
            return value.toLocaleString(language.code);
        }
    }
    Utils.formatNumber = formatNumber;
    function caseInsenstiveStringCompare(value1, value2) {
        // Both are null so the values are the same
        if (value1 === null && value2 === null) {
            return true;
        }
        // One of them is null, but the other one isn't so they are not the same
        if (value1 === null && value2 !== null ||
            value1 !== null && value2 === null) {
            return false;
        }
        return value1.toLowerCase() === value2.toLowerCase();
    }
    Utils.caseInsenstiveStringCompare = caseInsenstiveStringCompare;
    // Taken from underscore.js
    // Returns a function, that, as long as it continues to be invoked, will not
    // be triggered. The function will be called after it stops being called for
    // N milliseconds. If `immediate` is passed, trigger the function on the
    // leading edge, instead of the trailing.
    function debounce(func, wait, immediate) {
        var timeout;
        return function () {
            var context = this, args = arguments;
            var later = function () {
                timeout = null;
                if (!immediate)
                    func.apply(context, args);
            };
            var callNow = immediate && !timeout;
            clearTimeout(timeout);
            timeout = setTimeout(later, wait);
            if (callNow)
                func.apply(context, args);
        };
    }
    Utils.debounce = debounce;
    function generateSFUrl(metaServiceURL, projectId, projectVersion, configuration) {
        //Insert slash at the end of the metaServiceUrl if there isn't one already.
        if (metaServiceURL.lastIndexOf("/") !== metaServiceURL.length - 1) {
            metaServiceURL = metaServiceURL + "/";
        }
        if (configuration) {
            return metaServiceURL + "_sf/" + projectId + "/" + projectVersion + "/" + configuration;
        }
        else {
            return metaServiceURL + "_sf/" + projectId + "/" + projectVersion + "/(default)";
        }
    }
    Utils.generateSFUrl = generateSFUrl;
    var Color = /** @class */ (function () {
        function Color(r, g, b) {
            this.r = r;
            this.g = g;
            this.b = b;
        }
        Color.prototype.toNumber = function () {
            return (this.r << 16) + (this.g << 8) + this.b;
        };
        /*
         * Calculate the brightness according to
         * http://www.w3.org/TR/AERT#color-contrast
         */
        Color.prototype.getBrightness = function () {
            return ((this.r * 0.299) + (this.g * 0.587) + (this.b * 0.114)) / 255;
        };
        /*
         * Converts a number (argb format) to hex but leaves out the alpha
         * @param color A value specifying the 32-bit argb value.
         */
        Color.hexFromArgb = function (color) {
            function toHex(n) {
                // The modulo idea comes from: http://www.javascripter.net/faq/rgbtohex.htm
                return "0123456789ABCDEF".charAt((n - n % 16) / 16)
                    + "0123456789ABCDEF".charAt(n % 16);
            }
            return "#" + toHex((color >> 16) & 255) + toHex((color >> 8) & 255) + toHex((color) & 255);
        };
        Color.numberToColor = function (color) {
            var r = (color >> 16) & 255, g = (color >> 8) & 255, b = color & 255;
            return new Color(r, g, b);
        };
        Color.hexToColor = function (color) {
            var rgb = color.replace(/\s/g, '').match(/^rgba?\((\d+),(\d+),(\d+)/i);
            if (rgb) {
                return new Color(parseInt(rgb[1], 10), parseInt(rgb[2], 10), parseInt(rgb[3], 10));
            }
            return new Color(0, 0, 0); // Default black
        };
        return Color;
    }());
    Utils.Color = Color;
    var PasswordStorageType;
    (function (PasswordStorageType) {
        PasswordStorageType[PasswordStorageType["keytar"] = 1] = "keytar";
        PasswordStorageType[PasswordStorageType["SecureStorage"] = 2] = "SecureStorage";
    })(PasswordStorageType || (PasswordStorageType = {}));
    // tslint:disable-next-line:no-stateless-class
    var PasswordEncryption = /** @class */ (function () {
        function PasswordEncryption() {
        }
        PasswordEncryption.initialize = function () {
            this.latestSignedInPasswordKey = BaseModelReader.latestSignedInPasswordKey;
            var deferred = jQuery.Deferred();
            return (function () {
                var deferred = jQuery.Deferred();
                if (Utils.isElectron()) {
                    try {
                        PasswordEncryption.keytar = require("keytar-prebuild");
                        PasswordEncryption.encryptionMethod = PasswordStorageType.keytar;
                        deferred.resolve();
                    }
                    catch (ex) {
                        console.log("Couldn't initialize keytar. Falling back to localStorage. Error:", ex.message);
                        deferred.reject();
                    }
                }
                else if (Utils.isCordova()) {
                    //We need to wait for deviceready because else Cordova plugins won't be loaded yet
                    document.addEventListener("deviceready", function () {
                        if (!window["cordova"].plugins.SecureStorage) {
                            deferred.reject("SecureStorage plugin isn't installed.");
                        }
                        PasswordEncryption.secureStorage = new window["cordova"].plugins.SecureStorage(function () {
                            PasswordEncryption.encryptionMethod = PasswordStorageType.SecureStorage;
                            deferred.resolve();
                        }, function (err) {
                            deferred.reject(err);
                        }, PasswordEncryption.appName);
                    });
                }
                else {
                    deferred.reject("Can't initialize password encryption because the environment is neither Electron nor Cordova.");
                }
                return deferred.promise();
            }().then(function () {
                PasswordEncryption.initializedSuccessfully = true;
                return deferred.resolve().promise();
            }).fail(function (err) {
                PasswordEncryption.initializedSuccessfully = false;
                return deferred.reject(err).promise();
            }));
        };
        PasswordEncryption.isSupported = function () {
            return Utils.isElectron() || Utils.isCordova();
        };
        PasswordEncryption.getLatestPassword = function () {
            return __awaiter(this, void 0, void 0, function () {
                var _this = this;
                return __generator(this, function (_a) {
                    return [2 /*return*/, new Promise(function (resolve, reject) { return __awaiter(_this, void 0, void 0, function () {
                            var passwordFromLS;
                            return __generator(this, function (_a) {
                                switch (_a.label) {
                                    case 0: return [4 /*yield*/, Utils.Preferences.getItem(this.latestSignedInPasswordKey)];
                                    case 1:
                                        passwordFromLS = _a.sent();
                                        if (passwordFromLS !== null) {
                                            if (this.initializedSuccessfully && this.isSupported()) {
                                                //Migrate password from localStorage to keychain
                                                console.log("Migrating password from localStorage to keychain.");
                                                this.setLatestPassword(passwordFromLS).then(function () {
                                                    return __awaiter(this, void 0, void 0, function () {
                                                        return __generator(this, function (_a) {
                                                            switch (_a.label) {
                                                                case 0: return [4 /*yield*/, Utils.Preferences.removeItem(PasswordEncryption.latestSignedInPasswordKey)];
                                                                case 1:
                                                                    _a.sent();
                                                                    resolve(passwordFromLS);
                                                                    return [2 /*return*/];
                                                            }
                                                        });
                                                    });
                                                });
                                            }
                                            else {
                                                console.log("Password gotten from localStorage.");
                                                resolve(passwordFromLS);
                                            }
                                        }
                                        else {
                                            console.log("Password gotten from keychain.");
                                            if (this.encryptionMethod === PasswordStorageType.keytar) {
                                                this.keytar.getPassword(this.appName, this.latestSignedInPasswordKey)
                                                    .then(resolve)
                                                    .catch(reject);
                                            }
                                            else if (this.encryptionMethod === PasswordStorageType.SecureStorage) {
                                                this.secureStorage.get(function (password) {
                                                    resolve(password);
                                                }, function (err) {
                                                    //Prevent failing hard when the password wasn't found. 1st one is iOS, 2nd one is Android.
                                                    if (err.message === "Failure in SecureStorage.get() - The specified item could not be found in the keychain" ||
                                                        err.message === "Key [_SS_" + PasswordEncryption.latestSignedInPasswordKey + "] not found.") {
                                                        return resolve(null);
                                                    }
                                                    reject(err);
                                                }, this.latestSignedInPasswordKey);
                                            }
                                        }
                                        return [2 /*return*/];
                                }
                            });
                        }); })];
                });
            });
        };
        PasswordEncryption.removePassword = function () {
            return __awaiter(this, void 0, void 0, function () {
                var _this = this;
                return __generator(this, function (_a) {
                    return [2 /*return*/, new Promise(function (resolve, reject) { return __awaiter(_this, void 0, void 0, function () {
                            return __generator(this, function (_a) {
                                switch (_a.label) {
                                    case 0:
                                        if (!(!this.initializedSuccessfully || !this.isSupported())) return [3 /*break*/, 2];
                                        return [4 /*yield*/, Utils.Preferences.removeItem(this.latestSignedInPasswordKey)];
                                    case 1:
                                        _a.sent();
                                        resolve();
                                        return [2 /*return*/];
                                    case 2:
                                        if (this.encryptionMethod === PasswordStorageType.keytar) {
                                            this.keytar.deletePassword(this.appName, this.latestSignedInPasswordKey)
                                                .then(resolve)
                                                .catch(reject);
                                        }
                                        else if (this.encryptionMethod === PasswordStorageType.SecureStorage) {
                                            this.secureStorage.remove(function () {
                                                resolve();
                                            }, function (err) {
                                                reject(err);
                                            }, this.latestSignedInPasswordKey);
                                        }
                                        return [2 /*return*/];
                                }
                            });
                        }); })];
                });
            });
        };
        PasswordEncryption.setLatestPassword = function (password) {
            return __awaiter(this, void 0, void 0, function () {
                var _this = this;
                return __generator(this, function (_a) {
                    if (password.length === 0) {
                        return [2 /*return*/, this.removePassword()];
                    }
                    return [2 /*return*/, new Promise(function (resolve, reject) { return __awaiter(_this, void 0, void 0, function () {
                            return __generator(this, function (_a) {
                                switch (_a.label) {
                                    case 0:
                                        if (!(!this.initializedSuccessfully || !this.isSupported())) return [3 /*break*/, 2];
                                        return [4 /*yield*/, Utils.Preferences.setItem(this.latestSignedInPasswordKey, password)];
                                    case 1:
                                        _a.sent();
                                        resolve();
                                        return [2 /*return*/];
                                    case 2:
                                        if (this.encryptionMethod === PasswordStorageType.keytar) {
                                            this.keytar.setPassword(this.appName, this.latestSignedInPasswordKey, password)
                                                .then(resolve)
                                                .catch(reject);
                                        }
                                        else if (this.encryptionMethod === PasswordStorageType.SecureStorage) {
                                            this.secureStorage.set(function (value) {
                                                resolve();
                                            }, function (err) {
                                                reject(err);
                                            }, this.latestSignedInPasswordKey, password);
                                        }
                                        return [2 /*return*/];
                                }
                            });
                        }); })];
                });
            });
        };
        PasswordEncryption.initializedSuccessfully = false;
        //Prefix to prevent name conflicts with other apps
        PasswordEncryption.appName = "Thinkwise_Mobile_" + AppConfig.appName;
        return PasswordEncryption;
    }());
    Utils.PasswordEncryption = PasswordEncryption;
    var Preferences;
    (function (Preferences) {
        var isIOS = Utils.getiOSVersion() !== null;
        var pluginAvailable = null;
        function waitForPluginAvailable() {
            if (pluginAvailable === null) {
                document.addEventListener("deviceready", function () {
                    pluginAvailable = window.hasOwnProperty("plugins") && window["plugins"].hasOwnProperty("appPreferences");
                    if (!pluginAvailable) {
                        throw new Error("Cordova plugin isn't available.");
                    }
                    return Promise.resolve();
                });
            }
            else if (!pluginAvailable) {
                throw new Error("Cordova plugin isn't available.");
            }
            else {
                return Promise.resolve();
            }
        }
        function clear() {
            return __awaiter(this, void 0, void 0, function () {
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0:
                            /*  We also want localStorage to be cleared on iOS because else the migration in 'getItem'
                                will result in outdated localStorage entries getting returned. */
                            localStorage.clear();
                            if (!isIOS) {
                                return [2 /*return*/, Promise.resolve()];
                            }
                            return [4 /*yield*/, waitForPluginAvailable()];
                        case 1:
                            _a.sent();
                            return [4 /*yield*/, window["plugins"].appPreferences.clearAll()];
                        case 2: return [2 /*return*/, _a.sent()];
                    }
                });
            });
        }
        Preferences.clear = clear;
        function getItem(key) {
            return __awaiter(this, void 0, void 0, function () {
                var itemFromAppPreferences, itemFromLocalStorage;
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0:
                            if (!isIOS) {
                                return [2 /*return*/, Promise.resolve(localStorage.getItem(key))];
                            }
                            return [4 /*yield*/, waitForPluginAvailable()];
                        case 1:
                            _a.sent();
                            return [4 /*yield*/, window["plugins"].appPreferences.fetch(key)];
                        case 2:
                            itemFromAppPreferences = _a.sent();
                            if (!(itemFromAppPreferences === null)) return [3 /*break*/, 4];
                            itemFromLocalStorage = localStorage.getItem(key);
                            if (!(itemFromLocalStorage !== null)) return [3 /*break*/, 4];
                            return [4 /*yield*/, Utils.Preferences.setItem(key, itemFromLocalStorage)];
                        case 3:
                            _a.sent();
                            return [2 /*return*/, itemFromLocalStorage];
                        case 4: return [2 /*return*/, itemFromAppPreferences];
                    }
                });
            });
        }
        Preferences.getItem = getItem;
        function removeItem(key) {
            return __awaiter(this, void 0, void 0, function () {
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0:
                            if (!isIOS) {
                                localStorage.removeItem(key);
                                return [2 /*return*/, Promise.resolve()];
                            }
                            return [4 /*yield*/, waitForPluginAvailable()];
                        case 1:
                            _a.sent();
                            return [4 /*yield*/, window["plugins"].appPreferences.remove(key)];
                        case 2: return [2 /*return*/, _a.sent()];
                    }
                });
            });
        }
        Preferences.removeItem = removeItem;
        function setItem(key, data) {
            return __awaiter(this, void 0, void 0, function () {
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0:
                            if (!isIOS) {
                                localStorage.setItem(key, data);
                                return [2 /*return*/, Promise.resolve()];
                            }
                            return [4 /*yield*/, waitForPluginAvailable()];
                        case 1:
                            _a.sent();
                            return [4 /*yield*/, window["plugins"].appPreferences.store(key, data)];
                        case 2: return [2 /*return*/, _a.sent()];
                    }
                });
            });
        }
        Preferences.setItem = setItem;
    })(Preferences = Utils.Preferences || (Utils.Preferences = {}));
})(Utils || (Utils = {}));
