var badgeHelper;
(function (badgeHelper) {
    var BadgeInfo = /** @class */ (function () {
        function BadgeInfo(executor, objView) {
            this.executor = executor;
            this.objView = objView;
            this.subscribers = [];
        }
        BadgeInfo.prototype.notify = function (newValue, error) {
            if (error) {
                this.currentValue = null;
                this.subscribers.forEach(function (item) {
                    item.func(null, error);
                });
            }
            else if (this.currentValue !== newValue) {
                this.currentValue = newValue;
                this.subscribers.forEach(function (item) {
                    item.func(newValue, null);
                });
            }
        };
        BadgeInfo.prototype.subscribe = function (func) {
            var token = {};
            this.subscribers.push({ token: token, func: func });
            if (this.subscribers.length == 1) {
                this.execute();
            }
            return token;
        };
        BadgeInfo.prototype.unsubscribe = function (token) {
            for (var i = 0; i < this.subscribers.length; i++) {
                if (this.subscribers[i].token == token) {
                    this.subscribers.splice(i, 1);
                    break;
                }
            }
            if (this.subscribers.length == 0) {
                this.clearTimer();
            }
        };
        BadgeInfo.prototype.clearSubscribers = function () {
            this.subscribers = [];
            this.clearTimer();
        };
        BadgeInfo.prototype.execute = function () {
            var _this = this;
            if (this.subscribers.length == 0) {
                return;
            }
            var last = this.lastExecute = {};
            this.executor().then(function (value) {
                if (last == _this.lastExecute) {
                    _this.notify(value, null);
                }
            }, function (status) {
                if (last == _this.lastExecute) {
                    _this.notify(null, status);
                }
            }).always(function () {
                if (last == _this.lastExecute && _this.objView.badgeInterval != 0 && _this.subscribers.length > 0) {
                    _this.startTimer();
                }
            });
        };
        BadgeInfo.prototype.clearTimer = function () {
            clearTimeout(this.timer);
            this.timer = null;
        };
        BadgeInfo.prototype.startTimer = function () {
            var _this = this;
            this.timer = window.setTimeout(function () { return _this.execute(); }, this.objView.badgeInterval * 1000);
        };
        return BadgeInfo;
    }());
    badgeHelper.BadgeInfo = BadgeInfo;
    function clearTimers(badgeInfoMap) {
        for (var key in badgeInfoMap) {
            badgeInfoMap[key].clearTimer();
        }
    }
    badgeHelper.clearTimers = clearTimers;
    function clearSubscribers(badgeInfoMap) {
        for (var key in badgeInfoMap) {
            badgeInfoMap[key].clearSubscribers();
        }
    }
    badgeHelper.clearSubscribers = clearSubscribers;
    function updateBadges(badgeInfoMap) {
        for (var key in badgeInfoMap) {
            badgeInfoMap[key].execute();
        }
    }
    badgeHelper.updateBadges = updateBadges;
    function executeBadge(application, badgeInfo, contract) {
        var deferred = jQuery.Deferred();
        if (badgeInfo.objView.badgeInterval == 0 && badgeInfo.lastExecuted) {
            // Skip badge - already executed and interval was 0
            return deferred.resolve(badgeInfo.currentValue).promise();
        }
        // TODO: Add support for offline badges
        if (!AppConfig.online) {
            deferred.resolve(null);
            return deferred.promise();
        }
        badgeInfo.lastExecuted = new Date();
        var request = RequestBuilderFactory.createOnlineRequestBuilder(application, badgeInfo.objView);
        request.setCallbackMethod(function (reader, status) {
            if (status.success && reader.length == 1) {
                var value = reader.item(0).getNullableNumber("value");
                deferred.resolve(value);
            }
            else {
                deferred.reject(status);
            }
        });
        request.executeBadge(badgeInfo.objView, contract);
        return deferred.promise();
    }
    badgeHelper.executeBadge = executeBadge;
})(badgeHelper || (badgeHelper = {}));
