var taskHelper;
(function (taskHelper) {
    function showPageOrPopup(taskView, contextController) {
        if (contextController) {
            if (contextController.isInEditMode && contextController.currentEditor.allowActions()) {
                // first validate the editor when default form editmode is active to sync everything to the controller and save the changes
                contextController.currentEditor.validateEditors().always(function () { return showPageOrPopupInternal(taskView, contextController); });
            }
            else if (!contextController.isInEditMode) {
                showPageOrPopupInternal(taskView, contextController);
            }
        }
        else {
            showPageOrPopupInternal(taskView, null);
        }
    }
    taskHelper.showPageOrPopup = showPageOrPopup;
    function showPageOrPopupInternal(taskView, contextController) {
        var appl = Model.TSFApplicationManager.Current.activeAppl;
        var paramController = TSFController.getOrCreateTaskController(appl, taskView, contextController);
        // Initalize always returns with a resovled promise for tasks
        var promise = paramController.initialize(false).then(function () {
            return paramController.newRow(new TaskFormEditor(paramController));
        });
        promise.always(function () {
            var visibleCols = false;
            var props = paramController.obj.objProps.array;
            var layoutState = paramController.layoutState.fieldLayoutState;
            for (var i = 0; i < props.length; i++) {
                visibleCols = layoutState[props[i].objPropId] >= 2 /* ReadOnly */;
                if (visibleCols) {
                    break;
                }
            }
            if (!visibleCols) {
                var taskPromise_1 = taskHelper.tryExecuteTask(paramController, contextController, taskView);
                taskPromise_1.always(function () {
                    if (contextController) {
                        contextController.taskActionPerformed(taskView, taskPromise_1.state() === "resolved");
                    }
                });
                return;
            }
            var popup = new TaskPopup(taskView);
            popup.initialize(jQuery.mobile.activePage, paramController, contextController);
            //Reset the URL to menu.html without task parameter so going to the menu from a table doesn't result
            //in the task being opened again
            popup.onDestroy.push(function () {
                if (window.location.href.indexOf("menu.html") >= 0) {
                    var urlWithoutTaskParam = Utils.removeParameterFromUrl(window.location.href, "task");
                    NavigationManager.navigate(urlWithoutTaskParam, { transition: "none", allowSamePageTransition: true, reloadPage: true });
                }
            });
        });
    }
    function tryExecuteTask(taskController, contextController, taskView) {
        if (taskView.askConfirmation) {
            var deferred_1 = jQuery.Deferred(), messageOptions = [Enum.ConfirmationOption.Yes, Enum.ConfirmationOption.No], transl = taskController.application.language.transl(Model.TranslKeys.taskConfirmQuestion).replace("[taskTransl]", taskView.taskTransl.transl);
            NavigationManager.showMessageWhenReady(taskView.taskTransl.transl, [transl], messageOptions, Enum.ConfirmationOption.No).always(function (result) {
                if (result == Enum.ConfirmationOption.Yes) {
                    var execTaskPromise_1 = executeTask(taskController, contextController, taskView);
                    execTaskPromise_1.always(function () {
                        if (execTaskPromise_1.state() == "resolved") {
                            taskController.clearControllerEditor();
                            deferred_1.resolve();
                        }
                        else {
                            deferred_1.reject();
                        }
                    });
                    if (contextController) {
                        contextController.currentAction = execTaskPromise_1;
                    }
                }
                else {
                    deferred_1.reject();
                }
            });
            return deferred_1.promise();
        }
        else {
            var execTaskPromise = executeTask(taskController, contextController, taskView);
            if (contextController) {
                contextController.currentAction = execTaskPromise;
            }
            return execTaskPromise;
        }
    }
    taskHelper.tryExecuteTask = tryExecuteTask;
    function executeTask(taskController, contextController, taskView) {
        var taskPromise = taskController.executeTask(taskView, contextController ? contextController.obj : null);
        if (contextController) {
            var deferred_2 = jQuery.Deferred();
            taskPromise.always(function (syncGroupAlreadySynced) {
                taskController.clearControllerEditor();
                // Only refresh the context controller when refresh is enabled on the TaskLink.
                var taskLink = contextController.obj.tasks.object[taskView.task.taskId];
                // TODO: Implement the different refresh options
                if (taskLink && taskLink.refresh >= 0 /* ActiveRow */) {
                    contextController.refreshData(false, true).always(function () {
                        if (taskPromise.state() == "resolved") {
                            deferred_2.resolve();
                        }
                        else {
                            deferred_2.reject();
                        }
                    });
                }
                else {
                    if (taskPromise.state() == "resolved") {
                        deferred_2.resolve();
                    }
                    else {
                        deferred_2.reject();
                    }
                }
                // Also refresh the offline dataset when its enabled on the TaskLink.
                if (!syncGroupAlreadySynced && taskLink && taskLink.refreshOffline) {
                    // Execute a localsync for the current obj, it will do a full synchronize when its changed.
                    taskController.application.localSync.refreshSyncGroup(contextController.objView);
                }
            });
            return deferred_2.promise();
        }
        else {
            return taskPromise;
        }
    }
})(taskHelper || (taskHelper = {}));
