var electron = require("electron");
var app = electron.app;  // Module to control application life.
var BrowserWindow = electron.BrowserWindow;  // Module to create native browser window.
var path = require('path');


// Keep a global reference of the window object, if you don't, the window will
// be closed automatically when the javascript object is GCed.
var mainWindow = null;

app.setPath('userData', path.join(process.cwd(), "cache"));
app.setPath('userCache', path.join(process.cwd(), "cache"));

// Quit when all windows are closed.
app.on('window-all-closed', function() {
  if (process.platform != 'darwin') {
    app.quit();
  }
});

var preferredSize = undefined;

var argv = process.argv.slice(1);
var option = { debug: false};
for (var i in argv) {
  if (argv[i] == '--debug' || argv[i] == '-d') {
    option.debug = true;
  } else if (argv[i].indexOf('--page=') === 0) {
    option.page = argv[i].substring('--page='.length);
  } else if (argv[i] == '--phone') {
    preferredSize = { width: 360, height: 600, "use-content-size": true };
  } else if (argv[i] == '--tablet') {
    preferredSize = { width: 1024, height: 600, "use-content-size": true };
  } else if (argv[i].indexOf('--customconfig=') === 0) {
    option.customConfig = argv[i].substring('--customconfig='.length);
  }
}

// This method will be called when Electron has done everything
// initialization and ready for creating browser windows.
app.on('ready', function() {
  // App opens on the primary display and mimics the old viewer behavior for screen size
  if (preferredSize == undefined) {
    var atomScreen = electron.screen;
    var size = atomScreen.getPrimaryDisplay().workAreaSize;
    preferredSize = { width: size.width * 0.75, height: size.height * 0.75 };
  }
  // Create the browser window.
  mainWindow = new BrowserWindow(preferredSize);
  mainWindow.setMenu(null);

  // and load the index.html of the app.
  if (option.page) {
    mainWindow.loadURL('file://' + option.page);
  } else {
    mainWindow.loadURL('file://' + __dirname + '/www/index.html');
  }

  // Open the devtools on startup and on F12 if in debug mode
  if (option.debug) {
    mainWindow.openDevTools();
    electron.ipcMain.on('open-devtools', function() {
      mainWindow.openDevTools();
    });

    mainWindow.webContents.on('did-finish-load', function() {
      mainWindow.webContents.send('is-in-debug-mode');
    });
  }

  if (option.customConfig) {
    mainWindow.webContents.on('did-finish-load', function() {
      var fs = require('fs');
      fs.readFile(option.customConfig, function(err, data) {
        var newConfig = JSON.parse(data);
        mainWindow.webContents.send('set-custom-config', newConfig);
      });
    });
  }

  //Fix the context menu not working on right click (TCP 52837)
  //source: https://github.com/electron/electron/issues/4068#issuecomment-274159726
  const selectionMenu = electron.Menu.buildFromTemplate([
    {role: 'copy'},
    {type: 'separator'},
    {role: 'selectall'},
  ]);

  const inputMenu = electron.Menu.buildFromTemplate([
    {role: 'undo'},
    {role: 'redo'},
    {type: 'separator'},
    {role: 'cut'},
    {role: 'copy'},
    {role: 'paste'},
    {type: 'separator'},
    {role: 'selectall'},
  ]);

  mainWindow.webContents.on('context-menu', (e, props) => {
    const { selectionText, isEditable } = props;
    if (isEditable) {
      inputMenu.popup(mainWindow);
    } else if (selectionText && selectionText.trim() !== '') {
      selectionMenu.popup(mainWindow);
    }
  });

  //Prevent our app opening files dragged on it.
  mainWindow.webContents.on("will-navigate", (e, url) => {
    if (url !== mainWindow.webContents.getURL()) {
      e.preventDefault();
    }
  });

  //Prevent scrollwheel click opening new window (TCP 53148)
  mainWindow.webContents.on("new-window", (e, url) => {
    //Only prevent urls starting with file://,
    //because else we prevent opening images in a new window too
    if (url.indexOf("file://") === 0) {
      e.preventDefault();
    }
  });

  // Emitted when the window is closed.
  mainWindow.on('closed', function() {
    // Dereference the window object, usually you would store windows
    // in an array if your app supports multi windows, this is the time
    // when you should delete the corresponding element.
    mainWindow = null;
  });
});
